/**
 * 图表系列Glyph，包含了整个系列的画法
 * @example
 * var dataseries = new FR.Chart.DataSeries();
 *
 * @class FR.Chart.DataSeries
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 * @param {Array} [js.points = []] 系列点数组
 * @param  {Number} [js.seriesIndex = 0] 系列序号.
 * @param {String} [js.seriesName = ""] 系列名
 * @param {boolean} [js.seriesDragEnable = false] 系列是否支持拖拽
 * @param {Number} [js.paintState = FR.Chart.ChartPaintConstatns.STATE_ENABLE] 当前系列的绘画状态
 * @param {Array} [js.trendLineList = []] 趋势线数组.
 * @param {Object} [js.dataSeriesStyle = null] 系列风格
 *
 */
FR.Chart.DataSeries = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.points = [];
        if (js.dataPoints && js.dataPoints.length > 0 && js.dataPointType) {
            var dataPoints = js.dataPoints || {};
            for (var i = 0; i < dataPoints.length; i++) {
                this.points[i] = new FR.Chart[js.dataPointType](dataPoints[i]);
            }
        }

        this.seriesIndex = FR.pick(js.seriesIndex, 0);
        this.seriesName = js.seriesName;
        this.seriesDragEnable = FR.pick(js.seriesDragEnable, false);

        this.paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
        this.trendLineList = [];// 趋势线
        this.dataSeriesStyle = null;

        this.jsType = "DataSeries";
        // kunsnat: 因为这个draw的glyph是在layout之后创建art的
        this.glyph = null;
        this.lastGlyph = null;

        // 面积图的上层折线类型. FoldLine, drawAfter之后
        this.topFoldLine = null;
        this.shouldDrawLable = true;
    },

    /**
     * 合并两个系列
     * @param dataSeries 合并的系列
     */
    addDataSeries: function(dataSeries){
        for(var i = 0; i < this.points.length; i++){
            this.points[i].addDataPoint(dataSeries.points[i]);
        }
        this.seriesName = this.seriesName + "&" + dataSeries.seriesName;
    },

    toResult:function(columnName){
        if (FR.equals(columnName, 'SeriesIndex')){
            return this.seriesIndex + 1;
        } else if (FR.equals(columnName, 'SeriesName')) {
            return this.seriesName;
        } else if (FR.equals(columnName, 'ProjectID')){
            return this.seriesName;
        }
        return "";
    },

    equals:function(obj){
        if(!obj){
            return false;
        }

        return obj.jsType == this.jsType && obj.seriesIndex == this.seriesIndex;
    },

    /**
     * 遍历系列 为每个点创建热点.
     * @param hotTooltipStyle 热点提示格式
     * @param bounds plot
     * @param style 样式
     */
    createHotTooltip: function (hotTooltipStyle, bounds, style){
        this.outerBounds = bounds;
        this.isTooltip4Series = false;
        this.tooltipStyle = style;
        for (var i = 0; i < this.points.length; i++) {
            if (this.points[i]) {
                this.points[i].createHotTooltip(hotTooltipStyle, bounds, style);
            }
        }
    },

    /**
     * 遍历系列 为每个点创建热点.
     * @param hotTooltipStyle 热点提示格式
     * @param bounds plot
     * @param style 样式
     */
    createHotTooltip4Series: function (hotTooltipStyle, bounds, style, colors){
        this.outerBounds = bounds;
        this.isTooltip4Series = true;
        this.tooltipStyle = style;
        for (var i = 0; i < this.points.length; i++) {
            if (this.points[i]) {
                this.points[i].createHotTooltip4Series(hotTooltipStyle, bounds, style, colors);
            }
        }
    },

    /**
     * 返回系列点的个数
     * @returns {number} 系列点个数
     */
    getDataPointCount: function () {
        var count = 0;
        if (this.points) {
            count = this.points.length;
        }
        return count;
    },

    /**
     * 画出系列
     * @param ctx 画布
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctx, time, actionModel) {
        if (this.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
            || this.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {//透明也不画
            return;
        }
        if (this.glyph) {
            this.glyph.draw(ctx, time, actionModel);
        }

        var dataPoints = this.points;
        for (var i = 0; i < this.getDataPointCount(); i++) {
            if (dataPoints[i] != null) {
                dataPoints[i].draw(ctx, time);
            }
        }
    },

    /**
     * 系列 为缩略图画 dataSeries4Demo
     * @param ctx 画布
     */
    draw4Demo: function (ctx) {
        if (this.glyph) {
            this.glyph.draw4Demo(ctx);
        }
    },

    /**
     * 画标签
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawLabel: function (ctx, currentDivKey) {
        if(!this.shouldDrawLable){
            return;
        }

        for (var i = 0; i < this.getDataPointCount(); i++) {
            if (this.points[i]) {
                this.points[i].drawLabel(ctx, currentDivKey);
            }
        }
    },

    /**
     * kunsnat: 得到动态画法的每个步骤的画的glyph数组
     * @param step 动态步骤
     * @param actionModel 动态方式
     */
    getAnimateGlyph: function (step, actionModel) {
        if (this.paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {// 产生, 消失, 全部需要过程.
            return;
        }
        if (this.trendLineList) {
            this.trendLineAnimateGlyph = [];
            for (var i = 0; i < this.trendLineList.length; i++) {
                this.trendLineAnimateGlyph[i] = this.trendLineList[i].getAnimateGlyph(step, actionModel);
            }
        }

        if (this.glyph || this.lastGlyph) {
            this.animateGlyph = (this.glyph || this.lastGlyph).getAnimateGlyph(step, actionModel, this.lastGlyph, this.paintState);
            if (!this.isPointAnimate) {
                return;
            }
        }

        var dataPoints = this.points;
        for (var i = 0; i < this.getDataPointCount(); i++) {
            dataPoints[i].getAnimateGlyph(step, actionModel, this.paintState);
        }
    },

    /**
     * 返回已有的动态步骤个数.
     * @returns {Number} 动态步骤个数
     */
    getAnimateStepNumber: function () {
        var stepNumber = 0;
        if (this.animateGlyph) {
            stepNumber = this.animateGlyph.length;
        }

        if (this.trendLineAnimateGlyph && stepNumber <= 0 && this.trendLineAnimateGlyph[0]) {
            // 趋势线的步骤保持和系列一致. 类似折线不显示线的时候
            return this.trendLineAnimateGlyph[0].length;
        }

        if (this.points && stepNumber <= 0) {
            var i = 0;
            //这边加一个空值判断的原因是：空值 -- dataPoint.glyph=null -- animateGlyph.length=0。bug：74816
            //这边加一个length==0的原因是：dataPoint不在时间坐标轴范围内 dataPoint.glyph=null -- animateGlyph.length=0。bug:74678
            while (i < this.points.length && (!this.points[i].animateGlyph || this.points[i].valueIsNull || this.points[i].animateGlyph.length === 0)) {
                i++;
            }
            if (i < this.points.length && this.points[i].animateGlyph) {
                return this.points[i].animateGlyph.length;
            }
        }

        return stepNumber;
    },

    /**
     * 查找当前鼠标onMove响应的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件触发
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (this.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
            || this.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
            return;
        }

        var shape = this.glyph ? this.glyph.getShape().getInteractivePath() : null;

        // 先查找系列, 然后系列点 从效率上更合理.(鼠标按下点击的时候还是要先查找系列的数据点,见下面注释)
        if(this.seriesDragEnable && event._isMouseDown ) {// 查找拖拽情况, 不需要响应热点提示
            if(FR.Chart.GraphHelper.isDrawingEnd){
                //数据点优先判断选中，然后才是系列选中,不然会出现折线图和面积图等数据点选中了，但是点击超链没反应,bug65770
                for (var i = 0; i < this.getDataPointCount(); i++) {
                    this.points[i].findOnMoveGlyph(mousePos, event);
                    if(event._hasfound) {
                        break;
                    }
                }

                if(!event._hasfound && shape && shape.containsPoint(mousePos.x, mousePos.y)){
                    event._hasfound = true;
                    event._lastHover = this;
                }

                if(event._hasfound && !event._isMouseMoving) {
                    event._draggingTarget = this; //记录需要拖拽的系列
                    this.paintState = FR.Chart.ChartPaintConstants.STATE_TRANSPARENT;
                }
            }
        } else {
            this.nearByPoint = null;

            for (var i = this.getDataPointCount() - 1; i >= 0; i--) {
                this.points[i].findOnMoveGlyph(mousePos, event);
                if(event._hasfound) {
                    break;
                }
            }

            if(event._hasfound){
                return;
            }

            if (shape && shape.containsPoint(mousePos.x, mousePos.y)) {
                event._hasfound = true;
                event._lastHover = this;

                var min = Number.MAX_VALUE;

                for (var i = 0; i < this.getDataPointCount(); i++) {
                    if (this.points[i].glyph) {
                        var bounds = this.points[i].glyph.getShape().getBounds2D();
                        var tmpValue = this.getAbsLength(bounds, mousePos);
                        if (tmpValue < min && this.points[i].tooltip != null) {
                            min = tmpValue;
                            this.nearByPoint = this.points[i];
                        }
                    }
                }
            }

        }
    },

    /**
     * 计算绝对 直线距离.
     * @param bounds  边界
     * @param mousePos 鼠标位置
     * @return {number} 返回长度
     */
    getAbsLength : function(bounds, mousePos) {
        if(!bounds || !mousePos) {
            return Number.MAX_VALUE;
        }

        var cx = bounds.x + bounds.width/2;
        var cy = bounds.y + bounds.height/2;

        return (cx - mousePos.x) * (cx - mousePos.x) + (cy - mousePos.y)*(cy - mousePos.y);
    },

    /**
     * 在OnMove时响应时间 高亮 焦点响应
     * @param ctx 画布
     * @param mousePos 鼠标位置
     */
    doOnMove:function(ctx, mousePos, plotGlyph){
        //不是面积图并且没有标记点类型的时候
        var isMarkGlyph = this.points[0] && this.points[0].glyph &&
            this.points[0].glyph.marker && this.points[0].glyph.marker.getMarkerType;

        if(!plotGlyph instanceof FR.Chart.Area3DPlotGlyph) {
            FR.Chart.HighUtils.doFocus(this.glyph, ctx);
        }

        if(isMarkGlyph){
            if(!ctx){     return; } ctx.save();
            ctx.globalCompositeOperation = "destination-out";
            if(plotGlyph !== undefined && plotGlyph !== null){
                for(var seriesIndex = 0;seriesIndex<plotGlyph.dataSeries.length;seriesIndex++){
                    var series = plotGlyph.dataSeries[seriesIndex];
                    if(series.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
                                ||series.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
                        continue;
                    }
                    for(var index = 0;index<this.points.length;index++){
                        var glyph = series.points[index].glyph;
                        if(glyph) {
                            glyph.draw(ctx);
                        }
                    }

                }
            }else{
                for(var index = 0;index<this.points.length;index++){
                    var glyph = this.points[index].glyph;
                    if(glyph){
                        glyph.draw(ctx);
                    }
                }
            }

            ctx.restore();
        }

        //画数据点提示
        if(this.nearByPoint && plotGlyph.isTooltipEnable) {
            var point = this.nearByPoint;
            if (this.outerBounds && this.outerBounds.dataSeries) {
                this.outerBounds.currentChosenSeriesIndex = this.seriesIndex;
                if (this.isTooltip4Series) {
                    var colors = [];
                    var seriesIndex = 0;
                    var top = this.outerBounds.bounds.y + this.outerBounds.bounds.height;
                    var left = this.outerBounds.bounds.x;
                    for (var i = 0; i < this.outerBounds.dataSeries.length; i++) {
                        var dataSeries = this.outerBounds.dataSeries[i];
                        if (dataSeries && dataSeries.points && dataSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                            var dataPoint = dataSeries.points[point.categoryIndex];
                            var glyph = dataPoint.glyph;
                            this.drawChosenPoint(dataPoint, ctx);
                            if (dataSeries.glyph && dataSeries.glyph.toolTipBackground) {
                                colors[i] = dataSeries.glyph.toolTipBackground.color;
                            } else if (glyph.toolTipBackground){
                                colors[i] = glyph.toolTipBackground.color;
                            }
                            var x = 0;
                            var y = 0;
                            if (glyph.shape) {
                                x = glyph.shape.x;
                                y = glyph.shape.y;
                            } else if (glyph.bounds) {
                                x = glyph.bounds.x;
                                y = glyph.bounds.y;
                            } else {
                                x = glyph.marker.x;
                                y = glyph.marker.y;
                            }
                            if (y < top || y == top && x > left) {
                                top = y;
                                left = x;
                                seriesIndex = i;
                            }
                        }
                    }
                    if (this.outerBounds.dataSeries[seriesIndex] && this.outerBounds.dataSeries[seriesIndex].points[point.categoryIndex]) {
                        point = this.outerBounds.dataSeries[seriesIndex].points[point.categoryIndex];
                    }
                    if (point) {
                        FR.Chart.Tooltip.dataPointTip4Series(ctx, point.tooltip, mousePos, point.glyph, point.glyph.toolTipBackground,
                            this.outerBounds, this.tooltipStyle, colors);
                    }
                } else {
                    FR.Chart.Tooltip.dataPointTip4Point(ctx, point.tooltip, mousePos, point.glyph, this.glyph.toolTipBackground,
                        this.outerBounds, this.tooltipStyle);
                }
            }
        }

        //画选中效果
        if(this.nearByPoint){
            this.drawChosenPoint(this.nearByPoint, ctx);
        }
    },

    drawChosenPoint:function (point, ctx) {
        if(point.glyph && point.glyph.marker && point.glyph.marker.getMarkerType){//kunsnat: 热点提示线型标记的单独画法
            point.glyph.drawOnMove = true;
            point.glyph.draw(ctx);
            point.glyph.drawOnMove = false;
        } else {
            FR.Chart.HighUtils.doFocus(point.glyph, ctx);
        }
    },

    /**
     * 响应趋势线的热点提示.
     * @param newMousePos 鼠标位置
     * @param trendLineTips 趋势线的热点提示数组.
     */
    doTrendLineTooltip: function (newMousePos, trendLineTips) {
        if (this.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return;
        }
        if (this.trendLineList && this.trendLineList.length > 0) {
            for (var i = 0; i < this.trendLineList.length; i++) {
                var trendLine = this.trendLineList[i];

                var x = newMousePos.x; // PlotBounds的x, 上层计算好的.
                var plotValue = trendLine.calculator(x); // x 对应值轴下的时候. TODO 横向错误.
                var value = trendLine.valueAxisGlyph.getPointValue4TrendLine(plotValue, newMousePos.y);

                var tooltip = new FR.Chart.TooltipString();
                tooltip.formatValue(trendLine.trendLineName + " : " + value);
                // 通过trendLine, 坐标x计算出value.  String 然后统一去画
                trendLineTips[trendLineTips.length] = {text: [tooltip],
                    lineColor: trendLine.lineStyleInfo.attrLineColor.lineColor};
            }
        }
    },

    /**
     * 按照动态步骤 画系列
     * @param ctx 画布
     * @param index 动态步骤序号
     */
    drawAnimateGlyph: function (ctx, index) {// 画时, 产生和消失都需要动态过程.
        if (this.paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE
            || this.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
            return;
        }
        if (this.animateGlyph && this.animateGlyph.length > 0 && index < this.animateGlyph.length) {
            this.animateGlyph[index].draw(ctx);
        }
        var dataPoints = this.points;
        for (var i = 0; i < this.getDataPointCount(); i++) {
            dataPoints[i].drawAnimateGlyph(ctx, index);
            if(dataPoints[i].dataPointStyle && dataPoints[i].dataPointStyle.paintHighLight){
                dataPoints[i].dataPointStyle.paintHighLight(ctx);
            }
        }
    },

    /**
     * 按照动态步骤画出趋势线
     * @param ctx 画布
     * @param index 动态步骤序号
     */
  drawAnimateGlyph4TrendLine: function (ctx, index) {
        if (this.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
            || this.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {//产品确认，系列拖拽的时候趋势线不显示
            return;
        }
        if (this.trendLineAnimateGlyph[0] && this.trendLineAnimateGlyph.length > 0
            && index < this.trendLineAnimateGlyph[0].length) {
            for (var i = 0; i < this.trendLineAnimateGlyph.length; i++) {
                if (this.trendLineAnimateGlyph[i]) {
                    var trendLineArray = this.trendLineAnimateGlyph[i];
                    if (trendLineArray[index]) {
                        trendLineArray[index].draw(ctx);
                    }
                }
            }
        }
    }

});

/**
 * 面积图的系列
 * @example
 * var series4Area = new FR.Chart.DataSeries4Area();
 *
 * @class FR.Chart.DataSeries4Area
 * @extends FR.Chart.DataSeries
 *
 * @param {JSON} js 配置属性
 */
FR.Chart.DataSeries4Area = FR.extend(FR.Chart.DataSeries, {

    /**
     * 返回 动态步骤的Glyph
     * @param step 步骤步骤个数
     * @param actionModel 动态方式.
     */
    getAnimateGlyph: function (step, actionModel) {
        if (this.paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return;
        }
        if (this.topFoldLine && (this.glyph || this.lastGlyph)) {
            this.topFoldLineAnimateGlyph = (this.glyph || this.lastGlyph).getAnimateGlyph4TopFoldLine(step, actionModel, this.topFoldLine,
                this.lastGlyph, this.paintState);
        }

        FR.Chart.DataSeries4Area.superclass.getAnimateGlyph.apply(this, [step, actionModel]);
    },

    /**
     * 画出最终的面积图的最终 顶部折线.
     * @param ctx 画布
     * @param count 动态步骤序号
     */
    drawAnimateGlyph4TopFoldLine: function (ctx, count) {
        if (this.paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return;
        }
        if (this.topFoldLineAnimateGlyph && this.topFoldLineAnimateGlyph.length > 0
            && count < this.topFoldLineAnimateGlyph.length) {
            this.topFoldLineAnimateGlyph[count].draw(ctx);
        }
    }
});