/*
 * Copyright (c) 2001-2014,FineReport Inc, All Rights Reserved.
 */

/**
 * 数据表对应的Glyph
 * @class FR.Chart.DataSheetGlyph
 * @extends FR.Chart.RectangleGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Object} [js.font = new FR.Chart.FRFont()]; 数据表的字体
 * @param {Object} [js.format = null] 数据格式
 * @param {Object} [js.items = []] 数据表图例选项
 */
FR.Chart.DataSheetGlyph = FR.extend(FR.Chart.RectangleGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.font = js.font == undefined ? null : new FR.Chart.FRFont(js.font);
        this.format = js.format;
        this.items = null;
        if (js.items) {
            this.items = [];
            var legendItems = js.items;
            for (var i = 0; i < legendItems.length; i++) {
                this.items[i] = new FR.Chart.LegendItem(legendItems[i]);
            }
        }

        FR.Chart.DataSheetGlyph.superclass.initConstants.apply(this, [js]);

        this.seriesList = [];
        this.beginX = 0;
        this.unitLength = 0;
        this.seriesRowHeight = 0;
        this.endLegendWidth = 0;
        this.cateRowCout = 1;
        this.tooltipLabelBoundsList = [];
        this.tooltipLabelList = [];
        this.PADDING_GAP = 4;
        this.MAX_CATE_LINE = 4;
    },

    /**
     * 返回分类个数
     * @returns {Number} 分类个数
     */
    getCategoryCount: function () {
        var cateCount = 0;
        if (this.seriesList[0]) {
            cateCount = this.seriesList[0].points.length;
        }

        return cateCount;
    },

    /**
     * 返回当前数据表使用字体
     * @returns {FR.Chart.FRFont} 字体
     */
    getFont: function () {
        return this.font == null ? new FR.Chart.FRFont() : this.font;
    },

    /**
     * 返回文本格式
     * @returns {FR.Chart.TextAttr} 文本格式
     */
    getTextAttr: function () {
        var textAttr = new FR.Chart.TextAttr();
        textAttr.frFont = this.getFont();
        return textAttr;
    },

    /**
     * 设置单位 长度, 即每个分类占用的长度.
     * @param {Number} unit 长度
     */
    setUnitLength: function (unit) {
        this.unitLength = unit;
    },

    /**
     * 返回预计算的占据空间大小
     * @param chartGlyph 图表Glyph
     * @returns {FR.Chart.Dimension2D} 返回空间大小.
     */
    getPrefferedSize: function (chartGlyph) {
        if (this.seriesList.length < 1) {
            return new FR.Chart.Dimension2D();
        }

        this._initSeriesRowHeightAndEndLegendWidth(chartGlyph);

        var categoryMax = this.getMaxCateString(chartGlyph) || '';

        var tag = this.getCharNumber(this.getTextAttr(), this.unitLength);
        if (tag === 0) {
            this.cateRowCout = 1;
        } else {
            var times = Math.max(Math.ceil(categoryMax.length / tag), 1);// kunsnat: 多余的另起一行. 防止为0.
            this.cateRowCout = Math.min(this.MAX_CATE_LINE, times);
        }

        return new FR.Chart.Dimension2D(this.endLegendWidth + this.PADDING_GAP * 2,
            (this.seriesRowHeight + this.PADDING_GAP * 2) * (this.seriesList.length) + this.cateRowCout * this.seriesRowHeight + this.PADDING_GAP * 2);
    },

    /**
     * 初始化 数据表 需要的高度 宽度 图例列所占的宽度 分类实际的高度.
     * @private
     */
    _initSeriesRowHeightAndEndLegendWidth: function (chartGlyph) {
        this.seriesRowHeight = Number.MIN_VALUE;

        var width = Number.MIN_VALUE;
        var limit = chartGlyph.getBounds().width / 4;// 文档规定. 最多换行4
        //如果系列名比较长
        for (var i = 0; i < this.seriesList.length; i++) {
            var dim = null;
            if (this.items != null && this.items.length >= this.seriesList.length) {
                dim = this.items[i].prefferedSize(this.getFont());
            } else {
                dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.seriesList[i].seriesName, this.getTextAttr());
            }

            if (dim.width > width) {
                width = Math.min(dim.width, limit);
            }
            if (dim.height > this.seriesRowHeight) {
                this.seriesRowHeight = dim.height;
            }

        }
        this.endLegendWidth = width; // 最终图例使用的宽度
    },

    getMaxCateString: function (chartGlyph) {
        var count = chartGlyph.plotGlyph.getCategoryCount();
        //如果分类名比较长
        var dimMax = new FR.Chart.Dimension2D();
        var categoryMax = '';
        for (var i = 0; i < count; i++) {
            var categoryName = this.seriesList[0].points[i].categoryName;
            var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(categoryName, this.getTextAttr());
            if (dim.width > dimMax.width) {
                dimMax = dim;
                categoryMax = categoryName;
            }
        }

        return categoryMax
    },

    /**
     * 布局 位置计算.
     * @param plotGlyph 绘图区
     */
    doLayout: function (plotGlyph) {
        if (plotGlyph instanceof FR.Chart.CategoryPlotGlyph) {
            var position = plotGlyph.xAxisGlyph.position;
            if (position == FR.Chart.Constants.TOP || position == FR.Chart.Constants.BOTTOM) {
                this.beginX = plotGlyph.xAxisGlyph.getBounds().x;
                this.unitLength = plotGlyph.xAxisGlyph.getBounds().width / plotGlyph.getCategoryCount();
                this.getBounds().width = this.beginX + plotGlyph.xAxisGlyph.axisLength - this.getBounds().x;
            } else {
                this.beginX = plotGlyph.yAxisGlyph.getBounds().x;
                this.unitLength = (this.getBounds().x + this.getBounds().width - this.beginX) / plotGlyph.getCategoryCount();
                this.getBounds().width = this.beginX + plotGlyph.yAxisGlyph.axisLength - this.getBounds().x;
            }
        } else if (plotGlyph instanceof FR.Chart.StockPlotGlyph) {
            this.beginX = plotGlyph.xAxisGlyph.getBounds().x;
            this.unitLength = plotGlyph.xAxisGlyph.unitLength;
            this.getBounds().width = this.beginX + plotGlyph.xAxisGlyph.axisLength - this.getBounds().x;
        }

        this.crossValue = plotGlyph.xAxisGlyph.getCrossValue();
        this.arrowValue = plotGlyph.xAxisGlyph.getArrowValue();
    },

    /**
     * 只有出现省略符号的 数据(分类, 系列, 值) 参与onMove事件.
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     * @returns {FR.Chart.DataSheetGlyph} 返回找到的目标.
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (!this.tooltipLabelBoundsList || this.tooltipLabelBoundsList.length === 0) {
            return;
        }

        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
            return;
        }

        if (this.items) {
            for (var i = 0; i < this.items.length; i++) {
                this.items[i].findOnMoveGlyph(mousePos, event);
                if(event._hasfound) {
                    var item = this.items[i];
                    if(item.fontColor){
                        this.toolTipBackground = FR.Chart.ColorBackground.getInstance(item.fontColor);
                    }else if(item.lineMarkerIcon){
                        this.toolTipBackground = item.lineMarkerIcon.background;
                    }
                    break;
                }
            }
        }

        this.tooltip = null;
        for (var i = 0; i < this.tooltipLabelBoundsList.length; i++) {
            if (this.tooltipLabelBoundsList[i] && this.tooltipLabelBoundsList[i].containsPoint(mousePos.x, mousePos.y)) {
                this.tooltip = this.tooltipLabelList[i];
                event._hasfound = true;
                event._lastHover = this;
                return ;
            }
        }

    },

    /**
     * 响应鼠标移动事件.
     * @param ctx 画布
     * @param mousePos 鼠标位置.
     */
    doOnMove: function (ctx, mousePos) {
        FR.Chart.Tooltip.dataPointTip(ctx, this.tooltip, mousePos, this.toolTipBackground);
    },

    /**
     * 计算测试的中文字符 在指定最大宽度下 容纳的个数, (英文和数字的比较小, 全部以中文为基准)
     * @param textAttr 文本格式.
     * @param maxWidth 最大宽度
     * @returns {number} 个数.
     */
    getCharNumber: function (textAttr, maxWidth) {
        if (isNaN(maxWidth) || !textAttr || !textAttr.frFont) {
            return 0;
        }

        var width = textAttr.frFont.size;
        var number = parseInt(maxWidth / width);
        return isNaN(number) ? 0 : number;
    },

    _drawSeries: function (ctx, bounds) {// 同图例画法
        if (this.items != null && this.items.length >= this.seriesList.length) {

            var cateBoundsHeight = this.seriesRowHeight * this.cateRowCout + this.PADDING_GAP * 2;

            for (var i = 0; i < this.seriesList.length; i++) {

                this.items[i].bounds = new FR.Chart.Bounds(
                    bounds.x + this.PADDING_GAP,
                    bounds.y + (this.seriesRowHeight + this.PADDING_GAP * 2) * i + cateBoundsHeight + this.PADDING_GAP,
                    this.endLegendWidth, this.seriesRowHeight);
                this.items[i].draw(ctx, this.getFont()); // drawInCutOrNot = false 直接画省略号.

                this.tooltipLabelList.push(this.items[i].label);
                this.tooltipLabelBoundsList.push(this.items[i].bounds);
            }
        }
    },

    _drawCategory: function (ctx, bounds) {
        var cateTextAttr = this.getTextAttr();
        var numberInLine = this.getCharNumber(cateTextAttr, this.unitLength);// 字符个数 中文基准.
        if (numberInLine <= 0) {
            return;
        }

        var tmpName = '';
        var cateBoundsHeight = this.seriesRowHeight * this.cateRowCout + this.PADDING_GAP * 2;

        var dim = new FR.Chart.Dimension2D(0, 0);
        for (var j = 0; j < this.getCategoryCount() && j < this.arrowValue; j++) {
            var i = j - this.crossValue;
            if (i < 0) {
                continue;
            }

            var categoryName = this.seriesList[0].points[j].categoryName || '';
            var times = parseInt(categoryName.length / numberInLine);

            var cateBoundsX = this.beginX + i * this.unitLength;

            if (times < this.MAX_CATE_LINE) {
                for (var index = 0; index < this.MAX_CATE_LINE; index++) {
                    var endIndex = Math.min((index + 1) * numberInLine, categoryName.length);
                    tmpName = categoryName.substring(index * numberInLine, endIndex);

                    dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpName, cateTextAttr);
                    FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, tmpName, cateTextAttr, new FR.Chart.Bounds(
                        cateBoundsX + (this.unitLength - dim.width) / 2,
                        bounds.y + (this.seriesRowHeight - dim.height) / 2 + index * this.seriesRowHeight + this.PADDING_GAP,
                        dim.width, dim.height));

                    if (endIndex === categoryName.length) {
                        break;
                    }
                }
            } else {
                this.tooltipLabelList.push(categoryName);
                this.tooltipLabelBoundsList.push(new FR.Chart.Bounds(cateBoundsX, bounds.y, this.unitLength, cateBoundsHeight));

                for (var index = 0; index < this.MAX_CATE_LINE - 1; index++) {
                    tmpName = categoryName.substring(index * numberInLine, (index + 1) * numberInLine);
                    dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmpName, cateTextAttr);
                    FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, tmpName, this.getTextAttr(), new FR.Chart.Bounds(
                        cateBoundsX + (this.unitLength - dim.width) / 2,
                        bounds.y + (this.seriesRowHeight - dim.height) / 2 + index * this.seriesRowHeight + this.PADDING_GAP,
                        dim.width, dim.height));
                }

                var esdim = this._getEllipsisDim();
                var x = cateBoundsX + (this.unitLength - esdim.width) / 2;
                var y = bounds.y + (this.seriesRowHeight - esdim.height) / 2 + 3 * this.seriesRowHeight + this.PADDING_GAP;

                this._drawEllipsis(ctx, x, y);
            }
        }
    },

    /**
     * 根据宽度决定, 只要宽度不够就画省略号,  一旦省略号也不够, 就直接不画任何内容.
     */
    _drawValues: function (ctx, bounds) {
        var cateBoundsHeight = this.seriesRowHeight * this.cateRowCout + this.PADDING_GAP * 2;
        var seriesTextAttr = this.getTextAttr();

        for (var i = 0; i < this.seriesList.length; i++) {
            var dataSeries = this.seriesList[i];
            for (var j = 0; j < this.getCategoryCount() && j < this.arrowValue; j++) {
                var dataPoint = dataSeries.points[j];
                var k = j - this.crossValue;
                if (k < 0 || dataPoint.valueIsNull) {
                    continue;
                }

                var valueLabel = dataPoint.valueByUnit + "";
                if (dataPoint instanceof FR.Chart.DataPoint4Stock) {
                    valueLabel = dataPoint.values[4] + "";
                }
                if (this.format) {
                    valueLabel = FR.contentFormat(valueLabel, this.format);
                }

                var valueBoundsX = this.beginX + k * this.unitLength;
                var valueBoundsY = bounds.y + (this.seriesRowHeight + this.PADDING_GAP * 2) * i + cateBoundsHeight;

                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(valueLabel, seriesTextAttr);
                if (dim.width <= this.unitLength) {
                    FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, valueLabel, seriesTextAttr, new FR.Chart.Bounds(
                        valueBoundsX + (this.unitLength - dim.width) / 2,
                        valueBoundsY + this.PADDING_GAP,
                        dim.width, dim.height));
                } else {
                    this.tooltipLabelList.push(valueLabel);
                    var valueBounds = new FR.Chart.Bounds(valueBoundsX, valueBoundsY, this.unitLength, this.seriesRowHeight + this.PADDING_GAP * 2);
                    this.tooltipLabelBoundsList.push(valueBounds);

                    var ellDim = this._getEllipsisDim();
                    var x = valueBoundsX + (this.unitLength - ellDim.width) / 2;
                    var y = valueBoundsY + this.PADDING_GAP;

                    this._drawEllipsis(ctx, x, y);
                }
            }
        }
    },

    _getEllipsisDim: function () {
        if (!this.ellipsisDim) {
            this.ellipsisDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(FR.Chart.Constants.ESP, this.getTextAttr());
        }

        return this.ellipsisDim;
    },

    /**
     * 画省略符号, 分类, 系列 和值  统一的字体格式
     */
    _drawEllipsis: function (ctx, x, y) {
        var dim = this._getEllipsisDim();
        FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, FR.Chart.Constants.ESP, this.getTextAttr(), new FR.Chart.Bounds(x, y, dim.width, dim.height));
    },

    _newLine: function (ctx, x, y, x1, y1) {
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(x, y), new FR.Chart.Point2D(x1, y1));
        FR.Chart.GraphHelper.dealLine4JSDraw(line);
        FR.Chart.GraphHelper.draw(ctx, line, FR.Chart.Constants.LINE_THIN);
    },

    /**
     * 分为两部分:  第一: 图例部分, 左侧.  第二: 表格部分 (第一层 分类 以及下层的系列值 数据)
     */
    draw: function (ctx) {
        if (this.seriesList.length < 1) {
            return;
        }

        ctx.strokeStyle = "black";

        var maxBounds = this.getBounds();// 预计算出的边框 bounds 是最大的矩形.

        var cateBoundsHeight = this.seriesRowHeight * this.cateRowCout + this.PADDING_GAP * 2;// 第一层分类高度(额外的)

        var legendX = maxBounds.x;
        var legendY = maxBounds.y + cateBoundsHeight;

        //kunsnat: 杜绝两条线重复的可能, 重叠容易导致线型变化.
        this._newLine(ctx, legendX, legendY, legendX, maxBounds.y + maxBounds.height);
        this._newLine(ctx, legendX, maxBounds.y + maxBounds.height, legendX + maxBounds.width, maxBounds.y + maxBounds.height);
        this._newLine(ctx, maxBounds.x + maxBounds.width, maxBounds.y, maxBounds.x + maxBounds.width, maxBounds.y + maxBounds.height);
        this._newLine(ctx, this.beginX, maxBounds.y, maxBounds.x + maxBounds.width, maxBounds.y);

        for (var i = this.beginX; i < maxBounds.x + maxBounds.width; i += this.unitLength) { // 画纵线
            this._newLine(ctx, i, maxBounds.y, i, maxBounds.y + maxBounds.height);
        }

        // 画横线, 起始线从第一个系列的上侧开始, 最后一个系列的上侧终止.
        for (var i = 0; i < this.seriesList.length; i++) {
            var tmpY = maxBounds.y + cateBoundsHeight + (this.seriesRowHeight + this.PADDING_GAP * 2) * i;
            this._newLine(ctx, maxBounds.x, tmpY, maxBounds.x + maxBounds.width, tmpY);
        }

        this._drawSeries(ctx, maxBounds);
        this._drawCategory(ctx, maxBounds);
        this._drawValues(ctx, maxBounds);
    }
});
// DataSheetGlyph.end