// FineChart.start
/**
 * 
 * 每个图表所在dom节点的控件.
 *
 *     @example
 *     var $root = $('<div>').css({with:800,height:200}).appendTo('body');
 *     var option = {
 *               title : {text : 'test'},
 *               legend : {items : ['a', 'b', 'c']},
 *               xAxis : {
 *                   type : 'category',
 *                   categoryLabelList : ['jan', 'feb', 'test']
 *               },
 *               yAxis : {
 *                   type : 'value',
 *                   position : FR.Chart.Constants.LEFT,
 *                   maxValue : 200,
 *                   minValue : 0,
 *                   tickLength : 25
 *               },
 *               type : 'Bar2DPlotGlyph',
 *               series : [
 *                   {name : 'a', data : [112, 23, 45]},
 *                   {name : 'b', data : [112, 23, 45]}
 *               ]
 *           };
 *     new FineChart(option, $root);
 *
 * @class FineChart
 *
 * @param {JSON} option 图表定义属性
 * @param {String} [option.chartID='test'] 图表唯一标识符
 * @param {Number} [option.canvasCount=3] 图层数量
 * @param {Number} [option.width] 图表的宽度
 * @param {Number} [option.height] 图表的高度
 * @param {JSON} [option.chartAttr] 图表的属性
 *
 * @param {Number} [option.autoRefreshTime=0] 自动刷新的时间间隔
 * @param {String} [option.dependence] 依赖的参数
 *
 * @param dom  图表所在dom
 * @constructor
 */
FineChart = function(option, dom) {
    this.chartID = option.chartID || 'test';
    this.canvasCount = option.canvasCount || 3;// 默认3 没有图片层级.
    this.actionModel = option.actionModel || "0";
    this.width = option.width || dom.width();// 补充从dom获取.
    this.height = option.height || dom.height();
    this.sheetIndex = option.sheetIndex || 0;
	this.ecName = option.ecName || '';
    this.actionTime = option.actionTime || 1000;

    //report用
    this.chartAttr = option.chartAttr;

    //插件用
    this.chartConfig = option;

    this.isVanCharts = FR.pick(option.isVanCharts, false);

    if(FR.Chart.ChartUtils.isIE6Before()){
        this.actionModel = FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL;
    }

    this.ctxs = null;
    this.canvass = null;

    this.autoRefreshTime = option.autoRefreshTime || 0;
    if(option.interaction && option.interaction.autoRefresh){
        this.autoRefreshTime = option.interaction.autoRefresh;
    }

    this.dependence = option.dependence || "";

    this.dom = dom;
    dom.fineChart = this;
    this._init(option, dom);
}
$.extend(FineChart.prototype, /**@class FineChart*/{

    _init: function (option, dom) {

        this._updateCanvas(dom);

        var chartGlyph = new FR.Chart.ChartGlyph(this.chartAttr);

        chartGlyph.parentDiv = dom[0];
        chartGlyph.bounds = new FR.Chart.Bounds(0, 0, this.width, this.height);

        this.backgroundGlyph = this._createBackgroundGlyph();

        this.handler = new ChartHandler(dom, chartGlyph, this.backgroundGlyph);

        this.chartGlyph = chartGlyph;

        FR.Chart.WebUtils._storeChart(this, this.chartID);
        this.inits();
    },

    _updateCanvas:function(dom){

        this._createChartWithID(dom);

        this.ctxs = [];
        this.canvass = [];
        for (var i = 0; i < this.canvasCount; i++) {
            var canvas = document.getElementById(this.chartID + "" + i);
            if(!canvas){
                continue;
            }
            var ctx = canvas.getContext("2d");
            FR.Chart.ChartUtils.retinaOptimization(canvas, ctx, this.width, this.height);
            this.ctxs[this.ctxs.length] = ctx;
            this.canvass[this.canvass.length] = canvas;
        }
    },

    _createBackgroundGlyph:function(){
        var chartWidget = FR.Chart.WebUtils.getChartWidget(this.chartID);
        if(chartWidget && chartWidget.isNeedBackgroundGlyph){
            var items = chartWidget.getChartItems();
            var isNeedRefresh = chartWidget.isNeedRefreshButton;
            var selectedIndex = chartWidget.selectedIndex;
            var size = {width:this.width, height:this.height};
            var backgroundGlyph = new FR.Chart.BackgroundGlyph(this.dom, items, selectedIndex, isNeedRefresh, size);
            backgroundGlyph.chartWidget = chartWidget;
            return backgroundGlyph;
        }
    },

    _createChartWithID: function (parentDiv) {
        var canvasCount = this.canvasCount;

        for (var i = 0; i < canvasCount; i++) {
            this._createCanvas(this.chartID, i, parentDiv, i == canvasCount - 1);
        }
    },

    _createCanvas: function (cname, index, parentDiv, topCanvas) {// 名称传参数 simpleChartInShowID + uid
        var canvas = $('<canvas>').attr('id', cname + index).css('width', this.width + 'px').css('height', this.height + 'px')
            .css('position', 'absolute').css('left', 0).css('top', 0);

        if (topCanvas) {
            canvas.css('z-index', index);
        }

        canvas.appendTo(parentDiv);

        if (window.G_vmlCanvasManager) {
            window.G_vmlCanvasManager.initElement(canvas[0]);// 单独canvas加载
        }
    },

    /**
     * 重新根据现在大小 初始化
     */
    inits: function () {
        this.resize(this.width, this.height);

        this.chartGlyph.ctxs = this.ctxs;

        this._dealWithDependence();
        this.chartGlyph.draw(this.actionModel, this.actionTime);

        this._autoRefresh();
    },

    /**
     * 重新定义大小
     * @param width 宽度
     * @param height 高度
     */
    resize: function (width, height) {// 保证第一次初始化时的 大小.
        if(width) {
            this.width = width;
        }
        if(height) {
            this.height = height;
        }

        if (this.dom) {
            this.dom.css('width', this.width + 'px').css('height', this.height + 'px');
        }

        if(this.ctxs && this.canvass && this.ctxs.length == this.canvass.length){
            for(var i = 0, length = this.ctxs.length; i < length; i++){
                FR.Chart.ChartUtils.retinaOptimization(this.canvass[i], this.ctxs[i], this.width, this.height);
            }
        }


        if (this.chartGlyph) {
            this.chartGlyph.bounds = new FR.Chart.Bounds(0, 0, this.width, this.height);

            if(this.chartGlyph.legendGlyph) {
                this.chartGlyph.legendGlyph.resetLegendWhenResize();
            }
        }

        if(this.backgroundGlyph){
            this.backgroundGlyph.resize(width, height);
        }
    },

    /**
     * 处理 以来控件的 联动关系
     * @private
     */
    _dealWithDependence: function () {
        if (!window.contentPane || !window.contentPane.curLGP) {
            return;
        }

        var dependence = this.dependence;
        var form = window.contentPane.curLGP.form;

        if (FR.isArray(dependence) && form) {
            $.each(dependence, function (idx, p) {
                var widget = form.getWidgetByName(p);
                var event = FR.Events.STOPEDIT;
                if (widget instanceof FR.ComboBoxEditor || widget instanceof FR.CheckBoxEditor || widget instanceof FR.DateTimeEditor) {
                    event = FR.Events.AFTEREDIT;
                } else if (widget instanceof FR.RadioButton || widget instanceof FR.CheckBox || widget instanceof FR.ListEditor) {
                    event = FR.Events.CLICK;
                } else if (widget instanceof FR.RadioGroup || widget instanceof FR.CheckBoxGroup) {
                    event = FR.Events.STATECHANGE;
                }

                if (widget != null) {
                    widget.on(event, function () {
                        var para = {};
                        for (var i = 0; i < dependence.length; i++) {
                            var name = (dependence[i].indexOf("$") === 0) ? dependence[i].substr(1) : dependence[i];
                            para[name] = form.getWidgetByName(dependence[i]).getValue();
                        }
                        this.dealChartAjax(para);
                    });
                }
            })
        }
    },

    /**
     * 自动刷新 数据
     * @private
     */
    _autoRefresh:function(){
        var time = this.autoRefreshTime || 0;
        if (time < 2) {
            return;
        }
        var _this = this;
        this.autoRefreshID = setInterval(function () {
            _this._dealAuto();
        }, time * 1000);
    },

    /**
     * 处理图表关联
     * @param attrList 联动图表列表.
     * @private
     */
    _chartRelated: function (attrList) {
        for (var i = 0; i < attrList.length; i++) {
            var chartID = FR.Chart.WebUtils._getChartIDAndIndex(attrList[i].id);
            var chartSet = FR.ChartManager[chartID[0]];
            if (chartSet && chartSet[chartID[1]]) {
                var chartAttr = attrList[i].chartAttr;
                var attrModel =  attrList[i].actionModel;
                chartSet[chartID[1]].changeData(chartAttr, attrModel || 0);
            }
        }
    },

    _dealAuto:function(){
        if(this.isVanCharts){
            this._dealAuto4Plugin();
        }else{
            this._dealAuto4Report();
        }
    },

    /**
     * 图表插件的自动刷新
     * @private
     */
    _dealAuto4Plugin:function(){
        if(this.chartGlyph && this.chartGlyph.dataDefinition){
            var dataDefinition = this.chartGlyph.dataDefinition;
            if(dataDefinition.dataSource === 'remote'){
                this.changeTableData();
            }
        }else{
            var width = this.width || 0;
            var height = this.height || 0;
            var self = this;
            if(FR.servletURL) {
                FR.ajax({
                    type: 'GET',
                    url: FR.servletURL + '?op=chartrelate',
                    data: {
                        cmd: 'chart_refresh',
                        chartWidth: width,
                        chartHeight: height,
                        __time: new Date().getTime()
                    },
                    headers: {
                        sessionID: FR.SessionMgr.getSessionID()
                    },
                    dataType: 'json',
                    success: function (res) {
                        var attrList = res.relateChartList;
                        var chartID = FR.Chart.WebUtils._getChartIDAndIndex(self.chartID);
                        var chartSet = FR.ChartManager[chartID[0]];
                        for (var i = 0; i < attrList.length; i++) {
                            var tmpID = FR.Chart.WebUtils._getChartIDAndIndex(attrList[i].chartID);
                            if (chartSet && chartSet[tmpID[1]]) {
                                var chartAttr = attrList[i].chartAttr;
                                chartSet[chartID[1]].changeData(chartAttr, "0");
                            }
                        }
                    }
                });
            }
        }
    },

    /**
     * 报表自动刷新相关
     * @private
     */
    _dealAuto4Report: function () {
        var ID = this.chartID;
        var width = this.width || 0;
        var height = this.height || 0;
        var self = this;
        var sheetIndex = this.sheetIndex;
        var ecName = this.ecName;
        if(FR.servletURL) {
            FR.ajax({
                type: 'GET',
                url: FR.servletURL + '?op=chartauto',
                data: {
                    cmd: 'chart_auto_refresh',
                    chartID: ID,
                    chartWidth: width,
                    chartHeight: height,
                    sheetIndex : sheetIndex,
                    ecName : ecName,
                    __time: new Date().getTime()
                },
                headers: {
                    sessionID: FR.SessionMgr.getSessionID()
                },
                dataType: 'json',
                success: function (chartRelateJS) {
                    self._chartRelated(chartRelateJS.relateChartList);
                }
            });
        }
    },

    /**
     * 处理控件的 联动参数
     * @param {String} para 联动参数
     */
    dealChartAjax: function (para) {
        var data =  {
            'chartID': this.chartID,
            'chartWidth': this.width || 0,
            'chartHeight': this.height || 0,
            'sheetIndex' : this.sheetIndex,
            'ecName' : this.ecName,
            '__time': new Date().getTime()
        };

        para = para || {};

        for (var w in para) {
            var value = para[w];
            if (w.startWith("$")) {
                w = w.substring(1)
            }

            if (value == undefined) {
                value = null;
            }
            data[w] = value;
        }

        var self = this;
        FR.ajax({
            type: 'POST',  // @Chart-1185, 使用GET可能导致url过长抛414错误
            url: FR.servletURL + '?op=chartlink&cmd=refresh_relate_data',
            data: data,
            headers: {
                sessionID: FR.SessionMgr.getSessionID()
            },
            dataType: 'json',
            success: function (chartRelateJS) {
                self._chartRelated(chartRelateJS.relateChartList);
            }
        });
    },

    /**
     * 图表重新取数刷新
     */
    dataRefresh:function(){
        this._dealAuto4Report();
    },

    /**
     * 获取百度地图的map对象
     */
    getGis:function(){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            return this.chartGlyph.plotGlyph.map;
        }
    },

    /**
     * 设置标题是否可见
     * @param{Boolean} visible 是否可见
     */
    setTitleVisible:function(visible){
        if(this.chartGlyph && this.chartGlyph.titleGlyph){
            this.chartGlyph.titleGlyph.visible = FR.pick(visible, false);
            this.refreshWithoutAnimation();
        }
    },

    /**
     * 设置标题的值
     * @param{String} title 标题
     */
    setTitleContent:function(title){
        if(this.chartGlyph && this.chartGlyph.titleGlyph){
            this.chartGlyph.titleGlyph.text = FR.pick(title, "");
            this.refreshWithoutAnimation();
        }
    },

    getTitleContent:function(){
        if(this.chartGlyph && this.chartGlyph.titleGlyph){
            return this.chartGlyph.titleGlyph.text;
        }
        return '';
    },

    /**
     * 设置图例是否可见
     * @param{Boolean} visible 是否可见
     */
    setLegendVisible:function(visible){
        if(this.chartGlyph && this.chartGlyph.legendGlyph){
            this.chartGlyph.legendGlyph.visible = FR.pick(visible, false);
            this.refreshWithoutAnimation();
        }
    },

    /**
     * 设置系列是不可见
     * @param{Array} invisibleArray 不可见的系列的序号
     */
    setSeriesInvisible:function(invisibleArray){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;
            var legendGlyph = this.chartGlyph.legendGlyph;
            var dataSeries = plotGlyph.dataSeries;
            invisibleArray = invisibleArray || [];
            for(var i = 0; i < dataSeries.length; i++){
                var series = dataSeries[i];
                var paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
                var disable = false;
                if(FR.Chart.ChartUtils.inArray(invisibleArray, i)){
                    paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
                    disable = true;
                }

                series.paintState = paintState;
                if(legendGlyph && legendGlyph.items){
                    legendGlyph.items[i].disable = disable;
                }
            }
        }
        this.refresh();
    },

    /**
     * 设置标签是否可见
     * @param{Boolean} visible 是否可见
     */
    setLabelVisible:function(visible){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;
            var dataSeries = plotGlyph.dataSeries;
            if(dataSeries){
                for(var i = 0; i < dataSeries.length; i++){
                    dataSeries[i].shouldDrawLable = visible;
                }
            }
        }
        this.refreshWithoutAnimation();
    },

    /**
     * 设置标签的内容
     */
    setLabelValue:function(){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;
            var dataPoint = plotGlyph.getDataPointByPara(arguments);
            if(dataPoint){
                var contents = arguments[arguments.length - 1];
                dataPoint.setCustomLabel(contents, plotGlyph);
            }
        }
        this.refreshWithoutAnimation();
    },

    /**
     * 设置是否开启动画
     * @param{Boolean} animatable 是否开启动画
     */
    setAnimationEnable:function(animatable){
        this.actionModel = animatable ? FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE : FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL;
    },

    /**
     * 设置自动刷新
     * @param {Number}refreshTime 自动刷新的间隔时间，单位 秒
     */
    setAutoRefresh:function(refreshTime){
        clearInterval(this.autoRefreshID);
        this.autoRefreshTime = refreshTime;
        this._autoRefresh();
    },

    /**
     * 清除所有的警戒线
     */
    clearAlertLine:function(){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;

            function clearAlertValueList(axis){
                if(axis && axis.alertValueList){
                    axis.alertValueList = [];
                }
            }
            clearAlertValueList(plotGlyph.xAxisGlyph);
            clearAlertValueList(plotGlyph.yAxisGlyph);
            clearAlertValueList(plotGlyph.secondAxisGlyph);
        }

        this.refresh();
    },

    /**
     * 增加一条警戒线
     * @param{String} name 警戒线
     * @param{Number} value 警戒的值
     * @param{String} color 警戒的颜色
     * @param{"top"/"bottom"/"left"/"right"} align 警戒名称的位置
     * @param{Boolean} isToSecondValueAxis 需要加的坐标轴，默认是y轴
     */
    addAlertLine:function(name,value,color,align,isToSecondValueAxis){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;
            var axisGlyph = plotGlyph.yAxisGlyph;
            if(isToSecondValueAxis){
                axisGlyph = plotGlyph.getSecondValueAxisGlyph();
            }
            if(axisGlyph && axisGlyph.alertValueList){
                var alertGlyph = new FR.Chart.ChartAlertValue();
                axisGlyph.alertValueList.push(alertGlyph);
                var alertPosition = FR.Chart.Position[align];

                if(axisGlyph.position == FR.Chart.Constants.TOP
                    || axisGlyph.position == FR.Chart.Constants.BOTTOM){
                    if(alertPosition == FR.Chart.Constants.TOP){
                        alertPosition = FR.Chart.Constants.RIGHT;
                    }else{
                        alertPosition = FR.Chart.Constants.LEFT;
                    }
                }

                alertGlyph.alertContent = name;
                alertGlyph.formula = value;
                alertGlyph.attrLineColor.seriesColor = color;
                alertGlyph.alertPosition = alertPosition;
            }
        }
    },

    /**
     * 设置图表是否可以缩放
     * @param {Boolean}zoom 是否可以缩放
     */
    setZoom:function(isZoom){
        if(this.chartGlyph && this.chartGlyph.plotGlyph && this.chartGlyph.plotGlyph.xAxisGlyph){
            var xAxisGlyph = this.chartGlyph.plotGlyph.xAxisGlyph;
            xAxisGlyph.isZoom = isZoom;

            if(isZoom){
                this.chartGlyph.initDemoPlotGlyphIfNeed(this.chartAttr);
            }else{
                this.chartGlyph.preChartGlyph = null;
            }

            this.refreshWithoutAnimation();
        }
    },

    /**
     * 设置是否有坐标轴提示
     * @param {Boolean}axisTooltipEnable 是否有坐标轴提示
     */
    setAxisTooltipEnable:function(axisTooltipEnable){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            this.chartGlyph.plotGlyph.interactiveAxisTooltip = axisTooltipEnable;
        }
    },

    /**
     * 是否开启数据点提示
     * @param {Boolean}toolTipEnable 是否开启数据点提示
     */
    setTooltipEnable:function(toolTipEnable){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            this.chartGlyph.plotGlyph.isTooltipEnable = toolTipEnable;
        }
    },

    /**
     * 设置是否可以拖拽
     * @param {Boolean}draggable 是否可以拖拽
     */
    setDraggable:function(draggable){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            var plotGlyph = this.chartGlyph.plotGlyph;
            var series = plotGlyph.dataSeries;
            for(var i = 0; i < series.length; i++){
                series[i].seriesDragEnable = draggable;
            }
        }
    },

    /**
     * 按照分类总值由小到大排序排序
     * 柱形图，条形图，面积图，折线图支持这个功能
     * @para{Boolean} isIncreaseOrder 是否是从小到大
     */
    sort:function(isIncreaseOrder){

        if(isIncreaseOrder == null || isIncreaseOrder == undefined){
            isIncreaseOrder = true;
        }

        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            this.chartGlyph.plotGlyph.sort(isIncreaseOrder);
            this.refresh();
        }
    },

    /**
     * 图表按照现有动态方式刷新
     */
    refresh:function(){

        if(FR.Browser.isIE8Before()){
            this.dom.empty();
            this._updateCanvas(this.dom);
            this.chartGlyph.ctxs = this.ctxs;

            if(this.backgroundGlyph){
                this.backgroundGlyph.initBackgroundCtx(this.dom);
            }
        }

        this._refreshWithAnimationPara(this.actionModel);
    },

    refreshWithoutAnimation:function(){
        this._refreshWithAnimationPara(FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
    },

    refreshWithAnimation:function(){
        this._refreshWithAnimationPara(FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
    },

    _refreshWithAnimationPara:function(actionModel){
        if(this.chartGlyph){
            this.chartGlyph.clearAll();
            this.chartGlyph.draw(actionModel || FR.Chart.ChartPaintConstants.ACTION_MODEL_DEFAULT)
        }
    },

    /**
     * 判断图表是否可以继续钻取
     */
    hasNextLayer:function(){
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            return this.chartGlyph.plotGlyph.hasNextLayer();
        }
    },

    setTooltipValue:function(){
        var paras = [];
        for(var i = 0; i < arguments.length; i++){
            paras.push(arguments[i]);
        }
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            return this.chartGlyph.plotGlyph.setTooltipValue(paras);
        }
    },

    triggerTooltip:function(){
        var paras = [];
        for(var i = 0; i < arguments.length; i++){
            paras.push(arguments[i]);
        }
        if(this.chartGlyph && this.chartGlyph.plotGlyph){
            return this.chartGlyph.plotGlyph.triggerTooltip(paras, this.chartGlyph);
        }
    },

    /**
     * tableData改变以后
     */
    changeTableData:function(){
        if(this.chartGlyph){
            var ctxs = this.chartGlyph.ctxs;
            var bounds = this.chartGlyph.bounds;
            var pardiv = this.chartGlyph.parentDiv;
            var lastChartGlyph = this.chartGlyph;
            this.chartGlyph = new FR.Chart.ChartGlyph();
            this.chartGlyph.setOption(this.chartConfig);
            this.chartGlyph.setLastGlyph(lastChartGlyph);
            this.chartGlyph.parentDiv = pardiv;// 重新搭载div
            this.chartGlyph.bounds = bounds;
            this.chartGlyph.ctxs = ctxs;
            this.chartGlyph.clearAll();

            this.dom.unbind();

            this.handler.reStart(this.chartGlyph);

            this.chartGlyph.draw(this.actionModel);
        }
    },

    /**
     *  图表切换数据的接口, 用于改变数据.
     * @param chartAttr
     * @param actionModel
     */
    changeData: function (chartAttr, actionModel) {
        if(this.chartGlyph) {
            var ctxs = this.chartGlyph.ctxs;
            var bounds = this.chartGlyph.bounds;
            var pardiv = this.chartGlyph.parentDiv;

            //原来的chartGlyph还在往同一个div下面挂标签
            if(this.chartGlyph.plotGlyph){
                this.chartGlyph.plotGlyph.stopDataPointLabel();
            }
            var lastChartGlyph = this.chartGlyph;
            this.chartGlyph = new FR.Chart.ChartGlyph(chartAttr);
            this.chartGlyph.setLastGlyph(lastChartGlyph);
            this.chartGlyph.parentDiv = pardiv;// 重新搭载div
            this.chartGlyph.bounds = bounds;
            this.chartGlyph.ctxs = ctxs;
            this.chartGlyph.clearAll();

            this.dom.unbind();

            this.handler.reStart(this.chartGlyph);

            if (actionModel == null || actionModel == undefined || actionModel < 0 || actionModel > 2) {
                actionModel = FR.Chart.ChartPaintConstants.ACTION_MODEL_DEFAULT;
            }
            this.chartGlyph.draw(actionModel);
        }
    },

    setZoomScale: function (unitScale) {
        if(this.chartGlyph && this.chartGlyph.preChartGlyph) {
            var preGlyph = this.chartGlyph.preChartGlyph;
            var maxNumber = preGlyph.xAxisGlyph.getOriginalArrowValue() - preGlyph.xAxisGlyph.getOriginalCrossValue();
            var number = parseInt(unitScale * maxNumber);
            preGlyph.xAxisGlyph.zoomBackward = 0;
            preGlyph.xAxisGlyph.zoomForward = 0;
            preGlyph.xAxisGlyph.onZoomBarRightDown(number);
            preGlyph.initDragBar(preGlyph.xAxisGlyph);
        }
        this.refreshWithoutAnimation();
    },

    changeZoom: function (changeBorderEnable) {
        if(this.chartGlyph && this.chartGlyph.preChartGlyph) {
            this.chartGlyph.preChartGlyph.isScroll = changeBorderEnable;
        }
    }
});
// FineChart.end