/*
 * Copyright (c) 2001-2014,FineReport Inc, All Rights Reserved.
 */
/**
 * 图表图例区域
 *
 *     @example
 *     var legendGlyph = new FR.Chart.LegendGlyph();
 *
 * @class FR.Chart.LegendGlyph 图例区域
 * @extends FR.Chart.RectangleGlyph
 *
 * @param {JSON} js 图例配置
 * @param {Object} [js.font = new FR.Chart.FRFont()] 图例文字格式.
 * @param {Number} [js.position = FR.Chart.Constants.RIGHT] 图例位置
 * <br/>{@link FR.Chart.Constants#RIGHT}
 * <br/>{@link FR.Chart.Constants#LEFT}
 * <br/>{@link FR.Chart.Constants#TOP}
 * <br/>{@link FR.Chart.Constants#BOTTOM}
 * @param {Object} [js.items = []] 图例项
 */
FR.Chart.LegendGlyph = FR.extend(FR.Chart.RectangleGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.font = new FR.Chart.FRFont(js.font);
        this.position = FR.pick(FR.Chart.ChartUtils.getPosition(js.position), FR.Chart.Constants.RIGHT);

        this.items = [];
        if (js.items) {
            var legendItems = js.items;
            for (var i = 0; i < legendItems.length; i++) {
                this.items[i] = new FR.Chart.LegendItem(legendItems[i]);
            }
        }
        FR.Chart.LegendGlyph.superclass.initConstants.apply(this, [js]);

        this.seriesList = [];

        this.useButton = false;
        this.forwardButton = null;
        this.backButton = null;

        // 翻页用到的计数当前 图例序号
        this.startItemIndex = 0;
        this.endItemIndex = this.items.length;

        this.startItemY4Mobile = 0;
        this.startItemX4Mobile = 0;
        this.moveBarLength = 0;// 布局时计算是否需要这个长度
        this.legendPressing = false;

        this.maxLabelWidth = 0;
        this.drawInCutOrNot = false;

        this.isScroll = this.moveBarLength > 0;
        this.clickable = true;
    },

    //  拖到改变大小时, 重置图例的属性, 以便于重新计算问题.
    resetLegendWhenResize : function() {
        this.useButton = false;
        this.forwardButton = null;
        this.backButton = null;

        // 翻页用到的计数当前 图例序号
        this.startItemIndex = 0;
        this.endItemIndex = this.items.length;

        this.startItemY4Mobile = 0;
        this.startItemX4Mobile = 0;
        this.moveBarLength = 0;// 布局时计算是否需要这个长度

        this.maxLabelWidth = 0;
        this.drawInCutOrNot = false;
    },

    /**
     * 画出图例
     * @param ctx 画布.
     */
    draw:function (ctx) {
        if (this.visible) {
            FR.Chart.LegendGlyph.superclass.draw.apply(this, [ctx]);

            if(!ctx){     return; } ctx.save();

            if (this.items) {
                for (var i = 0; i < this.items.length; i++) {
                    this.items[i].draw(ctx, this.font);
                }
            }

            if (this.useButton) {
                this._checkLegendButton();
                if (this.forwardButton) {
                    this.forwardButton.draw(ctx);
                }
                if (this.backButton) {
                    this.backButton.draw(ctx);
                }
            }

            ctx.restore();
        }
    },

    /**
     * 布局图例并且 刷新图表相关的边界
     * @param chartBounds 图表边界
     * @param width_gap 边界的gap
     * @param height_gap 边界的高度gap
     */
    layoutLegendAndRefreshChartBounds: function (chartBounds, width_gap, height_gap) {
        var legendBounds = this.layoutLegendBounds(chartBounds);
        this.refreshItemBoundsWithLegendBounds(legendBounds);
        this._refreshChartBoundsWithLegendBounds(chartBounds, legendBounds, width_gap, height_gap);

        var newBounds = new FR.Chart.Bounds(parseInt(legendBounds.x) - 0.5, parseInt(legendBounds.y) - 0.5,
            legendBounds.width, legendBounds.height);
        this.bounds = FR.Chart.ChartUtils.rectangle2RoundRectangle(newBounds, this.isRoundBorder());
    },

    /**
     * 图例根据ChartBounds进行布局计算.
     * @param chartBounds 图表边界.
     * @returns {FR.Chart.Bounds} 返回最终的图例边界Bounds
     */
    layoutLegendBounds: function (chartBounds) {
        var legendX = this._getLegendX(chartBounds);
        var legendY = this._getLegendY(chartBounds);
        var legendWidth = this._getLegendWidth(chartBounds);
        var legendHeight = this._getLegendHeight(chartBounds);
        return new FR.Chart.Bounds(legendX, legendY, legendWidth, legendHeight);
    },

    changeLegendBounds:function(newBounds){
        this.refreshItemBoundsWithLegendBounds(newBounds);
        this.bounds = FR.Chart.ChartUtils.rectangle2RoundRectangle(newBounds, this.isRoundBorder());
    },

    /**
     *  根据图例的边界 刷新图表边界宽度
     * @param chartBounds 图表边界
     * @param legendBounds 图例边界
     * @param widthGap 宽度gap
     * @param heightGap 高度gap
     * @private
     */
    _refreshChartBoundsWithLegendBounds: function (chartBounds, legendBounds, widthGap, heightGap) {
        chartBounds.x = this.refreshChartBoundsXWithLegendBounds(chartBounds, legendBounds, widthGap);
        chartBounds.y = this.refreshChartBoundsYWithLegendBounds(chartBounds, legendBounds, heightGap);
        chartBounds.width = this.refreshChartBoundsWidthWithLegendBounds(chartBounds, legendBounds, widthGap);
        chartBounds.height = this.refreshChartBoundsHeightWithLegendBounds(chartBounds, legendBounds, heightGap);
    },

    /**
     * 刷新返回图例区边界的x位置
     * @param chartBounds 图表边界
     * @param legendBounds 图例边界
     * @param widthGap 默认Gap间隔
     * @returns {*} 返回图例x位置.
     */
    refreshChartBoundsXWithLegendBounds: function (chartBounds, legendBounds, widthGap) {
        if (this.position == FR.Chart.Constants.LEFT) {
            return chartBounds.x + legendBounds.width + widthGap;
        } else {
            return chartBounds.x;
        }
    },

    /**
     * 刷新返回图例区边界y的位置
     * @param chartBounds 图表边界
     * @param legendBounds 图例边界
     * @param heightGap 高度间隔gap
     * @returns {*} 返回图例y位置.
     */
    refreshChartBoundsYWithLegendBounds: function (chartBounds, legendBounds, heightGap) {
        if (this.position == FR.Chart.Constants.TOP) {
            return chartBounds.y + heightGap + legendBounds.height;
        } else {
            return chartBounds.y;
        }
    },

    /**
     * 刷新影响图表之后的图表宽度
     * @param chartBounds 图表边界
     * @param legendBounds 图例边界
     * @param widthGap 宽度间隔.
     * @returns {*} 返回图表宽度.
     */
    refreshChartBoundsWidthWithLegendBounds: function (chartBounds, legendBounds, widthGap) {
        if (this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM) {
            return chartBounds.width;
        } else {
            return chartBounds.width - widthGap - legendBounds.width;
        }
    },

    /**
     * 返回刷新影响图表的高度
     * @param chartBounds 图表边界
     * @param legendBounds  图例的边界
     * @param heightGap 高度间隔
     * @returns {*} 返回影响之后图表的高度.
     */
    refreshChartBoundsHeightWithLegendBounds: function (chartBounds, legendBounds, heightGap) {
        if (this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM) {
            return chartBounds.height - heightGap - legendBounds.height;
        } else {
            return chartBounds.height;
        }
    },

    /**
     * 返回图例x的位置
     * @param chartBounds 图表边界
     * @returns {Number}
     * @private
     */
    _getLegendX: function (chartBounds) {
        if (this.position == FR.Chart.Constants.LEFT) {
            return chartBounds.x;
        } else if (this.position == FR.Chart.Constants.RIGHT || this.position == FR.Chart.Constants.RIGHT_TOP) {
            return chartBounds.x + chartBounds.width - this._getLegendWidth(chartBounds);
        } else {
            return chartBounds.x + (chartBounds.width - this._getLegendWidth(chartBounds)) / 2;
        }
    },

    /**
     * 返回图例y的位置
     * @param chartBounds 图表边界
     * @returns {Number}
     * @private
     */
    _getLegendY: function (chartBounds) {
        if (this.position == FR.Chart.Constants.TOP) {
            return chartBounds.y;
        } else if (this.position == FR.Chart.Constants.BOTTOM) {
            return chartBounds.y + chartBounds.height - this._getLegendHeight(chartBounds);
        } else if(this.position == FR.Chart.Constants.RIGHT_TOP) {
            return chartBounds.y;
        } else {
            return chartBounds.y + (chartBounds.height - this._getLegendHeight(chartBounds)) / 2;
        }
    },

    /**
     * 返回图例宽度
     * @param chartBounds 图表边界
     * @returns {width|*|layout.width|tab.width|border.width|grid.el.width}
     * @private
     */
    _getLegendWidth: function (chartBounds) {
        return this._preferredLegendSizeWithChartBounds(chartBounds).width;
    },

    /**
     * 返回图例高度
     * @param chartBounds 图表边界
     * @returns {layout.height|*|tab.height|border.height|dlg.height|list.height}
     * @private
     */
    _getLegendHeight: function (chartBounds) {
        return this._preferredLegendSizeWithChartBounds(chartBounds).height;
    },

    /**
     * 预计算图例的大小
     * @param chartBounds 图表边界
     * @returns {FR.Chart.Dimension2D}
     * @private
     */
    _preferredLegendSizeWithChartBounds: function (chartBounds) {
        if (this.items == null || this.items.length === 0) {
            return new FR.Chart.Dimension2D();
        }

        if (this._isInTopAndBottom()) {
            return this._preLegendDim4TopAndBottom(chartBounds);
        } else {
            return this._preLegendDimLeftAndRight(chartBounds);
        }
    },

    /**
     * 刷新图例项中的所有项边界.
     * @param bounds 图例的整个大边界.
     */
    refreshItemBoundsWithLegendBounds: function (bounds) {
        if (this.items == null || this.items.length <= 0) {
            return;
        }

        if (this._isInTopAndBottom()) {
            this._refreshLegendItemTopAndBottom(bounds);
        } else {
            this._refreshLegendItemLeftAndRight(bounds);
        }
    },

    /**
     * 在上下位置时, 先简单检查下 是否需要翻页按钮
     * @param legendBounds 图例边界
     * @private
     */
    _checkIsUseButton4Top: function (legendBounds) {
        var useButton = false;

        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var startWidth = preWidth;

        var hasRelineInTop = false;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            if (this._isNotEnough4ItemInTopAndBottom(preWidth, itemDim, legendBounds)) {
                if (hasRelineInTop) {// 确定翻页了.
                    useButton = true;
                    break;
                } else {
                    hasRelineInTop = true;// 下面换行
                    preWidth = startWidth + itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;//初始到start 然后 加上当前的
                }
            } else {
                preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
            }
        }

        // 判断下 现有的宽度, 是否能够显示 按钮 和 两个字体...的宽度, 如果不能, 就不显示按钮.
        if (useButton) {
            var testWidth = FR.Chart.GraphHelper.stringDimension(FR.Chart.LegendGlyph.TWO_FONT_AND_MORE, this.font, 0).width;
            preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 + this.font.size / 2 + FR.Chart.LegendGlyph.ROW_COL_GAP * 2 + testWidth
                + FR.Chart.LegendButton.RIGHT_WIDTH * 2;

            useButton = preWidth < legendBounds.width;
        }

        return useButton;
    },

    /**
     * 刷新移动端 上下时  图例的每个项的边界
     * @param legendBounds 图例边界
     * @private
     */
    _refreshLegendItemTopAndBottom4Mobile: function (legendBounds) {
        var startX = this._getRealStartMoveBarFrom() + FR.Chart.LegendGlyph.PADDING_GAP + legendBounds.x;

        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);

            var x = startX;
            var y = FR.Chart.LegendGlyph.PADDING_GAP + legendBounds.y;

            var endX = 0;
            //最后一个item不要加间隔，不然移动端上最后一个item显示不了
            if(i == this.items.length-1){
                endX = parseInt(startX + itemDim.width)
            }else{
                endX = parseInt(startX + itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP)
            }

            if(endX > parseInt(legendBounds.x + legendBounds.width)
                || parseInt(startX) < parseInt(legendBounds.x)) {
                this.items[i].bounds = null;
            } else {
                this._refresItemBounds(i, x, y, itemDim);
            }

            startX += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
        }
    },

    /**
     * 返回滚动条 实际的开始位置
     * @returns {Number}
     * @private
     */
    _getRealStartMoveBarFrom: function () {// 根据比例
        if (!this.moveBarLength || !this.moveAllLength) {
            return 0;
        }

        var start = 0;
        if (this._isInTopAndBottom()) {
            start = this.moveAllLength * this.startItemX4Mobile / this.moveBarLength;
        } else {
            start = this.moveAllLength * this.startItemY4Mobile / this.moveBarLength;
        }
        return -start;
    },

    /**
     * 刷新pc端 图例在上下位置的图例项
     * @param legendBounds 图例边界
     * @private
     */
    _refreshLegendItemTopAndBottom: function (legendBounds) {
        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var maxSize = 0;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            maxSize = Math.max(maxSize, itemDim.height);
        }

        var useButton = this._checkIsUseButton4Top(legendBounds);
        this.useButton = useButton;

        //kunsnat:  对于resize, 需要强制startIndex = 0; 因为每行或者列的个数会变化, 不能用之前的.

        if(this.useButton) {
            var buttonWidth = FR.Chart.LegendButton.RIGHT_WIDTH;
            var y = legendBounds.y + FR.Chart.LegendGlyph.PADDING_GAP + maxSize
                + (FR.Chart.LegendGlyph.ROW_COL_GAP - FR.Chart.LegendButton.TOP_HEIGHT) / 2;
            var endX = legendBounds.x + legendBounds.width;
            var forX = endX - FR.Chart.LegendGlyph.PADDING_GAP - buttonWidth * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
            var backX = endX - FR.Chart.LegendGlyph.PADDING_GAP - buttonWidth;
            if(!this.forwardButton) {
                this.forwardButton = new FR.Chart.LegendButton(forX, y, FR.Chart.Constants.LEFT);
                this.forwardButton.isDisable = false;
            } else {
                this.forwardButton.initButtonBounds(forX, y);
            }

            if(!this.backButton) {
                this.backButton = new FR.Chart.LegendButton(backX, y, FR.Chart.Constants.RIGHT);
                this.backButton.isDisable = true;
            } else {
                this.backButton.initButtonBounds(backX, y);
            }
        }

        var hasRelineInTop = false;
        var makeEnd = true;
        var relineWrited = false;
        var startX = legendBounds.x;
        var startWidth = legendBounds.x + preWidth;

        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);

            if (this._isNotEnough4ItemInTopAndBottom(preWidth + this._getPreButtonWidth(), itemDim, legendBounds)) {

                if (hasRelineInTop && relineWrited) {// 已经换行, 还是不够. 支持翻页按钮
                    if (this.useButton && this.startItemIndex === 0 && makeEnd) {
                        this.endItemIndex = i - 1;
                        makeEnd = false;
                    }
                }

                if (startWidth == preWidth) {// 换行画法.
                    if (i >= this.startItemIndex && i <= this.endItemIndex) {// 单独的一个刷新边界
                        var dim = new FR.Chart.Dimension2D(legendBounds.width - startWidth - 20
                            - 2 * FR.Chart.LegendGlyph.PADDING_GAP, itemDim.height);
                        if (hasRelineInTop) {
                            this._refresItemBounds(i, startWidth + startX, legendBounds.y + FR.Chart.LegendGlyph.PADDING_GAP
                                + dim.height + FR.Chart.LegendGlyph.ROW_COL_GAP, dim);
                            relineWrited = true;
                        } else {
                            this._refresItemBounds(i, startWidth + startX, legendBounds.y + FR.Chart.LegendGlyph.PADDING_GAP, dim);
                            hasRelineInTop = true;
                        }
                        preWidth = startWidth;
                        preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 + maxSize;
                    } else {
                        this.items[i].bounds = null;
                    }
                } else {
                    if (i >= this.startItemIndex && i <= this.endItemIndex) {// 单独的一个刷新边界
                        if (hasRelineInTop) {
                            this.items[i].bounds = null;
                        } else {
                            //画第一个不需要加之前的最大的maxSize，也不需要换行，不然第一个过长就会显示在第二个的位置
                            preHeight =FR.Chart.LegendGlyph.PADDING_GAP * 2 + maxSize;
                            if(i === this.startItemIndex){
                                preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
                            }
                            hasRelineInTop = i !== this.startItemIndex;
                            relineWrited = i !== this.startItemIndex;
                            preWidth = startWidth - startX;

                            var x = preWidth + startX - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;
                            var y = legendBounds.y + preHeight - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;

                            // 放得下 就放 不然, 就省略画法. ------>
                            if (this._isNotEnough4ItemInTopAndBottom(preWidth + this._getPreButtonWidth(), itemDim, legendBounds)) {
                                var dim = new FR.Chart.Dimension2D(legendBounds.width - startWidth - 20
                                    - 2 * FR.Chart.LegendGlyph.PADDING_GAP, itemDim.height);
                                this._refresItemBounds(i, x, y, dim);
                                this.items[i].drawInCutOrNot = true;
                            } else {
                                this._refresItemBounds(i, x, y, itemDim);
                            }
                            preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                        }
                    } else {
                        this.items[i].bounds = null;
                    }
                }
            } else {
                if (i >= this.startItemIndex && i <= this.endItemIndex) {
                    var x = legendBounds.x + preWidth - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    var y = legendBounds.y + preHeight - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    this._refresItemBounds(i, x, y, itemDim);
                    preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                } else {
                    this.items[i].bounds = null;
                }
            }
        }
    },

    /**
     * 预计算图例在上下时的 高度 宽度 (移动端)
     * @param legendBounds 图例边界
     * @returns {FR.Chart.Dimension2D}
     * @private
     */
    _preLegendDim4TopAndBottom4Mobile: function (legendBounds) {
        this._initMaxLegendWidth4DrawisCut(legendBounds);

        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var maxWidth = preWidth;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);

            if (this._isNotEnough4ItemInTopAndBottom(preWidth, itemDim, legendBounds)) {
                preWidth = legendBounds.width;
            } else {
                preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
            }
            preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 + itemDim.height;
            maxWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
        }

        this.moveAllLength = preWidth;
        if (maxWidth > preWidth) {
            this.moveBarLength = preWidth * (preWidth / maxWidth);
        }
        this._checkScroll4Mobile();

        return new FR.Chart.Dimension2D(Math.round(preWidth), Math.round(preHeight));
    },

    /**
     * 预计算 图例上下时的高度 宽度 pc端
     * @param legendBounds 图例边界
     * @returns {FR.Chart.Dimension2D}
     * @private
     */
    _preLegendDim4TopAndBottom: function (legendBounds) {
        var useButton = this._checkIsUseButton4Top(legendBounds);
        this.useButton = useButton;

        // 根据是否使用按钮, 图例实际标签能够使用的大小 决定 画法 为剪切,
        this._initMaxLegendWidth4DrawisCut(legendBounds);

        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var startWidth = preWidth;

        var maxSize = 0;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            maxSize = Math.max(maxSize, itemDim.height);
        }

        var hasRelineInTop = false;

        var firstLineWidth = 0;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);

            if (this._isNotEnough4ItemInTopAndBottom(preWidth + this._getPreButtonWidth(), itemDim, legendBounds)) {
                if (hasRelineInTop) {// 确定翻页了.
                    preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 + maxSize * 2 + FR.Chart.LegendGlyph.ROW_COL_GAP;// 高度两行
                    preWidth = legendBounds.width;// 宽度最宽
                    break;
                } else {
                    hasRelineInTop = true;// 下面换行
                    firstLineWidth = preWidth;// 记录第一行的width.
                    preWidth = startWidth + itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;//初始到start 然后 加上当前的
                    preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 + maxSize * 2 + FR.Chart.LegendGlyph.ROW_COL_GAP;// 两行 高度固定
                }
            } else {
                preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                if (!hasRelineInTop) {
                    preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 + maxSize;
                }
            }
        }

        return new FR.Chart.Dimension2D(Math.max(Math.round(preWidth), firstLineWidth), Math.round(preHeight));
    },

    /**
     * 根据图例的宽度, 决定按钮的实际位置和间隔  靠右或者居中, true表示 靠右, 间隔一个ROW_COL_GAP, false表示居中, 间隔0
     */
    _checkButtonInRightOrCenter: function (legendWidth) {

        var buttonGapWidth = FR.Chart.LegendButton.TOP_WIDTH * 2 + FR.Chart.LegendGlyph.PADDING_GAP * 2
            + FR.Chart.LegendGlyph.ROW_COL_GAP;

        return legendWidth < buttonGapWidth;
    },

    /**
     * 刷新图例 左右位置时的图例项 边界 (移动端)
     * @param legendBounds 图例边界     * @private
     */
    _refreshLegendItemLeftAndRight4Mobile: function (legendBounds) {
        var realMaxItemWidth = legendBounds.width - FR.Chart.LegendGlyph.PADDING_GAP * 2;

        var startY = legendBounds.y + this._getRealStartMoveBarFrom() + FR.Chart.LegendGlyph.PADDING_GAP;

        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);

            var x = FR.Chart.LegendGlyph.PADDING_GAP + legendBounds.x;
            var y = startY + i * itemDim.height + i * FR.Chart.LegendGlyph.ROW_COL_GAP;

            if(parseInt(y) + itemDim.height > legendBounds.y + legendBounds.height
                || parseInt(y) < legendBounds.y) {
                this.items[i].bounds = null;
            } else {
                if (itemDim.width <= realMaxItemWidth) {
                    this._refresItemBounds(i, x, y, itemDim);
                } else {
                    var realDim = new FR.Chart.Dimension2D(realMaxItemWidth, itemDim.height);
                    this._refresItemBounds(i, x, y, realDim);
                    this.items[i].drawInCutOrNot = true;
                }
            }
        }
    },

    /**
     * 刷新图例 在左右位置时的图例项
     * @param legendBounds 图例边界
     * @private
     */
    _refreshLegendItemLeftAndRight: function (legendBounds) {
        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var realMaxItemWidth = legendBounds.width - FR.Chart.LegendGlyph.PADDING_GAP * 2;

        if(this.useButton) {
            var buttonWidth = FR.Chart.LegendButton.TOP_WIDTH;
            var buttonGap = 4;
            var inCenter = this._checkButtonInRightOrCenter(legendBounds.width);
            var y = legendBounds.height - FR.Chart.LegendButton.TOP_HEIGHT - buttonGap;
            var forX = inCenter ? legendBounds.width * 0.5 - buttonWidth
                : legendBounds.width - FR.Chart.LegendGlyph.PADDING_GAP - buttonWidth * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
            var backX = inCenter ? legendBounds.width * 0.5 : legendBounds.width - FR.Chart.LegendGlyph.PADDING_GAP - buttonWidth;
            if(!this.forwardButton) {
                this.forwardButton = new FR.Chart.LegendButton(legendBounds.x + forX, legendBounds.y + y, FR.Chart.Constants.TOP);
                this.forwardButton.isDisable = false;
            } else {
                this.forwardButton.initButtonBounds(legendBounds.x + forX, legendBounds.y + y);
            }

            if(!this.backButton) {
                this.backButton = new FR.Chart.LegendButton(legendBounds.x + backX, legendBounds.y + y, FR.Chart.Constants.BOTTOM);
                this.backButton.isDisable = true;
            } else {
                this.backButton.initButtonBounds(legendBounds.x + backX, legendBounds.y + y);
            }
        }

        var makeEnd = true;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            if (this._isNoEnough4ItemInLeftAndRight(preHeight, itemDim, legendBounds)) {// 高度不够, 肯定不画, 翻页.

                if (this.useButton && this.startItemIndex === 0 && makeEnd) {// 根据实际的图例宽度 决定按钮的实际位置 居中or靠右
                    this.endItemIndex = i - 1;
                    makeEnd = false;
                }

                this.items[i].bounds = null;
            } else {
                if (i >= this.startItemIndex && i <= this.endItemIndex) {
                    var x = legendBounds.x + preWidth - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    var y = legendBounds.y + preHeight - FR.Chart.LegendGlyph.PADDING_GAP + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    // 判定宽度, 再决定是何种画法.
                    if (itemDim.width <= realMaxItemWidth) { // 正常画法.
                        this._refresItemBounds(i, x, y, itemDim);
                    } else {
                        var realDim = new FR.Chart.Dimension2D(realMaxItemWidth, itemDim.height);
                        this._refresItemBounds(i, x, y, realDim);
                        this.items[i].drawInCutOrNot = true;
                    }

                    preHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
                } else {
                    this.items[i].bounds = null;
                }
            }
        }
    },

    /**
     * 检查图例在左右时 是否使用翻页按钮.
     * @param legendBounds 图例边界
     * @returns {Boolean} 是否使用按钮
     * @private
     */
    _checkIsUseButton4LeftAndRight: function (legendBounds) {
        var useButton = false;

        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            if (this._isNoEnough4ItemInLeftAndRight(preHeight, itemDim, legendBounds)) {
                useButton = true;
                break;
            } else {
                preHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
            }
        }

        if (useButton) {// 判断宽度, 如果两个翻页按钮横向宽度 大于 图例百分比的大小, 则不用按钮
            var width = this.maxLabelWidth;
            var buttonWidth = this._getMinButtonWidth4LeftAndRight();
            useButton = width > buttonWidth;
        }

        return useButton;
    },

    _getMinButtonWidth4LeftAndRight: function () {
        return FR.Chart.LegendButton.TOP_WIDTH * 2 + FR.Chart.LegendGlyph.PADDING_GAP * 2;
    },

    _preLegendDimLeftAndRight4Mobile: function (legendBounds) {
        this._initMaxLegendWidth4DrawisCut(legendBounds);

        var preWidth = 0;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var maxHeight = preHeight;
        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            if (this._isNoEnough4ItemInLeftAndRight(preHeight, itemDim, legendBounds)) {
                if (preHeight + FR.Chart.LegendGlyph.ROW_COL_GAP + itemDim.height / 2 > legendBounds.height) {
                    preHeight -= itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    preHeight += itemDim.height / 2 - FR.Chart.LegendGlyph.PADDING_GAP;
                } else {
                    preHeight += itemDim.height / 2 - FR.Chart.LegendGlyph.PADDING_GAP;
                }
            } else {
                preWidth = Math.max(preWidth, FR.Chart.LegendGlyph.PADDING_GAP * 2 + itemDim.width);
                preHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
            }
            maxHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
        }

        this.moveAllLength = preHeight;

        if (maxHeight > preHeight) {
            this.moveBarLength = preHeight * (preHeight / maxHeight);
        }
        this._checkScroll4Mobile();

        return new FR.Chart.Dimension2D(Math.min(Math.round(preWidth), this.maxLabelWidth), Math.round(preHeight));
    },

    _preLegendDimLeftAndRight: function (legendBounds) {
        var useButton = this._checkIsUseButton4LeftAndRight(legendBounds);
        this.useButton = useButton;

        this._initMaxLegendWidth4DrawisCut(legendBounds);

        var preWidth = 0;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var hasDealButton = false;

        for (var i = 0; i < this.items.length; i++) {
            var itemDim = this.items[i].prefferedSize(this.font);
            if (this._isNoEnough4ItemInLeftAndRight(preHeight, itemDim, legendBounds)) {
                var buttonHeight = FR.Chart.LegendButton.RIGHT_WIDTH;
                var buttonGap = 4;
                if (!hasDealButton && useButton) {
                    if (preHeight + buttonGap + buttonHeight > legendBounds.height) {// 按钮也放不下了, 回少一个图例, 然后再放
                        preHeight -= itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
                        preHeight += buttonGap + buttonHeight;
                    } else {
                        preHeight += buttonGap + buttonHeight;
                    }
                    hasDealButton = true;
                }
            } else {
                preWidth = Math.max(preWidth, FR.Chart.LegendGlyph.PADDING_GAP * 2 + itemDim.width);
                preHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
            }
        }

        if (useButton) {// 按钮宽度肯定要小于 maxLabelWidth, 不然怎么画按钮.
            preWidth = Math.max(preWidth, this._getMinButtonWidth4LeftAndRight());
        }

        // 取实际的标签宽度和允许最大的标签(0.28)中较小的
        return new FR.Chart.Dimension2D(Math.min(Math.round(preWidth), this.maxLabelWidth), Math.round(preHeight));
    },

    _initMaxLegendWidth4DrawisCut: function (legendBounds) {
        if (legendBounds) {
            if (this._isInTopAndBottom()) {
                if (this.useButton) {
                    this.maxLabelWidth = legendBounds.width - this._getPreButtonWidth() - FR.Chart.LegendGlyph.ROW_COL_GAP;
                } else {
                    this.maxLabelWidth = legendBounds.width;
                }
            } else {
                this.maxLabelWidth = legendBounds.width * FR.Chart.LegendGlyph.MAX_LEFT_WIDTH;
            }
            var testWidth = FR.Chart.GraphHelper.stringDimension(FR.Chart.LegendGlyph.TWO_FONT_AND_MORE, this.font, 0).width;
            this.drawInCutOrNot = testWidth > this.maxLabelWidth
                - FR.Chart.LegendGlyph.PADDING_GAP * 2 - this.font.size / 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        }
    },

    _refresItemBounds: function (index, x, y, dim) {
        if (this.items[index]) {
            this.items[index].bounds = new FR.Chart.Bounds(x, y, dim.width, dim.height);
            this.items[index].drawInCutOrNot = this.drawInCutOrNot;
        }
    },

    _isInTopAndBottom: function () {
        return (this.position == FR.Chart.Constants.TOP || this.position == FR.Chart.Constants.BOTTOM);
    },

    _isNotEnough4ItemInTopAndBottom: function (preWidth, itemDim, legendItemBounds) {
        return parseInt(preWidth + itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP) > legendItemBounds.width;
    },

    _isNoEnough4ItemInLeftAndRight: function (preHeight, itemDim, legendItemBounds) {
        return parseInt(preHeight + itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP) > legendItemBounds.height;
    },

    _refreshItemWithForwardButton: function () {
        var count = this._calInt4ItemWithLegendBoundsForward();

        if (count > 0) {
            this.endItemIndex = Math.min(this.startItemIndex - 1, this.items.length);
            this.startItemIndex = Math.max(this.endItemIndex - count + 1, 0);
        }

        this._checkLegendButton();
    },

    // 因为上下的时候 宽度不确定. 需要预先排除掉 按钮的大小
    _getPreButtonWidth: function () {
        var width = 0;
        if (this.useButton) {// 只有TopAndBottom的时候用到.
            width = FR.Chart.LegendButton.RIGHT_WIDTH * 2 + FR.Chart.LegendGlyph.ROW_COL_GAP;
        }

        return width;
    },

    /**
     * 预算. 从0开始计算每次后翻的个数, 保证后翻和前翻两次的效果一样. 不然从start前推 个数可能不同.
     */
    _calInt4ItemWithLegendBoundsForward: function () {
        var count = 0;
        if (this.startItemIndex === 0) {
            return 0;
        }
        var form = 0;
        while (true) {
            count = this._checkStartValueCount(form, this.startItemIndex);
            if (count === 0 || count + form == this.startItemIndex || count + form > this.startItemIndex) {
                break;
            } else {
                form += count;
            }
        }
        return count;
    },

    _checkLegendButton: function () {
        var forwardCount = this._calInt4ItemWithLegendBoundsForward();
        var backCount = this._calInt4ItemWithLegendBoundsBack();

        this.forwardButton.isDisable = this.startItemIndex > 0 && forwardCount > 0;
        this.backButton.isDisable = this.endItemIndex < this.items.length - 1 && backCount > 0;
    },

    /**
     * 刷新后翻按钮
     */
    _refreshItemWithBackButton: function () {
        var count = this._calInt4ItemWithLegendBoundsBack();

        if (count > 0) {
            this.startItemIndex = Math.min(this.endItemIndex + 1, this.items.length);
            this.endItemIndex = Math.min(this.startItemIndex + count - 1, this.items.length);
        }

        this._checkLegendButton();
    },

    /**
     * 检查 当前后翻页 endItemIndex + 1 开始往后能放几个.
     */
    _calInt4ItemWithLegendBoundsBack: function () {// 从目前的start 往前
        return this._checkStartValueCount(this.endItemIndex + 1, this.items.length);
    },

    /**
     * 检查从startValue到endValue 能放几个.
     */
    _checkStartValueCount: function (startValue, endValue) {
        var preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
        var preHeight = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;

        var hasReline = false;

        var count = 0;
        for (var start = startValue; start < endValue; start++) {
            var itemDim = this.items[start].prefferedSize(this.font);
            if (this._isInTopAndBottom()) {
                if (this._isNotEnough4ItemInTopAndBottom(preWidth + this._getPreButtonWidth(), itemDim, this.bounds)) {
                    if (start == startValue) {// 起始第一个 省略的放下
                        preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                        count++;
                        continue;
                    } else {
                        if (!hasReline) {
                            hasReline = true;
                            preWidth = FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP;
                            if (this._isNotEnough4ItemInTopAndBottom(preWidth + this._getPreButtonWidth(), itemDim, this.bounds)) {
                                count++;// 省略的放下这一个
                                break;
                            } else {
                                preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                                count++;
                            }
                        } else {
                            if (preWidth == FR.Chart.LegendGlyph.PADDING_GAP * 2 - FR.Chart.LegendGlyph.ROW_COL_GAP) {
                                count++;
                            }
                            break;
                        }
                    }
                } else {
                    preWidth += itemDim.width + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    count++;
                }
            } else {
                if (this._isNoEnough4ItemInLeftAndRight(preHeight, itemDim, this.bounds)) {
                    break;
                } else {
                    preHeight += itemDim.height + FR.Chart.LegendGlyph.ROW_COL_GAP;
                    count++;
                }
            }
        }

        return count;
    },

    /**
     * 图例响应按压事件.
     * @param mousePos 鼠标位置.
     * @param event  鼠标事件.
     */
    onPress: function (mousePos, event) {
        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
            return;
        }

        if (this.useButton) {
            this.forwardButton.onPress = false;
            this.backButton.onPress = false;

            if (this.forwardButton) {
                if (this.forwardButton.containsPoint(mousePos.x, mousePos.y) && this.forwardButton.isDisable) {
                    this.forwardButton.onPress = true;
                    return;
                }
            }
            if (this.backButton) {
                if (this.backButton.containsPoint(mousePos.x, mousePos.y) && this.backButton.isDisable) {
                    this.backButton.onPress = true;
                    return;
                }
            }
        }

    },

    /**
     * 鼠标响应 点击事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件.
     */
    onClick: function (mousePos, event) {
        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
//            return;
        }

        if (!(this.bounds && this.bounds.getBounds2D().containsPoint(mousePos.x, mousePos.y))) {
            return;
        }

        if (this.useButton) {
            if (this.forwardButton && this.forwardButton.bounds && this.forwardButton.isDisable) {
                if (this.forwardButton.containsPoint(mousePos.x, mousePos.y)) {
                    this._refreshItemWithForwardButton();
                    event.needRefresh = true;
                    event.needAnimateRefresh = false;
                    return;
                }
            }
            if (this.backButton && this.backButton.bounds && this.backButton.isDisable) {
                if (this.backButton.containsPoint(mousePos.x, mousePos.y)) {
                    this._refreshItemWithBackButton();
                    event.needRefresh = true;
                    event.needAnimateRefresh = false;
                    return;
                }
            }
        }

        if (this.items) {
            for (var i = 0; i < this.items.length; i++) {
                if (!this.items[i].getBounds()) {
                    continue;
                }
                var dim = this.items[i].prefferedSize(this.font);
                var itemBounds = new FR.Chart.Bounds(this.items[i].getBounds().x, this.items[i].getBounds().y, dim.width, dim.height);
                if (itemBounds && itemBounds.containsPoint(mousePos.x, mousePos.y) && i < this.seriesList.length) {

                    var currentStatus = this.items[i].disable;
                    this.items[i].disable = !currentStatus;

                    if(event._chartGlyph && event._chartGlyph.hasCombineGlyph()){
                        event.needRefresh = true;
                        this._changeCombineGlyphStatus(i, event);
                    }else{
                        event.needRefresh = true;
                        event.needRefreshLabel = true;
                        this._changeLegendStatus(this.seriesList, i);
                        if(event._chartGlyph){
                            event._chartGlyph.plotGlyph.createHotTooltip();
                        }
                    }
                    return;
                }
            }
        }
    },

    _changeCombineGlyphStatus:function(seriesIndex){
        for(var i = 0; i < this.seriesList.length; i++){
            var dataSeries = this.seriesList[i];
            for(var j = 0, count = dataSeries.points.length; j < count; j++){
                var dataPoint = dataSeries.points[j];
                if(dataPoint.combineGlyph && dataPoint.combineGlyph.plotGlyph){
                    var plotGlyph = dataPoint.combineGlyph.plotGlyph;
                    this._changeLegendStatus(plotGlyph.dataSeries, seriesIndex);
                }
            }
        }
    },

    _changeLegendStatus:function(dataSeries, seriesIndex){
        if(dataSeries && dataSeries[seriesIndex]){
            if (dataSeries[seriesIndex].paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                dataSeries[seriesIndex].paintState = FR.Chart.ChartPaintConstants.STATE_DISAPPEAR;
            } else if (dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries[seriesIndex].paintState = FR.Chart.ChartPaintConstants.STATE_APPEAR;
            }
        }
    },

    /**
     * 鼠标按压事件时 一些图例的画法
     * @param ctx 画布.
     */
    drawOnPress: function (ctx) {

        if (this.bounds) {
            if (this.backButton && this.backButton.onPress) {
                this.backButton.drawOnPress(ctx);
            }
            if (this.forwardButton && this.forwardButton.onPress) {
                this.forwardButton.drawOnPress(ctx);
            }
        }
    },

    /**
     * 在移动端 检查 图例是否可滚动
     */
    _checkScroll4Mobile: function () {
        this.isScroll = this.moveBarLength > 0;
    },

    _drawMoveBarInPressing: function (ctx) {
        if (this.legendPressing && this.moveBarLength && this.moveBarLength > 0) {
            if(!ctx){     return; } ctx.save();
            var arcLength = 2.5;
            var barDee = 5;
            if (this._isInTopAndBottom()) {
                var moveBar = new FR.Chart.RoundBounds(this.startItemX4Mobile, 0, this.moveBarLength, barDee, arcLength, arcLength);
                ctx.fillStyle = 'rgba(51, 51, 51, 0.6)';
                moveBar.paint(ctx);
            } else {
                var moveBar = new FR.Chart.RoundBounds(this.bounds.width - barDee, this.startItemY4Mobile,
                    barDee, this.moveBarLength, arcLength, arcLength);
                ctx.fillStyle = 'rgba(51, 51, 51, 0.6)';
                moveBar.paint(ctx);
            }
            ctx.restore();
        }
    },

    /**
     * 鼠标在图例区移动时 一些响应
     * @param ctx 画布
     * @param mousePos 鼠标位置.
     */
    doOnMove: function (ctx, mousePos) {
        if (this.bounds && this.backButton && this.forwardButton) {
            if (this.backButton) {
                this.backButton.draw4OnMove(ctx);
            }
            if (this.forwardButton) {
                this.forwardButton.draw4OnMove(ctx);
            }
        }

        if(this.onmoveitem) {
            this.onmoveitem.doOnMove(ctx, mousePos);
        }
    },

    /**
     * 查找鼠标在移动时 当前找到的对应的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (!(this.bounds && this.bounds.getBounds2D().containsPoint(mousePos.x, mousePos.y))) {
            return;
        }

        this.findOnMoveLegendButtonGlyph(mousePos, event);

        if(event._hasfound){
            return;
        }

        this.onmoveitem = null;

        if (this.items) {
            for (var i = 0; i < this.items.length; i++) {
                this.items[i].findOnMoveGlyph(mousePos, event);
                if(event._hasfound) {
                    this.items[i].font = this.font;
                    this.onmoveitem =  this.items[i];
                    event._lastHover = this;
                    break;
                }
            }
        }

        if(this.moveBarLength > 0) {// 图例选项优先, 否则 备选整个图例 用于拖拽等.
            event._lastHover = event._lastHover || this;
            event._hasfound = true;
            event._draggingTarget = this;
        }
    },

    findOnMoveLegendButtonGlyph: function (mousePos, event) {
        if (this.useButton) {
            this.forwardButton.onMove = false;
            this.backButton.onMove = false;

            if (this.forwardButton) {
                if (this.forwardButton.containsPoint(mousePos.x, mousePos.y) && this.forwardButton.isDisable) {
                    this.forwardButton.onMove = true;
                    event.cursor = ChartHandler.POINTER;
                    event._hasfound = true;
                    event._lastHover = this;
                    return;
                }
            }
            if (this.backButton) {
                if (this.backButton.containsPoint(mousePos.x, mousePos.y) && this.backButton.isDisable) {
                    this.backButton.onMove = true;
                    event.cursor = ChartHandler.POINTER;
                    event._hasfound = true;
                    event._lastHover = this;
                }
            }
        }
    },

    /**
     * 在移动端 移动时 响应的移动事件.
     * @param downPos 按压时的鼠标位置
     * @param mousePos 当前的鼠标位置.
     * @param ctx  画布.
     */
    move4Mobile: function (downPos, mousePos, ctx) {
        // 变动起始的mobile x 或者y位置, 然后根据 比例关系(滚动条的长度, 整个图例对应的长度) 计算图例实际的起始位置
        var startPos = downPos;
        if (this._isInTopAndBottom()) {
            this.startItemX4Mobile -= mousePos.x - startPos.x;

            this.startItemX4Mobile = Math.min(this.startItemX4Mobile, this.moveAllLength - this.moveBarLength);
            this.startItemX4Mobile = Math.max(this.startItemX4Mobile, 0);

        } else {
            this.startItemY4Mobile -= mousePos.y - startPos.y;

            this.startItemY4Mobile = Math.min(this.startItemY4Mobile, this.moveAllLength - this.moveBarLength);
            this.startItemY4Mobile = Math.max(this.startItemY4Mobile, 0);
        }

        ctx.translate(this.bounds.x, this.bounds.y);
        this._drawMoveBarInPressing(ctx);
        ctx.translate(-this.bounds.x, -this.bounds.y);

        event.refreshLegend = true;
    },

    /**
     * 拖拽开始时
     * @param mousePos 鼠标位置
     * @param event 鼠标事件.
     */
    onDragStart: function (mousePos, event) {
        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
            return;
        }

        if (this.bounds.containsPoint(mousePos.x, mousePos.y)) {
            event._draggingTarget = this;
        }
    },

    /**
     * 拖拽结束 响应事件.
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onDragEnd   : function(mousePos, event) {
        // nothing just refresh
    },

    /**
     * 拖拽进行时
     */
    onDrag  :   function() {

    },

    /**
     * 系列删除以后同步系列和图例的颜色
     * @param dataSeries 需要同步的系列
     */
    synchronizeLegendColorWithSeries:function(plotFillStyle, dataSeries){
        if(dataSeries){
            var colorArray = FR.Chart.ChartUtils.createFillColorArray(plotFillStyle, dataSeries.length);
            for(var index = 0; index < this.items.length; index++){
                var lineIcon = this.items[index].lineMarkerIcon;
                if(lineIcon.background.backgroundType == "ColorBackground"){
                    lineIcon.background.color = colorArray[index];
                }

                if(lineIcon.marker && lineIcon.marker.background && lineIcon.marker.background == "ColorBackground"){
                    lineIcon.marker.background.color = colorArray[index];
                }
            }
        }

    }
});
$.extend(FR.Chart.LegendGlyph,{
    /**
     * @static
     * @property 填充间隙
     */
    PADDING_GAP: 4,

    /**
     * @static
     * @property 行列间隙
     */
    ROW_COL_GAP: 5,

    /**
     * @static
     * @property 最大的图表宽度比例
     */
    MAX_LEFT_WIDTH: 0.28,// 固定的最大图表宽度比例

    /**
     * @static
     * @property 多个字符时的现实文本
     */
    TWO_FONT_AND_MORE: FR.i18nText('Fine-Dec_Chart_Chart_Name') + FR.Chart.Constants.ESP // 两个字符 + "..."
});