/**
 * 热点提示文字
 *    @example
 *    var js = {
 *         value : '热点',
 *         font : new FR.Chart.FRFont(),
 *         newLine : false,
 *         height : 20,
 *         width : 20
 *    };
 *    var tooltip = new FR.Chart.TooltipString(js);
 *
 *
 * @class FR.Chart.TooltipString
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 * @param {String} [js.value = ''] 字符值
 * @param {FR.Chart.FRFont}[js.font = new FR.Chart.FRFont()] 热点提示文本
 * @param {Number} [js.height = 0] 高度
 * @param {Number}[js.width = 0] 宽度
 * @param {Boolean} [js.newLine = false] 文本结束是否换行
 */
FR.Chart.TooltipString = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};
        this.value = js.value ||  "";
        this.font = new FR.Chart.FRFont(js.font);
        this.height = 0;
        this.width = 0;

        this.newLine = false;// 标签之后是否换行.
        this.rightAlign = false;//是否是右对齐
    },

    /**
     *  对字符串进行格式处理.
     * @param string
     */
    formatValue: function (string) {
        var pattern = /^-?(\d*\.\d*)$/;//纯数字
        if (pattern.exec(string)) {
            this.value = string;
            this.font.fontName = "Century Gothic";
            return;
        }
        pattern = /^-?(\d*\.\d*)%$/;//百分比
        if (pattern.exec(string)) {
            this.value = parseFloat(string) + "%";
            this.font.fontName = "Century Gothic";
            return;
        }
        pattern = /^[￥$]-?(\d*\.\d*)$/;//货币
        if (pattern.exec(string)) {
            this.value = string.substring(0, 1) + parseFloat(string.substring(1, string.length));
            this.font.fontName = "Century Gothic";
            return;
        }
        if (!isNaN(string)) {//整数、科学计数
            this.value = parseFloat(string);
            this.font.fontName = "Century Gothic";
            return;
        } else {
            this.value = string;
            this.font.fontName = "Microsoft YaHei"
        }
    }
});


//Tooltip.start
/**
 * 图表热点提示的处理画法
 * @class FR.Chart.Tooltip
 */
FR.Chart.Tooltip = {

    /**
     * @static
     * @property 左侧间隙
     */
    LEFTGAP : 6,

    /**
     * @static
     * @property 上边间隙
     */
    TOPGAP : 6,

    /**
     * @static
     * @property 角落间隙
     */
    ROUNDGAP : 6,

    /**
     * 动画的步骤
     */
    STEP:20,

    /**
     * 数据点提示默认的css样式
     */
    DEFAULT_CSS_TEXT:'position:absolute;border-style:solid;white-space:nowrap;z-Index:10;opacity:1;font-family:Microsoft YaHei;font-size:12px;line-height:17px;padding:6px;border-width:2px;',

    /**
     * 画坐标轴的热点提示.
     * @static
     * @param ctx 画布
     * @param axisTooltip 坐标轴值提示.
     */
    drawAxisTooltip: function (ctx, axisTooltip) {
        if (axisTooltip.tooltips) {
            var dim = FR.Chart.GraphHelper.calculateTooltipDimension(axisTooltip.tooltips);

            var x = 0, y = 0;

            if (axisTooltip.pos == FR.Chart.Constants.LEFT) {
                x = axisTooltip.point.x + dim.width + this.LEFTGAP * 2;
                y = axisTooltip.point.y + dim.height + this.TOPGAP * 2;
            } else if (axisTooltip.pos == FR.Chart.Constants.RIGHT) {
                x = axisTooltip.point.x;
                y = axisTooltip.point.y + dim.height + this.TOPGAP * 2;
            } else if (axisTooltip.pos == FR.Chart.Constants.BOTTOM) {
                x = axisTooltip.point.x + dim.width + this.LEFTGAP * 2;
                y = axisTooltip.point.y;
            } else if (axisTooltip.pos == FR.Chart.Constants.TOP) {
                x = axisTooltip.point.x + dim.width + this.LEFTGAP * 2;
                y = axisTooltip.point.y + dim.height + this.TOPGAP * 2;
            }

            x = x - dim.width - this.LEFTGAP * 2;  // x,y转化为tooltip左上角
            y = y - dim.height - this.TOPGAP * 2;

            var bounds = new FR.Chart.Bounds(x, y, dim.width + this.LEFTGAP * 2, dim.height + this.TOPGAP * 2);
            FR.Chart.Tooltip._drawWithCanvas(ctx, axisTooltip.tooltips, bounds, "rgb(230,230,230)", 1);
        }
    },

    /**
     * 图例项 热点提示.
     * @static
     * @param tooltip 热点提示.
     * @param ctx 画布
     * @param pos 鼠标位置.
     * @param borderBackground 边界背景.
     */
    dataPointTip:function(ctx, tooltip, pos, borderBackground) {
        if(tooltip) {
            var dim = this._calculateTooltipTextDimension(tooltip);

            pos = new FR.Chart.Point2D((pos.x - dim.width) < 0 ? 0 : pos.x - dim.width, (pos.y - dim.height) < 0 ? 0 : pos.y - dim.height);
            FR.Chart.Tooltip._drawWithDiv(ctx, tooltip, pos, 'white', borderBackground);
        }
    },

    /**
     * dataPoint的数据点提示
     * @param ctx 画布
     * @param tooltip 热点提示内容
     * @param pos 鼠标位置
     * @param glyph dataPoint的形状
     * @param strokeStyle 边框颜色
     * @param plotGlyph 数据点说在的plotGlyph
     * @param fillStyle 填充的颜色{'black', white}
     */
    dataPointTip4Point:function (ctx, tooltip, pos, glyph, strokeStyle, plotGlyph, fillStyle) {
        if(ctx && tooltip) {
            var tooltipPos = FR.Chart.Tooltip._dealTooltipPos(tooltip, pos, glyph, plotGlyph);
            strokeStyle = FR.Chart.Tooltip._getBorderBackground(glyph, strokeStyle);
            FR.Chart.Tooltip._drawWithDiv(ctx, tooltip, tooltipPos, fillStyle, strokeStyle);
        }
    },

    /**
     * dataPoint的数据点提示
     * @param ctx 画布
     * @param tooltip 热点提示内容
     * @param pos 鼠标位置
     * @param glyph dataPoint的形状
     * @param strokeStyle 边框颜色
     * @param plotGlyph 数据点说在的plotGlyph
     * @param fillStyle 填充的颜色{'black', white}
     */
    dataPointTip4Series:function (ctx, tooltip, pos, glyph, strokeStyle, plotGlyph, fillStyle, colors) {
        if(tooltip && ctx) {
            var bounds = FR.Chart.Tooltip._dealTooltipPos(tooltip, pos, glyph, plotGlyph);
            strokeStyle = FR.Chart.Tooltip._getBorderBackground(glyph, strokeStyle);
            FR.Chart.Tooltip._draw4Series(ctx, tooltip, bounds, fillStyle, strokeStyle, colors);
        }
    },

    _getBorderBackground: function(glyph, borderBackground) {
        if(glyph && glyph.marker){
            return glyph.marker.background;
        }else if(borderBackground){
            return borderBackground;
        }
        return null;
    },

    _calculateTooltipTextDimension:function(tooltip){
        var body = document.getElementsByTagName("body")[0];
        var testDiv = document.createElement('div');
        testDiv.innerHTML = tooltip;
        testDiv.style.cssText = this.DEFAULT_CSS_TEXT + 'visibility:hidden;';
        body.appendChild(testDiv);
        var width = testDiv.offsetWidth;
        var height = testDiv.offsetHeight;
        body.removeChild(testDiv);
        return new FR.Chart.Dimension2D(width, height);
    },

    _dealTooltipPos: function(tooltip, mousePos, glyph, plotGlyph){
        var dim = this._calculateTooltipTextDimension(tooltip);
        var bounds = new FR.Chart.Bounds(mousePos.x, mousePos.y, dim.width, dim.height);
        var plotBounds = plotGlyph.bounds;

        if(!glyph || !plotBounds){
            return;
        }

        if(glyph.shape || (glyph.jsType && FR.equals(glyph.jsType, "bubble"))){
            bounds = glyph.getTooltipBounds4Shape(bounds, plotBounds);
        }else{
            var x, y;
            var shape;
            if (glyph.bounds) {
                shape = glyph.bounds;
            } else if (glyph.marker){
                shape = new FR.Chart.Bounds(glyph.marker.x, glyph.marker.y, glyph.marker.size, glyph.marker.size);
            }

            x = shape.x + shape.width;
            if (x + bounds.width > plotBounds.x + plotBounds.width) {
                x = shape.x - bounds.width;
            }
            if (x < plotBounds.x){
                x = Math.max(plotBounds.x, plotBounds.x + plotBounds.width - bounds.width);
            }
            y = shape.y + shape.height;
            if (y + bounds.height > plotBounds.y + plotBounds.height) {
                y = shape.y - bounds.height;
            }
            bounds = new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
        }

        return new FR.Chart.Point2D(bounds.x, Math.max(bounds.y, plotBounds.y));
    },

    /**
     * 画趋势线方法
     * @static
     * @param ctx 画布
     * @param trendLineTooltips 趋势线数组
     * @param plotBounds 绘图区边界.
     */
    doTrendLineTooltip: function (ctx, trendLineTooltips, plotBounds) {
        if (trendLineTooltips && trendLineTooltips.length > 0) {
            if(!ctx){     return; } ctx.save();
            ctx.strokeStyle = "rgb(128,128,128)";

            var allWidth = 0;
            for (var i = 0; i < trendLineTooltips.length; i++) {
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(trendLineTooltips[i].text);
                trendLineTooltips[i].dim = dim;
                allWidth += dim.width + this.LEFTGAP * 2;
            }

            var x = plotBounds.width / 2 - allWidth / 2;
            for (var i = 0; i < trendLineTooltips.length; i++) {

                var textDim = trendLineTooltips[i].dim;

                var bounds = new FR.Chart.Bounds(x, 0, textDim.width + this.LEFTGAP * 2, textDim.height + this.TOPGAP * 2);
                FR.Chart.Tooltip._drawWithCanvas(ctx, trendLineTooltips[i].text, bounds, 'white', 1, trendLineTooltips[i].lineColor);
                x += bounds.width;
            }

            ctx.restore();
        }
    },

    _drawWithDiv:function(ctx, tipText, pos, fillStyle, strokeStyle){
        if(ctx._tooltipDiv){
            var tooltipDiv = ctx._tooltipDiv;
            tooltipDiv.innerHTML = tipText;
            tooltipDiv.style.cssText = this._getCssText(pos, fillStyle, strokeStyle);
            if(ctx._hyperlink){
                var tipBounds = this._calculateTooltipTextDimension(tipText);
                var url = FR.servletURL + '?op=resource&resource=/com/fr/chart/base/mobilelink.png';
                var xPos = tipBounds.width - 25;
                var background = "background:url(" + url + ") no-repeat " + xPos + "px 5px";
                if(fillStyle == 'black'){
                    background += ' #000000;' + 'filter:alpha(opacity=50);' + 'background-color:rgba(0,0,0,0.5);';
                }else{
                    background += ' #FFFFFF;' + 'filter:alpha(opacity=75);' + 'background-color:rgba(255,255,255,0.75);';
                }
                tooltipDiv.style.cssText += background;
            }
        }
    },

    _drawWithCanvas:function(ctx, tipText, bounds, fillStyle, lineWidth, strokeStyle){
        FR.Chart.Tooltip._drawBack(ctx, bounds, fillStyle, lineWidth, strokeStyle);
        FR.Chart.PaintUtils.paintTooltip(ctx, tipText, bounds.x + this.LEFTGAP, bounds.y, fillStyle);
    },

    _getCssText:function(pos, fillStyle, strokeStyle){
        var strokeColor = 'rgb(85,85,85)';
        if (strokeStyle && strokeStyle.color) {
            strokeColor = strokeStyle.color;
        }

        var borderCss = 'border-color:'+strokeColor+';';
        if(strokeColor.startWith('rgba')){
            var hexAlpha = FR.Chart.Color.rgb2Hex(strokeColor);
            if(hexAlpha){
                borderCss = 'border-color:' + hexAlpha.hex + ';' + 'filter:alpha(opacity=' + hexAlpha.alpha + ');' + borderCss;
            }
        }

        var backgroundCSS;
        var fontColor = 'black';
        if(fillStyle == 'black'){
            backgroundCSS = 'background-color:#000000;' + 'filter:alpha(opacity=50);' + 'background-color:rgba(0,0,0,0.5);';
            fontColor = 'white';
        }else{
            backgroundCSS = 'background-color:#FFFFFF;' + 'filter:alpha(opacity=75);' + 'background-color:rgba(255,255,255,0.75);';
        }

        var transitionText = 'left ' + 0.4 + 's,' + 'top ' + 0.4 + 's,'+ 'opacity ' + 0.4 + 's;';

        return this.DEFAULT_CSS_TEXT + backgroundCSS + borderCss
            + 'left:'+pos.x+'px;'
            + 'top:'+pos.y+'px;'
            + 'border-radius:'+5+'px;'
            + '-moz-border-radius:'+5+'px;'
            + '-webkit-border-radius:'+5+'px;'
            + '-o-border-radius:'+5+'px;'
            + 'transition:'+transitionText
            + '-moz-transition:' + transitionText
            + '-webkit-transition:' + transitionText
            + '-o-transition:' + transitionText
            + 'color:'+ fontColor + ';'
            + 'display:block;'
            + 'box-shadow:2px 2px 2px rgba(0,0,0,0.25);'
    },

    /**
     *
     * @param ctx 画布
     * @param tipText 文本
     * @param bounds 实际的整个边框样式边界
     * @param fillStyle 边界是否填充颜色
     * @param lineWidth 边框宽度
     * @param strokeStyle 边框颜色
     * @param colors 系列颜色
     */
    _draw4Series: function (ctx, tipText, bounds, fillStyle, strokeStyle, colors) {
        if(ctx._tooltipDiv){
            var tooltipDiv = ctx._tooltipDiv;
            tooltipDiv.style.cssText = this._getCssText(bounds, fillStyle, strokeStyle);
            tooltipDiv.innerHTML = tipText;
        }
    },

    /**
     * 画数据点提示
     * @param ctx 画布
     * @param bounds 边框
     * @param fillStyle 填充颜色
     * @param lineWidth 边框
     * @param strokeStyle 边框颜色
     * @private
     */
    _drawBack: function(ctx, bounds, fillStyle, lineWidth, strokeStyle) {
        var useBounds = new FR.Chart.RoundBounds(bounds.x, bounds.y, bounds.width, bounds.height, 5, 5);

        if(!ctx){     return; } ctx.save();
        ctx.fillStyle = fillStyle;
        ctx.globalAlpha = 0.75;
        if (fillStyle == "black") {
            ctx.globalAlpha = 0.5;
        }
        useBounds.paint(ctx);

        ctx.globalAlpha = 1;
        ctx.lineWidth = lineWidth;
        if (strokeStyle && strokeStyle.color) {
            strokeStyle.draw(ctx, useBounds);
            ctx.strokeStyle = strokeStyle.color;
        } else {
            ctx.strokeStyle = 'rgb(85,85,85)';
            useBounds.draw(ctx);
        }

        useBounds.draw(ctx);

        ctx.restore();
    }

};
//Tooltip.end