/*
 * 各种Glyph. PlotGlyph等
 * 大概三种: Glyph的initConstants(初始化) glyph的Layout(布局) draw(animateGlyphDraw动态的画法)
 */

FR.Chart = FR.Chart || {};

/**
 * 线性图例对象
 *
 *     @example
 *     var glyph = new FR.Chart.LineGlyph();
 *
 * @class FR.Chart.LineGlyph
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 线型配置属性
 *
 * @param {Number} [js.lineStyle=FR.Chart.Constants.LINE_THIN] 线型
 * @param {Color} [js.lineColor = 'black'] 线颜色
 * @param {Number} [js.alpha = 1.0] 线的透明度
 *
 */
FR.Chart.LineGlyph = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};
        this.lineStyle = FR.pick(js.lineStyle, FR.Chart.Constants.LINE_THIN);
        this.lineColor = FR.pick(js.lineColor, 'black');
        this.alpha = FR.pick(js.alpha, 1.0);

        this.line = null;
    },

    /**
     * 画出线型.
     * @param ctx 画布
     */
    draw: function (ctx) {
        if (this.line && this.lineColor) {
            if(!ctx){     return; } ctx.save();

            ctx.globalAlpha = this.alpha;
            ctx.strokeStyle = this.lineColor;
            ctx.lineWidth = FR.Chart.GraphHelper.getLineStyleSize(this.lineStyle);

            var start = this.line.startPoint;
            var end = this.line.endPoint;

            ctx.moveTo(start.x, start.y);
            ctx.lineTo(end.x, end.y);
            ctx.stroke();

            ctx.restore();
        }
    },

    /**
     * 返回线的shape
     * @returns {null|*}
     */
    getShape: function () {
        return this.line;
    }
});
// LineGlyph.end

// TrendLineGlyph.start
/**
 * 趋势线
 *
 *      @example
 *      var glyph = new FR.Chart.TrendLineGlyph(js);
 *
 * @class FR.Chart.TrendLineGlyph 趋势线Glyph
 * @extends FR.Chart.FoldLine
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.equation = FR.Chart.Constants.TRENDLINE_POLYNOMIAL] 趋势线的拟合方式.
 * <br/>{@link FR.Chart.Constants#TRENDLINE_LINEAR}
 * <br/>{@link FR.Chart.Constants#TRENDLINE_POLYNOMIAL}
 * <br/>{@link FR.Chart.Constants#TRENDLINE_LOG}
 * <br/>{@link FR.Chart.Constants#TRENDLINE_EXPONENT}
 * <br/>{@link FR.Chart.Constants#TRENDLINE_POWER}
 * <br/>{@link FR.Chart.Constants#TRENDLINE_MA}
 *
 * @param {Number} [js.square = 6] 拟合的最高次数
 * @param {Number} [js.moveAverage = 5] 移动平均周期
 */
FR.Chart.TrendLineGlyph = FR.extend(FR.Chart.FoldLine, {
    initConstants: function (js) {
        js = js || {};

        this.equation = FR.pick(js.equation, FR.Chart.Constants.TRENDLINE_POLYNOMIAL);
        this.square = FR.pick(js.square, 6);
        this.moveAverage = FR.pick(js.moveAverage, 5);
        FR.Chart.TrendLineGlyph.superclass.initConstants.apply(this, [js]);

        this.drawBounds = null;//记录绘图区大小
        // MA的动态数组.
        this.actionXVal = [];
        this.actionYVal = [];
        this.val = [];
    },

    /**
     * 画出具体的趋势线shape
     * @param bounds 边界范围
     * @param actionNumber 动态Number
     */
    drawShape: function (bounds, actionNumber) { // 得到拟合曲线, 动态
        this.drawBounds = bounds;
        if (actionNumber && this.equation === 5) {
            var number = parseInt(this.actionXVal.length * actionNumber)

            var linePath = new FR.Chart.GeneralPath();
            for (var i = 0; i < number; i++) {
                if (i === 0) {
                    linePath.moveTo(this.actionXVal[i], this.actionYVal[i]);
                } else {
                    linePath.lineTo(this.actionXVal[i], this.actionYVal[i]);
                }
            }

            var path = new FR.Chart.GeneralPath();
            FR.Chart.FoldLine.curveTo(path, linePath);

            this.generalPath = path;
        } else {
            var isStartPoint = true;
            var path = new FR.Chart.GeneralPath();

            for (var x = bounds.x, y; x < bounds.width + bounds.x; x += 1) {
                y = this.calculator(x);
                if (y > bounds.height + bounds.y || y < 0) {
                    isStartPoint = true;
                } else {
                    if (isStartPoint) {
                        path.moveTo(x, y);
                        isStartPoint = false;
                    } else {
                        path.lineTo(x, y);
                    }
                }
            }
            this.generalPath = path;
        }
    },

    /**
     *    计算趋势线的动态步骤path, 其中width, height提前在getAnimateGlyph时初始化的
     * @param stepNumber 动态步骤序号
     * @param actionModel 动态方式
     * @returns {Array} 返回动态数组.
     */
    getAnimateGlyph: function (stepNumber, actionModel) {
        var width = this.drawBounds.width;
        var height = this.drawBounds.height;

        if (width <= 0 || height <= 0) {
            return [];
        }

        var animateGlyph = [];
        for (var i = 0; i < stepNumber; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                animateGlyph[i] = this;
            } else {
                var line = new FR.Chart.TrendLineGlyph();
                line.val = this.val;

//				line.generalPath = this.generalPath;

                line.lineStyleInfo = this.lineStyleInfo;
                line.equation = this.equation;
                line.actionXVal = this.actionXVal;
                line.actionYVal = this.actionYVal;

                // (i + 1) / stepNumber  actionNumber 只有MA用到.
                line.drawShape(new FR.Chart.Bounds(this.drawBounds.x, this.drawBounds.y, FR.Chart.ChartEase.easeOutQuint(i + 1, 0, width, stepNumber), height), (i + 1) / stepNumber);

                animateGlyph[i] = line;
            }
        }

        return animateGlyph;
    },

    /**
     * 根据坐标轴点数组进行拟合
     * @param xVal x数组
     * @param yVal y数组.
     */
    fitting: function (xVal, yVal) {
        if (xVal.length != yVal.length || xVal.length === 0 || yVal.length === 0) {
            return;
        }

        this.square = this.square > xVal.length - 1 ? xVal.length - 1 : this.square; // 根据点的个数判断多项式拟合时最高次数

        if (this.equation == FR.Chart.Constants.TRENDLINE_LINEAR) {
            this.linear(xVal, yVal);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_POLYNOMIAL) {
            this.polynomial(xVal, yVal, this.square);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_LOG) {
            this.log(xVal, yVal);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_EXPONENT) {
            this.exponent(xVal, yVal);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_POWER) {
            this.power(xVal, yVal);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_MA) {
            this.ma(xVal, yVal);
        }
    },

    /**
     * 计算, 根据x值 计算出拟合的y值
     * @param x  x值
     * @returns 返回y值.
     */
    calculator: function (x) {
        if (this.equation == FR.Chart.Constants.TRENDLINE_LINEAR) {
            return this.linearFunc(x);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_POLYNOMIAL) {
            return this.polynomialFunc(x);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_LOG) {
            return this.logFunc(x);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_EXPONENT) {
            return this.exponentFunc(x);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_POWER) {
            return this.powerFunc(x);
        } else if (this.equation == FR.Chart.Constants.TRENDLINE_MA) {
            return this.maFunc(x);
        }

        return NaN;
    },

    linear: function (xVal, yVal) {
        var mX = 0, mY = 0, mXX = 0, mXY = 0;
        var n = xVal.length;

        for (var i = 0; i < xVal.length; i++) {
            mX += xVal[i];
            mY += yVal[i];
            mXX += xVal[i] * xVal[i];
            mXY += xVal[i] * yVal[i];
        }

        if (mX * mX - mXX * n === 0) {
            return;
        }
        this.val[0] = (mY * mX - mXY * n) / (mX * mX - mXX * n);
        this.val[1] = (mY - mX * this.val[0]) / n;
    },

    polynomial: function (xVal, yVal, M) {
        M++;
        var t;
        var b = [M];
        var A = [M];
        for (var i = 0; i < M; i++) {
            b[i] = [xVal.length];
            A[i] = [M];
        }
        var B = [M];
        var y = [M];
        this.val = [M];

        var i, j, k, l;
        for (i = 0; i < M; i++) {
            for (j = 0; j < xVal.length; j++) {
                t = 1;
                for (l = 0; l < i; l++) {
                    t = t * xVal[j];
                }
                b[i][j] = t;
            }
        }

        for (i = 0; i < M; i++) {
            for (k = 0; k < M; k++) {
                t = 0;
                for (j = 0; j < xVal.length; j++) {
                    t += b[i][j] * b[k][j];
                }
                A[i][k] = t;
            }
        }
        for (i = 0; i < M; i++) {
            t = 0;
            for (j = 0; j < xVal.length; j++) {
                t += yVal[j] * b[i][j];
            }
            B[i] = t;
        }

        for (i = 1; i < M; i++) {
            A[i][0] = A[i][0] / A[0][0];
        }
        for (i = 1; i < M; i++) {
            for (j = i; j < M; j++) {
                var tmp = 0;
                for (k = 0; k < i; k++) {
                    var a = A[k][j];
                    var b = A[i][k];
                    tmp = tmp + a * b;
                }
                var test = A[i][j] - tmp;
                A[i][j] = test;
                if (j + 1 != M) {
                    var tmp = 0;
                    for (k = 0; k < i; k++) {
                        tmp += A[k][i] * A[j + 1][k];
                    }
                    A[j + 1][i] = (A[j + 1][i] - tmp) / A[i][i];
                }

            }
        }
        y[0] = B[0];
        for (i = 1; i < M; i++) {
            t = 0;
            for (j = 0; j < i; j++) {
                t += A[i][j] * y[j];
            }
            y[i] = B[i] - t;
        }

        this.val[M - 1] = y[M - 1] / A[M - 1][M - 1];
        for (i = M - 2; i >= 0; i--) {
            t = 0;
            for (j = i + 1; j < M; j++) {
                t += A[i][j] * this.val[j];
            }
            this.val[i] = (y[i] - t) / A[i][i];
        }
    },

    log: function (xVal, yVal) {
        var newX = [];
        var newY = [];
        var l = 0;
        for (var i = 0; i < xVal.length; i++) {
            if (xVal[i] > 0) {
                newX[l] = Math.log(xVal[i]);
                newY[l] = yVal[i];
                l++;
            }
        }

        this.linear(newX, newY);
    },

    exponent: function (xVal, yVal) {
        var newX = [];
        var newY = [];
        var l = 0;
        for (var i = 0; i < yVal.length; i++) {
            if (yVal[i] > 0) {
                newX[l] = xVal[i];
                newY[l] = Math.log(yVal[i]);
                l++;
            }
        }

        this.linear(newX, newY);

        var temp = this.val[0];
        this.val[0] = Math.exp(this.val[1]);
        this.val[1] = temp;
    },

    power: function (xVal, yVal) {
        var newX = [];
        var newY = [];
        var l = 0;
        for (var i = 0; i < xVal.length; i++) {
            if (xVal[i] > 0 && yVal[i] > 0) {
                newX[l] = Math.log(xVal[i]);
                newY[l] = Math.log(yVal[i]);
                l++;
            }
        }

        this.linear(newX, newY);

        var temp = this.val[0];
        this.val[0] = Math.exp(this.val[1]);
        this.val[1] = temp;
    },

    ma: function (xVal, yVal) {
        this.actionXVal = xVal;
        this.actionYVal = [];

        var linePath = new FR.Chart.GeneralPath();

        for (var i = 0; i < xVal.length; i++) {
            if (i === 0) {
                linePath.moveTo(xVal[i], yVal[i]);

                this.actionYVal[i] = yVal[i];
            } else if (i < this.moveAverage) {
                var tmpY = 0;
                for (var tmp = 0; tmp < i + 1; tmp++) {
                    tmpY += yVal[tmp];
                }
                linePath.lineTo(xVal[i], tmpY / (i + 1));

                this.actionXVal[i] = xVal[i];
                this.actionYVal[i] = tmpY / (i + 1);
            } else {
                var tmpY = 0;
                for (var j = 0; j < this.moveAverage; j++) {
                    tmpY += yVal[i - j];
                }
                linePath.lineTo(xVal[i], tmpY / this.moveAverage);

                this.actionYVal[i] = tmpY / this.moveAverage;
            }
        }

        var path = new FR.Chart.GeneralPath();
        FR.Chart.FoldLine.curveTo(path, linePath);

        this.generalPath = path;
    },

    linearFunc: function (x) {
        return this.val[0] * x + this.val[1];
    },

    polynomialFunc: function (x) {
        var result = 0;
        for (var i = 0; i < this.val.length; i++) {
            result += this.val[i] * Math.pow(x, i);
        }
        return result;
    },

    logFunc: function (x) {
        if (x <= 0) {
            return NaN;
        }
        return this.val[0] * Math.log(x) + this.val[1];
    },

    exponentFunc: function (x) {
        return this.val[0] * Math.exp(x * this.val[1]);
    },

    powerFunc: function (x) {
        return this.val[0] * Math.pow(x, this.val[1]);
    },

    // 这个function是x的坐标值通过ma的已知点和贝塞尔公式去得到y坐标点.
    maFunc: function (x) {
        // 只有两点的时候. 连成直线
        if (this.actionXVal.length === 2) {
            if (x < this.actionXVal[0] || x > this.actionXVal[1]) {
                return NaN;
            }

            if (this.actionXVal[0] == this.actionXVal[1]) {
                return this.actionYVal[0];
            } else {
                var t = Math.abs(x - this.actionXVal[0]) / Math.abs(this.actionXVal[1] - this.actionXVal[0]);
                return this.actionYVal[0] - Math.abs(this.actionYVal[0] - this.actionYVal[1]) * t;
            }
        }

        var actionXVal = this.actionXVal;
        var actionYVal = this.actionYVal;

        var minX, maxX = 0;
        for (var i = 0; i < this.actionXVal.length; i++) {
            if (x >= this.actionXVal[i] && x <= this.actionXVal[i + 1]) {
                minX = i;
                maxX = i + 1;
                break;
            }
        }

        if (maxX <= minX) {
            return NaN;
        }

        var startY = this.actionYVal[minX], endY = this.actionYVal[maxX];

        // 下面的计算控制点 和FoldLine的CurveTo(Path) 是一样的
        var scale = 0.6;
        var midX = [];
        var midY = [];

        for (var i = 0; i < actionXVal.length - 1; i++) {
            midX[i] = (actionXVal[i] + actionXVal[i + 1]) / 2;
            midY[i] = (actionYVal[i] + actionYVal[i + 1]) / 2;
        }

        var ctrlX = [];
        var ctrlY = [];

        for (var i = 1; i < midX.length; i++) {
            var offsetX = actionXVal[i] - (midX[i - 1] + midX[i]) / 2;
            var offsetY = actionYVal[i] - (midY[i - 1] + midY[i]) / 2;

            ctrlX[(i - 1) * 2] = actionXVal[i] + (midX[i - 1] + offsetX - actionXVal[i]) * scale;
            ctrlY[(i - 1) * 2] = actionYVal[i] + (midY[i - 1] + offsetY - actionYVal[i]) * scale;

            ctrlX[(i - 1) * 2 + 1] = actionXVal[i] + (midX[i] + offsetX - actionXVal[i]) * scale;
            ctrlY[(i - 1) * 2 + 1] = actionYVal[i] + (midY[i] + offsetY - actionYVal[i]) * scale;
        }

        var y = NaN;
        var t = Math.abs(x - actionXVal[minX]) / Math.abs(actionXVal[maxX] - actionXVal[minX]);
        if (maxX === 1 || maxX == actionXVal.length - 1) { // 二阶的
            y = (1 - t) * (1 - t) * startY + 2 * t * (1 - t) * ctrlY[0] + t * t * endY;
        } else { // 三阶的
            y = (1 - t) * (1 - t) * (1 - t) * startY
                + 3 * ctrlY[minX * 2 - 1] * t * (1 - t) * (1 - t) + 3 * ctrlY[minX * 2] * t * t * (1 - t) + endY * t * t * t;
        }
        return y;
    }
});
// TrendLineGlyph.end

// GeneralGlyph.start
/**
 *      @example
 *      var glyph = new FR.Chart.GeneralGlyph(); 新建路径Glyph
 *
 * @class FR.Chart.GeneralGlyph
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 * @param {Object} [js.info = new FR.Chart.GeneralInfo()];
 */
FR.Chart.GeneralGlyph = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.info = new FR.Chart.GeneralInfo(js.background || js.info);
    },

    getBackground: function () {
        return this.info.getBackground();
    },

    setBackground: function (background) {
        this.info.setBackground(background);
    },

    getBorderStyle: function () {
        return this.info.getBorderStyle();
    },

    setBorderStyle: function (borderStyle) {
        this.info.setBorderStyle(borderStyle);
    },

    getBorderColor: function () {
        return this.info.getBorderColor();
    },

    setBorderColor: function (borderBackground) {
        this.info.setBorderColor(borderBackground);
    },

    isRoundBorder: function () {
        return this.info.isRoundBorder();
    },

    setRoundBorder: function (isRoundBorder) {
        this.info.setRoundBorder(isRoundBorder);
    },

    getAlpha: function () {
        return this.info.getAlpha();
    },

    setAlpha: function (alpha) {
        this.info.setAlpha(alpha);
    },

    draw: function (ctx) {
        this.info.paint(ctx, this.getOutline4Fill());
    },

    draw4Demo: function (ctx) {
        var shape = this.getOutline4Fill();
        if (shape) {
            shape.paint(ctx);
        }
    }

});
// GeneralGlyph.end

// RectangleGlyph.start
/**
 *矩形区域图形
 *
 *      @example
 *      var glyph = new FR.Chart.RectangleGlyph();
 *
 * @class FR.Chart.RectangleGlyph
 * @extends FR.Chart.GeneralGlyph
 *
 * @param {JSON} js 配置属性
 * @param {Boolean} [js.visible = true] Glyph是否可见
 *
 */
FR.Chart.RectangleGlyph = FR.extend(FR.Chart.GeneralGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.visible = FR.pick(js.visible, true);

        this.bounds = null;

        FR.Chart.RectangleGlyph.superclass.initConstants.apply(this, [js]);
    },

    getBounds: function () {
        if (this.bounds) {
            return this.bounds.getBounds2D();
        }

        return null;
    },

    /**
     * 返回填充 区域
     * @returns {Object} 填充区域.
     */
    getOutline4Fill: function () {
        return this.bounds;
    },

    getShape: function () {
        return this.bounds;
    }
});
// RectangleGlyph.end

// ShapeGlyph.start
/**
 * Shape图形
 * @class FR.Chart.ShapeGlyph
 * @extends FR.Chart.GeneralGlyph
 *
 * @param {JSON} js 配置属性
 */
FR.Chart.ShapeGlyph = FR.extend(FR.Chart.GeneralGlyph, {
    initConstants: function (js) {
        js = js || {};
        // kunsnat: shape js这只用到一些background, border, borderColor等, 不会用到shape.
        this.shape = null;

        FR.Chart.ShapeGlyph.superclass.initConstants.apply(this, [js]);
    },

    setShape: function (shape) {
        this.shape = shape;
    },

    getShape: function () {
        return this.shape;
    },

    /**
     * 返回填充区域, 根据Shape而定.
     * @returns 填充区域.
     */
    getOutline4Fill: function () {
        return this.getShape();
    },

    /**
     * 画出形状  风格.
     * @param ctx 画布.
     */
    draw: function (ctx) {
        if (this.dataPointStyle) {
            this.dataPointStyle.paintStyle(ctx);
        } else {
            this.info.paint(ctx, this.getOutline4Fill());
        }
    },

    /**
     * 画出形状  风格.
     * @param ctx 画布.
     */
    drawWithOutBorder: function (ctx) {
        if (this.dataPointStyle) {
            this.dataPointStyle.paintStyle(ctx);
        } else {
            this.info.paintWithOutBorder(ctx, this.getOutline4Fill());
        }
    },

    //只画边框
    drawOnlyBorder:function(ctx,count){
        if(this.dataPointStyle){
            return ;
        }
        this.info.paintOnlyBorder(ctx, this.getOutline4Fill());
    },


    /**
     * 边框是否是默认的
     */
    isBorderDefault:function (){
        if (this.dataPointStyle || !this.info || !this.info.seriesAttrBorder) {
            return true;
        }

        return  this.info.seriesAttrBorder.borderStyle === FR.Chart.Constants.LINE_THIN &&
            this.info.seriesAttrBorder.borderColor === 'rgb(255,255,255)';
    },

    drawWithOutStyle: function (ctx) {
        this.info.paint(ctx, this.getOutline4Fill());
    },

    getFillStyleColor:function(){
        if(this.info && this.info.seriesAttrBackground.seriesBackground){
            return this.info.seriesAttrBackground.seriesBackground.color;
        }
        return null;
    },

    /**
     * 设置tooltip位置
     */
    getTooltipBounds4Shape:function(bounds, outerBounds) {
        if (this.shape instanceof FR.Chart.Arc2D || this.shape instanceof FR.Chart.Ring) {
            var point = this.shape.getHalfAnglePoint();
            var angle = this.shape.getHalfAngle();
            return this.getTooltipBounds4ArcRing(bounds, outerBounds, point, angle);
        }
        if (this.shape instanceof FR.Chart.Pie3D) {
            var point = this.shape.top().getMiddlePoint();
            var angle = this.shape.top().getMiddleAngle();
            return this.getTooltipBounds4ArcRing(bounds, outerBounds, point, angle);
        }
        if (this.shape instanceof FR.Chart.Ring3D) {
            var pie3D = this.shape.outerCylinderList[0];
            if (pie3D) {
                var point = pie3D.top().getMiddlePoint();
                var angle = pie3D.top().getMiddleAngle();
                return this.getTooltipBounds4ArcRing(bounds, outerBounds, point, angle);
            }
        }
        if (this.shape instanceof FR.Chart.Bounds) {
            return this.getTooltipBounds4Bounds(bounds, outerBounds);
        }
        if (this.shape instanceof FR.Chart.Cubic) {
            return this.getTooltipBounds4Cubic(bounds, outerBounds);
        }
        if (this.shape instanceof FR.Chart.GeneralPath) {
            return this.getTooltipBounds4GeneralPath(bounds, outerBounds);
        }
    },

    getTooltipBounds4GeneralPath: function(bounds, outerBounds) {
        var x;
        var y;
        var shape = this.shape.getBounds2D();
        x = shape.x + shape.width / 2;
        if (x + bounds.width > outerBounds.x + outerBounds.width) {
            x -= bounds.width;
        }
        x = this.getTooltipStartX(x, bounds, outerBounds, false);
        y = this.shape.getVerticalMiddlePointY();
        if (y + bounds.height > outerBounds.y + outerBounds.height) {
            y -= bounds.height;
        }
        return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
    },

    getTooltipBounds4Cubic: function(bounds, outerBounds) {
        var x;
        var y;
        if (this.shape.horizontalDrawBar) {
            x = this.shape.points[3].x - bounds.width;
            if (x < outerBounds.x) {
                x = this.shape.points[3].x;
            }
            x = this.getTooltipStartX(x, bounds, outerBounds, true);
            y = this.shape.points[3].y + 3;
            if (y + bounds.height > outerBounds.y + outerBounds.height) {
                y = this.shape.points[1].y - bounds.height - 3;
            }
            return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
        } else {
            y = this.shape.points[1].y;
            if (y + bounds.height > outerBounds.y + outerBounds.height) {
                y = this.shape.points[1].y - bounds.height;
            }
            x = this.shape.points[1].x + 3;
            if (x + bounds.width > outerBounds.x + outerBounds.width) {
                x = this.shape.points[0].x - bounds.width - 3;
                y = this.shape.points[0].y - bounds.height;
            }
            x = this.getTooltipStartX(x, bounds, outerBounds, false);
            return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
        }
    },

    getTooltipBounds4Bounds: function(bounds, outerBounds) {
        var x;
        var y;
        if (this.shape.isHorizontal4Bounds) {
            x = this.shape.x + this.shape.width - bounds.width;
            if (x < outerBounds.x) {
                x = this.shape.x + this.shape.width;
            }
            x = this.getTooltipStartX(x, bounds, outerBounds, true);
            y = this.shape.y + this.shape.height + 3;
            if (y + bounds.height > outerBounds.y + outerBounds.height) {
                y = this.shape.y - bounds.height - 3;
            }
            y = Math.max(y, outerBounds.y - 3);
            return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
        } else {
            x = this.shape.x + this.shape.width + 3;
            if (x + bounds.width > outerBounds.x + outerBounds.width) {
                x = this.shape.x - bounds.width - 3;
            }
            x = this.getTooltipStartX(x, bounds, outerBounds, false);
            y = this.shape.y;
            if (y + bounds.height > outerBounds.y + outerBounds.height) {
                y = this.shape.y - bounds.height;
            }
            return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
        }
    },

    getTooltipStartX: function (x, bounds, outerBounds, isHorizontal) {
        if(isHorizontal){
            if (x + bounds.width > outerBounds.x + outerBounds.width){
                return outerBounds.x;
            }
        } else {
            if (x < outerBounds.x){
                return Math.max(outerBounds.x, outerBounds.x + outerBounds.width - bounds.width);
            }
        }
        return x;
    },

    getTooltipBounds4ArcRing: function(bounds, outerBounds, point, angle) {
        var x;
        var y;
        angle %= 360;
        if (angle < 90) {
            x = point.x - bounds.width - 1;
            y = point.y - bounds.height - 1;
            if (x < outerBounds.x || y < outerBounds.y) {
                x = point.x + 1;
                y = point.y + 1;
            }
        } else if (angle < 180) {
            x = point.x + 1;
            y = point.y - bounds.height - 1;
            if (x + bounds.width > outerBounds.x + outerBounds.width || y < outerBounds.y) {
                x = point.x - bounds.width - 1;
                y = point.y + 1;
            }
        } else if (angle < 270) {
            x = point.x + 1;
            y = point.y + 1;
            if (x + bounds.width > outerBounds.x + outerBounds.width || y + bounds.height > outerBounds.y + outerBounds.height) {
                x = point.x - bounds.width - 1;
                y = point.y - bounds.height - 1;

            }
        } else {
            x = point.x - bounds.width - 1;
            y = point.y + 1;

            if (x < outerBounds.x || y + bounds.height > outerBounds.y + outerBounds.height) {
                x = point.x + 1;
                y = point.y - bounds.height - 1;
            }
        }

        if(90 <= angle && angle < 270){
            if(x < outerBounds.x){
                x = outerBounds.x;
            }
        } else {
            if(x + bounds.width >  outerBounds.x + outerBounds.width){
                x = outerBounds.x;
            }
        }
        return new FR.Chart.Bounds(x, y, bounds.width, bounds.height);
    },

    /**
     * 返回动态步骤的Glyph数组
     * @param step 步骤个数
     * @param actionModel 变化方式
     * @param lastGlyph 上一步的Glyph
     * @param paintState 当前系列的状态.
     * @returns  动态步骤的数组.
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph, paintState) {
        if (this.isFade) {
            return this.getAnimateGlyphWithFade(step, actionModel, lastGlyph, paintState);
        }

        if (this.shape) {
            var lastShape = null;
            if (lastGlyph && lastGlyph.shape && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
                lastShape = lastGlyph.shape;
            }
            this.animateShape = this.shape.getAnimateShape(step, actionModel, lastShape, paintState);
        }

        var animateGlyph = [];
        for (var i = 0; i < this.animateShape.length; i++) {
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                animateGlyph[i] = this;
            } else {
                animateGlyph[i] = new FR.Chart.ShapeGlyph();
                animateGlyph[i].shape = this.animateShape[i];

                animateGlyph[i].info.seriesAttrAlpha = this.info.seriesAttrAlpha;
                animateGlyph[i].info.seriesAttrBorder = this.info.seriesAttrBorder;
                animateGlyph[i].info.seriesAttrBackground = this.info.seriesAttrBackground;

                animateGlyph[i].info.shadow = this.info.shadow;
                animateGlyph[i].info.outerLight = this.info.outerLight;
            }

            if (this.dataPointStyle) {
                animateGlyph[i].dataPointStyle = this.dataPointStyle.clone(this.animateShape[i]);
                animateGlyph[i].dataPointStyle.animateGlyph = animateGlyph[i];
            }
        }

        return animateGlyph;
    },

    /**
     * 地图,淡入淡出 动态变化方式
     * @param step 步骤
     * @param actionModel 变化方式
     * @param lastGlyph 上一步Glyph
     * @param paintState 当前状态
     * @returns {Array} 动态变化数组.
     */
    getAnimateGlyphWithFade: function (step, actionModel, lastGlyph, paintState) {
        if (paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        var animateGlyph = [];
        for (var i = 0; i < step; i++) {
            // 不变
            if ((actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL)
                || (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE
                && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE)) {
                animateGlyph[i] = this;
                // 出现
            } else if ((actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_DEFAULT
                && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE)
                || (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE
                && paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR)) {

                animateGlyph[i] = new FR.Chart.ShapeGlyph();
                animateGlyph[i].shape = this.shape;

                animateGlyph[i].info.seriesAttrAlpha.alpha =
                    FR.Chart.ChartEase.easeInCirc(i + 1, 0, this.info.seriesAttrAlpha.alpha, step);

                animateGlyph[i].info.seriesAttrBorder = this.info.seriesAttrBorder;
                animateGlyph[i].info.seriesAttrBackground = this.info.seriesAttrBackground;

                animateGlyph[i].info.shadow = this.info.shadow;
                animateGlyph[i].info.outerLight = this.info.outerLight;
                // 消失
            } else if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE
                && paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {

                animateGlyph[i] = new FR.Chart.ShapeGlyph();
                animateGlyph[i].shape = this.shape;

                animateGlyph[i].info.seriesAttrAlpha.alpha =
                    FR.Chart.ChartEase.easeOutCirc(i + 1, this.info.seriesAttrAlpha.alpha, 0 - this.info.seriesAttrAlpha.alpha, step);

                animateGlyph[i].info.seriesAttrBorder = this.info.seriesAttrBorder;
                animateGlyph[i].info.seriesAttrBackground = this.info.seriesAttrBackground;

                animateGlyph[i].info.shadow = this.info.shadow;
                animateGlyph[i].info.outerLight = this.info.outerLight;
            }
        }

        return animateGlyph;
    },

    /**
     *  面积图上边缘动态
     * @param step 步骤
     * @param actionModel 动态方式
     * @param topFoldLine 面积图上边缘的线
     * @param lastGlyph 上一步的Glyph
     * @param paintState 当恰状态
     * @returns {Array} 动态数组.
     */
    getAnimateGlyph4TopFoldLine: function (step, actionModel, topFoldLine, lastGlyph, paintState) {
        if (paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }

        if (this.dataPointStyle && this.dataPointStyle instanceof FR.Chart.AreaPlane3DStyle) {
            return [];
        }

        if (this.shape && topFoldLine) {
            var lastShape = null;
            if (lastGlyph && actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE) {
                lastShape = lastGlyph.shape;
            }
            topFoldLine.animateShape = this.shape.getTopFoldLineAnimateShape(step, actionModel, lastShape, paintState);
        }

        var animateGlyph = [];
        for (var i = 0; i < topFoldLine.animateShape.length; i++) {
            animateGlyph[i] = new FR.Chart.FoldLine(topFoldLine.animateShape[i]);

            animateGlyph[i].lineStyleInfo = topFoldLine.lineStyleInfo;
        }

        return animateGlyph;
    }
});
// ShapeGlyph.end


// TextGlyph.start
/**
 * 文本图形
 * @class FR.Chart.TextGlyph
 * @extends FR.Chart.RectangleGlyph
 *
 * @param {JSON} js 配置
 *
 * @param {String} [js.text = null] 文本值
 * @param {Number} [js.position = FR.Chart.Constants.CENTER]  文本位置
 * <br/>{@link FR.Chart.Constants#CENTER}
 * <br/>{@link FR.Chart.Constants#TOP}
 * <br/>{@link FR.Chart.Constants#BOTTOM}
 * <br/>{@link FR.Chart.Constants#LEFT}
 * <br/>{@link FR.Chart.Constants#RIGHT}
 * <br/>{@link FR.Chart.Constants#RIGHT_TOP}
 *
 * @param {FR.Chart.TextAttr} [js.textAttr = new FR.Chart.TextAttr()] 文本格式.
 *
 */
FR.Chart.TextGlyph = FR.extend(FR.Chart.RectangleGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.text = js.text ? $.trim(js.text) : null;

        this.position = FR.Chart.ChartUtils.getPosition(FR.pick(js.align || js.position ||FR.Chart.Constants.CENTER));
        this.textAttr = new FR.Chart.TextAttr(js.textAttr);
        if(js.font){
            this.textAttr.frFont = new FR.Chart.FRFont(js.font);

        }

        FR.Chart.TextGlyph.superclass.initConstants.apply(this, [js]);

        this.boundsGap = FR.pick(js.boundsGap, 0);
    },

    /**
     * 画出文本
     * @param ctx 画布
     * @param currentDivKey 当前图表控件div 的key
     */
    draw: function (ctx, currentDivKey) {
        if (!this.getBounds() || !this.visible) {
            return;
        }

        var rotation = 0;
        if (this.textAttr != null) {
            rotation = this.textAttr.rotation;
        }
        var testAngle = rotation * Math.PI / 180;
        var rx = 0;
        var ry = 0;
        if (rotation > 0) {
            rx = Math.sin(testAngle) * this.getBounds().height;
        } else {
            ry = Math.sin(testAngle) * this.getBounds().width;
        }

        if(!ctx){     return; } ctx.save();

        ctx.translate(this.getBounds().x + rx, this.getBounds().y - ry);
        ctx.rotate(testAngle);
        FR.Chart.RectangleGlyph.superclass.draw.apply(this, [ctx]);//  边框  无背景, 肯定会用到旋转坐标系.
        ctx.translate(-this.getBounds().x + rx, -this.getBounds().y - ry);

        ctx.globalAlpha = 1.0;
        this.drawText(ctx, currentDivKey);
        ctx.restore();
    },

    /**
     * 直接画出文本
     * @param ctx 画布
     * @param currentDivKey 当前图表控件的key
     */
    drawText: function (ctx, currentDivKey) {
        var bounds = this.getBounds();
        var gapBounds = this.getTextBoundsWithOutGap(bounds);
        if (gapBounds != null && this.text) {
            var reText = FR.Chart.TextUtils.changeAllReline2Normal(this.text);

            var al = reText.split('\n');
            for (var relineIndex = 0; relineIndex < al.length; relineIndex++) {
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(al[relineIndex], this.textAttr);

                var labelX = this.boundsGap;
                var labelY = this.boundsGap + (dim.height) * relineIndex;

                if (this.textAttr != null && this.textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
                    labelX = this.boundsGap + (dim.width) * relineIndex;
                    labelY = this.boundsGap;
                } else {
                    if (this.position == FR.Chart.Constants.CENTER) {// 非旋转的情况
                        labelX = this.boundsGap + (gapBounds.width - dim.width) / 2;
                    } else if (this.position == FR.Chart.Constants.LEFT) {
                        labelX = this.boundsGap;
                    } else if (this.position == FR.Chart.Constants.RIGHT) {
                        labelX = this.boundsGap + (gapBounds.width - dim.width);
                    }
                }
                var boun = new FR.Chart.Bounds(labelX + bounds.x, labelY + 2 + bounds.y, dim.width, dim.height);
                FR.Chart.TextUtils.drawLabelWithNoRotation(ctx, al[relineIndex], this.textAttr, boun, currentDivKey);
            }
        }
    },

    drawAxisTitle: function (ctx, currentDivKey) {
        if (this.textAttr != null && this.textAttr.alignText == FR.Chart.Style.VERTICALTEXT) {
            if (!this.getBounds() || !this.visible) {
                return;
            }
            var gapBounds = this.getTextBoundsWithOutGap(this.getBounds());
            if (gapBounds != null && this.text) {
                if (!this.textAttr.frFont) {
                    this.textAttr.frFont = new FR.Chart.FRFont();
                }

                var tmpTextAttr = new FR.Chart.TextAttr();
                tmpTextAttr.frFont = this.textAttr.frFont;
                tmpTextAttr.alignText = FR.Chart.Style.HORIZONTALTEXT;
                var tmpDim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(this.text, tmpTextAttr);

                var dim = new FR.Chart.Dimension2D();
                dim.width = this.textAttr.frFont.size + this.boundsGap * 2;
                dim.height = tmpDim.width + this.boundsGap * 4;

                var labelX = this.boundsGap + this.bounds.x;
                var labelY = this.boundsGap + (gapBounds.height - dim.height) / 2 + this.bounds.y;

                var boun = new FR.Chart.Bounds(labelX, labelY, dim.width, dim.height);

                if (this.textAttr.rotation === -90) {
                    boun.x += this.textAttr.frFont.size;
                }

                this.textAttr.alignText = FR.Chart.Style.HORIZONTALTEXT;

                FR.Chart.TextUtils.drawLabelWithRotation(ctx, this.text, this.textAttr, boun);

                this.textAttr.alignText = FR.Chart.Style.VERTICALTEXT;
            }

        }else{
            this.drawText(ctx, currentDivKey);
        }
    },


    /**
     * 返回当前文本的填充区域
     * @returns 返回边界.
     */
    getOutline4Fill: function () {
        var bounds = this.bounds;
        if (bounds && bounds.arcHeight) {
            return new FR.Chart.RoundBounds(bounds.x, bounds.y, bounds.width, bounds.height, bounds.arcWidth, bounds.arcHeight);
        } else {
            return new FR.Chart.Bounds(bounds.x, bounds.y, bounds.width, bounds.height);
        }
    },

    /**
     *返回带有outGap边界的 文本区域.
     * @param bounds 文本占用数据.
     * @returns 返回区域.
     */
    getTextBoundsWithOutGap: function (bounds) {
        if (this.notEnoughBounds4Text(bounds)) {
            return null;
        }

        return new FR.Chart.Bounds(bounds.x + this.boundsGap, bounds.y + this.boundsGap,
            bounds.width - this.boundsGap * 2, bounds.height - this.boundsGap * 2);
    },

    /**
     * 判断区域是否足够 文本
     * @param bounds 目标bounds
     * @returns {boolean}  是否足够, false表示足够
     */
    notEnoughBounds4Text: function (bounds) {
        return bounds == null || Math.abs(bounds.width) < Math.abs(this.boundsGap) * 2
            || Math.abs(bounds.height) < Math.abs(this.boundsGap) * 2;
    },

    /**
     * 预计算文本占用大小
     * @returns 返回文本占用大小
     */
    preferredDimension: function () {
        return FR.Chart.TextUtils.calculateTextDimensionWithRotation(this.text, this.textAttr);
    },

    /**
     * 处理条件属性
     * @param collection  图表条件属虚构in
     * @param obj 对应的条件
     */
    dealCondition: function (collection, obj) {
        var attrAlpha = collection.getConditionAttr(new FR.Chart.AttrAlpha(), obj);
        this.setAlpha(attrAlpha.alpha);
    }
});
// TextGlyph.end

//TitleGlyph.start
/**
 * 标题文本
 * @class FR.Chart.TitleGlyph 标题文本.
 * @extends FR.Chart.TextGlyph
 */
FR.Chart.TitleGlyph = FR.extend(FR.Chart.TextGlyph, {

    /**
     * 预计算文本占用大小
     * @param chartWidth 图表宽度
     * @param chartHeight 图表高度
     * @returns {FR.Chart.Dimension2D} 返回占用的空间大小.
     */
    preferredDimension: function (chartWidth, chartHeight) {
        var TITLE_GAP = 4;
        var MAX_TITLE_HEIGHT_PERCENT = 0.2;

        var titleText = FR.Chart.ChartUtils.dealTitleStringAutoWrap(this.text, this.textAttr, chartWidth - TITLE_GAP * 2);

        var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(titleText, this.textAttr);

        titleText = FR.Chart.TextUtils.changeAllReline2Normal(titleText);
        var strs = titleText.split('\n');
        var gapCount = strs.length;

        return new FR.Chart.Dimension2D(Math.min(dim.width + TITLE_GAP * 2 * gapCount, chartWidth),
            Math.min(dim.height + TITLE_GAP * 2 * gapCount, chartHeight * MAX_TITLE_HEIGHT_PERCENT));
    },

    /**
     * 画出文本图例
     * @param backctx  使用的画布.
     */
    draw: function (backctx) {
        if (!this.getBounds() || !this.visible) {
            return;
        }

        if(!backctx){
            return;
        }

        backctx.save();
        FR.Chart.RectangleGlyph.superclass.draw.apply(this, [backctx]);
        backctx.restore();

        this.drawText();
    },

    /**
     * 画出文本
     */
    drawText: function () {

        var bounds = this.getBounds();
        var gapBounds = this.getTextBoundsWithOutGap(bounds);
        if (gapBounds) {

            var alignValue = 'left';
            if (this.position == FR.Chart.Constants.CENTER) {
                alignValue = 'center';
            } else if (this.position == FR.Chart.Constants.RIGHT) {
                alignValue = 'right';
            }

            var font = '12px Microsoft YaHei';
            var fontcolor = 'black';
            if (this.textAttr && this.textAttr.frFont) {
                var frfont = this.textAttr.frFont;

                var size = parseInt(frfont.size);// kunsnat: 取整, 因为大于半数, 容易有截取的现象.
                font = frfont.style + ' ' + size + 'px  ' + frfont.fontName;
                fontcolor = frfont.color;
            }

            var offWidth = (FR.Chart.ChartUtils.isIE11()) ? 2 : 0;

            var titleText = FR.Chart.ChartUtils.dealTitleStringAutoWrap(this.text, this.textAttr, gapBounds.width);
            var reText = FR.Chart.TextUtils.changeAllReline2Normal(titleText);
            reText = reText.replaceAll('\n', '<br>');
            var newdiv = $("<div>" + reText + "</div>").css('position', 'absolute')
                .css('font', font).css('color', fontcolor)
                .css('overflow', 'hidden')// visible  css 属性.
                //					.css('text-overflow', 'ellipsis')// css3
                .css('text-align', alignValue)
                .css('left', gapBounds.x).css('top', gapBounds.y)
                .css('width', gapBounds.width + offWidth).css('height', gapBounds.height + 3);

            var currentDivKey = FR.Chart.currentDivKey;
            FR.Chart.DivCon.addDiv(currentDivKey, newdiv[0]);
        }
    }
});
//TitleGlyph.end

// PlotGlyph.start
/**
 * 抽象的绘图区
 * @class FR.Chart.PlotGlyph
 * @extends FR.Chart.RectangleGlyph
 * @abstract
 *
 * @param {JSON} js 绘图区配置属性.
 *
 * @param {Object} [js.conditionCollection ] 条件属性配置
 * @param {Boolean} [js.isNullValueBreak = true];  空值断开.
 * @param {Object} [js.plotFillStyle] 图表颜色风格.
 * @param {Number} [js.plotStyle] 图表系列默认风格
 * @param {Object} [js.hotTooltipStyle = new FR.Chart.AttrContents()] 热点提示标签内容.
 * @param {Array} seriesList 系列数组.
 */
FR.Chart.PlotGlyph = FR.extend(FR.Chart.RectangleGlyph, {

    initConstants: function (js) {
        js = js || {};

        this.conditionCollection = new FR.Chart.ConditionCollection(js.conditionCollection);
        this.isNullValueBreak = FR.pick(js.isNullValueBreak, true);
        this.plotFillStyle = new FR.Chart.AttrFillStyle(js.color || js.plotFillStyle);// 颜色填充风格
        this.plotStyle = FR.pick(js.plotStyle, FR.Chart.Constants.STYLE_NONE);// 系列默认的风格
        this.hotTooltipStyle = new FR.Chart.AttrContents(js.tooltip || js.hotTooltipStyle);
        this.combinedSize = FR.pick(js.combinedSize, 50);
        this.wholeChartBackground = null;
        this.isTooltipEnable = true;//数据点提示是否起作用
        if(js.theme){
            this.plotStyle = FR.Chart.PlotStyle[js.theme];
        }
        this.dataSeries = [];
        if (js.seriesList && js.seriesList.length > 0) {
            var array = js.seriesList;
            for (var i = 0; i < array.length; i++) {
                this.dataSeries[i] = new FR.Chart[array[i].type](array[i]);
            }
        }

        this.actionModel = null;//记录下绘图区的变化方式

        FR.Chart.PlotGlyph.superclass.initConstants.apply(this, [js]);

        this.createHotTooltip();
        this.createDataLabel4EverySeries();

        //当前选中系列默认是0
        this.currentChosenSeriesIndex = -1;

        this.isStockChart = false; //是否是stockcharts
    },

    isSupportLegend:function(){
        return true;
    },

    seriesLabelsInSide:function(){
        return false;
    },

    /**
     * 获取第二个值轴
     */
    getSecondValueAxisGlyph:function(){

    },

    //从导出属性中配置系列的一些属性
    configSeries:function(seriesConfig){
        var defaultAttr = this.conditionCollection.defaultAttr;
        if(seriesConfig.trendLine){
            var trendLine = new FR.Chart.AttrTrendLine(seriesConfig.trendLine)
            defaultAttr.addAttr(trendLine);
        }
    },

    layoutDemoGlyph:function(){

    },

    /**
      * 得到保存的stock charts里是否有cate属于同一统计区间的
      * @param stockDataPoints
      * @param cateDate
      * @param xAxisGlyph
      * @param adjustType
      * @returns {number}
      */
     getStockDateRangeIndex:function(stockDataPoints,cateDate,xAxisGlyph,adjustType){
         for(var index = stockDataPoints.length-1; index>=0 ;index-- ){
             var tempDate = xAxisGlyph.getDate4Value(stockDataPoints[index].categoryIndex);
             if(FR.Chart.ChartUtils.isBelongToSameUnit(tempDate,cateDate,adjustType)){
                  return index;
             }
         }
         return -1;
     },

    /**
     * 得到stock charts 数据重叠的两个日期，以此来调整求均值或者和的范围
     * @param cateDate 当前日期
     * @param lastDate 前面与当前日期数据重叠的日期
     * @returns {*} 范围
     */
    getStockChartAdjustDateType:function(cateDate, lastDate){
        var diff = FR.Chart.ChartUtils.getTwoDay(cateDate, lastDate, FR.Chart.Constants.Second);  //秒为单位的
        if(diff/60<1) {
            return FR.Chart.Constants.Minute;
        }
        if(diff /3600 <1){
            return FR.Chart.Constants.Hour;
        }
        if(diff/(3600 * 24) < 1){
            return FR.Chart.Constants.DAY;
        }

        if(diff/(3600 * 24)< cateDate.getMonthDays()){
            return FR.Chart.Constants.MONTH;
        }
        return FR.Chart.Constants.YEAR;
    },

    getStockChartDatePointX:function(xAxisGlyph,dataPoint,adjustDataType,plotBounds){
        var cateDate = xAxisGlyph.getDate4Value(dataPoint.categoryIndex);
        var midDate = FR.Chart.ChartUtils.getMiddleDate(cateDate,adjustDataType);
        var date2Int = FR.Chart.ChartUtils.date2Int(midDate, xAxisGlyph.mainType);
        var dataX = Math.round(xAxisGlyph.getPoint2D(date2Int + xAxisGlyph.labelOffset).x);

        if (dataX < plotBounds.x) {
            //取坐标轴开始日期与cateDate结束日期中间的日期
            var endDate = FR.Chart.ChartUtils.getEndDate4Type(cateDate,adjustDataType);
            date2Int = FR.Chart.ChartUtils.date2Int(endDate, xAxisGlyph.mainType);
            date2Int = xAxisGlyph.getCrossValue() + (date2Int - xAxisGlyph.getCrossValue())/2;
            dataX =Math.round(xAxisGlyph.getPoint2D(date2Int + xAxisGlyph.labelOffset).x);
        }else if(dataX > (plotBounds.x+ plotBounds.width)){
            //取坐标轴结束日期和cate开始日期的中间值
            var startDate = FR.Chart.ChartUtils.getStartDate4Type(cateDate,adjustDataType);
            date2Int = FR.Chart.ChartUtils.date2Int(startDate, xAxisGlyph.mainType);
            date2Int = xAxisGlyph.getArrowValue() - (xAxisGlyph.getArrowValue() - date2Int)/2;
            dataX =Math.round(xAxisGlyph.getPoint2D(date2Int + xAxisGlyph.labelOffset).x);
        }

        if(xAxisGlyph.stockChartMinDateInt){
            xAxisGlyph.stockChartMinDateInt = Math.min(xAxisGlyph.stockChartMinDateInt,date2Int);
        }else{
            xAxisGlyph.stockChartMinDateInt = date2Int;
        }
        return dataX;
    },


    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        var cateName = para[0];
        var seriesName = para[1];
        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            for(var j = 0, cateCount = this.getCategoryCount(); j < cateCount; j++){
                var dataPoint = series.points[j];
                if(dataPoint && FR.equals(cateName, dataPoint.categoryName) && FR.equals(seriesName, dataPoint.seriesName)){
                    return dataPoint;
                }
            }
        }
    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    _getDataPointByXY:function(para){
        var seriesName = para[0];
        var x = para[1];
        var y = para[2];

        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            for(var j = 0, cateCount = this.getCategoryCount(); j < cateCount; j++){
                var dataPoint = series.points[j];
                if(FR.equals(seriesName, dataPoint.seriesName)
                    && FR.equals(x, dataPoint.x) && FR.equals(y, dataPoint.value)){
                    return dataPoint;
                }
            }
        }
    },

    /**
     * 自定义数据点提示的内容
     * @param para
     */
    setTooltipValue:function(para){
        var dataPoint = this.getDataPointByPara(para);
        if(dataPoint){
            dataPoint.tooltip = para[para.length - 1];
        }
    },

    triggerTooltip:function(para, chartGlyph){
        var dataPoint = this.getDataPointByPara(para);
        if(dataPoint && dataPoint.glyph && dataPoint.glyph.shape && dataPoint.glyph.shape.getBounds2D && chartGlyph){
            var pointBounds = dataPoint.glyph.shape.getBounds2D();
            if(pointBounds && chartGlyph.handler){
                var mousePos = new FR.Chart.Point2D(pointBounds.getMiddleX(), pointBounds.getMiddleY());
                var plotGlyph = chartGlyph.plotGlyph;

                var ctx4OnMove = chartGlyph.getOnMoveCtx();
                chartGlyph.clearOnMoveCtx();

                var tooltipDiv = chartGlyph.handler.getTooltipDiv();
                ctx4OnMove._tooltipDiv = tooltipDiv;

                dataPoint.doTooltip(ctx4OnMove, mousePos, plotGlyph);
            }
        }
    },

    /**
     * 按照分类总值由小到大排序排序
     * 柱形图，条形图，面积图，折线图支持这个功能
      * @param isIncreaseOrder 是否是从小到大的顺序
     */
    sort:function(isIncreaseOrder){
        var cateTotal = [];
        var order = [];
        for(var i = 0, cateCount = this.getCategoryCount(); i < cateCount; i++){
            var total = 0;
            for(var j = 0,seriesCount = this.dataSeries.length; j < seriesCount; j++){
                var points = this.dataSeries[j].points;
                if(points){
                    var dataPoint = points[i];
                    total += dataPoint.value;
                }
            }
            cateTotal.push(total);
            order.push(i);
        }

        for(var i = 0; i < cateTotal.length; i++){
            var tmpIndex = i;
            var tmpValue = cateTotal[i];
            for(var j = i + 1; j < cateTotal.length; j++){
                if(isIncreaseOrder){
                    if(cateTotal[j] < tmpValue){
                        tmpValue = cateTotal[j];
                        tmpIndex = j;
                    }
                }else{
                    if(cateTotal[j] > tmpValue){
                        tmpValue = cateTotal[j];
                        tmpIndex = j;
                    }
                }
            }
            var tmp = cateTotal[i];
            cateTotal[i] = cateTotal[tmpIndex];
            cateTotal[tmpIndex] = tmp;

            tmp = order[i];
            order[i] = order[tmpIndex];
            order[tmpIndex] = tmp;
        }

        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            var points = series.points;
            var newPoints = [];
            series.points = newPoints;
            for(var j = 0; j < points.length; j++){
                var cateIndex = order[j];
                var dataPoint = points[cateIndex];
                dataPoint.categoryIndex = j;
                newPoints.push(dataPoint);
            }
        }

        if(this.xAxisGlyph && this.xAxisGlyph.categoryLabelList){
            var originLabelList = this.xAxisGlyph.categoryLabelList;
            var newLabelList = [];

            for(var i = 0; i < originLabelList.length; i++){
                newLabelList.push(originLabelList[order[i]])
            }

            this.xAxisGlyph.categoryLabelList = newLabelList;
        }

    },

    configTooltipAndLabels:function(options){
        var defaultAttr = this.conditionCollection.defaultAttr;

        if(options.label){//标签
            var labelAttr = new FR.Chart.AttrContents(options.label);
            defaultAttr.addAttr(labelAttr);
        }

        if(options.tooltip){//数据点提示
            this.hotTooltipStyle = new FR.Chart.AttrContents(options.tooltip);
        }
    },
    //配置颜色和风格
    configStyle:function(options){
        this.plotStyle = FR.pick(options.plotStyle, 0);
        this.plotFillStyle = new FR.Chart.AttrFillStyle(options.color);
    },

    //用chartData来构造系列和数据点对象,默认是normalChartData，子类不同的覆盖
    installSeriesWithData:function(chartData, plotTypes){

        this.addSeries2PlotGlyph(chartData, plotTypes);

        this.calculatePercentValue();
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        if(data && data.category && data.series){
            var category = data.category;
            var series = data.series;
            for(var i = 0; i < series.length; i++){
                var seryConfig = series[i];
                var sery = new FR.Chart[plotTypes.dataSeries]();
                sery.points = [];
                sery.seriesIndex = seryConfig.seriesIndex;
                sery.seriesName = seryConfig.seriesName;
                for(var cateIndex = 0, count = category.length; cateIndex < count; cateIndex++){
                    var dataPoint = new FR.Chart[plotTypes.dataPoint]();
                    dataPoint.categoryIndex = cateIndex;
                    dataPoint.categoryName = category[cateIndex];
                    dataPoint.seriesIndex = seryConfig.seriesIndex;
                    dataPoint.seriesName = seryConfig.seriesName;
                    var value = seryConfig.value[cateIndex];
                    if(FR.equals(value, 'null')){
                        dataPoint.value = 0;
                        dataPoint.valueIsNull  = true;
                    }else{
                        dataPoint.value = value;
                    }

                    sery.points[cateIndex] = dataPoint;
                }
                this.dataSeries[i] = sery;
            }
        }
    },

    configAxisLabels:function(){

    },

    configAxisLabelsWithOptionData:function(){

    },

    //装载完数据才能初始化条件显示
    configCondition:function(config){
        var options = config.conditionConfig || {};
        this.conditionCollection.addAll(options, this.dataSeries);
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.dataSeries = [];
        for(var i = 0, len = chartData.getSeriesCount(); i < len; i++){
            var series = new FR.Chart[plotTypes.dataSeries]();
            series.points = [];
            series.seriesIndex = i;
            series.seriesName = chartData.getSeriesLabel(i);

            for(var cateIndex = 0, count = chartData.getCategoryLabelCount(); cateIndex < count; cateIndex++){
                var dataPoint = new FR.Chart[plotTypes.dataPoint]();

                dataPoint.categoryIndex = cateIndex;
                dataPoint.seriesIndex = i;
                var value = chartData.getValueAt(i, cateIndex);

                if(isNaN(parseFloat(value))){
                    value = 0;
                    dataPoint.valueIsNull = true;
                }else{
                    value = parseFloat(value);
                }
                dataPoint.value = value;
                dataPoint.categoryName = chartData.getCategoryLabel(cateIndex);
                dataPoint.seriesName = chartData.getSeriesLabel(i);

                //添加多分类
                this.addMoreCate2DataPoint(dataPoint, chartData);

                series.points[cateIndex] = dataPoint;
            }

            this.dataSeries[i] = series;
        }
    },

    //计算百分比的值
    calculatePercentValue:function(){
        //计算下percentValue
        for(var i = 0; i < this.getCategoryCount(); i++){
            var total = 0;
            for(var j = 0; j < this.dataSeries.length; j++){
                if(this.dataSeries[j].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                    continue;
                }
                total += Math.abs(this.dataSeries[j].points[i].value);
            }

            for(var j = 0; j < this.dataSeries.length; j++){
                var dataPoint = this.dataSeries[j].points[i];
                var value = Math.abs(dataPoint.value);
                dataPoint.percentValue = value / total;
            }
        }
    },

    refreshDataPointLabel:function(){
        this.calculatePercentValue();
        this.createDataLabel4EverySeries();
        this.createHotTooltip();
    },


    //创建标签
    createDataLabel4EverySeries:function(){
        for (var seriesIndex = 0, len = this.dataSeries.length; seriesIndex < len; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                if (dataPoint.valueIsNull) {
                    continue;
                }
                var attr = this.conditionCollection.getAttr(dataPoint);
                var attrContents = attr.isContains(new FR.Chart.AttrContents());
                if(!attrContents){
                    attrContents = this.conditionCollection.defaultAttr.isContains(new FR.Chart.AttrContents());
                    if(!attrContents){
                        continue;
                    }
                }
                var seriesLabel = attrContents.seriesLabel;
                var separator = ',';
                var separators = FR.Chart.Separator;
                for(var i = 0; i < separators.length; i++){
                    if(seriesLabel.indexOf(separators[i]) != -1){
                        separator = separators[i];
                        break;
                    }
                }
                this.createDataLabel4DataPointWithCondition(dataPoint, attrContents, separator);
            }
        }
    },

    //创建图表插件的超链
    createHyperlink:function(hyperlinks){
        for(var i = 0; i < hyperlinks.length; i++){
            var hyperlink = hyperlinks[i];
            for(var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++){
                var series = this.dataSeries[seriesIndex];
                for(var cateIndex = 0; cateIndex < series.points.length; cateIndex++){
                    var dataPoint = series.points[cateIndex];
                    var resultParameter = dataPoint.resultParameter(hyperlink.parameter);
                    var resultContent = this._substituteContentsWithParameters(hyperlink.content, resultParameter);
                    switch(hyperlink.type){
                        case FR.Chart.HyperlinkType.JS:
                            dataPoint.jsHyperlink.push(resultContent);
                            break;
                        case FR.Chart.HyperlinkType.WEB:
                            var isStart = true;
                            for(var para in resultParameter){
                                if(isStart){
                                    resultContent += "?" + para + "=" + resultParameter[para];
                                }else{
                                    resultContent += "&" + para + "=" + resultParameter[para];
                                }
                            }
                            dataPoint.webHyerlink.push(resultContent);
                            break;
                    }
                }
            }
        }
    },

    //参数替换
    _substituteContentsWithParameters:function(content, parameter){
        for(var para in parameter){
            content = content.replaceAll(para, parameter[para]);
        }
        return content;
    },

    //标签支持条件显示的设置
    createDataLabel4DataPointWithCondition:function(dataPoint, attrContents, separator){
        //开始构造标签
        var label = dataPoint.getDataPointLabelString(attrContents, separator);

        //如果是自定义的标签就不是根据参数创建
        if(dataPoint.isCustomLabel && dataPoint.dataLabel){
            label = dataPoint.dataLabel.text;
        }

        var textGlyph = new FR.Chart.TextGlyph();
        textGlyph.text = label;
        textGlyph.textAttr = attrContents.textAttr;
        dataPoint.dataLabel = textGlyph;
        if(dataPoint.isShowLabel){
            dataPoint.showLabel = label;
        }
    },

    isNeedTimeSwitchWidget:function(){
        return false;
    },

    //根据装载好的数据，创建图例项
    createLegendItems:function(){
        var legendItems = [];

        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            var item = new FR.Chart.LegendItem();
            var colors = this.createColors4Series();
            item.label = series.seriesName;
            item.lineMarkerIcon = this.getLegendMarkerIcon(series, colors);
            legendItems[i] = item;
        }

        if(this.conditionCollection){
            for (var i = 0; i < this.dataSeries.length; i++) {
                var series = this.dataSeries[i];
                var attrTrendLine = this.conditionCollection.getConditionAttr(new FR.Chart.AttrTrendLine(), series);
                if (attrTrendLine) {
                    var trendLineList = attrTrendLine.trendLineList;
                    for (var lineIndex = 0; lineIndex < trendLineList.length; lineIndex++) {
                        var trendLine = trendLineList[lineIndex];
                        var lineInfo = trendLine.lineStyleInfo;
                        var item = new FR.Chart.LegendItem();
                        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();
                        item.label = trendLine.trendLineName;
                        item.lineMarkerIcon = lineMarkerIcon;
                        var color = 'black';
                        if(lineInfo && lineInfo.attrLineColor && lineInfo.attrLineColor.seriesColor){
                            color = lineInfo.attrLineColor.seriesColor;
                        }
                        lineMarkerIcon.background = FR.Chart.ColorBackground.getInstance(color);
                        lineMarkerIcon.lineStyle = FR.Chart.Constants.LINE_THIN;
                        lineMarkerIcon.marker = new FR.Chart.NullMarker();
                        legendItems.push(item);
                    }
                }
            }
        }

        return legendItems;
    },

    dealAttr4LegendGlyph:function(legendGlyph){
    },

    getLegendMarkerIcon:function(dataSeries, colors){
        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();

        this.dealCondition(lineMarkerIcon, dataSeries, colors);

        return lineMarkerIcon;
    },

    //折线图各种的标记点类型
    getLegendMarkerIcon4Line:function(dataSeries, colors){
        var itemIcon = new FR.Chart.LineMarkerIcon();

        this.dealLineCondition(itemIcon, dataSeries, colors);

        return itemIcon;
    },

    //处理lineMarkerIcon的背景等信息
    dealCondition:function(lineMarkerIcon, dataSeries, colors){

        var attrAlpha = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAlpha(), dataSeries);
        lineMarkerIcon.alpha = attrAlpha.alpha;

        var attrBackground = this.conditionCollection.getConditionAttr(new FR.Chart.AttrBackground(), dataSeries, colors);
        lineMarkerIcon.background = attrBackground.seriesBackground;
    },

    //折线图的各种标记点类型
    dealLineCondition:function(lineMarkerIcon, dataSeries, colors){
        var attrLine = this.conditionCollection.getConditionAttr(new FR.Chart.AttrColor(), dataSeries, colors);
        lineMarkerIcon.background = FR.Chart.ColorBackground.getInstance(attrLine.seriesColor);

        var attrLineStyle = this.conditionCollection.getConditionAttr(new FR.Chart.AttrLineStyle(), dataSeries, colors);
        lineMarkerIcon.lineStyle = attrLineStyle.lineStyle;

        var seriesAttrAlpha = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAlpha(), dataSeries, colors);
        lineMarkerIcon.alpha = seriesAttrAlpha.alpha;

        var markerType = this.conditionCollection.getConditionAttr(new FR.Chart.AttrMarkerType(), dataSeries, colors);
        var marker = new FR.Chart[markerType.markerType]();
        marker.size = marker.size * 0.8;
        marker.background = FR.Chart.ColorBackground.getInstance(attrLine.seriesColor);
        lineMarkerIcon.marker = marker;
    },

    addMoreCate2DataPoint:function(dataPoint, chartData){
        var secondCates = chartData.secondCates;
        if(!secondCates || secondCates.length === 0){
            return;
        }

        var categoryIndex = dataPoint.categoryIndex;

        var firstAllCount = chartData.getCategoryLabelCount();
        var secondCount = Math.max(1, secondCates.length);

        var thirdCates = chartData.thirdCates;
        var thirdCount = 1;
        if(thirdCates && thirdCates.length > 0){
            thirdCount = thirdCates.length;
        }

        var oneThirdConsCates = firstAllCount / thirdCount;
        var oneSecondConsCates = oneThirdConsCates/ secondCount;

        var thirdIndex = categoryIndex / oneThirdConsCates;
        var secondIndex = (categoryIndex - thirdIndex * oneThirdConsCates) / oneSecondConsCates;

        if(secondCates && secondCates.length > 0) {
            if(secondIndex < secondCates.length && secondCates[secondIndex]) {
                dataPoint.secondCate = secondCates[secondIndex];
            }
        }

        if(thirdCates && thirdCates.length > 0) {
            if(thirdIndex < thirdCount && thirdCates[thirdIndex]) {
                dataPoint.thirdCate = thirdCates[thirdIndex];
            }
        }
    },

    /**
     * 设置联动和刷新之前的plotGlyph
     * @param lastGlyph 联动之前的glyh
     */
    setLastGlyph:function(lastPlotGlyph){
        this.lastPlotGlyph = lastPlotGlyph;
    },

    /**
     *获取数据点的lastGlyph
     * @param dataPoint
     */
    getLastDataPointGlyph:function(dataPoint){
        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;

        if(this.lastPlotGlyph && this.lastPlotGlyph.dataPointDoExit(seriesIndex, categoryIndex)){
            return this.lastPlotGlyph.dataSeries[seriesIndex].points[categoryIndex].glyph;
        }

        return dataPoint.glyph;
    },

    getLastSeriesGlyph:function(series){
        var seriesIndex = series.seriesIndex;
        if(this.lastPlotGlyph && this.lastPlotGlyph.dataSeries.length > seriesIndex){
            var lastSeries = this.lastPlotGlyph.dataSeries[seriesIndex];
            if(lastSeries.points.length == series.points.length){
                lastSeries.glyph;
            }
        }

        return series.glyph;
    },

    /**
     * 指定分类和系列的数据点是否存在
     * @param seriesInde
     * @param categoryIndex
     */
    dataPointDoExit:function(seriesIndex, categoryIndex){
        return this.dataSeries.length > seriesIndex && this.dataSeries[seriesIndex].points.length > categoryIndex;
    },

    /**
     * 遍历Plot系列 为每个系列点创建热点提示文字
     */
    createHotTooltip: function () {
        if(this.hotTooltipStyle && this.hotTooltipStyle.seriesLabel){
            var is4Series = this.hotTooltipStyle.isShowMutiSeries;
            var style = this.hotTooltipStyle.isWhiteBackground ? 'white' : 'black';

            var colors = [];
            if (this.dataSeries.length === 1) {
                colors = this.createColors4Cate();
            } else {
                colors = this.createColors4Series();
            }

            if (is4Series) {
                for (var i = 0; i < this.dataSeries.length; i++) {
                    if (this.dataSeries[i]) {
                        this.dataSeries[i].createHotTooltip4Series(this.hotTooltipStyle, this, style, colors);
                    }
                }
            } else {
                for (var i = 0; i < this.dataSeries.length; i++) {
                    if (this.dataSeries[i]) {
                        this.dataSeries[i].createHotTooltip(this.hotTooltipStyle, this, style);
                    }
                }
            }
        }
    },

    /**
     * 返回分类个数.
     * @returns 分类个数.
     */
    getCategoryCount: function () {
        if (this.dataSeries[0] && this.dataSeries[0].points) {
            return this.dataSeries[0].points.length;
        }

        return 0;
    },

    /**
     * 显示的系列个数
     * @returns {number}  显示的系列个数.  TODO 只有柱形图 用到这几个方法
     */
    getSeriesViewCount: function () {
        var seriesCount = 0;
        for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
            if (this.dataSeries[i] && this.dataSeries[i].paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                seriesCount++;
            }
        }

        return seriesCount;
    },

    //显示的系列中的最后一个系列
    getLastSeriesViewIndex: function () {
        var seriesIndex = -1;
        for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
            if (this.dataSeries[i] && this.dataSeries[i].paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                seriesIndex = i;
            }
        }
        return seriesIndex;
    },

    // 当前系列为显示系列中的位置
    getSeriesViewIndex: function (seriesIndex) {
        var seriesViewIndex = 0;
        for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
            if (i == seriesIndex) {
                return seriesViewIndex;
            }
            if (this.dataSeries[i] && this.dataSeries[i].paintState >= FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                seriesViewIndex++;
            }
        }

        return -1;
    },

    /**
     * 返回系列 画的顺序.
     * @returns {Array} 返回顺序数组.
     */
    getDrawSequence: function () {
        var seq = [];
        for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
            seq[i] = i;
        }
        return seq;
    },

    /**
     * 返回 为坐标轴上方单位预留的间隔Gap
     * @returns {number}
     */
    getPreGap4AxisTopUnit: function () {
        return 0;
    },

    /**
     * 返回为坐标轴右侧单位 预留的间隔Gap
     * @returns {number}
     */
    getPreGap4AxisRightUnit: function () {
        return 0;
    },

    /**
     * 返回为 数据表左侧预留的间隔位置.
     * @returns {number}
     */
    getPreLeft4DataSheet: function () {
        return 0;
    },

    _isShowInScaleChartCate: function (dataPoint) {// 对于缩放轴 必须是在显示分类范围内
        var index = dataPoint.categoryIndex;
        if (this.xAxisGlyph) {
            var back = this.xAxisGlyph.zoomBackward;
            var forward = this.xAxisGlyph.zoomForward;
            return index >= Math.abs(back) && dataPoint.categoryIndex < this.getCategoryCount() - Math.abs(forward);
        }
        return true;
    },

    /**
     * 判断系列点的x值是否在绘图范围内  暂时先只判断x值, 因为y值对绘图效果的影响不大. x值会直接导致确缺失.
     * @param px 系列点x值
     * @returns {boolean} 是否在绘图区内. true,不在内.
     */
    isDataPointXNotInPlotBounds: function (px) {
        //如果不是数值，肯定不在内
        if(isNaN(px)){
            return true;
        }

        var bounds = this.getBounds();

        return (px < bounds.x - 1 || px > bounds.x + bounds.width + 1);
    },

    /**
     * 按照顺序 一般是左上角 到右下角 处理标签的位置
     */
    dealLabelBoundsInOrder: function () {
        var disOrderList = [];

        for (var seriesIndex = 0; this.dataSeries && seriesIndex < this.dataSeries.length; seriesIndex++) {
            for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
                var dataSeries = this.dataSeries[seriesIndex];
                if (dataSeries == null || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    continue;
                }

                var dataPoint = dataSeries.points[categoryIndex];


                if (dataPoint && dataPoint.dataLabel && dataPoint.dataLabel.bounds != null) {
                    if (this._isShowInScaleChartCate(dataPoint)) {
                        disOrderList[disOrderList.length] = dataPoint;
                    } else {// 如果不在分类显示范围内, 则不加入list, 并且强制不显示.
                        dataPoint.dataLabel.bounds = null;
                    }
                }
            }
        }

        if (disOrderList.length <= 0) {
            return;
        }

        if(FR.Browser.isIE8Before()){
            var allowMaxSizeLabel = 50;
            if (disOrderList.length > allowMaxSizeLabel) {// kunsnat: 先加个限制条件 因为太都的话 吃不消 就挂掉了 其余的全部不显示
                var newList = [];
                var count = parseInt(disOrderList.length / allowMaxSizeLabel);
                for (var i = 0; i < disOrderList.length; i++) {
                    if (i % count === 0) {
                        newList[newList.length] = disOrderList[i];
                    } else {
                        disOrderList[i].dataLabel.bounds = null;
                    }
                }
                disOrderList = newList;
            }
        }

        var orderList = this._dealDataPointOrder(disOrderList);
        var plotBounds = this.getBounds();

        var boundsManager = new FR.Chart.BoundsManager();

        for (var orderListIndex = 0; orderListIndex < orderList.length; orderListIndex++) {
            var dataPoint = orderList[orderListIndex];

            if (dataPoint.dataLabel == null) {
                continue;
            }

            var labelBounds = dataPoint.dataLabel.bounds;

            this.dealLabelBoundsOutPlotBounds(dataPoint,labelBounds,plotBounds,orderList,orderListIndex,boundsManager);
        }

        //bug:59653 产品说只判断超出范围，不判断重叠
        for (var orderListIndex = 0; orderListIndex < orderList.length; orderListIndex++) {
            var dataPoint = orderList[orderListIndex];

            if (dataPoint.dataLabel == null) {
                continue;
            }
            var labelBounds = dataPoint.dataLabel.bounds;
            if (labelBounds == null || labelBounds.x < this.bounds.x-3 || labelBounds.y < this.bounds.y-3) {
                dataPoint.dataLabel.bounds = null;
            }
        }
    },


    dealLabelBoundsOutPlotBounds:function(dataPoint,labelBounds,plotBounds,orderList,orderListIndex,boundsManager){
        labelBounds = this.avoidCoverLabelBounds(boundsManager, labelBounds);

        if (labelBounds == null) {// 没有找到空值 返回
            dataPoint.dataLabel.bounds = null;
            return ;
        }

                    if (plotBounds.contains(labelBounds.x, labelBounds.y, labelBounds.width, labelBounds.height)) {
                        dataPoint.dataLabel.bounds = labelBounds;
                    } else {
                        while (labelBounds.y + labelBounds.height > plotBounds.y + plotBounds.height) {
                            labelBounds.y = labelBounds.y - 1;
                        }
                        this._dealBoundsIntersects(orderList, orderListIndex, boundsManager, labelBounds, 0, -1);
                        this._reAddLabelPath(boundsManager, orderList, orderListIndex);

                        while (labelBounds.x < plotBounds.x) {
                            labelBounds.x = labelBounds.x + 1;
                        }
                        this._dealBoundsIntersects(orderList, orderListIndex, boundsManager, labelBounds, 1, 0);
                        this._reAddLabelPath(boundsManager, orderList, orderListIndex);

                        while (labelBounds.x + labelBounds.width > plotBounds.x + plotBounds.width) {
                            labelBounds.x = labelBounds.x - 1;
                        }
                        this._dealBoundsIntersects(orderList, orderListIndex, boundsManager, labelBounds, -1, 0);
                        this._reAddLabelPath(boundsManager, orderList, orderListIndex);

                        while (labelBounds.y < plotBounds.y) {
                            labelBounds.y = labelBounds.y + 1;
                        }
                        this._dealBoundsIntersects(orderList, orderListIndex, boundsManager, labelBounds, 0, 1);
                        this._reAddLabelPath(boundsManager, orderList, orderListIndex);

                        dataPoint.dataLabel.bounds = labelBounds;// 逆推 上移所有的相关位置
                    }

        boundsManager.addBounds(labelBounds);
    },

    _reAddLabelPath: function (boundsManager, orderList, orderListIndex) {
        boundsManager = new FR.Chart.BoundsManager();
        for (var tmp = 0; tmp < orderListIndex; tmp++) {// 之前的bounds重新加入
            var nPoint = orderList[tmp];
            if (nPoint != null && nPoint.dataLabel != null && nPoint.dataLabel.bounds != null) {
                boundsManager.addBounds(nPoint.dataLabel.bounds);
            }
        }
    },

    _dealBoundsIntersects: function (orderList, orderListIndex, boundsManager, labelBounds, leftOffset, topOffset) {
        var intersectsPointList = [];

        if (orderListIndex <= 0) {
            return;
        }

        var orderIndex = 1;
        while (boundsManager.isInteract(labelBounds)) {// 已经排序好的 倒序 上移
            if ((orderListIndex - orderIndex) < 0) {
                break;
            }
            var tmpDataPoint = orderList[orderListIndex - orderIndex];
            orderIndex++;

            if (orderIndex > orderList.length) {
                break;
            }

            if (tmpDataPoint == null || tmpDataPoint.dataLabel == null || tmpDataPoint.dataLabel.bounds == null) {
                continue;
            }

            var tmpBounds = tmpDataPoint.dataLabel.bounds;

            var isIntersect = false;

            var labelBoundsManager = new FR.Chart.BoundsManager();
            labelBoundsManager.addBounds(labelBounds);

            //这边在往上调之前先把tmppoint之后的点的bounds加进去，防止tmppoint的bounds和后面的bounds重叠了。见bug:20431
            var tmpIndex = 1;
            while(tmpIndex<orderIndex-1){
                var behindDataPoint = orderList[orderListIndex - tmpIndex];
                tmpIndex++;
                if (behindDataPoint == null || behindDataPoint.dataLabel == null || behindDataPoint.dataLabel.bounds == null) {
                    continue;
                }
                labelBoundsManager.addBounds(behindDataPoint.dataLabel.bounds);
            }


            if (tmpBounds && labelBoundsManager.isInteract(tmpBounds)) {
                isIntersect = true;
            }

            while (tmpBounds && labelBoundsManager.isInteract(tmpBounds)) {// 单独判断和这个边界是否相交
                tmpBounds.x = tmpBounds.x + leftOffset;
                tmpBounds.y = tmpBounds.y + topOffset;
                tmpDataPoint.dataLabel.bounds = tmpBounds;
            }

            if (isIntersect) {
                intersectsPointList[intersectsPointList.length] = tmpDataPoint;
            }

            boundsManager = new FR.Chart.BoundsManager();
            for (var tmp = 0; tmp < orderListIndex; tmp++) {
                var nPoint = orderList[tmp];
                if (nPoint != null && nPoint.dataLabel != null && nPoint.dataLabel.bounds != null) {
                    boundsManager.addBounds(nPoint.dataLabel.bounds);
                }
            }
        }

        for (var test = 0; test < intersectsPointList.length; test++) {
            // 判断移动之后相关Bounds 是否和以前的重叠
            var tmpPoint = intersectsPointList[test];
            var tBounds = tmpPoint.dataLabel.bounds;

            for (var index = 0; index < orderList.length; index++) {
                if (tmpPoint && orderList[index]
                    && orderList[index].categoryIndex == tmpPoint.categoryIndex
                    && tmpPoint.seriesIndex == orderList[index].seriesIndex) {
                    this._dealBoundsIntersects(orderList, index, boundsManager, tBounds, leftOffset, topOffset);
                }
            }
        }
    },

    _dealDataPointOrder: function (disorderList) {
        var resultOrderDataPoint = [];

        var n = disorderList.length;
        for (var i = 0; i < n && disorderList.length >= 1; i++) {
            var tmp = disorderList[0];

            for (var k = 0; k < disorderList.length; k++) {// 比较找出y最大, x最大的
                var dataPoint = disorderList[k];

                if (dataPoint.dataLabel == null || dataPoint.dataLabel.bounds == null) {
                    continue;
                }

                var labelBounds = dataPoint.dataLabel.bounds;

                if (tmp.dataLabel == null || tmp.dataLabel.bounds == null) {
                    continue;
                }
                var tmpBounds = tmp.dataLabel.bounds;

                var tmpX = tmpBounds.x;
                var tmpY = tmpBounds.y;

                var x = labelBounds.x;
                var y = labelBounds.y;

                if (tmpY < y) {

                } else if (y < tmpY) {
                    tmp = dataPoint;
                } else if (tmpY == y) {
                    if (tmpX > x) {
                        tmp = dataPoint;
                    }
                }

            }
            disorderList.remove(tmp);
            for (var listIndex = 0; listIndex < disorderList.length; listIndex++) {
                var point = disorderList[listIndex];
                if (point.categoyrIndex == tmp.categoryIndex && point.seriesIndex == tmp.seriesIndex) {
                    for (tmpIndex = listIndex; tmpIndex < disorderList.length - 1; tmpIndex++) {
                        disorderList[listIndex] = disorderList[listIndex + 1];
                    }
                    disorderList.length = disorderList.length - 1;
                    break;
                }
            }

            resultOrderDataPoint[resultOrderDataPoint.length] = tmp;
        }

        return resultOrderDataPoint;
    },

    /**
     * 避免覆盖 set之前看下bounds
     * @param boundsManager  标签边界管理
     * @param bounds 边界
     * @return 返回最终的边界
     */
    avoidCoverLabelBounds: function (boundsManager, bounds) {
        if (bounds == null) {
            return null;
        }

        var x = bounds.x;
        var y = bounds.y;
        var width = bounds.width;
        var height = bounds.height;

        var leftTop = new FR.Chart.Bounds(x, y, 1, 1);
        var rightTop = new FR.Chart.Bounds(x + width - 1, y, 1, 1);
        var leftBottom = new FR.Chart.Bounds(x, y + height - 1, 1, 1);
        var rightBottom = new FR.Chart.Bounds(x + width - 1, y + height - 1, 1, 1);

        // 先通过 角的相交判断出 向哪边移动合理.

        var isLeftTop = boundsManager.isInteract(leftTop);
        var isRightTop = boundsManager.isInteract(rightTop);
        var isLeftBottom = boundsManager.isInteract(leftBottom);
        var isRightBottom = boundsManager.isInteract(rightBottom);

        if (!isLeftTop && !isRightTop && !isLeftBottom && !isRightBottom) {// 四个都未相交 直接set
            if (boundsManager.isInteract(bounds)) {
                this.whileTmp(boundsManager, bounds, 0, 1, height);
            } else {
                var oldBounds = new FR.Chart.Bounds(bounds.x, bounds.y, bounds.width + 2, bounds.height);
                return oldBounds;
            }
        } else if (!isLeftTop && isRightTop && isLeftBottom && isRightBottom) {// 只有左上角 未相交
            this.whileTmp(boundsManager, bounds, -1, -1, width);
        } else if (isLeftTop && !isRightTop && isLeftBottom && isRightBottom) {// 只有右上角 未相交
            this.whileTmp(boundsManager, bounds, 1, -1, width);
        } else if (isLeftTop && isRightTop && !isLeftBottom && isRightBottom) {// 只有左下角 未相交
            this.whileTmp(boundsManager, bounds, -1, 1, width);
        } else if (isLeftTop && isRightTop && isLeftBottom && !isRightBottom) {// 只有右下角 未相交
            this.whileTmp(boundsManager, bounds, 1, 1, width);
        } else if (!isLeftTop && !isRightTop && isLeftBottom && isRightBottom) {// 左下和右下相交  上方移动
            this.whileTmp(boundsManager, bounds, 0, -1, height);
        } else if (isLeftTop && !isRightTop && isLeftBottom && !isRightBottom) {// 左上和左下相交  右移动
            this.whileTmp(boundsManager, bounds, 1, 0, width);
        } else if (isLeftTop && isRightTop && isLeftBottom && isRightBottom || // 四角相交  向下移动
            isLeftTop && isRightTop && !isLeftBottom && !isRightBottom) {// 左上和右上相交 下移动
            this.whileTmp(boundsManager, bounds, 0, 1, height);
        } else if (!isLeftTop && isRightTop && !isLeftBottom && isRightBottom) {// 右下和右上相交 左移动
            this.whileTmp(boundsManager, bounds, -1, 0, width);
        } else if (isLeftTop && !isRightTop && !isLeftBottom && !isRightBottom) {// 只有左上角 相交
            this.whileTmp(boundsManager, bounds, 1, 0, width);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 0, 1, height);

            if (boundsManager.isInteract(bounds)) {
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 1, 1, width);

        } else if (!isLeftTop && isRightTop && !isLeftBottom && !isRightBottom) { // 只有右上相交
            this.whileTmp(boundsManager, bounds, -1, 0, width);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 0, 1, height);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, -1, 1, width);

        } else if (!isLeftTop && !isRightTop && isLeftBottom && !isRightBottom) { // 只有左下相交
            this.whileTmp(boundsManager, bounds, 0, -1, height);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 1, 0, width);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 1, -1, width);
        } else if (!isLeftTop && !isRightTop && !isLeftBottom && isRightBottom) {// 只有右下相交
            this.whileTmp(boundsManager, bounds, -1, 0, width);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 0, -1, height);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, -1, -1, width);

        } else if (isLeftTop && !isRightTop && !isLeftBottom && isRightBottom) { // 左上 右下 相交
            this.whileTmp(boundsManager, bounds, -1, 1, width);

            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 1, -1, width);
        } else if (!isLeftTop && isRightTop && isLeftBottom && !isRightBottom) { // 左下 右上 相交
            this.whileTmp(boundsManager, bounds, -1, -1, width);
            if (boundsManager.isInteract(bounds)) {// 复原
                bounds = new FR.Chart.Bounds(x, y, width, height);
            }

            this.whileTmp(boundsManager, bounds, 1, 1, width);
        }

        if (!boundsManager.isInteract(bounds)) {
            var oldBounds = new FR.Chart.Bounds(bounds.x, bounds.y, bounds.width + 2, bounds.height);
            return oldBounds;
        }

        return null;
    },

    /**
     * 移动判断
     * @param boundsManager 边界集合
     * @param bounds 目标边界
     * @param tmpX 移动变量x
     * @param tmpY 移动变量y
     * @param tmpBreak 中断条件.
     */
    whileTmp: function (boundsManager, bounds, tmpX, tmpY, tmpBreak) {
        var tmp = 0;
        while (boundsManager.isInteract(bounds)) {
            bounds.x = bounds.x + tmpX;
            bounds.y = bounds.y + tmpY;
            tmp += 1;
            if (tmp > tmpBreak) {
                break;
            }
        }
    },

    /**
     * 处理Plot的Bounds为整型的0.5, 便于画线精确, 同时在Plotbounds的内部都用整型就可以了
     *
     * 尽量扩大像素范围, 因为 在layoutSeries的时候 判断是否在内, 容易判断误差1像素.
     */
    dealPlotBounds4JSDraw: function (bounds) {
        bounds.x = parseInt(bounds.x) - 0.5;
        bounds.y = parseInt(bounds.y) - 0.5;
        bounds.width = Math.round(bounds.width + 1);
        bounds.height = Math.round(bounds.height + 1);
    },

    /**
     * 返回动态步骤数目.
     * @returns  {Number}步骤数目
     */
    getAnimateStepNumber: function () {
        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var i = 0;
        while (this.dataSeries[i] && !(this.dataSeries[i].getAnimateStepNumber() > 0
            && this.dataSeries[i].paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE)) {
            i++;
        }
        if (this.dataSeries[i]) { // 暂时判断是否为空值, 还有其他空值情况..
            return this.dataSeries[i].getAnimateStepNumber();
        } else {
            return -1;
        }
    },

    /**
     * 按照系列个数 顺序创建颜色数组
     **/
    createColors4Series: function () {
        return FR.Chart.ChartUtils.createFillColorArray(this.plotFillStyle, this.dataSeries.length);
    },

    /**
     * 按照分类个数 顺序创建颜色数组
     */
    createColors4Cate: function () {
        return FR.Chart.ChartUtils.createFillColorArray(this.plotFillStyle, this.getCategoryCount());
    },

    // PlotGlyph.draw
    draw: function (ctxs, time, actionModel) {
        this.actionModel = actionModel;

        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            var stepNumber = 6; // actionModel 动态变化的方式

            //IE678只有通过少画才能减少闪烁
            if(!FR.Browser.isIE() || FR.Chart.ChartUtils.isIE9Later()){
                if (actionModel) {
                    stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[actionModel];
                    time = FR.Chart.ChartPaintConstants.ACTION_TIME[actionModel];
                }
            }

            // 先给趋势线赋值 width.height 再去搞动态步骤
            if (this.dataSeries[i].trendLineList) {
                for (var listIndex = 0; listIndex < this.dataSeries[i].trendLineList.length; listIndex++) {
                    this.dataSeries[i].trendLineList[listIndex].width = this.getBounds().width;
                    this.dataSeries[i].trendLineList[listIndex].height = this.getBounds().height;
                }
            }

            this.dataSeries[i].getAnimateGlyph(stepNumber, actionModel);
        }

        if (this.arc3DStyleBase) {
            this.arc3DStyleBase.paintStyle(ctxs[1]);
            this.arc3DStyleBase = null; //画一遍，画完重置，每次layout以后位置不一样
        }

        if (this.arcOuterBase) {
            this.arcOuterBase.paintStyle(ctxs[1]);
            this.arcOuterBase = null;
        }

        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var step = this.getAnimateStepNumber();
        if (step > 0) { // 暂时判断是否为空值, 还有其他空值情况..
            this.animateGlyphDraw(ctxs[2], time, step);
        } else {
            this.animateGlyphDraw(ctxs[2], 1, 1);
        }
    },

    /**
     * 设置组合图形的ctx
     * @param ctx
     */
    setCombieGlyphCtx:function(ctx){
        this.combineGlyphCtx = ctx;
    },

    /**
     * 当前的图表画图结束以后，画标签之前，画组合在该图表上的形状
     */
    drawCombineGlyph:function(){

    },

    /**
     * 是否支持两步的画图,散点，气泡，雷达，地图支持
     */
    isSupportTwoStepDraw:function(){
        return false;
    },

    /**
     * @param ctx 画布
     * @param time 动态事件
     * @param stepNumber 动态步骤
     */
    animateGlyphDraw: function (ctx, time, stepNumber){
        if(this.lastPlotGlyph && this.isSupportTwoStepDraw()){
            var lastPlotGlyph = this.lastPlotGlyph;
            var count = 0;
            var lastSeries = this.lastPlotGlyph.dataSeries;
            var self = this;
            for(var index = 0; index < lastSeries.length; index++){
                var series = lastSeries[index];
                series.paintState = FR.Chart.ChartPaintConstants.STATE_DISAPPEAR;
                series.getAnimateGlyph(stepNumber, FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
            }

            function callback() {
                if(!ctx){     return; } ctx.save();
                lastPlotGlyph._saveClip4SeriesDrawThrowOutOfPlot(ctx, lastPlotGlyph);
                lastPlotGlyph.doAnimate4SeriesDraw(ctx, lastPlotGlyph, count);
                count++;
                ctx.restore();
                if (count <= stepNumber - 1) {
                    requrestAnimationFrame(callback);
                }else{
                    self.animateGlyphDrawCurrent(ctx, time, stepNumber);
                }
            }

            var requrestAnimationFrame = window.requrestAnimationFrame
                || window.msRequestAnimationFrame
                || window.mozRequestAnimationFrame
                || window.webkitRequestAnimationFrame
                || function (func) {
                    setTimeout(func, time / stepNumber);
                }

            requrestAnimationFrame(callback);
        }else{
            this.animateGlyphDrawCurrent(ctx, time, stepNumber);
        }
    },

    /**
     * 画当前展示的图表的绘图区内容
     * @param ctx 画布
     * @param time 时间
     * @param stepNumber 步骤
     */
    animateGlyphDrawCurrent:function(ctx, time, stepNumber) {
        var plotGlyph = this;
        var count = 0;
        var currentDivKey = FR.Chart.currentDivKey;

        function callback() {
            if(!ctx){     return; } ctx.save();// 整个动态开始前 保存, 以便于clip截取绘图区. 然后在整个动态(包括after 最后的标记结束后) 再复位.

            plotGlyph._saveClip4SeriesDrawThrowOutOfPlot(ctx, plotGlyph);
            plotGlyph.doAnimate4SeriesDraw(ctx, plotGlyph, count);
            count++;
            if (count > stepNumber - 1) {
                plotGlyph.drawMarkersLabelAfterSeriesAnimateDraw(ctx, currentDivKey);
                plotGlyph.drawCombineGlyph(time);
                FR.Chart.GraphHelper.isDrawingEnd = true;
            }

            ctx.restore();// 整个动态结束 包括最后的标记类型, 复位.

            if (count <= stepNumber - 1) {
                FR.Chart.GraphHelper.isDrawingEnd = false;
                requrestAnimationFrame(callback);
            }
        }

        var requrestAnimationFrame = window.requrestAnimationFrame
            || window.msRequestAnimationFrame
            || window.mozRequestAnimationFrame
            || window.webkitRequestAnimationFrame
            || function (func) {
                setTimeout(func, time / stepNumber);
            }

        requrestAnimationFrame(callback);
    },

    _saveClip4SeriesDrawThrowOutOfPlot: function (ctx, plotGlyph) {
        // kunsnat: 加上阴影后, plot有部分清除不了~ 改为整个canvas清除 ctx.canvas.width
        ctx.clearRect(0, 0, ctx.canvas.width, ctx.canvas.height);

        var bounds = plotGlyph.bounds;
        // 裁剪前保存状态 有可能Plot被反转, 出现负值. 清除范围需要扩大.	bug 11266
        var gap = FR.Browser.r.webkit ? 1 : 0;// 各浏览器不同 专为Chrome下 多预留 -gap的空间
        var rectX = (0 < bounds.width ? -gap : bounds.width - gap) + bounds.x;
        var rectY = (0 < bounds.height ? -gap : bounds.height - gap) + bounds.y;
        var rectWidth = Math.abs(bounds.width) + gap * 2;
        var rectHeight = Math.abs(bounds.height) + gap * 2;

        ctx.beginPath();
        ctx.moveTo(rectX, rectY);
        ctx.lineTo(rectX, rectY + rectHeight);
        ctx.lineTo(rectX + rectWidth, rectY + rectHeight);
        ctx.lineTo(rectX + rectWidth, rectY);
        ctx.closePath();
        ctx.clip();
    },

    /**
     * 系列的动态 画法. 画法顺序可能不同. 画法结束 也可能不同.
     * @param ctx 画布
     * @param plotGlyph 绘图区Glyph
     * @param count 步骤count Index
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);

            // 面积图画上边缘
            if (plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine) {
                plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine(ctx, count);
            }
        }

        for (var i = plotGlyph.dataSeries.length - 1; i >= 0; i--) {
            plotGlyph.dataSeries[i].drawAnimateGlyph4TrendLine(ctx, count);
        }
    },

    drawBar3DStackSeries: function (ctx, plotGlyph, count, dataSeriesArry) {
        var cateCount = plotGlyph.getCategoryCount();
        var seriesCount = dataSeriesArry.length;
        for (var i = 0; i < cateCount; i++) {
            var dp = [];
            for (var j = 0; j < seriesCount; j++) {
                var dataSeries = dataSeriesArry[j];
                if (dataSeries != null && (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_ENABLE || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_APPEAR)) {
                    dp[j] = dataSeries.points[i];
                }
            }

            for (var j = 0; j < seriesCount; j++) {
                for (var k = seriesCount - 1; k > j; k--) {
                    if (dp[k] != null && !dp[k].valueIsNull && dp[k].valueByUnit < 0) {
                        var temp = dp[k];
                        dp[k] = dp[k - 1];
                        dp[k - 1] = temp;
                    }
                }
            }

            for (var j = 0; j < seriesCount; j++) {
                if (dp[j] != null) {
                    dp[j].drawAnimateGlyph(ctx, count);
                    if(dp[j].dataPointStyle && dp[j].dataPointStyle.paintHighLight){
                        dp[j].dataPointStyle.paintHighLight(ctx);
                    }
                }
            }
        }
    },

    // PlotGlyph.demodraw
    draw4Demo: function (ctx) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            this.dataSeries[i].draw4Demo(ctx);
        }
    },

    /**
     * 刷新坐标轴 位置等.
     * 默认 对于没有坐标轴的 是donothing
     */
    refreshAxisGlyph: function () {

    },

    /**
     * 从已有数据中 返回最大值.
     * @returns {number}
     */
    getMaxValueFromData: function () {
        var maxValue = 0.0;

        for (var i = 0, len = this.getCategoryCount(); i < len; i++) {
            for (var j = 0; j < this.dataSeries.length; j++) {
                if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    continue;
                }

                var number = this.dataSeries[j].points[i];
                if (!number.valueIsNull) {
                    maxValue = (number.value > maxValue) ? number.value : maxValue;
                }
            }
        }
        if (maxValue == this.getMinValueFromData(false)) {
            return maxValue + 10;
        }

        return maxValue;
    },

    /**
     * 从当前显示的系列数据中取到最小值. 判断是否对数.
     * @param isLog
     * @returns {number}
     */
    getMinValueFromData: function (isLog) {
        var minValue = 0.0;
        if (isLog) {
            minValue = Number.MAX_VALUE;
        }

        for (var i = 0, len = this.getCategoryCount(); i < len; i++) {
            for (var j = 0; j < this.dataSeries.length; j++) {
                if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    continue;
                }

                var number = this.dataSeries[j].points[i];
                if (!number.valueIsNull) {
                    if (isLog) {
                        minValue = (number.value < minValue && number.value < 1 && number.value > 0) ? number.value : minValue;
                    } else {
                        minValue = (number.value < minValue) ? number.value : minValue;
                    }
                }
            }
        }
        if (minValue == Number.MAX_VALUE) {
            return 1;
        }
        return minValue;
    },

    /**
     * 是否支持 图例显示交互
     * @returns {boolean}  true 表示支持, false 不支持.
     */
    isLegendInteractive: function () {
        if (this instanceof FR.Chart.StockPlotGlyph) {
            return false;
        }
        if (this instanceof FR.Chart.MeterPlotGlyph) {
            return false;
        }
        if (this instanceof FR.Chart.GanttPlotGlyph) {
            return false;
        }
        if (this instanceof FR.Chart.GisMapPlotGlyph) {
            return false;
        }

        return true;
    },

    /**
     * 按照分类 分组, 列表每个系列对应的超级链接 如果只有一个, 直接响应, 如果是地图 就只提供钻取 不提供超级链接
     */
    getHyperMenuWithPosition: function (mousePos, chartEvent) {
        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
            return;
        }

        var item = this.getMenuItemsInBounds(mousePos);

        var result = [];
        for (var i = 0; i < item.length; i++) {
            var cate = item[i];
            if (cate.length > 1) {
                for (var index = 0; index < cate.length; cate++) {
                    var menus = this._createHyperItemMenu(cate[index], chartEvent);
                    if (menus.src) {
                        result[result.length] = menus;
                    }
                }
            } else {
                var menus = this._createHyperItemMenu(cate[0], chartEvent);
                if (menus.src) {
                    result[result.length] = menus;
                }
            }
        }

        var resultMenu = {};
        resultMenu.src = "showMenu";
        resultMenu.items = result;

        return resultMenu;
    },

    /**
     * 返回在鼠标位置 局部区域内 包括显示的项目列表
     * @param mousePos 鼠标位置
     * @returns {Array} 显示的列表项
     */
    getMenuItemsInBounds: function (mousePos) {
        var item = [];
        var roundArea = 5;
        var pressBounds = new FR.Chart.Bounds(mousePos.x - roundArea, mousePos.y - roundArea, roundArea * 2, roundArea * 2);

        // 判定某个DataPoint的bounds 是否path 某个 intersects
        for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
            var cate = [];
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataSeries = this.dataSeries[seriesIndex];

                var dataPoint = dataSeries.points[categoryIndex];
                if (dataPoint
                    && dataPoint.hyperlink
                    && dataPoint.glyph && dataPoint.glyph.getShape()
                    ) {// shapeglyph, bubble, meter
                    var shape = dataPoint.glyph.getShape();
                    if (shape
                        && (this._dealEveryPointInBounds(shape, pressBounds)
                        || shape.containsPoint(mousePos.x, mousePos.y))
                        ) {
                        cate[cate.length] = dataPoint;
                    }
                }
            }

            if (cate.length > 0) {
                item[item.length] = cate;
            }
        }
        return item;
    },

    _dealEveryPointInBounds: function (shape, pressBounds) {
        if (shape && pressBounds) {
            var width = pressBounds.width || 0;
            var height = pressBounds.height || 0;
            for (var i = pressBounds.x; i < pressBounds.x + width; i++) {
                for (var h = pressBounds.y; h < pressBounds.y + height; h++) {
                    if (shape.containsPoint(i, h)) {
                        return true;
                    }
                }
            }
        }
        return false;
    },

    /**
     * 创建超链列表
     * @param dataPoint 数据点.
     * @param chartEvent 图表事件
     * @returns {{}} 返回数组.
     */
    _createHyperItemMenu: function (dataPoint, chartEvent) {
        var newItem = {};
        newItem.src = dataPoint.categoryName + " " + dataPoint.seriesName;

        if (dataPoint.glyph) {
            var glyph = dataPoint.glyph;

            if (glyph.info && glyph.info.seriesAttrBackground.seriesBackground) {
                newItem.color = glyph.info.seriesAttrBackground.seriesBackground.color;
            } else if (glyph.lineStyleInfo && glyph.lineStyleInfo.attrLineColor.seriesColor) {
                newItem.color = glyph.lineStyleInfo.attrLineColor.seriesColor;
            } else {
                newItem.color = 'gray';
            }
        }

        newItem.dataPoint = dataPoint;
        if (dataPoint.nameLayerTo && dataPoint.hasNextTo) {
            newItem.chartEvent = chartEvent;
        } else {
            if (dataPoint.hyperlink) {
                var obj = new Function("return " + dataPoint.hyperlink)();
                var menuItems = this._createItems4Menu(obj);
                if (menuItems.length === 1) {
                    newItem.handler = menuItems[0].handler;
                } else {
                    newItem.submenu = menuItems;
                }
            }
        }

        return newItem;
    },

    _createItems4Menu: function (obj) {
        return $.map(obj, function createCallback(cfg) {
            return {
                src: cfg.name,
                handler: new Function(unescape(cfg.data))
            };
        });
    },

    findClickOnGlyph:function(mousePos, event){
        this.findOnMoveGlyph(mousePos, event);
    },


    _findSeriesGlyphPoint:function(series, mousePos, event){
        var shape = series.glyph ? series.glyph.getShape().getInteractivePath() : null;
        if(shape && shape.containsPoint(mousePos.x, mousePos.y)){
            var minDistance = Number.MAX_VALUE;
            var nearbyPoint;
            for(var j = 0; j < series.points.length && !event._hasfound; j++) {
                var point = series.points[j];
                if(point.glyph && point.glyph.getBounds2D){
                    var bounds = point.glyph.getBounds2D();
                    if(bounds.containsPoint(mousePos.x, mousePos.y)){
                        event._hasfound = true;
                        event._lastHover = point;
                        break;
                    }
                    var distance = this._getDistance(mousePos, bounds);
                    if(distance < minDistance){
                        nearbyPoint = point;
                        minDistance = distance;
                    }
                }
            }

            if(nearbyPoint && !event._hasfound){
                event._hasfound = true;
                event._lastHover = nearbyPoint;
            }
        }
    },


    _findNearByDataPoint:function(mousePos, event){
        var minDistance = Number.MAX_VALUE;
        for(var i = 0; i < this.dataSeries.length && !event._hasfound; i++){
            var series = this.dataSeries[i];
            if(series.paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                continue;
            }
            this._findSeriesGlyphPoint(series, mousePos, event);
            var nearbyPoint;
            for(var j = 0; j < series.points.length && !event._hasfound; j++){
                var point = series.points[j];
                if(point && point.glyph && point.glyph.shape){

                    if(point.glyph.shape && point.glyph.shape.containsPoint){
                        if(point.glyph.shape.containsPoint(mousePos.x, mousePos.y)){
                            event._hasfound = true;
                            event._lastHover = point;
                            break;
                        }
                    }

                    var distance = this._getDistance(mousePos, point.glyph.shape);
                    if(distance < minDistance){
                        nearbyPoint = point;
                        minDistance = distance;
                    }
                }
            }
        }

        if(nearbyPoint && !event._hasfound){
            event._hasfound = true;
            event._lastHover = nearbyPoint;
        }
    },

    _getDistance:function(mousePos, shape){
        if(this.isHorizontal){//条形图
            if(shape.getMiddleY){
                var middleY = shape.getMiddleY();
                return Math.abs(middleY - mousePos.y);
            }
        }else{
            if(shape.getMiddleX){
                var middleX = shape.getMiddleX();
                return Math.abs(middleX - mousePos.x);
            }
        }

        return Number.MAX_VALUE;
    },

    /**
     * 寻找鼠标移动 所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件
     * @param ctx4OnMove 最上层触发的ctx图层.
     */
    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        var bounds = this.getBounds();

        if(bounds == null || !bounds.containsPoint(mousePos.x,mousePos.y)) {
            return;
        }

        if (this.xAxisGlyph) {
            this.xAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }
        if (this.yAxisGlyph) {
            this.yAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }
        if (this.secondAxisGlyph) {
            this.secondAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }

        if (event._hasfound || !this.dataSeries || this.dataSeries.length <= 0) {
            return;
        }

        if (mousePos.y < bounds.y || mousePos.x < bounds.x
            || mousePos.x > bounds.x + bounds.width || mousePos.y > bounds.y + bounds.height) {
            return;
        }

        var seq = this.getDrawSequence();
        for (var i = this.dataSeries.length - 1; i >= 0 && i < this.dataSeries.length; i += -1) {
            this.dataSeries[seq[i]].findOnMoveGlyph(mousePos, event, ctx4OnMove);
            if (event._hasfound) {
                return;
            }
        }
    },

    /**
     * 绘图区 处理 onMove时的总体响应事件 (坐标轴提示, 趋势线提示, 空值提示)
     * @param ctx 画布
     * @param mousePos  鼠标位置
     */
    doOnMove: function (ctx, mousePos) {
        this.doAxisTooltip(ctx, mousePos);
        this.doTrendLineTooltip(ctx, mousePos);
        this.doNullDataTootip(ctx, mousePos)
    },

    /**
     * 绘图区 处理 onMove的 坐标轴提示
     * @param ctx  画布
     * @param mousePos  鼠标位置
     */
    doAxisTooltip: function (ctx, mousePos) {

        if (this.interactiveAxisTooltip && this.getBounds().containsPoint(mousePos.x, mousePos.y)) {
            var axistips = [];
            $.each([this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph], function (idx, obj) {
                if (obj) {
                    var value = obj.getPointValue(mousePos);
                    var text = "";
                    var format = obj.format;
                    if (obj instanceof FR.Chart.DateAxisGlyph) {
                        text = FR.contentFormat(FR.Chart.ChartUtils.int2Date(value, obj.mainType),
                            FR.Chart.ChartUtils.accDateFormat(obj.mainType, obj.tickLength)) + "";
                    } else if (obj instanceof FR.Chart.NumberAxisGlyph
                        || obj.markAxisType == "valueaxis"
                        || obj instanceof FR.Chart.ValueAxisGlyph) {
                        format = FR.Chart.ChartUtils.accFormat(format);
                        text = FR.contentFormat(value, format) + "";
                    } else {
                        if (this instanceof FR.Chart.GanttPlotGlyph && this.yAxisGlyph == obj) {
                            text = this.yAxisGlyph.categoryLabelList[obj.getCategoryCount() - parseInt(value) - 1];
                        } else {
                            if (obj.categoryLabelList && parseInt(value) < obj.categoryLabelList.length) {
                                text = obj.categoryLabelList[parseInt(value)];
                            }
                        }
                    }
                    var tooltip = new FR.Chart.TooltipString();
                    tooltip.value = text;

                    axistips[axistips.length] = {
                        point: obj.getPoint2D(value),
                        tooltips: [tooltip],
                        pos: obj.position,
                        x: obj.getBounds().x,
                        y: obj.getBounds().y,
                        gridLine: obj.getGridLine(value)[0]
                    };
                }
            });

            if (axistips && axistips.length > 0) {
                if(!ctx){     return; } ctx.save();
                ctx.strokeStyle = "rgb(128,128,128)";

                for (var i = 0; i < axistips.length; i++) {
                    axistips[i].gridLine.draw(ctx, FR.Chart.Constants.LINE_DOT);
                    FR.Chart.Tooltip.drawAxisTooltip(ctx, axistips[i]);
                }
                ctx.restore();
            }
        }
    },

    /**
     * 绘图区 处理 onMove时 每个系列的趋势线提示.
     * @param ctx 画布
     * @param mousePos 鼠标位置
     */
    doTrendLineTooltip: function (ctx, mousePos) {
        if (!this.interactiveAxisTooltip) {
            return;// 先保持和坐标轴提示同步 如果有另外需要再加
        }

        var trendLineTips = [];
        if (this.getBounds().containsPoint(mousePos.x, mousePos.y)) {
            for (var i = 0; i < this.dataSeries.length; i++) {
                this.dataSeries[i].doTrendLineTooltip(mousePos, trendLineTips);
            }
        }

        FR.Chart.Tooltip.doTrendLineTooltip(ctx, trendLineTips, this.getBounds());
    },

    /**
     * 绘图区 处理空值时的onMove 先处理股价图和甘特图的数据错误 提示.
     * @param ctx 画布
     * @param mousePos 鼠标位置.
     */
    doNullDataTootip: function (ctx, mousePos) {
        if (!this.errorDataTip) {
            return;
        }

        var borderBackground = new FR.Chart.ColorBackground();
        borderBackground.color = 'red';

        FR.Chart.Tooltip.dataPointTip(ctx, this.errorDataTip, mousePos, borderBackground);
    },

    /**
     * 画出背景等相关信息
     * @param ctxs
     */
    drawInfo: function (ctxs) {
        this.info.paint(ctxs[0], this.getOutline4Fill());
    },

    /**
     * kunsnat: 需要传入ValueAxisGlyph, 计算趋势线提示的像素对应的Value值(太乱了. 先实现功能..)
     * @param xVal  x数组值
     * @param yVal y数组值
     * @param dataSeries 对应系列
     * @param valueAxisGlyph 对应值轴
     */
    trendLineFitting: function (xVal, yVal, dataSeries, valueAxisGlyph) {
        dataSeries.trendLineList = [];

        var attrTrendLine = this.conditionCollection.getConditionAttr(new FR.Chart.AttrTrendLine(), dataSeries.seriesIndex);

        if (attrTrendLine && attrTrendLine.trendLineList) {

            for (var i = 0; i < attrTrendLine.trendLineList.length; i++) {
                var trendLineGlyph = attrTrendLine.trendLineList[i].createTrendLineGlyph();
                trendLineGlyph.fitting(xVal, yVal);
                trendLineGlyph.drawShape(this.bounds.clone());

                if (valueAxisGlyph instanceof FR.Chart.ValueAxisGlyph) {
                    trendLineGlyph.valueAxisGlyph = valueAxisGlyph;
                }
                dataSeries.trendLineList[i] = trendLineGlyph;
            }
        }
    },

    /**
     * 快速联动的时候会导致数据点标签重复
     */
    stopDataPointLabel:function(){
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            this.dataSeries[i].shouldDrawLable = false;
        }
    },

    /**
     *  kunsnat: 放一些要动态后画的东西. 图例由于Chrome中经常会拖延. 所以不放了. 而且也没有覆盖的问题了
     * @param ctx 画布
     * @param currentDivKey 当前显示文本div的key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            if (this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {
                this.dataSeries[i].paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
            }
            if (this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
                this.dataSeries[i].paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
            }

            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            this.dataSeries[i].drawLabel(ctx, currentDivKey);
        }

        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            if (this.dataSeries[i] && this.dataSeries[i].dataSeriesStyle) {
                if (this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE) {
                    this.dataSeries[i].dataSeriesStyle.paintStyle(ctx);// 这个只是针对饼图的
                    break;
                }
            }
        }
    },

    /**
     * 个别折线类型需要画标记, 折线图, 散点图 雷达图等 在动态之后画标记.
     * @param ctx  画布
     * @param currentDivKey 当前divkey
     */
    drawAfterAnimate4LineMarker: function (ctx, currentDivKey) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            if (this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {
                this.dataSeries[i].paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
            }
            if (this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_APPEAR) {
                this.dataSeries[i].paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
            }

            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
                || this.dataSeries[i].paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
                continue;
            }
            this.dataSeries[i].drawLabel(ctx, currentDivKey);

            var dataPoints = this.dataSeries[i].points;
            for (var j = 0; j < dataPoints.length; j++) {
                dataPoints[j].draw(ctx, currentDivKey); // Marker
            }
        }
    },

    /**
     * 处理系列点标签
     * @param dataPoint 系列点
     */
    dealDataPointLabel: function (dataPoint) {
        var labelGlyph = dataPoint.dataLabel;
        if (!labelGlyph) {
            return;
        }

        if (this.notDealDataPointLabel(dataPoint, labelGlyph)) {
            labelGlyph.bounds = null;
        } else {
            var labelPreDim = labelGlyph.preferredDimension();
            var dataPointRect = dataPoint.getShape().getBounds2D();
            var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
            var isNegativeValue = dataPoint.value < 0;
            var bounds = this.getDataPointLabelBoundsWithPosition(labelPreDim, dataPointRect, attrContents.position, isNegativeValue);
            labelGlyph.bounds = bounds;
        }
    },

    /**
     * 某些情况下 不需要处理标签.
     * @param dataPoint 系列点.
     * @param labelGlyph  文本标签Glyph
     * @returns {boolean} 是否处理.
     */
    notDealDataPointLabel: function (dataPoint, labelGlyph) {
        return (!labelGlyph || !labelGlyph.text || dataPoint.getShape() == null || dataPoint.getShape().getBounds2D() == null);
    },

    /**
     * 根据标签的位置 (上下左右等) 得到系列点标签的边界.
     * @param labelPreDim  标签占用大小
     * @param dataPointRect 系列点占用的范围
     * @param position 文本位置.
     * @returns {null} 返回空值 或者 标签边界.
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return null;
    },

    /**
     * 折线类图形中 系列点 文本标签的位置.
     * @param labelPreDim 标签占用大小.
     * @param dataPointRect 系列点占用的范围
     * @param position 标签位置 (折线类中没有效果)
     * @returns {FR.Chart.Bounds} 返回边界.
     */
    getDataPointLabelBoundsWithPosition4Line: function (labelPreDim, dataPointRect, position) {
        var gap2Top = 2;
        var labelX = dataPointRect.x + (dataPointRect.width - labelPreDim.width) / 2.0;
        var labelY = dataPointRect.y - labelPreDim.height - gap2Top;
        return new FR.Chart.Bounds(labelX, labelY, labelPreDim.width, labelPreDim.height);
    },

    // 返回上一层
    returnBackToLayer: function () {

    },

    /**
     * 根据点击的dataPoint, 钻取到下一层
     * @param dataPoint 点击的系列点
     * @param event 鼠标事件.
     */
    layerToDataPointNext: function (dataPoint, event) {

    },

    /**
     * 开始拖拽.
     * @param mousePos 鼠标位置.
     * @param event 鼠标事件.
     * @param series 系列
     */
    onDragStart: function (ctxOnMove, series) {
        var backupAlpha;
        if (series.glyph) {
            backupAlpha = series.glyph.getAlpha();
            if (series instanceof FR.Chart.DataSeries4Area) {
                series.glyph.setAlpha(0.2);
            } else {
                series.glyph.setAlpha(0.5);
            }
            series.glyph.draw(ctxOnMove, 1000, FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
            series.glyph.setAlpha(backupAlpha);
        }

        var dataPoints = series.points;
        var startIndex = 0;
        var endIndex = series.getDataPointCount();

        if (this.xAxisGlyph && this.xAxisGlyph.isZoom) {
            startIndex = this.xAxisGlyph.getCrossValue();
            endIndex = this.xAxisGlyph.getArrowValue();
        }

        for (var i = startIndex; i < endIndex; i++) {
            var dataPoint = dataPoints[i];
            if (dataPoint && dataPoint.glyph) {
                backupAlpha = dataPoint.glyph.getAlpha();
                dataPoint.glyph.setAlpha(0.5);
                dataPoint.draw(ctxOnMove, 1000);
                dataPoint.glyph.setAlpha(backupAlpha);
            }
        }
    },

    /**
     * 拖拽过程中系列移动
     * @param mouseStartPos 鼠标单击的位置
     * @param mousePos 当前鼠标的位置
     * @param ctx4OnMove move的ctx
     */
    onSeriesDrag: function (mouseStartPos, mousePos, ctxOnMove, draggingSeries) {
        var startIndex = 0;
        var endIndex = draggingSeries.getDataPointCount();

        if (this.xAxisGlyph && this.xAxisGlyph.isZoom) {
            startIndex = this.xAxisGlyph.getCrossValue();
            endIndex = this.xAxisGlyph.getArrowValue();
        }

        var seriesIndex = this._combineSeriesIndex(mousePos, draggingSeries);
        if (seriesIndex !== -1) {//鼠标移动到了某个系列之上
            var series = this.dataSeries[seriesIndex];
            //画边框
            if (series.glyph) {
                FR.Chart.HighUtils.doSeriesDragBorder(series.glyph, ctxOnMove);
            }

            var dataPoints = series.points;
            for (var i = startIndex; i < endIndex; i++) {
                var dataPoint = dataPoints[i];
                if (dataPoint != null && dataPoint.glyph) {
                    FR.Chart.HighUtils.doSeriesDragBorder(dataPoint.glyph, ctxOnMove);
                }
            }
        }

        var stepX = mousePos.x - mouseStartPos.x;
        var stepY = mousePos.y - mouseStartPos.y;
        ctxOnMove.translate(stepX, stepY);

        //有系列的先画系列的shape
        var backupAlpha;
        if (draggingSeries.glyph) {
            backupAlpha = draggingSeries.glyph.getAlpha();
            if (draggingSeries instanceof FR.Chart.DataSeries4Area) {
                draggingSeries.glyph.setAlpha(0.2);
            } else {
                draggingSeries.glyph.setAlpha(0.5);
            }
            draggingSeries.glyph.draw(ctxOnMove, 1000, FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL);
            draggingSeries.glyph.setAlpha(backupAlpha);
        }

        var dataPoints = draggingSeries.points;
        for (var i = startIndex; i < endIndex; i++) {
            var dataPoint = dataPoints[i];
            if (dataPoint && dataPoint.glyph) {
                backupAlpha = dataPoint.glyph.getAlpha();
                dataPoint.glyph.setAlpha(0.5);
                dataPoint.draw(ctxOnMove, 1000);
                dataPoint.glyph.setAlpha(backupAlpha);
            }
        }
        ctxOnMove.translate(-stepX, -stepY);
    },

    /**
     * 系列拖拽结束.
     * @param series 拖拽系列.
     * @param startPos 起始位置.
     * @param mousePos 鼠标位置.
     * @param event 事件.
     */
    onSeriesDragEnd: function (series, startPos, mousePos, event) {//绘图区中有系列正在被拖拽
        var seriesIndex = this._combineSeriesIndex(mousePos, series);
        if (seriesIndex !== -1) { //系列合并
            this.dataSeries[seriesIndex].addDataSeries(series);
            series.paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
            series.lastState = FR.Chart.ChartPaintConstants.STATE_TRANSPARENT;
            this.refreshDataPointLabel();
            event.needRefresh = true;
        } else if (!this.bounds.containsPoint(mousePos.x, mousePos.y)) { //删除操作
            series.paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
            series.lastState = FR.Chart.ChartPaintConstants.STATE_TRANSPARENT;
            this.refreshDataPointLabel();
            event.needRefresh = true;
        } else {//无效操作
            series.paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
        }
    },

    /**
     * 拖拽的时候鼠标移除
     * @param series 拖拽出的系列.
     */
    onDragMouseOut: function (series) {
        series.paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
        series.lastState = FR.Chart.ChartPaintConstants.STATE_TRANSPARENT;
    },

    /**
     * 是否支持缩放
     * @returns {boolean}
     */
    isSupportZoom: function() {
      return false;
    },

    /**
     * 设置缩放
     * @param zoom
     */
    setZoom:function(zoom) {
        return false;
    },

    _combineSeriesIndex: function (mousePos, draggingSeries) {
        var seriesToCombine = -1;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var series = this.dataSeries[seriesIndex];
            if (series.paintState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT
                || series.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            //先从数据点找，再从系列的shape找
            for (var index = 0; index < series.points.length; index++) {
                var shape = series.points[index].getShape();
                if (shape && shape.containsPoint(mousePos.x, mousePos.y)) {
                    seriesToCombine = seriesIndex;
                    break;
                }
            }
            if (series.glyph && seriesToCombine == -1) {
                if (series.glyph.getShape().containsPoint(mousePos.x, mousePos.y)) {
                    seriesToCombine = seriesIndex;
                    break;
                }
            }
        }
        return seriesToCombine; //表示没有系列需要合并
    },

    /**
     * 判断图表能否继续钻取
     * @returns {boolean} 默认都是不能继续钻取，地图only
     */
    hasNextLayer:function(){
        return false;
    },


    /**
     * 设置过程偏移
     * @param x
     * @param y
     */
    setCurrentOffset:function(x, y) {

    },

    /**
     * 设置结果偏移
     * @param x
     * @param y
     */
    setTotalOffset:function() {

    }
});

/**
 * @class FR.Chart.RectanglePlotGlyph
 * @extends FR.Chart.PlotGlyph
 * @abstract
 *
 * @param  {JSON} js 配置属性
 *
 * @param {Color} horizontalIntervalBackgroundColor 横向间隔颜色
 * @param {Color} verticalIntervalBackgroundColor 纵向间隔颜色
 * @param {Boolean} [js.interactiveAxisTooltip = false] 坐标轴热点提示
 *
 */
FR.Chart.RectanglePlotGlyph = FR.extend(FR.Chart.PlotGlyph, {

    initConstants: function (js) {
        js = js || {};

        this.horizontalIntervalBackgroundColor = js.horizontalIntervalBackgroundColor;
        this.verticalIntervalBackgroundColor = js.verticalIntervalBackgroundColor;
        this.interactiveAxisTooltip = js.interactiveAxisTooltip || false;
        var axisConfig = FR.Chart.AxisConfig;
        if (js.xAxis || js.xAxisGlyph) {
            if(js.xAxis){
                var axisName = axisConfig[js.xAxis.axisType];
                this.xAxisGlyph = new FR.Chart[axisName](js.xAxis);
            }else if(js.xAxisGlyph){
                this.xAxisGlyph = new FR.Chart[js.xAxisGlyphType](js.xAxisGlyph);
            }
        } else {
            this.xAxisGlyph = null;
        }

        if (js.yAxis || js.yAxisGlyph) {
            if(js.yAxis){
                var axisName = axisConfig[js.yAxis.axisType];
                this.yAxisGlyph = new FR.Chart[axisName](js.yAxis);
            }else if(js.yAxisGlyph){
                this.yAxisGlyph = new FR.Chart[js.yAxisGlyphType](js.yAxisGlyph);
            }
        } else {
            this.yAxisGlyph = null;
        }

        if (js.secondAxis || js.secondAxisGlyph) {
            if(js.secondAxis){
                var axisName = axisConfig[js.secondAxis.axisType];
                this.secondAxisGlyph = new FR.Chart[axisName](js.secondAxis);
            }else if(js.secondAxisGlyph){
                this.secondAxisGlyph = new FR.Chart[js.secondAxisGlyphType](js.secondAxisGlyph);
            }
        } else {
            this.secondAxisGlyph = null;
        }

        if(this.xAxisGlyph){
            this.xAxisGlyph.isLabelWrap = this.xAxisGlyph.isLabelWrap &&  this.isSupportLabelWrap();
            this.xAxisGlyph.isLabelWrap = this.xAxisGlyph.isLabelWrap && this.isSupportLabelWrap();
            this.xAxisGlyph.isLabelWrap = this.xAxisGlyph.isLabelWrap && this.isSupportLabelWrap();
        }

        FR.Chart.RectanglePlotGlyph.superclass.initConstants.apply(this, [js]);
    },


    /**
     * 在plotGlyph里响应拖拽事件
     * @param mouseStartPos 拖拽开始的位置
     * @param mousePos 当前的鼠标位置
     * @param ctx4OnMove 矩形画的canvas
     */
    onDrag: function (mouseStartPos, mousePos, ctx4OnMove) {

        if(!this.bounds.containsPoint(mouseStartPos.x, mouseStartPos.y)
            || !this.bounds.containsPoint(mousePos.x, mousePos.y)){
            return ;
        }

        var pointX = Math.min(mouseStartPos.x, mousePos.x);
        var pointY = Math.min(mouseStartPos.y, mousePos.y);
        var width = Math.abs(mouseStartPos.x - mousePos.x);
        var height = Math.abs(mouseStartPos.y - mousePos.y);
        var drawBounds = new FR.Chart.Bounds(pointX, pointY, width, height);

        ctx4OnMove.save();
        ctx4OnMove.fillStyle = 'rgba(64,194,239, 0.5)';
        ctx4OnMove.strokeStyle = 'rgba(64,194,239, 1.0)';
        drawBounds.paint(ctx4OnMove);
        drawBounds.draw(ctx4OnMove);
        ctx4OnMove.restore();

    },

    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey){
        FR.Chart.RectanglePlotGlyph.superclass.drawMarkersLabelAfterSeriesAnimateDraw.apply(this, [ctx, currentDivKey]);
        this.drawAlertLinesAfterSeriesAnimateDraw(ctx);
    },

    drawAlertLinesAfterSeriesAnimateDraw: function (ctx){
        if(this.xAxisGlyph){
            this.xAxisGlyph.drawAlertValueLines(ctx);
        }

        if(this.yAxisGlyph && !this.yAxisGlyph.cubic){
            this.yAxisGlyph.drawAlertValueLines(ctx);
        }

        if(this.secondAxisGlyph && !this.secondAxisGlyph.cubic){
            this.secondAxisGlyph.drawAlertValueLines(ctx);
        }
    },

    isSupportLabelWrap:function(){
        return false;
    },

    isNeedTimeSwitchWidget:function(){
        return this.xAxisGlyph && this.xAxisGlyph.isNeedTimeSwitch();
    },


    layoutDemoGlyph:function(){
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries.glyph = null;
                continue;
            }

            this.dealLine4DemoGlyph(dataSeries, this, this.xAxisGlyph, this.getSeriesAxisGlyph(seriesIndex));
        }
    },

    dealLine4DemoGlyph: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph) {

    } ,

    /**
     * 布局坐标轴
     * @param left 间隔.
     */
    layoutAxisGlyph: function (left) {
        this.layoutDoubleAxisGlyph(this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph, left);
    },

    /**
     * 布局双值轴
     * @param categoryAxisGlyph  分类轴
     * @param valueAxisGlyph 值轴
     * @param secondValueAxisGlyph 第二值轴
     */
    layoutDoubleAxisGlyph: function (categoryAxisGlyph, valueAxisGlyph, secondValueAxisGlyph) {
        var plotZeroBounds = this.getBounds().clone();
        var plotBoundsLeftWithoutAxis = this.getBounds();

        this.initAxisGlyphBoundsAndZeroBounds(categoryAxisGlyph, plotZeroBounds);
        this.initAxisGlyphBoundsAndZeroBounds(valueAxisGlyph, plotZeroBounds);
        this.initAxisGlyphBoundsAndZeroBounds(secondValueAxisGlyph, plotZeroBounds);

        this.dealPlotBoundsWithAxisGlyph(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        this.dealPlotBoundsWithAxisGlyph(valueAxisGlyph, plotBoundsLeftWithoutAxis);
        this.dealPlotBoundsWithAxisGlyph(secondValueAxisGlyph, plotBoundsLeftWithoutAxis);

        if(!categoryAxisGlyph.isLabelWrap){
            this.dealPlotBoundsWidthAxisInHorizontal(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        }else{
            this.dealWithAreaAxisWrap(categoryAxisGlyph,plotBoundsLeftWithoutAxis);
        }
        this.dealPlotBoundsWidthAxisInHorizontal(valueAxisGlyph, plotBoundsLeftWithoutAxis);
        this.dealPlotBoundsWidthAxisInHorizontal(secondValueAxisGlyph, plotBoundsLeftWithoutAxis);

        this.calculateAxisGlyphAttr(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        if(categoryAxisGlyph.isLabelWrap  && !(categoryAxisGlyph instanceof FR.Chart.DateAxisGlyph)){
            //有换行，重新计算高度
            var diff = categoryAxisGlyph.dealWithPlotBounds();
            categoryAxisGlyph.calculateBoundsWidthOrientationAndTitle(plotBoundsLeftWithoutAxis,diff,new FR.Chart.Dimension2D(0, 0));
            this.calculateAxisGlyphAttr(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        }
        this.calculateAxisGlyphAttr(valueAxisGlyph, plotBoundsLeftWithoutAxis);
        this.calculateAxisGlyphAttr(secondValueAxisGlyph, plotBoundsLeftWithoutAxis);

        // 处理0值对齐
        if (secondValueAxisGlyph && secondValueAxisGlyph.alignZeroValue) {
            var valueZeroPoint = valueAxisGlyph.getPointInBounds(0);
            var secondValueZeroPoint = secondValueAxisGlyph.getPointInBounds(0);

            var isVertical = valueAxisGlyph.position == FR.Chart.Constants.LEFT || valueAxisGlyph.position == FR.Chart.Constants.RIGHT;

            var valueY = isVertical ? valueZeroPoint.y : valueZeroPoint.x;
            var secondValueY = isVertical ? secondValueZeroPoint.y : secondValueZeroPoint.x;

            if (valueY != secondValueY) {
                var bounds = valueAxisGlyph.getBounds();
                var allLength = isVertical ? bounds.height : bounds.width;
                if (valueY > allLength * 0.2 && valueY < allLength * 0.8) {// 接近中间区域. 那么只变动第二值轴
                    this.dealAxisGlyphZeroPoint(secondValueAxisGlyph, isVertical, valueY);
                } else {
                    this.dealAxisGlyphZeroPoint(valueAxisGlyph, isVertical, (valueY + secondValueY) / 2);
                    this.dealAxisGlyphZeroPoint(secondValueAxisGlyph, isVertical, (valueY + secondValueY) / 2);
                }
            }
        }

        this.dealPlotBounds4JSDraw(this.getBounds());
    },

    dealWithAreaAxisWrap:function(categoryAxisGlyph,leftBounds){
        if(this instanceof FR.Chart.AreaPlotGlyph || this instanceof FR.Chart.Area3DPlotGlyph){
            categoryAxisGlyph.dealPlotBounds4AreaWrapInHorizontal(leftBounds);
        }
    },

    /**
     * 布局初始化 坐标轴 边界 原点等. TODO 删除
     * @param axisGlyph 坐标轴glyph
     * @param zeroBounds 原点边界.
     */
    initAxisGlyphBoundsAndZeroBounds: function (axisGlyph, zeroBounds) {
        if (axisGlyph) {
            axisGlyph.bounds = zeroBounds.clone();
            axisGlyph.plotZeroBounds = zeroBounds.clone();
        }
    },

    /**
     * 根据坐标轴调整 绘图区剩余的边界.
     * @param axisGlyph 坐标轴
     * @param plotLeftBounds 剩余的边界.
     */
    dealPlotBoundsWithAxisGlyph: function (axisGlyph, plotLeftBounds) {
        if (axisGlyph) {
            axisGlyph.dealPlotBoundsWithAxisLabel(plotLeftBounds);
        }
    },

    /**
     * 在水平方向上 根据坐标轴调整绘图区的剩余边界.
     * @param axisGlyph 坐标轴
     * @param lastPlotBounds 剩余的边界.
     */
    dealPlotBoundsWidthAxisInHorizontal: function (axisGlyph, lastPlotBounds) {
        if (axisGlyph) {
            axisGlyph.dealPlotBoundsWithLabelInHorizontal(lastPlotBounds);
        }
    },

    /**
     * 计算初始化坐标轴的属性
     * @param axisGlyph 坐标轴
     * @param endPlotBounds 剩余的绘图区边界.
     */
    calculateAxisGlyphAttr: function (axisGlyph, endPlotBounds) {
        if (axisGlyph) {
            axisGlyph.calculateAxisGlyph(endPlotBounds);
        }
    },

    /**
     * 处理坐标轴的0值点
     * @param axisGlyph 坐标轴
     * @param isVertical 是否垂直
     * @param suitableValue  处理的结果值.
     */
    dealAxisGlyphZeroPoint: function (axisGlyph, isVertical, suitableValue) {
        if (axisGlyph == null) {
            return;
        }

        var bounds = axisGlyph.getBounds();

        var allLength = isVertical ? bounds.height : bounds.width;

        var oldZeroPoint = axisGlyph.getPointInBounds(0);
        var oldY = isVertical ? oldZeroPoint.y : oldZeroPoint.x;

        var scale = 1;
        if (oldY > suitableValue) {//kunsnat: 用实际的坐标轴0点到上方或者下方的实际长度  计算合适的缩放比例
            scale = (suitableValue - bounds.y) / (oldY - bounds.y);

            var oPoint = axisGlyph.originPoint;
            oPoint.y = bounds.y + allLength * scale;
        } else if (oldY < suitableValue) {
            scale = (allLength - (suitableValue - bounds.y)) / (allLength - (oldY - bounds.y));
        }
        axisGlyph.axisLength = allLength * scale;
        axisGlyph.unitLength = axisGlyph.unitLength * scale;
    },

    /**
     * 根据序列号 获取对应的值轴.
     * @param seriesIndex 系列序号
     * @returns {axisGlyph} 对应的值轴
     */
    getSeriesAxisGlyph: function (seriesIndex) {
        var position = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAxisPosition(), seriesIndex);
        return position.axisPosition == FR.Chart.CustomAttr.Axis_RIGHT ? this.secondAxisGlyph : this.yAxisGlyph;
    },

    /**
     * RectanglePlotGlyph中 为xyscatter和bubble准备的获取y值 不根据值轴(选取)  不支持次要坐标轴
     * @returns {FR.Chart.NumberAxisGlyph.UNIT_HEIGHT|*}
     */
    getStartY : function(dataPoint) {
        var axisGlyph = this.yAxisGlyph;//kunsnat: 直接用y值, 不考虑双值轴, 不支持, 且兼容问题. bug: 04961
        if(axisGlyph == null) {
            return 0;
        }
        return axisGlyph.getPoint2D(dataPoint.value).y;
    },

    /**
     * 获取为坐标轴顶部单位预留的空间
     * @returns {number} 单位预留值.
     */
    getPreGap4AxisTopUnit: function () {
        return (this._isAxisShowUnit(this.yAxisGlyph) || this._isAxisShowUnit(this.secondAxisGlyph)) ? FR.Chart.NumberAxisGlyph.UNIT_HEIGHT : 0;
    },

    _isAxisShowUnit: function (axisGlyph) {
        return axisGlyph && (axisGlyph.showUnit || axisGlyph.unitPadding);
    },

    /**
     * 获取为坐标轴 右侧 单位预留的空间
     * @returns {number} 单位预留值.
     */
    getPreGap4AxisRightUnit: function () {
        return this._isAxisShowUnit(this.xAxisGlyph) ? FR.Chart.NumberAxisGlyph.UNIT_HEIGHT : 0;
    },

    /**
     * 获取为数据表预留的左侧 空间
     * @returns {number} 左侧预留空间大小.
     */
    getPreLeft4DataSheet: function () {
        var first = (this.xAxisGlyph == null) ? 0 : this.xAxisGlyph.getPreLeftWidth4DataSheet();
        var second = (this.yAxisGlyph == null) ? 0 : this.yAxisGlyph.getPreLeftWidth4DataSheet();
        var third = (this.secondAxisGlyph == null) ? 0 : this.secondAxisGlyph.getPreLeftWidth4DataSheet();
        return first + second + third;
    },

    /**
     * RectanglePlotGlyph 矩形绘图区 画内容.
     * @param ctxs 画布层
     * @param time 动态时间
     * @param actionModel 动态方式.
     */
    draw: function (ctxs, time, actionModel) {
        if (!this.xAxisGlyph.cubic) {
            this.drawInfo(ctxs);// info 应该在坐标轴之前. 因为刻度会挤压info
        }

        // 坐标轴压过了系列(柱形) RectanglePlot 坐标轴线放在系列之上的canvas
        if (this.xAxisGlyph != null && !this.xAxisGlyph.cubic) {
            this.xAxisGlyph.draw(ctxs[1]);
        }
        if (this.yAxisGlyph != null && !this.xAxisGlyph.cubic) {
            this.yAxisGlyph.draw(ctxs[1]);
        }

        if (this.secondAxisGlyph != null && !this.secondAxisGlyph.cubic) {
            this.secondAxisGlyph.draw(ctxs[1]);
        }

        FR.Chart.RectanglePlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    /**
     *  画出矩形区域的 背景信息
     * @param ctxs 画布层.
     */
    drawInfo: function (ctxs) {

        //绘图背景
        this.info.paint(ctxs[0], this.getOutline4Fill());

        if (this.verticalIntervalBackgroundColor != null) {
            var xBackground = new FR.Chart.IntervalColorBackground();
            xBackground.bgColor = this.verticalIntervalBackgroundColor;
            xBackground.position = FR.Chart.Constants.BOTTOM;
            xBackground.intervalLength = this._getIntervalLength(false);

            xBackground.paint(ctxs[1], this.getOutline4Fill());
        }

        if (this.horizontalIntervalBackgroundColor != null) {
            var yBackground = new FR.Chart.IntervalColorBackground();
            yBackground.bgColor = this.horizontalIntervalBackgroundColor;
            yBackground.position = FR.Chart.Constants.LEFT;
            yBackground.intervalLength = this._getIntervalLength(true);
            yBackground.paint(ctxs[1], this.getOutline4Fill());
        }

        // 网格线和间隔背景
        this._drawBackgroundGrid(ctxs[1])
    },

    _getIntervalLength: function (isHorizontal) {
        var xLength = this.xAxisGlyph.unitLength * this.xAxisGlyph.tickLength;
        var yLength = this.yAxisGlyph.unitLength * this.yAxisGlyph.tickLength;
        if (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP) {
            return isHorizontal === true ? yLength : xLength;
        } else {
            return isHorizontal === true ? xLength : yLength;
        }
    },

    _drawBackgroundGrid: function (ctx) {
        if (this.yAxisGlyph && this.yAxisGlyph.getBounds()) {
            this.yAxisGlyph.drawAxisGrid(ctx);
        }

        if (this.xAxisGlyph && this.xAxisGlyph.getBounds()) {
            this.xAxisGlyph.drawAxisGrid(ctx);
        }

        if (this.secondAxisGlyph && this.secondAxisGlyph.getBounds()) {
            this.secondAxisGlyph.drawAxisGrid(ctx);
        }
    },

    /**
     *  刷新x轴, y轴 两个坐标轴的最大最小值.
     */
    refreshAxisGlyph4XY: function () {
        this.xAxisGlyph.oldMin = this.xAxisGlyph.minValue;
        this.xAxisGlyph.oldMax = this.xAxisGlyph.maxValue;
        this.xAxisGlyph.oldTick = this.xAxisGlyph.tickLength;

        var min = this._getMinValue4XValue(this.xAxisGlyph.isLog);  // 从系列中获取.
        var max = this._getMaxValue4XValue();
        var dis = (max - min) * this.getMarkerSize() / 500;
        max = max + dis;
        min = min - dis;

        this.xAxisGlyph.initMinMaxValue(min, max);

        this.yAxisGlyph.oldMin = this.yAxisGlyph.minValue;
        this.yAxisGlyph.oldMax = this.yAxisGlyph.maxValue;
        this.yAxisGlyph.oldTick = this.yAxisGlyph.tickLength;

        min = this.getMinValueFromData(this.yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT);
        max = this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT);
        dis = (max - min) * this.getMarkerSize() / 300;
        max = max + dis;
        min = min - dis;
        this.yAxisGlyph.initMinMaxValue(min, max);
    },

    /**
     * 返回标记点的 预留大小 默认0
     * @returns {number} 标记的预留大小.
     */
    getMarkerSize: function () {
        return 0;
    },

    _getMinValue4XValue: function (isLog) {
        var minValue = 0;
        if (isLog) {
            minValue = Number.MAX_VALUE;
        }

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex += 1) {
            var dataSeries = this.dataSeries[seriesIndex];

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            for (var i = 0; dataSeries && i < dataSeries.getDataPointCount(); i++) {
                var dataPoint = dataSeries.points[i];
                var value = dataPoint.x;

                if (isLog) {
                    if (value > 0 && value < minValue) {
                        minValue = value;
                    }
                } else {
                    minValue = minValue < value ? minValue : value;
                }
            }
        }

        if (isLog) {
            if (minValue == Number.MAX_VALUE) {
                minValue = 1;
            }
        }

        return minValue;
    },

    _getMaxValue4XValue: function () {
        var maxX = 0;

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex += 1) {
            var dataSeries = this.dataSeries[seriesIndex];

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            for (var i = 0; dataSeries && i < dataSeries.getDataPointCount(); i++) {
                var dataPoint = dataSeries.points[i];
                maxX = Math.max(maxX, dataPoint.x);
            }
        }
        return maxX;
    },

    /**
     *  值轴重新布局 RectanglePlotGlyph
     */
    refreshAxisGlyph: function () {
        this.yAxisGlyph.oldMin = this.yAxisGlyph.minValue;
        this.yAxisGlyph.oldMax = this.yAxisGlyph.maxValue;
        this.yAxisGlyph.oldTick = this.yAxisGlyph.tickLength;

        this.yAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));

        if (this.secondAxisGlyph) {
            this.secondAxisGlyph.oldMin = this.secondAxisGlyph.minValue;
            this.secondAxisGlyph.oldMax = this.secondAxisGlyph.maxValue;
            this.secondAxisGlyph.oldTick = this.secondAxisGlyph.tickLength;

            this.secondAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.secondAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_RIGHT),
                this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_RIGHT));
        }
    }
});

/**
 * 分类轴绘图区
 * @class FR.Chart.CategoryPlotGlyph
 * @extends FR.Chart.RectanglePlotGlyph
 *
 * @param {JSON} js配置属性
 * @param {Boolean} [js.isStacked] 堆积属性. TODO 可以转移到BarPlotGlyph中.. 因为 折线 面积都是确定的了.
 */
FR.Chart.CategoryPlotGlyph = FR.extend(FR.Chart.RectanglePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isStacked = js.isStacked;

        FR.Chart.CategoryPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    configAxisLabels:function(chartData){
        this.xAxisGlyph.configLabels(chartData);
    },

    configAxisLabelsWithOptionData:function(data){
        this.xAxisGlyph.configAxisLabelsWithOptionData(data);
    },

    /**
     *从数据中计算出一个最大值
     * @returns {Number} 计算出的最大值
     */
    getMaxValueFromData: function () {
        var maxValue = 0.0;
        var maxPercent = 0.0;
        var startIndex = 0;
        var endIndex = this.getCategoryCount();
        if (this.xAxisGlyph.isZoom) {
//            startIndex = this.xAxisGlyph.getCrossValue();// 日期轴的取值 位置不对, 需要另外计算.
//            endIndex = this.xAxisGlyph.getArrowValue();
        }
        if (this.isStacked) {
            for (var i = startIndex; i < endIndex; i++) {
                var sum = 0.0;
                for (var j = 0; j < this.dataSeries.length; j++) {
                    if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        continue;
                    }
                    var number = this.dataSeries[j].points[i];
                    if (!number.valueIsNull && number.value >= 0.0 && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                        sum += number.value;
                    }
                }
                maxValue = sum > maxValue ? sum : maxValue;

                if (this.yAxisGlyph.isPercentage) {
                    var sum4Percent = 0;
                    for (var j = 0; j < this.dataSeries.length; j++) {
                        var number = this.dataSeries[j].points[i];
                        if (!number.valueIsNull && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                            sum4Percent += Math.abs(number.value);
                        }
                    }

                    maxPercent = sum / sum4Percent > maxPercent ? sum / sum4Percent : maxPercent;
                }
            }
            if (this.yAxisGlyph.isPercentage) {
                return maxPercent;
            }

        } else {
            for (var i = startIndex; i < endIndex; i++) {
                for (var j = 0; j < this.dataSeries.length; j++) {
                    if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        continue;
                    }

                    var number = this.dataSeries[j].points[i];
                    if (!number.valueIsNull && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                        maxValue = (number.value > maxValue) ? number.value : maxValue;
                    }
                }
            }
        }

        // kunsnat: 处理没有值最大值和最小值相等 值轴显示不出的情况.
        if (this.getMinValueFromData(false) == maxValue) {
            return maxValue + 10;
        }

        return maxValue;
    },

    /**
     * 从数据中计算一个最小值
     * @param isLog{boolean} 判断是否为对数底数
     * @returns {number} 计算出的最小值
     */
    getMinValueFromData: function (isLog) {
        var minValue = 0.0;
        var minPercent = 0.0;
        var startIndex = 0;
        var endIndex = this.getCategoryCount();
        if (this.xAxisGlyph.isZoom) {//缩放以后重新计算
//            startIndex = this.xAxisGlyph.getCrossValue();
//            endIndex = this.xAxisGlyph.getArrowValue();
        }

        if (isLog) {
            minValue = Number.MAX_VALUE;
        }

        if (this.isStacked) {
            for (var i = startIndex; i < endIndex; i++) {
                var sum = 0.0;

                // 取出第一个系列中的分类, 判断是否在范围内. categoryName

                for (var j = 0; j < this.dataSeries.length; j++) {
                    if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        continue;
                    }

                    var number = this.dataSeries[j].points[i];
                    if (!number.valueIsNull && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                        if (number.value <= 0.0) {
                            sum += number.value;
                        } else if (isLog) {
                            minValue = (number.value < minValue) ? number.value : minValue;
                        }
                    }
                }
                if (!isLog) {
                    minValue = sum < minValue ? sum : minValue;
                }

                if (this.yAxisGlyph.isPercentage) {
                    var sum4Percent = 0;
                    for (var j = 0; j < this.dataSeries.length; j++) {
                        if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                            continue;
                        }

                        var number = this.dataSeries[j].points[i];
                        if (!number.valueIsNull && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                            sum4Percent += Math.abs(number.value);
                        }
                    }

                    minPercent = sum / sum4Percent < minPercent ? sum / sum4Percent : minPercent;
                }
            }
            if (this.yAxisGlyph.isPercentage) {
                return minPercent;
            }

        } else {
            for (var i = startIndex; i < endIndex; i++) {
                for (var j = 0; j < this.dataSeries.length; j++) {
                    if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        continue;
                    }

                    var number = this.dataSeries[j].points[i];
                    if (!number.valueIsNull && this.isInCategoryField(this.xAxisGlyph, number.categoryName)) {
                        if (isLog) {
                            minValue = (number.value < minValue && number.value > 0) ? number.value : minValue;
                        } else {
                            minValue = (number.value < minValue) ? number.value : minValue;
                        }
                    }
                }
            }
        }
        if (minValue == Number.MAX_VALUE) {
            return 1;
        }
        return minValue;
    },

    /**
     * 判断是否在 轴显示的范围内
     * @param dateAxisGlyph 日期坐标轴
     * @returns {boolean} 返回是否在显示的区域范围内.
     */
    isInCategoryField: function (dateAxisGlyph, cateOb) {
        var minValue = dateAxisGlyph.minValue;
        var maxValue = dateAxisGlyph.maxValue;

        if (dateAxisGlyph.jsType == 'DateAxis') {
            var date = FR.Chart.ChartUtils.object2Date(cateOb);
            var number = FR.Chart.ChartUtils.date2Int(date, dateAxisGlyph.mainType);
            if (!isNaN(number)) {
                if (number < minValue || number > maxValue) {
                    return false;
                }
            }
        }

        return true;
    },

    //判断某一个系列是不是曲线
    isSeriesCurve:function(dataSeries){
        if(dataSeries && this.conditionCollection){
            var attCurveLine = this.conditionCollection.getConditionAttr(new FR.Chart.AttrCurveLine({isCurve:this.isCurve}), dataSeries);
            if(attCurveLine){
                return attCurveLine.isCurve;
            }
        }
        return this.isCurve;
    },

    //判断一个系列是否是空值断开
    isSeriesNullValueBreak:function(dataSeries){
        if(dataSeries && this.conditionCollection){
            if(this.conditionCollection.isContainsAttr(dataSeries,new FR.Chart.AttrNullValueBreak())){
                var attrNullValue = this.conditionCollection.getConditionAttr(new FR.Chart.AttrNullValueBreak({isNullValueBreak:this.isNullValueBreak}), dataSeries);
                return attrNullValue.isNullValueBreak;
            }
        }
        return this.isNullValueBreak;
    },

    draw4Demo: function (ctx) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            this.dataSeries[i].draw4Demo(ctx);
        }
    },

    /**
     * 返回 指定 系列 分类的堆积值
     * @param seriesIndex 指定系列
     * @param categoryIndex 指定分类
     * @returns {*} 返回堆积值.
     */
    getPreSum4Line: function (seriesIndex, categoryIndex) {
        return this.getPreSum4Area(seriesIndex, categoryIndex);
    },

    /**
     * 返回 指定 系列 分类的堆积值
     * @param seriesIndex 指定系列
     * @param categoryIndex 指定分类
     * @returns {*} 返回堆积值.
     */
    getPreSum4Area: function (seriesIndex, categoryIndex) {
        var pre_sum = 0;
        for (var i = seriesIndex - 1; i >= 0; i--) {
            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            pre_sum += this.getDataPointPercentValue(i, categoryIndex);
        }

        return pre_sum;
    },


    /**
     *返回指定系列 日期范围的堆积值
     * @param seriesIndex 系列
     * @param date 日期
     * @param dateType 范围类型
     */
    getPreSum4AreaDataRange:function(seriesIndex,baseDate,dateType,xAxisGlyph){
        var pre_sum = 0;
        for (var i = seriesIndex - 1; i >= 0; i--) {
            var dataSeries =this.dataSeries[i];
            var seriesSum = 0;
            var seriesCount = 0;
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];

                if(dataPoint.valueIsNull){
                    continue ;
                }
                var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
                if(FR.Chart.ChartUtils.isBelongToSameUnit(baseDate,cateDate,dateType)){
                    seriesSum += this.getDataPointPercentValue(i, categoryIndex);
                    seriesCount++;
                }
            }
            if(seriesCount>0){
                pre_sum += seriesSum/seriesCount;
            }
        }

        return pre_sum;
    },


    /**
     * 返回指定系列 分类 数据正负的 堆积值.
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @param isNegative 是否为正值
     * @returns {number} 堆积值.
     */
    getPreSum4Bar: function (seriesIndex, categoryIndex, isNegative) {
        var pre_sum = 0;
        // 小于0的累积
        var pre_sum_negative = 0;

        for (var i = seriesIndex - 1; i >= 0; i--) {
            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var seriesValue = this.getDataPointPercentValue(i, categoryIndex);
            if (seriesValue > 0) {
                pre_sum += seriesValue;
            } else {
                pre_sum_negative += seriesValue;
            }
        }

        if (isNegative) {
            return pre_sum_negative;
        } else {
            return pre_sum;
        }
    },

    /**
     * 处理折线的图表缩放区系列.
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类绘图区.
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     */
    dealLine4DemoGlyph: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph) {
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }

        var isStartPoint = true;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];

            if (dataPoint.valueIsNull) {
                if (this.isNullValueBreak) {
                    isStartPoint = true;  //空值断开
                }
                continue;
            }

            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);
            var point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x;
            if (this.isDataPointXNotInPlotBounds(point_x)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                continue;
            }
            var point_y = yAxisGlyph.getPoint2D(value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            lineDataCount++;

            if (this.isCurve) {
                if (isStartPoint) {
                    FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
                    curvePaths.clear();

                    curvePaths.moveTo(point_x, point_y);
                    linePaths.moveTo(point_x, point_y);
                    isStartPoint = false;
                } else {
                    curvePaths.lineTo(point_x, point_y);
                }
            } else {
                if (isStartPoint) {
                    linePaths.moveTo(point_x, point_y);
                    isStartPoint = false;
                } else {
                    linePaths.lineTo(point_x, point_y);
                }
            }
        }

        if (this.isCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }
    },


    /**
     * 返回系列点的 百分比
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {*} 百分比
     */
    getDataPointPercentValue: function (seriesIndex, categoryIndex) {
        var dataPointValue = this.dataSeries[seriesIndex].points[categoryIndex].value;

        if (!this.yAxisGlyph.isPercentage) {
            return dataPointValue;
        }

        var cateSum = 0;
        for (var series = 0; series < this.dataSeries.length; series++) {
            var dataSeries = this.dataSeries[series];
            cateSum = cateSum + Math.abs(dataSeries.points[categoryIndex].value);
        }

        if (cateSum === 0) {
            return 0;
        }

        return dataPointValue / cateSum;
    },

    /**
     * 判断数据点对应的shape的部分是否在鼠标画出来的矩形中
     * @param dataPoint{DataPoint} 数据点对象，里面存有该数据点对应的图形对象
     * @param drawBounds{Bounds} 鼠标画出来的矩形
     * @returns {boolean} 返回矩形是否包含图形
     */
    categoryInBounds: function (dataPoint, drawBounds) {
        return false;
    },

    /**
     * 查找鼠标位置当前的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     * @param ctx4OnMove 画布
     */
    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        if (this.xAxisGlyph && this.xAxisGlyph.isZoom && this.bounds.containsPoint(mousePos.x, mousePos.y)) {
            event._lastHover = this;
            event._draggingTarget = event._isMouseDown ? this : null;// kunsnat: 移动端不支持, 和页面拖拽冲突.
        }
        FR.Chart.CategoryPlotGlyph.superclass.findOnMoveGlyph.apply(this, [mousePos, event, ctx4OnMove]);
    },

    _findOnMoveGlyph4Line:function(mousePos, event, ctx4OnMove){

        if(this.bounds && !this.bounds.containsPoint(mousePos.x, mousePos.y)){
            return;
        }

        if (this.xAxisGlyph && this.xAxisGlyph.isZoom && this.bounds.containsPoint(mousePos.x, mousePos.y)) {
            event._lastHover = this;
            event._draggingTarget = event._isMouseDown ? this : null;// kunsnat: 移动端不支持, 和页面拖拽冲突.
        }
        FR.Chart.CategoryPlotGlyph.superclass.findOnMoveGlyph.apply(this,[mousePos, event, ctx4OnMove]);
        var bounds = this.getBounds();
        if(!event._hasfound && bounds.containsPoint(mousePos.x,mousePos.y)){
            this._resetCurrentLineSeries();
            var chosenSeries = this.dataSeries[this.currentChosenSeriesIndex];
            if (chosenSeries && chosenSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                var points = chosenSeries.points;
                var gap = Number.MAX_VALUE;
                var n = 0;
                for (var i = 0; i < points.length; i++) {
                    if(!points[i].glyph || !points[i].glyph.marker){
                        continue;
                    }
                    var temp = Math.abs(points[i].glyph.marker.x - mousePos.x);
                    if (gap > temp) {
                        gap = temp;
                        n = i;
                    }
                }
                points[n].dealTooltip(mousePos, event);
            }
        }
    },

    //重置选中的系列
    _resetCurrentLineSeries:function(){
        var index = this.currentChosenSeriesIndex;
        var chosenSeries = this.dataSeries[index];

        if(!chosenSeries || chosenSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE
                                                                    && this._isLineSeries(chosenSeries)){
            return;
        }

        for(var i = 0; i < this.dataSeries.length; i++){
            if(this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                continue;
            }
            if(this._isLineSeries(this.dataSeries[i])){
                this.currentChosenSeriesIndex = i;
                break;
            }
        }
    },


    _isLineSeries:function(dataSeries){
        if(dataSeries && dataSeries.points && dataSeries.points[0]){
            var point = dataSeries.points[0];
            return point.glyph && point.glyph.marker;
        }
        return false;
    },

    /**
     * 在plotGlyph里响应拖拽事件
     * @param mouseStartPos 拖拽开始的位置
     * @param mousePos 当前的鼠标位置
     * @param ctx4OnMove 矩形画的canvas
     */
    move4Mobile: function (mouseStartPos, mousePos, ctx4OnMove) {

        var pointX = Math.min(mouseStartPos.x, mousePos.x);
        var pointY = Math.min(mouseStartPos.y, mousePos.y);
        var width = Math.abs(mouseStartPos.x - mousePos.x);
        var height = Math.abs(mouseStartPos.y - mousePos.y);
        var drawBounds = new FR.Chart.Bounds(pointX, pointY, width, height);

        ctx4OnMove.save();
        ctx4OnMove.fillStyle = 'rgba(64,194,239, 0.5)';
        ctx4OnMove.strokeStyle = 'rgba(64,194,239, 1.0)';
        drawBounds.paint(ctx4OnMove);
        drawBounds.draw(ctx4OnMove);
        ctx4OnMove.restore();

    },


    /**
     * 拖拽结束时 响应事件
     * @param mouseStartPos 鼠标拖拽开始位置
     * @param mousePos 鼠标当前位置
     */
    onDragEnd: function (mouseStartPos, mousePos) {

        //鼠标按下或者释放的时候是否是在绘图区,不在则不支持拖拽缩放
        if(!this.bounds.containsPoint(mouseStartPos.x, mouseStartPos.y)
            || !this.bounds.containsPoint(mousePos.x, mousePos.y)){
            return ;
        }

        var pointX = Math.min(mouseStartPos.x, mousePos.x);
        var pointY = Math.min(mouseStartPos.y, mousePos.y);
        var width = Math.abs(mouseStartPos.x - mousePos.x);
        var height = Math.abs(mouseStartPos.y - mousePos.y);
        var drawBounds = new FR.Chart.Bounds(pointX, pointY, width, height);

        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var minCategory = this.getCategoryCount();
            var maxCategory = 0;
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++){
                var series = this.dataSeries[seriesIndex];
                for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
                    var dataPoint = series.points[categoryIndex];
                    if (dataPoint.glyph != null && this.categoryInBounds(dataPoint, drawBounds)) {
                        if(minCategory >series.points.length -1 ||
                            !this.categoryInBounds(series.points[minCategory], drawBounds)){
                            minCategory = categoryIndex;
                        }
                        if(!this.categoryInBounds(series.points[maxCategory], drawBounds)){
                            maxCategory = categoryIndex ;
                        }
                        minCategory = this.xAxisGlyph.getTickIndex4Value(minCategory) < this.xAxisGlyph.getTickIndex4Value(categoryIndex) ?minCategory: categoryIndex;
                        maxCategory = this.xAxisGlyph.getTickIndex4Value(maxCategory) > this.xAxisGlyph.getTickIndex4Value(categoryIndex) ?maxCategory: categoryIndex;
                    }
                }
            }

            if (minCategory < this.getCategoryCount()) {//有选中的分类
                this.xAxisGlyph.zoomBackward = this.xAxisGlyph.getOriginalCrossValue() -  this.xAxisGlyph.getTickIndex4Value(minCategory) ;
                this.xAxisGlyph.zoomForward =  this.xAxisGlyph.getTickIndex4Value(maxCategory)+1-this.xAxisGlyph.getOriginalArrowValue();
            }
        }
    },

    /**
     * 根据三维格式和深度重新调整绘图区的边界.
     */
    resetPlotBoundsWith3DDeepAndProjectionBeforeAxisGlyph: function () {
        var deepScaleBase3D = 0.1;
        this.deepBase = deepScaleBase3D * this.bounds.height;
        var deep = this.get3DDeep();

        var plotHeight = this.bounds.height;
        var plotWidth = this.bounds.width;
        var deve3DHeight = plotHeight - this.projection.projecteeRY(plotWidth, plotHeight, deep);
        var deve3DWidth = this.projection.projecteeRX(plotWidth, plotHeight, deep) - plotWidth;

        this.bounds.setRect(this.bounds.x, this.bounds.y + deve3DHeight, plotWidth - deve3DWidth, plotHeight - deve3DHeight);

        //设置一下底座的cubic
        this.threeDBase = new FR.Chart.Cubic(0, 0, this.bounds.width, this.bounds.height);
        this.threeDBase.deep = deep;
        this.threeDBase.projection = this.projection;
        this.threeDBase.covers = [FR.Chart.Cubic.BOTTOM];

        //存一下，用来区分坐标轴的画法什么的
        this.xAxisGlyph.cubic = this.threeDBase;
        this.yAxisGlyph.cubic = this.threeDBase;
        if(this.secondAxisGlyph){
            this.secondAxisGlyph.cubic = this.threeDBase;
        }
    },

    /**
     * 布局坐标轴的三维位置
     * @param categoryAxisGlyph 分类坐标轴
     * @param valueAxisGlyph 值轴.
     */
    layoutAxisGlyph4ThreeD: function (categoryAxisGlyph, valueAxisGlyph, secValueAxis) {

        this.resetPlotBoundsWith3DDeepAndProjectionBeforeAxisGlyph();

        var plotZeroBounds = this.getBounds().clone();
        var plotBoundsLeftWithoutAxis = this.getBounds();

        this.initAxisGlyphBoundsAndZeroBounds(categoryAxisGlyph, plotZeroBounds);
        this.initAxisGlyphBoundsAndZeroBounds(valueAxisGlyph, plotZeroBounds);
        this.initAxisGlyphBoundsAndZeroBounds(secValueAxis, plotZeroBounds);

        this.dealPlotBoundsWithAxisGlyph(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        if (valueAxisGlyph) {
            valueAxisGlyph.dealPlotBoundsWithAxisTitle(plotBoundsLeftWithoutAxis)
        }

        if(secValueAxis){
            valueAxisGlyph.dealPlotBoundsWithAxisTitle(plotBoundsLeftWithoutAxis);
        }

        this.threeDBase.setCubicWidthHeight(plotBoundsLeftWithoutAxis.width, plotBoundsLeftWithoutAxis.height);
        this.threeDBase.setCubicPosition(plotBoundsLeftWithoutAxis.x, plotBoundsLeftWithoutAxis.y);

        //布局值轴的标签
        if (valueAxisGlyph) {
            valueAxisGlyph.dealPlotBoundsWithLabel(plotBoundsLeftWithoutAxis);
            valueAxisGlyph.dealPlotBoundsWithAlert(plotBoundsLeftWithoutAxis);
            valueAxisGlyph.dealPlotBoundsWithLabelInHorizontal(plotBoundsLeftWithoutAxis);
        }

        if (secValueAxis) {
            secValueAxis.dealPlotBoundsWithLabel(plotBoundsLeftWithoutAxis);
            secValueAxis.dealPlotBoundsWithAlert(plotBoundsLeftWithoutAxis);
            secValueAxis.dealPlotBoundsWithLabelInHorizontal(plotBoundsLeftWithoutAxis);
        }

        var isHorizontal = !!this.isHorizontal;

        this._dealPlotBoundsBefore(isHorizontal, plotBoundsLeftWithoutAxis);
        this.calculateAxisGlyphAttr(categoryAxisGlyph, plotBoundsLeftWithoutAxis);
        this.calculateAxisGlyphAttr(valueAxisGlyph, plotBoundsLeftWithoutAxis);
        this.calculateAxisGlyphAttr(secValueAxis, plotBoundsLeftWithoutAxis);
        this._dealPlotBoundsAfter(isHorizontal, plotBoundsLeftWithoutAxis);

        this._adjustValueAxisBounds(valueAxisGlyph, true, isHorizontal);
        this._adjustValueAxisBounds(secValueAxis, false, isHorizontal);

        this.dealPlotBounds4JSDraw(this.getBounds());
    },

    _adjustValueAxisBounds:function(valueAxisGlyph, isFirstValueAxis, isHorizontal){
        if(!valueAxisGlyph){
            return ;
        }
        var points = this.threeDBase.getPoints();
        var axisBounds = valueAxisGlyph.bounds;
        var basePoint = isFirstValueAxis ? points[4] : points[5];
        if (isHorizontal) {
            axisBounds.x = basePoint.x;
            axisBounds.y = basePoint.y + this.threeDBase.height - axisBounds.height;
            valueAxisGlyph.originPoint = new FR.Chart.Point2D(axisBounds.x, axisBounds.y);
            valueAxisGlyph.axisGridLength = this.bounds.height;
        } else {
            if(valueAxisGlyph.position == FR.Chart.Constants.LEFT){
                axisBounds.x = basePoint.x;
                axisBounds.y = basePoint.y;
                axisBounds.height = this.threeDBase.height;
                valueAxisGlyph.originPoint = new FR.Chart.Point2D(axisBounds.x + axisBounds.width, axisBounds.y + axisBounds.height);
                valueAxisGlyph.axisGridLength = this.bounds.width;
            }else if(valueAxisGlyph.position == FR.Chart.Constants.RIGHT){
                axisBounds.x = basePoint.x - axisBounds.width;
                axisBounds.y = basePoint.y;
                axisBounds.height = this.threeDBase.height;
                valueAxisGlyph.originPoint = new FR.Chart.Point2D(basePoint.x - axisBounds.width, basePoint.y + axisBounds.height);
                valueAxisGlyph.axisGridLength = this.bounds.width;
            }
        }
    },

    _dealPlotBoundsBefore:function(isHorizontal, plotBoundsLeftWithoutAxis){
        if (isHorizontal) {
            var gapHeight = this.threeDBase.getGapHeight();
            var x = plotBoundsLeftWithoutAxis.x;
            var y = plotBoundsLeftWithoutAxis.y + gapHeight;
            var width = plotBoundsLeftWithoutAxis.width;
            var height = plotBoundsLeftWithoutAxis.height - 2 * gapHeight;
            plotBoundsLeftWithoutAxis.setRect(x, y, width, height);
        } else {
            var gapWidth = this.threeDBase.getGapWidth();
            var x = plotBoundsLeftWithoutAxis.x + gapWidth;
            var y = plotBoundsLeftWithoutAxis.y;
            var width = plotBoundsLeftWithoutAxis.width - 2 * gapWidth;
            var height = plotBoundsLeftWithoutAxis.height;
            plotBoundsLeftWithoutAxis.setRect(x, y, width, height);
        }
    },

    _dealPlotBoundsAfter:function(isHorizontal, plotBoundsLeftWithoutAxis){
        if (isHorizontal) {
            plotBoundsLeftWithoutAxis.width += this.threeDBase.getGapWidth();
            plotBoundsLeftWithoutAxis.height += 2 * this.threeDBase.getGapHeight();
            plotBoundsLeftWithoutAxis.y -= this.threeDBase.getGapHeight();
        } else {
            plotBoundsLeftWithoutAxis.width += 2 * this.threeDBase.getGapWidth();
            plotBoundsLeftWithoutAxis.height += this.threeDBase.getGapHeight();
            plotBoundsLeftWithoutAxis.y -= this.threeDBase.getGapHeight();
        }
    },

    /**
     * 返回3D的深度
     * @returns {*} 深度
     */
    get3DDeep: function(){
        var deep = this.deep;
        if (deep == -1) {
            deep = this.projection.projecteeDeep(this.deepBase);
        }
        return deep;
    },

    /**
     * 3D的信息,网格线，背景，底座
     * @param ctx 画布
     */
    draw3DInfo: function (ctxs) {
        this.drawInfo(ctxs);
        if (this.yAxisGlyph && this.yAxisGlyph.getBounds()) {
            this.yAxisGlyph.drawAxisGrid(ctxs[1]);
        }
        if (this.threeDBase) {
            if (this.isHorizontal) {
                this.threeDBase.draw3DBase(ctxs[1], FR.Chart.Cubic.LEFT);
            } else {
                this.threeDBase.draw3DBase(ctxs[1], FR.Chart.Cubic.BOTTOM);
            }
        }
    }

});

/**
 *
 *      @example
 *      var marker = new FR.Chart.MarkerGlyph();
 *
 * @class FR.Chart.MarkerGlyph 标记对应的图形
 * @extends FR.Chart.ChartJsInit
 */
FR.Chart.MarkerGlyph = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function () {
        this.alpha = 1.0;
        this.background = null;
        this.marker = null;
        this.x = 0;
        this.y = 0;
        this.width = 0;
        this.height = 0;
    },

    /**
     * 设置透明度
     * @param alpha 透明值
     */
    setAlpha: function (alpha) {
        this.alpha = alpha;
    },

    /**
     * 返回透明度
     * @returns {number} 透明值.
     */
    getAlpha: function () {
        return this.alpha;
    },

    /**
     * 画出标记类型
     * @param ctx 画布
     */
    draw: function (ctx) {

        if (this.marker) {
            if(!ctx){     return; } ctx.save();
            ctx.globalAlpha = this.alpha;
            if (this.drawOnMove) { //kunsnat: 默认在热点提示的时候, 空时为圆， 或者放大处理
                if (this.marker.getMarkerType() == FR.Chart.Marker.NullMarker) {
                    var roundMarker = new FR.Chart.RoundFilledMarker();
                    roundMarker.color = this.marker.color;
                    if(this.background === null||! (this.background instanceof FR.Chart.ColorBackground)){
                        roundMarker.background =FR.Chart.ColorBackground.getInstance("rgb(255,255,255)");
                    }else{
                        roundMarker.background = this.background;
                    }
                    roundMarker.size = 6.5;
                    roundMarker.paintBackgroundBiggerMarker(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);

                    roundMarker.background = this.marker.background;
                    roundMarker.size = 4.5;
                    roundMarker.paint(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);
                } else {
                    var oldLine = ctx.lineWidth;
                    ctx.lineWidth = 2;
                    var originalBackground = this.marker.background;
                    if(this.background === null||! (this.background instanceof FR.Chart.ColorBackground)){
                        this.marker.background =FR.Chart.ColorBackground.getInstance("rgb(255,255,255)");
                    }else{
                        this.marker.background = this.background;
                    }
                    this.marker.size = 8.5;
                    this.marker.paintBackgroundBiggerMarker(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);

                    this.marker.size = 6.5;
                    this.marker.background = originalBackground;
                    this.marker.paint(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);
                    ctx.lineWidth = oldLine;
                }
            } else {
                if(!FR.Chart.ChartUtils.isExcanvas()){
                    this.marker.size = 6.5;
                    if(!ctx){     return; } ctx.save();
                    ctx.globalCompositeOperation = "destination-out";
                    this.marker.paintBackgroundBiggerMarker(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);
                    ctx.restore();
                }
                this.marker.size = 4.5;
                this.marker.paint(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);
            }

            ctx.restore();
        }
    },

    /**
     * 缩略图时 donothing
     * @param ctx
     */
    draw4Demo: function (ctx) {// 缩略图的时候 不画marker.
        if (this.marker) {
//			this.marker.paint(ctx, this.x + this.width / 2.0, this.y + this.height / 2.0);
        }
    },

    /**
     * 返回 动态的步骤Glyph
     * @param step 步骤
     * @param actionModel 动态方式
     * @param lastGlyph 上一个glyph
     * @param paintState 动态方式
     * @returns {Array} 动态步骤数组
     */
    getAnimateGlyph: function (step, actionModel, lastGlyph, paintState) {
        if (paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return [];
        }
        var random = Math.random() * step;
        var glyphArray = [];

        for (var i = 0; i < step; i++) {
            // 不动
            if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_NULL) {
                glyphArray[i] = this;
            } else if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE
                && paintState == FR.Chart.ChartPaintConstants.STATE_ENABLE && lastGlyph) {

                glyphArray[i] = new FR.Chart.MarkerGlyph();
                glyphArray[i].alpha = this.alpha;
                glyphArray[i].marker = this.marker;
                glyphArray[i].width = this.width;
                glyphArray[i].height = this.height;

                glyphArray[i].x = FR.Chart.ChartEase.easeOutQuint(i + 1, lastGlyph.x, this.x - lastGlyph.x, step);
                glyphArray[i].y = FR.Chart.ChartEase.easeOutQuint(i + 1, lastGlyph.y, this.y - lastGlyph.y, step);
                // 消失
            } else if (actionModel == FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE
                && paintState == FR.Chart.ChartPaintConstants.STATE_DISAPPEAR) {

                if (i + 1 > random) {
                    glyphArray[i] = new FR.Chart.MarkerGlyph();
                } else {
                    glyphArray[i] = this;
                }
                // 出现
            } else {
                if (i + 1 > random) {
                    glyphArray[i] = this;
                } else {
                    glyphArray[i] = new FR.Chart.MarkerGlyph();
                }
            }
        }

        return glyphArray;
    },

    /**
     * 返回占用shape
     * @returns {FR.Chart.Bounds} 占用bounds
     */
    getShape: function () {
        return new FR.Chart.Bounds(this.x, this.y, this.width, this.height);
    },

    /**
     * 设置占用的bounds
     * @param bounds
     */
    setShape: function (bounds) {
        this.x = bounds.x;
        this.y = bounds.y;
        this.width = bounds.width;
        this.height = bounds.height;
    },

    /**
     * 处理条件属性
     * @param collection 条件属性
     * @param index 序号
     */
    dealCondition: function (collection, index) {
        var markerType = collection.getConditionAttr(new FR.Chart.AttrMarkerType(), index);
        this.marker = FR.Chart.MarkerFactory.createMarker(markerType.markerType);

        var attrAlpha = collection.getConditionAttr(new FR.Chart.AttrAlpha(), index);
        this.alpha = attrAlpha.alpha;
    },

    /**
     * 处理折线类的条件属性
     * @param collection 条件属性
     * @param index
     * @param colors
     */
    dealCondition4Line: function (collection, index, colors) {
        var markerType = collection.getConditionAttr(new FR.Chart.AttrMarkerType(), index);
        this.marker = FR.Chart.MarkerFactory.createMarker(markerType.markerType);

        var attrColor = collection.getConditionAttr(new FR.Chart.AttrColor(), index, colors);
        this.marker.background = FR.Chart.ColorBackground.getInstance(attrColor.seriesColor);

        var attrAlpha = collection.getConditionAttr(new FR.Chart.AttrAlpha(), index);
        this.alpha = attrAlpha.alpha;
    },

    /**
     * 处理面积图的条件属性
     * @param collection 条件属性
     * @param index
     * @param colors
     */
    dealCondition4Area: function (collection, index, colors) {
        var markerType = collection.getConditionAttr(new FR.Chart.AttrMarkerType(), index);
        this.marker = FR.Chart.MarkerFactory.createMarker(markerType.markerType);

        var attrBackground = collection.getConditionAttr(new FR.Chart.AttrBackground(), index, colors);
        this.marker.background = attrBackground.seriesBackground;

        var attrAlpha = collection.getConditionAttr(new FR.Chart.AttrAlpha(), index);
        this.alpha = attrAlpha.alpha;
    },

    getBounds2D: function () {
        return this.getShape();
    },

    isNeedRadialGradient:function(){
       return false;
    }

});

/**
 * 抽象的柱形
 * @class FR.Chart.BarPlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 * @abstract
 */
FR.Chart.BarPlotGlyph = FR.extend(FR.Chart.CategoryPlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isHorizontal = FR.pick(js.isHorizontal, false);
        this.seriesOverlapPercent = FR.pick(js.seriesOverlapPercent, -0.2);
        this.categoryIntervalPercent = FR.pick(js.categoryIntervalPercent, 1);
        this.LABEL_BAR_GAP = 3;

        FR.Chart.BarPlotGlyph.superclass.initConstants.apply(this, [js]);
    },


    isSupportLabelWrap:function(){
        return true;
    },

    configSeries:function(seriesConfig){
        this.isHorizontal = FR.pick(seriesConfig.isHorizontal, false);
        this.seriesOverlapPercent = FR.pick(seriesConfig.seriesGap, -0.2);
        this.categoryIntervalPercent = FR.pick(seriesConfig.categoryGap, 1);
        FR.Chart.BarPlotGlyph.superclass.configSeries.apply(this, [seriesConfig]);
    },


    seriesLabelsInSide:function(dataPoint){
        if(!this.conditionCollection || !dataPoint){
            return true;
        }
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(!attrContents){
            return true;
        }
        return attrContents.position === FR.Chart.Constants.INSIDE;
    },

    /**
     * 返回柱子的具体位置
     * @param dataSeries 系列
     * @param categoryPlotGlyph 当前的CategoryPlotGlyph
     * @param categoryIndex 分类序号
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     * @param barNumber 分类的柱子个数
     * @param barIndex 分类中柱子的序号
     * @returns {FR.Chart.Bounds} 返回柱子的边界.
     */
    getBarShape: function (dataSeries, categoryPlotGlyph, categoryIndex, xAxisGlyph, yAxisGlyph, barNumber, barIndex) {

        // 柱形宽度,  间隔,    起始的x点 (或者y点) 都可以知道.  只需要求一个确定的y点.(高度)

        var bar_percent = categoryPlotGlyph.getBarPercent(barNumber);
        var bar_begin_percent = categoryPlotGlyph.getBarStartPercent(bar_percent, barIndex, xAxisGlyph, categoryIndex);
        var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);

        var crossValue = 0;
        if (yAxisGlyph.isLog) {
            crossValue = 1;
        }

        var valueStart, valueLength, cateStart= xAxisGlyph.unitLength * bar_percent;
        var cateLength =  xAxisGlyph.unitLength * bar_percent *
            (this.isStockChart ? dataSeries.points[categoryIndex].stockDataPointCount : 1);

        if (this.isHorizontal) {
            if (!xAxisGlyph.axisReversed) {
                cateStart = xAxisGlyph.getPoint2D(bar_begin_percent + bar_percent).y;
            } else {
                cateStart = xAxisGlyph.getPoint2D(bar_begin_percent).y;
            }
        } else {
            if (!xAxisGlyph.axisReversed) {
                cateStart = xAxisGlyph.getPoint2D(bar_begin_percent).x;
            } else {
                cateStart = xAxisGlyph.getPoint2D(bar_begin_percent + bar_percent).x;
            }
        }
        // kunsnat: 用于判断是否0值以下, 倒立
        var isNigativeValue = false;

        if (this.isStacked) {
            if (value >= 0) {
                var pre_sum = categoryPlotGlyph.getPreSum4Bar(dataSeries.seriesIndex, categoryIndex, false);

                if (this.isHorizontal) {
                    if (yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum + value).x;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum).x;
                    }
                } else {
                    if (yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum).y;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum + value).y;
                    }
                }
                valueLength = yAxisGlyph.get2ValueLength(pre_sum, pre_sum + value);
            } else {
                var pre_sum_negative = categoryPlotGlyph.getPreSum4Bar(dataSeries.seriesIndex, categoryIndex, true);

                if (this.isHorizontal) {
                    if (yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum_negative).x;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum_negative + value).x;
                    }
                } else {
                    if (yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum_negative + value).y;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(pre_sum_negative).y;
                    }
                }
                valueLength = yAxisGlyph.get2ValueLength(pre_sum_negative + value, pre_sum_negative);
                isNigativeValue = true;
            }
        } else {
            if (value > crossValue) {
                if (this.isHorizontal) {
                    if (!yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(crossValue).x;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(value).x;
                    }
                } else {
                    if (!yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(value).y;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(crossValue).y;
                    }
                }
            } else {
                if (this.isHorizontal) {
                    if (!yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(value).x;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(crossValue).x;
                    }
                } else {
                    if (!yAxisGlyph.axisReversed) {
                        valueStart = yAxisGlyph.getPointInBounds(crossValue).y;
                    } else {
                        valueStart = yAxisGlyph.getPointInBounds(value).y;
                    }
                }
                isNigativeValue = true;
            }
            valueLength = yAxisGlyph.get2ValueLength(value, crossValue);
        }

        var bounds = new FR.Chart.Bounds();
        bounds.isHorizontal4Bounds = this.isHorizontal;
        bounds.isNigativeValue = isNigativeValue;
        //值轴逆序影响动态效果
        bounds.axisReversed = yAxisGlyph.axisReversed;

        var gap = 0.05;
        var len = gap * 2;
        if (this.isHorizontal) {// kunsnat: 不再进行取整操作, 容易偏差 bug 15253
            bounds.x = valueStart - gap;
            bounds.y = cateStart - gap;
            bounds.width = valueLength;
            bounds.height = cateLength;
        } else {
            bounds.x = cateStart - gap;
            bounds.y = valueStart - gap;
            bounds.width = cateLength + len;
            bounds.height = valueLength + len;
        }

        return bounds;
    },

    _getLabelBoundsInHorizontal: function (dim, rect, position, isNegativeDraw) {
        var labelX;
        if (isNegativeDraw) {
            switch (position) {
                case FR.Chart.Constants.INSIDE :
                {
                    labelX = rect.x + this.LABEL_BAR_GAP;
                    break;
                }
                case FR.Chart.Constants.OUTSIDE :
                {
                    labelX = rect.x - dim.width - this.LABEL_BAR_GAP;
                    break;
                }
                default  :
                {
                    labelX = rect.x + (rect.width - dim.width) / 2;
                    break;
                }
            }
        } else {
            switch (position) {
                case FR.Chart.Constants.INSIDE :
                {
                    labelX = rect.x + rect.width - dim.width - this.LABEL_BAR_GAP;
                    break;
                }
                case FR.Chart.Constants.OUTSIDE :
                {
                    labelX = rect.x + rect.width + this.LABEL_BAR_GAP;
                    break;
                }
                default  :
                {
                    labelX = rect.x + (rect.width - dim.width) / 2;
                    break;
                }
            }
        }

        return new FR.Chart.Bounds(labelX, rect.y + (rect.height - dim.height) / 2,
            dim.width, dim.height);
    },

    _getLabelBoundsInVertical: function (dim, rect, position, isNegativeDraw) {
        var labelY;
        if (isNegativeDraw) {
            switch (position) {
                case FR.Chart.Constants.INSIDE :
                {
                    labelY = rect.y + rect.height - this.LABEL_BAR_GAP - dim.height;
                    break;
                }
                case FR.Chart.Constants.OUTSIDE :
                {
                    labelY = rect.y + rect.height + this.LABEL_BAR_GAP;
                    break;
                }
                default :
                {
                    labelY = rect.y + (rect.height - dim.height) / 2.0;
                    break;
                }
            }
        } else {
            switch (position) {
                case FR.Chart.Constants.INSIDE :
                {
                    labelY = rect.y + this.LABEL_BAR_GAP;
                    break;
                }
                case FR.Chart.Constants.OUTSIDE :
                {
                    labelY = rect.y - dim.height - this.LABEL_BAR_GAP;
                    break;
                }
                default :
                {
                    labelY = rect.y + (rect.height - dim.height) / 2.0;
                    break;
                }
            }
        }

        return new FR.Chart.Bounds(rect.x + (rect.width - dim.width) / 2.0,
            labelY, dim.width, dim.height);
    },

    /**
     * 根据柱子的个数, 计算柱子的系列实际间隔百分比
     * @param seriesCount 系列个数
     * @returns {number} 间隔百分比
     */
    getBarPercent: function (seriesCount) {
        return 1.0 / (seriesCount - ((seriesCount - 1 ) * this.seriesOverlapPercent) + this.categoryIntervalPercent);
    },

    getBarStartPercent:function(bar_percent, barIndex, xAxisGlyph, categoryIndex){
        return bar_percent * (1 - this.seriesOverlapPercent) * barIndex + xAxisGlyph.getTickIndex4Value(categoryIndex) +
                                                0.5 * this.categoryIntervalPercent * bar_percent;
    },

    /**
     * 根据系列点的位置 返回系列点的标签位置
     * @param labelPreDim 标签占用大小
     * @param dataPointRect 系列点的位置
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签位置
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position, isNegativeValue) {
        var isNegativeDraw = isNegativeValue ^ this.yAxisGlyph.axisReversed;
        if (this.isHorizontal) {
            return this._getLabelBoundsInHorizontal(labelPreDim, dataPointRect, position, isNegativeDraw);
        } else {
            return this._getLabelBoundsInVertical(labelPreDim, dataPointRect, position, isNegativeDraw);
        }
    },

    /**
     * 返回 甘特特起始, 结束值 对应的分类柱形
     * @param start 起始值
     * @param end 结束值
     * @param categoryIndex 分类序号
     * @param categoryAxisGlyph 分类坐标轴
     * @param dateAxisGlyph 日期轴
     * @param barNumber 柱形个数
     * @param barIndex 柱形序号
     * @returns {FR.Chart.Bounds} 返回柱形边界.
     */
    getBarShape4GanttPlot: function (start, end, categoryIndex, categoryAxisGlyph, dateAxisGlyph, barNumber, barIndex) {
        this.seriesOverlapPercent = -0.2;
        this.categoryIntervalPercent = 0.1;
        var bar_percent = this.getBarPercent(barNumber);
        var bar_begin_percent = bar_percent * (1 - this.seriesOverlapPercent) * barIndex
            + categoryAxisGlyph.getTickIndex4Value(categoryIndex) + 0.5 * this.categoryIntervalPercent * bar_percent;

        var startDate = FR.Chart.ChartUtils.int2Date(start, FR.Chart.Constants.Second);
        var axisStart = FR.Chart.ChartUtils.date2Int(startDate, dateAxisGlyph.mainType);

        var endDate = FR.Chart.ChartUtils.int2Date(end, FR.Chart.Constants.Second);
        var axisEnd = FR.Chart.ChartUtils.date2Int(endDate, dateAxisGlyph.mainType);

        var x = dateAxisGlyph.getPointInBounds(axisStart).x;
        var y = categoryAxisGlyph.getPoint2D(bar_begin_percent).y;
        var width = dateAxisGlyph.getPointInBounds(axisEnd).x - x;
        var height = categoryAxisGlyph.unitLength * bar_percent;

        var maxBarWidth = 40;// UI规定数值.
        if(height > maxBarWidth) {
            y = y + (height - maxBarWidth)/2;
            height = maxBarWidth;
        }

        return new FR.Chart.Bounds(x, y, width, height);
    }
});

/**
 * 柱形和条形图绘图区 Bar2DPlotGlyph.start
 *
 * @class FR.Chart.Bar2DPlotGlyph
 * @extends FR.Chart.BarPlotGlyph
 *
 * @param {JSON} js配置属性
 *
 * @param {Boolean} [js.isSimulation3D=false] 是否模拟3D效果
 *
 */
FR.Chart.Bar2DPlotGlyph = FR.extend(FR.Chart.BarPlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.Bar2DPlotGlyph.superclass.initConstants.apply(this, [js]);
    },


    findClickOnGlyph:function(mousePos, event){
        if(this.bounds && this.bounds.containsPoint(mousePos.x, mousePos.y)){
            this._findNearByDataPoint(mousePos, event);
        }
    },

    /**
     *  布局系列
     */
    layoutDataSeriesGlyph: function () {
        var seriesViewCount = this.getSeriesViewCount(), seriesViewIndex = -1;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                    var dataPoint = dataSeries.points[categoryIndex];

                    dataPoint.lastGlyph = dataPoint.glyph;
                    dataPoint.glyph = null;
                }
                continue;
            }

            seriesViewIndex++;

            var adjustType = this.dealShape4Bar(dataSeries, this, this.xAxisGlyph, this.getSeriesAxisGlyph(seriesIndex),
                seriesViewCount, seriesViewIndex);
            //-1表示布局成功
            if(adjustType !== -1){
                this.dealBarShape4LargeDataStockChart(dataSeries, this, this.xAxisGlyph, this.getSeriesAxisGlyph(seriesIndex),
                                seriesViewCount, seriesViewIndex,adjustType);
            }

            if(this.isStockChart) {
                this.refreshDataPointLabel();
            }

        }
        if (this.isStacked) {
            this._setTotalRectWhenStacked();
        }
        this.dealLabelBoundsInOrder();

        this._dealBoundsGap4Unstacked();
    },

    _getBoundsGap:function(seriesInViw){
        var bar_percent = this.getBarPercent(seriesInViw);
        var cateLength = this.xAxisGlyph.unitLength * bar_percent;
        var firstColumnPercent = bar_percent * (1 - this.seriesOverlapPercent) * 0 + this.xAxisGlyph.getTickIndex4Value(0) +
            0.5 * this.categoryIntervalPercent * bar_percent;
        var secondColumnPercent = bar_percent * (1 - this.seriesOverlapPercent) * 1 + this.xAxisGlyph.getTickIndex4Value(0) +
            0.5 * this.categoryIntervalPercent * bar_percent;

        var firstStart  = 0;
        var secondStart = 0;
        if (this.isHorizontal) {
            if (this.xAxisGlyph.axisReversed) {
                firstStart = this.xAxisGlyph.getPoint2D(firstColumnPercent).y;
                secondStart = this.xAxisGlyph.getPoint2D(secondColumnPercent).y;
            } else {
                firstStart = this.xAxisGlyph.getPoint2D(firstColumnPercent + bar_percent).y;
                secondStart = this.xAxisGlyph.getPoint2D(secondColumnPercent + bar_percent).y;
            }
        } else {
            if (this.xAxisGlyph.axisReversed) {
                firstStart = this.xAxisGlyph.getPoint2D(firstColumnPercent + bar_percent).x;
                secondStart = this.xAxisGlyph.getPoint2D(secondColumnPercent + bar_percent).x;
            } else {
                firstStart = this.xAxisGlyph.getPoint2D(firstColumnPercent).x;
                secondStart = this.xAxisGlyph.getPoint2D(secondColumnPercent).x;
            }
        }
        return Math.abs(secondStart - firstStart) - cateLength;
    },

    _dealBoundsGap4Unstacked:function(){
        if(!this.isStacked){
            var seriesInViw = this.getSeriesViewCount();
            if(seriesInViw <= 1 || this.getCategoryCount() <= 0){
                return;
            }
            var bar_percent = this.getBarPercent(seriesInViw);
            var unitLength = this.xAxisGlyph.unitLength;
            var boundsGap = Math.round(this._getBoundsGap(seriesInViw));
            var cateLength = Math.round(unitLength * bar_percent);
            var totalWidth = cateLength * seriesInViw + boundsGap * (seriesInViw - 1);

            for(var cateIndex = 0; cateIndex < this.getCategoryCount(); cateIndex++) {
                var cateStart = 0;
                if (this.isHorizontal) {
                    cateStart = this.xAxisGlyph.getPoint2D(this.xAxisGlyph.getTickIndex4Value(cateIndex)).y;
                } else {
                    cateStart = this.xAxisGlyph.getPoint2D(this.xAxisGlyph.getTickIndex4Value(cateIndex)).x;
                }
                if(this.isHorizontal){
                    var startY = Math.round(cateStart - (unitLength - totalWidth) / 2);
                    if(this.xAxisGlyph.axisReversed){
                        startY = Math.round(cateStart + (unitLength - totalWidth) / 2);
                    }
                    var viewIndex = 0;
                    for(var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++){
                        if(this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                            continue;
                        }
                        var glyph = this.dataSeries[seriesIndex].points[cateIndex].glyph
                        if(glyph && glyph.shape){
                            var shape = glyph.shape;
                            if(this.xAxisGlyph.axisReversed){
                                shape.y = startY + viewIndex * (cateLength + boundsGap)
                            }else{
                                shape.y = startY - viewIndex * (cateLength + boundsGap) - cateLength;
                            }
                            shape.height = cateLength;
                        }
                        viewIndex++;
                    }
                }else{
                    var startX = Math.round(cateStart + (unitLength - totalWidth) / 2);
                    if(this.xAxisGlyph.axisReversed){
                        startX = Math.round(cateStart - (unitLength + totalWidth) / 2);
                    }
                    var viewIndex = 0;
                    for(var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++){
                        if(this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                            continue;
                        }
                        var glyph = this.dataSeries[seriesIndex].points[cateIndex].glyph;
                        if(glyph && glyph.shape){
                            var shape = glyph.shape;
                            if(this.xAxisGlyph.axisReversed){
                                shape.x = startX + (this.dataSeries.length - viewIndex - 1) * (cateLength + boundsGap);
                            }else{
                                shape.x = startX + viewIndex * (cateLength + boundsGap);
                            }
                            shape.width = cateLength;
                        }
                        viewIndex++;
                    }
                }
            }
        }
    },

    _isOutPlotBounds: function (pointBounds) {
        var plotBounds = this.getBounds();

        var px = parseInt(pointBounds.x);
        var pxwidth = parseInt(px + pointBounds.width);

        var py = parseInt(pointBounds.y);
        var pyheight = parseInt(py + pointBounds.height);

        var plotx = parseInt(plotBounds.x);
        var ploty = parseInt(plotBounds.y);

        if (px < plotx || pxwidth > plotx + plotBounds.width + 1) {
            return true;
        }

        if (py < ploty || pyheight > ploty + plotBounds.height + 1) {
            return true;
        }

        return false;
    },


    /**
     * 柱形图或者条形图的stock charts大数据量的整合调整
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类
     * @param xAxisGlyph x坐标轴
     * @param yAxisGlyph yzubiaozhou
     * @param barNumber
     * @param barIndex
     * @param adjustType
     */
    dealBarShape4LargeDataStockChart:function(dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph, barNumber, barIndex,adjustType){

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            dataSeries.points[categoryIndex].revert2Original();
        }

        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        var currentCombineDate = null;
        var combineDataPoint = null;

        var stockDataPoints = [];

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.glyph = null;
            dataPoint.lastGlyph = null;
            dataPoint.animateGlyph = null;
            if (dataPoint.dataLabel) {
                dataPoint.dataLabel.bounds = null;
            }

            if (dataPoint.valueIsNull) {
                continue;
            }

            var bounds = this.getBarShape(dataSeries, categoryPlotGlyph, categoryIndex, xAxisGlyph, yAxisGlyph, barNumber, barIndex);

            if (this._isOutPlotBounds(bounds)) {
                continue;
            }

            var cateDate = xAxisGlyph.getDate4Value(categoryIndex);

            if(stockDataPoints.length === 0){
                currentCombineDate = cateDate;
                stockDataPoints[0] = dataPoint;
                combineDataPoint = dataPoint;
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                combineDataPoint.stockDataPointCount = 1;
                continue;
            }else if(FR.Chart.ChartUtils.isBelongToSameUnit(currentCombineDate, cateDate,adjustType)){
                combineDataPoint.addDataPoint4StockCategory(dataPoint);
                combineDataPoint.stockDataPointCount++;
                dataPoint.valueIsNull = true;
                continue;
            }

            var stockIndex = this.getStockDateRangeIndex(stockDataPoints,cateDate,xAxisGlyph,adjustType);
            //是否属于之前的一个统计区间
            if(stockIndex !== -1){
                stockDataPoints[stockIndex].addDataPoint4StockCategory(dataPoint);
                dataPoint.valueIsNull = true;
                combineDataPoint.stockDataPointCount++;
            }else{ //属于新的统计区间
                stockDataPoints[stockDataPoints.length] = dataPoint;
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                currentCombineDate = cateDate;
                combineDataPoint = dataPoint;
                combineDataPoint.stockDataPointCount = 1;
            }
        }


        yAxisGlyph.oldMin = yAxisGlyph.minValue;
        yAxisGlyph.oldMax = yAxisGlyph.maxValue;
        yAxisGlyph.oldTick = yAxisGlyph.tickLength;

        yAxisGlyph.initMinMaxValue(this.getMinValueFromData(yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));
        yAxisGlyph.calculateAxisGlyph(this.bounds);
        xAxisGlyph.stockChartMainType = adjustType;


        for(var stockIndex = 0; stockIndex< stockDataPoints.length ;stockIndex++ ){
            var dataPoint = stockDataPoints[stockIndex];


            var bounds = this.getBarShape(dataSeries, categoryPlotGlyph, dataPoint.categoryIndex, xAxisGlyph, yAxisGlyph, barNumber, barIndex);

            var cateDate = xAxisGlyph.getDate4Value(dataPoint.categoryIndex);

            if (this._isOutPlotBounds(bounds)) {// kunsnat: 标签肯定要去掉.  因为超出之后 也会进行重叠等判断.
                dataPoint.glyph = null;
                dataPoint.lastGlyph = null;
                dataPoint.animateGlyph = null;
                if (dataPoint.dataLabel) {
                    dataPoint.dataLabel.bounds = null;
                }
                continue;
            }

            var tempInt = FR.Chart.ChartUtils.date2Int(cateDate, xAxisGlyph.mainType);
            if(!xAxisGlyph.stockChartMinDateInt){
                xAxisGlyph.stockChartMinDateInt = tempInt;
            }else{
                xAxisGlyph.stockChartMinDateInt = Math.min(tempInt,xAxisGlyph.stockChartMinDateInt);
            }

            var shapeGlyph = new FR.Chart.ShapeGlyph();
            dataPoint.glyph = shapeGlyph;
            shapeGlyph.setShape(bounds);
            if (this.dataSeries.length === 1) {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
                shapeGlyph.info.dealConditionWithSeriesColor(this.conditionCollection, dataPoint, this.createColors4Cate());
            } else {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            }

            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();

            var value = dataPoint.value;

            // kunsnat: 超出范围的东西 在趋势范围内计算, TODO 等待产品定位. 趋势线的计算是以所有的系列值为准计算, 还是以在绘图区范围内的为准. bug 23067
            var isDealTrend = true;
            if (this.isHorizontal) {
                if ((bounds.y < 0 && bounds.y + bounds.height < 0)) {
                    isDealTrend = false;
                }
            } else {
                if ((bounds.x < 0 && bounds.x + bounds.width < 0)
                    || (bounds.x > this.getBounds().x + this.getBounds().width)) {
                    isDealTrend = false;
                }
            }

            if (isDealTrend) {
                var flag = (value > 0) != (yAxisGlyph.axisReversed);
                if (flag) {
                    xVal[lineDataCount] = bounds.x + bounds.width / 2;
                    yVal[lineDataCount] = bounds.y;
                } else {
                    xVal[lineDataCount] = bounds.x + bounds.width / 2;
                    yVal[lineDataCount] = bounds.y + bounds.height;
                }
                lineDataCount++;
            }

            // 切入风格 渐变高亮 3D
            dataPoint.dataPointStyle = null;
            if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
                this.addChartStyle4DataPoint(this.plotStyle, dataPoint, shapeGlyph, bounds);
                shapeGlyph.dataPointStyle = dataPoint.dataPointStyle;//shapeGlyph里存下一后面动画要用
                if (dataPoint.seriesIndex == this.getLastSeriesViewIndex() && dataPoint.dataPointStyle) {
                    dataPoint.dataPointStyle.isFirst = true;
                }
            }

            this.dealDataPointLabel(dataPoint);
        }


        // 拟合计算
        if (!this.isStacked && !this.isHorizontal) {
            this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
        }
    },



    /**
     * 根据 序号, 个数等 处理相关的柱形位置
     * @param dataSeries 系列
     * @param categoryPlotGlyph 相关分类glyph
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     * @param barNumber 柱形个数
     * @param barIndex 柱形序号
     */
    dealShape4Bar: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph, barNumber, barIndex) {
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        var lastDate = null;

        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
        }

        xAxisGlyph.stockChartMainType = null;
        xAxisGlyph.stockChartMinDateInt = null;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = this.getLastDataPointGlyph(dataPoint);

            if (dataPoint.valueIsNull) {
                dataPoint.glyph = null;
                dataPoint.lastGlyph = null;
                continue;
            }

            var shapeGlyph = new FR.Chart.ShapeGlyph();
            dataPoint.glyph = shapeGlyph;

            var bounds = this.getBarShape(dataSeries, categoryPlotGlyph, categoryIndex, xAxisGlyph, yAxisGlyph, barNumber, barIndex);

            if (this._isOutPlotBounds(bounds)) {// kunsnat: 标签肯定要去掉.  因为超出之后 也会进行重叠等判断.
                dataPoint.glyph = null;
                dataPoint.lastGlyph = null;
                dataPoint.animateGlyph = null;
                if (dataPoint.dataLabel) {
                    dataPoint.dataLabel.bounds = null;
                }
                continue;
            }


            if(this.isStockChart){
                var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
                if((this.isHorizontal ? bounds.height : bounds.width) <2 && lastDate){
                    return this.getStockChartAdjustDateType(cateDate,lastDate);
                }
                lastDate = cateDate;
            }

            shapeGlyph.setShape(bounds);
            if (this.dataSeries.length === 1) {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
                shapeGlyph.info.dealConditionWithSeriesColor(this.conditionCollection, dataPoint, this.createColors4Cate());
            } else {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            }

            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();

            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);

            // kunsnat: 超出范围的东西 在趋势范围内计算, TODO 等待产品定位. 趋势线的计算是以所有的系列值为准计算, 还是以在绘图区范围内的为准. bug 23067
            var isDealTrend = true;
            if (this.isHorizontal) {
                if ((bounds.y < 0 && bounds.y + bounds.height < 0)) {
                    isDealTrend = false;
                }
            } else {
                if ((bounds.x < 0 && bounds.x + bounds.width < 0)
                    || (bounds.x > this.getBounds().x + this.getBounds().width)) {
                    isDealTrend = false;
                }
            }

            if (isDealTrend) {
                var flag = (value > 0) != (yAxisGlyph.axisReversed);
                if (flag) {
                    xVal[lineDataCount] = bounds.x + bounds.width / 2;
                    yVal[lineDataCount] = bounds.y;
                } else {
                    xVal[lineDataCount] = bounds.x + bounds.width / 2;
                    yVal[lineDataCount] = bounds.y + bounds.height;
                }
                lineDataCount++;
            }

            // 切入风格 渐变高亮 3D
            dataPoint.dataPointStyle = null;
            if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
                this.addChartStyle4DataPoint(this.plotStyle, dataPoint, shapeGlyph, bounds);
                shapeGlyph.dataPointStyle = dataPoint.dataPointStyle;//shapeGlyph里存下一后面动画要用
                if (dataPoint.seriesIndex == this.getLastSeriesViewIndex() && dataPoint.dataPointStyle) {
                    dataPoint.dataPointStyle.isFirst = true;
                }
            }

            this.dealDataPointLabel(dataPoint);
        }

        // 拟合计算
        if (!this.isStacked && !this.isHorizontal) {
            this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
        }

        return -1;
    },

    /**
     * 为系列点 添加图表相关风格
     * @param plotStyle 图表风格
     * @param dataPoint 系列点
     * @param shapeGlyph 相关图形
     * @param bounds 边界
     */
    addChartStyle4DataPoint: function (plotStyle, dataPoint, shapeGlyph, bounds) {
        if (shapeGlyph.info.getBackground().color) {
            var seriesColor = shapeGlyph.info.getBackground().color;
            var para = {'color': seriesColor, 'rect': bounds, 'avoidOriginDraw': true, 'isHorizontal': this.isHorizontal, 'axisReversed': this.yAxisGlyph.axisReversed};
            var paraOther = {'color': seriesColor, 'rect': bounds, 'avoidOriginDraw': false, 'isHorizontal': this.isHorizontal};
            if (plotStyle == FR.Chart.Constants.STYLE_3D) {
                if (this.isStacked) {
                    dataPoint.dataPointStyle = new FR.Chart.StackedBar3DStyle(para);
                } else {
                    if (this.isHorizontal) {
                        dataPoint.dataPointStyle = new FR.Chart.BarHorizontal3DStyle(para);
                    } else {
                        dataPoint.dataPointStyle = new FR.Chart.BarVertical3DStyle(para);
                    }
                }
            } else if (plotStyle == FR.Chart.Constants.STYLE_OUTER) {
                if (this.isStacked) {
                    dataPoint.dataPointStyle = new FR.Chart.StackedBarHighLightStyle(para);
                } else {
                    dataPoint.dataPointStyle = new FR.Chart.BarHighLightStyle(para);
                }
                dataPoint.dataPointStyle.plotBounds = this.bounds;
            } else if (plotStyle == FR.Chart.Constants.STYLE_SHADE) {
                if (this.isStacked) {
                    dataPoint.dataPointStyle = new FR.Chart.StackedBarTopDownShadeStyle(paraOther);
                    dataPoint.dataPointStyle.animateGlyph = shapeGlyph;
                } else {
                    dataPoint.dataPointStyle = new FR.Chart.BarTopDownShadeStyle(para);
                }

            } else if (plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
                if (this.isStacked) {
                    dataPoint.dataPointStyle = new FR.Chart.StackedBarTransparentChartStyle(para);
                } else {
                    dataPoint.dataPointStyle = new FR.Chart.BarTransparentChart(para);
                }
            }
            if(this.conditionCollection.isContainsAttr(dataPoint,new FR.Chart.AttrAlpha())){
                dataPoint.dataPointStyle.alpha = shapeGlyph.info.getAlpha();
            }
            if(this.conditionCollection.isContainsAttr(dataPoint,new FR.Chart.AttrBorder())){
                dataPoint.dataPointStyle.borderColor = shapeGlyph.info.getBorderColor();
                dataPoint.dataPointStyle.borderWidth = shapeGlyph.info.getBorderStyle();
            }
        }
    },

    /**
     * 判断数据点对应的shape的部分是否在鼠标画出来的矩形中
     * @param dataPoint{DataPoint} 数据点对象，里面存有该数据点对应的图形对象
     * @param drawBounds{Bounds} 鼠标画出来的矩形
     * @returns {boolean} 返回矩形是否包含图形
     */
    categoryInBounds: function (dataPoint, drawBounds) {
        if(!dataPoint.glyph){
            return false;
        }
        var pointBounds = dataPoint.glyph.getShape();
        if (pointBounds != null) {
            return FR.Chart.ChartUtils.rectangleOverlapped(pointBounds, drawBounds);
        }
    },

    //组合图用来设置高光风格
    setTotalRectForStackedBar:function(customPlot){
        var seriesList = customPlot.stackedList;
        for(var index = 0; index < seriesList.length; index++){
            var singleBar = seriesList[index];
            var singleBarSeries = [];
            for(var i = 0; i < singleBar.length; i++){
                var series = customPlot.dataSeries[singleBar[i]]
                singleBarSeries[i] = series;
                if(i == singleBar.length - 1){
                    this.setSeriesIsFirst(series, true);
                }else{
                    this.setSeriesIsFirst(series, false);
                }
            }

            if(singleBarSeries.length <=0){
                continue ;
            }
            for(var cateIndex = 0; cateIndex < this.getCategoryCount(); cateIndex++){
                this.getTotalRectWhenStacked(singleBarSeries, cateIndex);
            }
        }
    },

    setSeriesIsFirst:function(dataSeries, isFirst){
        for(var index = 0; index < dataSeries.getDataPointCount(); index++){
            var dataPoint = dataSeries.points[index];
            if(dataPoint.dataPointStyle){
                dataPoint.dataPointStyle.isFirst = isFirst;
            }
        }
    },

    getTotalRectWhenStacked:function(seriesList, categoryIndex){
        var x = this.bounds.x + this.bounds.width, y = this.bounds.y + this.bounds.height;
        var width = 0, height = 0;
        var topPoint = null;
        for (var seriesIndex = 0; seriesIndex < seriesList.length; seriesIndex++) {
            var dataPoint = seriesList[seriesIndex].points[categoryIndex];
            if (dataPoint.glyph) {
                var shape = dataPoint.glyph.getShape();
                x = Math.min(x, shape.x);
                y = Math.min(y, shape.y);
                if (this.isHorizontal) {
                    width += shape.width;
                    height = shape.height;
                } else {
                    width = shape.width;
                    height += shape.height;
                }
            }
        }
        var totalRect = new FR.Chart.Bounds(x, y, width, height);
        for (var index = 0; index < seriesList.length; index++) {
            var dataPoint = seriesList[index].points[categoryIndex];
            if (dataPoint && dataPoint.glyph && dataPoint.dataPointStyle) {
                dataPoint.dataPointStyle.totalRect = totalRect;
            }
        }
    },

    _setTotalRectWhenStacked: function () {
        for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
            var x = this.bounds.x + this.bounds.width, y = this.bounds.y + this.bounds.height;
            var width = 0, height = 0;
            var topPoint = null;
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
                if (dataPoint.glyph) {
                    var shape = dataPoint.glyph.getShape();
                    x = Math.min(x, shape.x);
                    y = Math.min(y, shape.y);
                    if (this.isHorizontal) {
                        width += shape.width;
                        height = shape.height;
                    } else {
                        width = shape.width;
                        height += shape.height;
                    }
                }
            }
            var totalRect = new FR.Chart.Bounds(x, y, width, height);
            for (var index = 0; index < this.dataSeries.length; index++) {
                var dataPoint = this.dataSeries[index].points[categoryIndex];
                if (dataPoint && dataPoint.glyph && dataPoint.dataPointStyle) {
                    dataPoint.dataPointStyle.totalRect = totalRect;
                }
            }
        }
    }
});

/**
 * 3D图形绘图区
 *
 *      @example
 *      var bar3d = new FR.Chart.Bar3DPlotGlyph();
 *
 * @class FR.Chart.Bar3DPlotGlyph();
 * @extends FR.Chart.BarPlotGlyph
 *
 * @param {JSON} js配置属性
 * @param {Number} [js.deep = 70] 默认的图表深度
 * @param {Number} [js.seriesIntervalPerent = 0] 默认的系列间隔百分比
 * @param {Boolean} [js.horizontalDrawBar = false] 是否横排画柱形
 * @param {FR.Chart.Projection} 三维属性
 */
FR.Chart.Bar3DPlotGlyph = FR.extend(FR.Chart.BarPlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.ZP = 70;
        this.LABEL_GAP = 8;
        this.deep = FR.pick(js.deep, -1);
        this.horizontalDrawBar = FR.pick(js.horizontalDrawBar, false);
        this.projection = new FR.Chart.Projection(js.projection);
        FR.Chart.Bar3DPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    /***
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        this.layoutAxisGlyph4ThreeD(this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph);
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        var labelPath = new FR.Chart.GeneralPath();
        var seriesViewCount = this.getSeriesViewCount(), seriesViewIndex = -1;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                    var dataPoint = dataSeries.points[categoryIndex];

                    dataPoint.lastGlyph = dataPoint.glyph;
                    dataPoint.glyph = null;
                }
                continue;
            }

            seriesViewIndex++;

            this.dealShape4CubicBar(dataSeries, this, this.xAxisGlyph, this.yAxisGlyph, seriesViewCount, seriesViewIndex);
            this.dealLabelBoundsInOrder();
        }
    },

    dealShape4CubicBar: function (dataSeries, categoryPlotGlyph, xAxisGlyph, valueAxisGlyph, barNumber, barIndex) {
        var lineDataCount = 0;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = dataPoint.glyph;

            if (dataPoint.valueIsNull) {
                dataPoint.glyph = null;
                continue;
            }

            var shapeGlyph = new FR.Chart.ShapeGlyph();
            dataPoint.glyph = shapeGlyph;

            var bounds = this.getBarShape(dataSeries, categoryPlotGlyph, categoryIndex, xAxisGlyph, valueAxisGlyph, barNumber, barIndex);
            var cubic = new FR.Chart.Cubic(bounds.x, bounds.y, bounds.width, bounds.height);
            var series_deep = categoryPlotGlyph.get3DDeep();
            var z = 0;

            var horizontalDrawBar = !!categoryPlotGlyph.horizontalDrawBar;
            var isStacked = !!categoryPlotGlyph.isStacked;

            if (!isStacked) {
                series_deep = series_deep / barNumber;
                z = - series_deep * barIndex;
            }
            if (categoryPlotGlyph.isNeedDefaultDeep()) {
                z = 0;
                series_deep = this.ZP;
            }
            cubic.horizontalDrawBar = categoryPlotGlyph.isHorizontal;
            cubic.deep = series_deep;
            cubic.projection = categoryPlotGlyph.projection;
            cubic.z = z;

            //值轴比正常的位置有一定的偏移，算出来的cubic的位置需要调整
            if (categoryPlotGlyph.isHorizontal) {
                var gapWidth = ((horizontalDrawBar || isStacked) ? cubic.getGapWidth() : categoryPlotGlyph.threeDBase.getGapWidth());
                cubic.setCubicPosition(cubic.x - gapWidth, cubic.y);
            } else {
                var gapHeight = ((horizontalDrawBar || isStacked) ? cubic.getGapHeight() : categoryPlotGlyph.threeDBase.getGapHeight());
                cubic.setCubicPosition(cubic.x, cubic.y + gapHeight);
            }
            shapeGlyph.setShape(cubic);

            if (categoryPlotGlyph.dataSeries.length === 1) {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
                shapeGlyph.info.dealConditionWithSeriesColor(this.conditionCollection, dataPoint, this.createColors4Cate());
            } else {
                shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            }
            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();

            //布局标签
            this.dealDataPointLabel(dataPoint);
        }
    },

    isNeedDefaultDeep:function(){
        return this.horizontalDrawBar || this.isStacked;
    },

    /**
     * 返回3D的深度
     * @returns {*} 深度
     */
    get3DDeep:function(){
        var deep = this.deep;
        if (deep == -1) {
            deep = this.projection.projecteeDeep(this.deepBase);
        }
        if(this.isNeedDefaultDeep()){
            return this.ZP;
        }
        return deep;
    },

    /**
     * 获取 系列的绘制顺序
     * @returns {Array} 绘制顺序数组
     */
    getDrawSequence: function () {
        if (this.horizontalDrawBar || this.isStacked) {
            return FR.Chart.Bar3DPlotGlyph.superclass.getDrawSequence.apply(this, []);
        } else {
            var seq = [];
            for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
                seq[i] = this.dataSeries.length - i - 1;
            }
            return seq;
        }
    },

    /**
     * 画出三维柱形
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        this.draw3DInfo(ctxs);
        if (this.xAxisGlyph != null) {
            this.xAxisGlyph.draw4ThreeD(ctxs[1]);
        }
        if (this.yAxisGlyph != null) {
            this.yAxisGlyph.draw4ThreeD(ctxs[1]);
        }

        FR.Chart.Bar3DPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    drawWhenStacked: function (ctx, plotGlyph, count) {
        //三维堆积柱形图，按分类从下到上画。bug：74880
        this.drawBar3DStackSeries(ctx, plotGlyph, count, plotGlyph.dataSeries);
    },

    drawWhenUnStacked: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);
        }
    },

    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        if(plotGlyph.isStacked){
          this.drawWhenStacked(ctx, plotGlyph, count);
        } else {
           this.drawWhenUnStacked(ctx, plotGlyph, count);
        }

        for (var i = plotGlyph.dataSeries.length - 1; i >= 0; i--) {
            plotGlyph.dataSeries[i].drawAnimateGlyph4TrendLine(ctx, count);
        }
    },

    /**
     * 画出背景信息
     * @param ctxs 画布层级
     */
    drawInfo: function (ctxs) {
        this.info.paint(ctxs[0], this.getOutline4Fill());
    },

    getOutline4Fill: function () {
        var basePoint = this.threeDBase.getPoints()[4];
        var axisWidth = this.yAxisGlyph.bounds.width;
        var axisHeight = this.yAxisGlyph.bounds.height;
        var width = this.threeDBase.width;
        var height = this.threeDBase.height;
        if (this.isHorizontal) {
            return new FR.Chart.Bounds(basePoint.x, basePoint.y, width, height - axisHeight)
        } else {
            return new FR.Chart.Bounds(basePoint.x + axisWidth, basePoint.y, width - axisWidth, height)
        }
    }
});

/**
 * 折线的绘图区 LinePlotGlyph.start
 *
 *      @example
 *      var linePlot = new FR.Chart.LinePlot();
 *
 * @class FR.Chart.LinePlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Boolean} [js.isCurve=false] 是否曲线连接
 */
FR.Chart.LinePlotGlyph = FR.extend(FR.Chart.CategoryPlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isCurve = js.isCurve;

        FR.Chart.LinePlotGlyph.superclass.initConstants.apply(this, [js]);

    },


    isSupportLabelWrap:function(){
        return true;
    },

    configSeries:function(seriesConfig){
        this.isCurve = FR.pick(seriesConfig.isCurve, false);
        this.isNullValueBreak = FR.pick(seriesConfig.isNullValueBreak, true);
        var defaultAttr = this.conditionCollection.defaultAttr;
        if(seriesConfig.lineStyle != null && seriesConfig.lineStyle != undefined){
            var attrLine = new FR.Chart.AttrLineStyle();
            attrLine.lineStyle = seriesConfig.lineStyle;
            defaultAttr.addAttr(attrLine);
        }

        var markerType = seriesConfig.markerType || 'NullMarker';
        var attrMarker = new FR.Chart.AttrMarkerType();
        attrMarker.markerType = markerType;
        defaultAttr.addAttr(attrMarker);
        FR.Chart.LinePlotGlyph.superclass.configSeries.apply(this, [seriesConfig]);
    },

    getLegendMarkerIcon:function(dataSeries, colors){
        return this.getLegendMarkerIcon4Line(dataSeries, colors);
    },

    dealAttr4LegendGlyph:function(legendGlyph){
        if(!legendGlyph || !legendGlyph.items){
            return;
        }

        for(var index = 0; index < this.dataSeries.length ; index++){
            if(!this.dataSeries[index].glyph || !this.dataSeries[index].glyph.lineStyleInfo || !this.dataSeries[index].glyph.lineStyleInfo.attrLineColor){
                continue;
            }
            var icon = legendGlyph.items[index].lineMarkerIcon;
            if(icon && icon.marker && icon.marker.background && icon.marker.background instanceof FR.Chart.ColorBackground) {
                icon.marker.background.color = this.dataSeries[index].glyph.lineStyleInfo.attrLineColor.seriesColor;
            }
        }
    },

    /**
     * 布局系列.
     */
    layoutDataSeriesGlyph: function () {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            dataSeries.lastGlyph = this.getLastSeriesGlyph(dataSeries);

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries.glyph = null;
                continue;
            }

            var yAxisGlyph = this.getSeriesAxisGlyph(seriesIndex);

            var adjustType = this.dealLine4PlotGlyph(dataSeries, this, this.xAxisGlyph,yAxisGlyph);//调整的类型
            //-1为成功布局
            if(adjustType !== -1){
                this.dealLine4StockChartLargeData(dataSeries, this, this.xAxisGlyph,yAxisGlyph,adjustType);
            }

            if(this.isStockChart){
                this.refreshDataPointLabel();
            }
        }

        this.dealLabelBoundsInOrder();
    },


    /**
      * 布局大数据系列,先从年往下一次，年，月，日，时，分，秒
      */
    dealLine4StockChartLargeData: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph,adjustType) {

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            dataSeries.points[categoryIndex].revert2Original();
        }

        var isSeriesCurve = this.isSeriesCurve(dataSeries);

        var currentCombineDate = null;
        var combineDataPoint = null;
        var currentCombineX = -1;

        var stockDataPoints = [];


        //调整合并
        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.valueIsNull) {
                dataPoint.glyph = null;
                continue;
            }

            var dataPointX = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x;
            if (this.isDataPointXNotInPlotBounds(dataPointX)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                dataPoint.glyph = null;
                continue;
            }

            var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
            if (stockDataPoints.length === 0) {
                currentCombineDate = cateDate;
                stockDataPoints[0] = dataPoint;
                combineDataPoint = dataPoint;
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
                continue;
            } else if (FR.Chart.ChartUtils.isBelongToSameUnit(cateDate, currentCombineDate, adjustType)) {//同一范围数值累加
                combineDataPoint.addDataPoint4StockCategory(dataPoint);
                combineDataPoint.stockDataPointCount++;
                dataPoint.valueIsNull = true;
                continue;
            }


            var stockIndex = this.getStockDateRangeIndex(stockDataPoints, cateDate, xAxisGlyph, adjustType);
            //是否属于之前的一个统计区间
            if (stockIndex !== -1) {
                stockDataPoints[stockIndex].addDataPoint4StockCategory(dataPoint);
                dataPoint.valueIsNull = true;
                combineDataPoint.stockDataPointCount++;
            } else { //属于新的统计区间
                currentCombineX = parseInt(dataPointX);
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                stockDataPoints[stockDataPoints.length] = dataPoint;
                currentCombineDate = cateDate;
                combineDataPoint = dataPoint;
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
            }
        }

        //折线图面积图stock chart调整区间内是以平均值来展示的
        var stockCount = stockDataPoints.length;
        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];
            stockPoint.value = stockPoint.value / stockPoint.stockDataPointCount;
            stockPoint.percentValue = stockPoint.percentValue / stockPoint.stockDataPointCount;
        }


        //刷新值轴
        yAxisGlyph.oldMin = yAxisGlyph.minValue;
        yAxisGlyph.oldMax = yAxisGlyph.maxValue;
        yAxisGlyph.oldTick = yAxisGlyph.tickLength;

        yAxisGlyph.initMinMaxValue(this.getMinValueFromData(yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));
        yAxisGlyph.calculateAxisGlyph(this.bounds);
        xAxisGlyph.stockChartMainType = adjustType;


        //布局图形
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;
        var isStartPoint = true;
        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }


        var plotBounds = this.getBounds();
        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];

            var point_x = this.getStockChartDatePointX(xAxisGlyph,stockPoint,adjustType,this.getBounds());
            var point_y = yAxisGlyph.getPoint2D(stockPoint.value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            lineDataCount++;

            this.deal4LinePath(isSeriesCurve,isStartPoint,linePaths,curvePaths,point_x,point_y,stockPoint);
            if(isStartPoint){
                isStartPoint = false;
            }
        }

        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
    },


    deal4LinePath:function(isSeriesCurve,isStartPoint,linePaths,curvePaths,pointX,pointY,dataPoint){
        if (isSeriesCurve){
            if (isStartPoint) {
                FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
                curvePaths.clear();

                curvePaths.moveTo(pointX, pointY);
                linePaths.moveTo(pointX, pointY);
            } else {
                curvePaths.lineTo(pointX, pointY);
            }
        } else {
            if (isStartPoint) {
                linePaths.moveTo(pointX, pointY);
            } else {
                linePaths.lineTo(pointX, pointY);
            }
        }

        if (!this.isDataPointXNotInPlotBounds(pointX)) {
            var markerGlyph = new FR.Chart.MarkerGlyph();
            markerGlyph.dealCondition4Line(this.conditionCollection, dataPoint, this.createColors4Series());
            markerGlyph.setShape(new FR.Chart.Bounds(pointX - FR.Chart.Constants.MarkerDefaultSize/2, pointY - FR.Chart.Constants.MarkerDefaultSize/2, FR.Chart.Constants.MarkerDefaultSize, FR.Chart.Constants.MarkerDefaultSize));
            markerGlyph.background = this.getBackground() !== null ? this.getBackground():this.wholeChartBackground;
            dataPoint.glyph = markerGlyph;
            markerGlyph.toolTipBackground = markerGlyph.background;

            this.dealDataPointLabel(dataPoint);
        }
    },




    /**
     * 处理折线的绘图区系列.
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类绘图区.
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     */
    dealLine4PlotGlyph: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph) {
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        xAxisGlyph.stockChartMinDateInt = null;
        xAxisGlyph.stockChartMainType = null;

        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }

        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
        }

        var isStartPoint = true;
        var isSeriesCurve = this.isSeriesCurve(dataSeries);
        var isSeriesNullValueBreak = this.isSeriesNullValueBreak(dataSeries);

        var lastPointX = -1;
        var lastDate = null;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];

            if (dataPoint.valueIsNull) {
                if (isSeriesNullValueBreak) {
                    isStartPoint = true;  //空值断开
                }
                continue;
            }
            var isZoom = xAxisGlyph.zoomBackward !== 0 || xAxisGlyph.zoomForward !== 0;
            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);
            var point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x;
            if (this.isDataPointXNotInPlotBounds(point_x)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                dataPoint.glyph = null;
                if(!isZoom || (isZoom && dataPoint.isOutBounds)){
                    dataPoint.isOutBounds  = true;
                    continue;
                }
            }

            if(this.isStockChart && lastPointX === parseInt(point_x)){
                //纪录一下是在年月日还是时分秒重叠,返回下面要用的时间单位
                var cateDate =xAxisGlyph.getDate4Value(categoryIndex);
                return this.getStockChartAdjustDateType(cateDate,lastDate);
            }else{
                lastPointX = parseInt(point_x);
            }

            if(this.isStockChart){
                lastDate = xAxisGlyph.getDate4Value(categoryIndex);
            }

            var point_y = yAxisGlyph.getPoint2D(value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            lineDataCount++;

            this.deal4LinePath(isSeriesCurve,isStartPoint,linePaths,curvePaths,point_x,point_y,dataPoint);
            if(isStartPoint){
                isStartPoint = false;
            }
        }

        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, yAxisGlyph);
        return -1;
    },

    /**
     * 根据标签位置 返回系列点的标签边界
     * @param labelPreDim 标签占用大小
     * @param dataPointRect 系列点的范围
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签边界
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return this.getDataPointLabelBoundsWithPosition4Line(labelPreDim, dataPointRect, position);
    },

    /**
     *  linePlot 动态之后补画折线Marker
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        this.drawAfterAnimate4LineMarker(ctx, currentDivKey);
        this.drawAlertLinesAfterSeriesAnimateDraw(ctx);
    },

    /**
     * 判断数据点对应的shape的部分是否在鼠标画出来的矩形中
     * @param dataPoint{DataPoint} 数据点对象，里面存有该数据点对应的图形对象
     * @param drawBounds{Bounds} 鼠标画出来的矩形
     * @returns {boolean} 返回矩形是否包含图形
     */
    categoryInBounds: function (dataPoint, drawBounds) {
        if(!dataPoint.glyph){
            return false;
        }
        var pointBounds = dataPoint.glyph.getShape();
        if (pointBounds != null) {
            //判断折线上的点是否在矩形内
            return drawBounds.containsPoint(pointBounds.x, pointBounds.y);
        }
    },

    /**
     * 寻找鼠标移动 所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件
     * @param ctx4OnMove 最上层触发的ctx图层.
     */
    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        this._findOnMoveGlyph4Line(mousePos, event, ctx4OnMove);
    }
});

/**
 * Donut2DPlotGlyph.start
 * 二维圆环图绘图区
 *
 * @class FR.Chart.Donut2DPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性.
 *
 * @param {Number} [js.innerRadiusPercent = 0.5]  内半径百分比
 * @param {Number} [js.seriesGap = 0] 系列间隔
 * @param {Number} [js.categoryGap = 0.1] 分类间隔
 */
FR.Chart.Donut2DPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.innerRadiusPercent = FR.pick(js.innerRadiusPercent, 0.5);
        this.seriesGap = FR.pick(js.seriesGap, 0.05);
        this.categoryGap = FR.pick(js.categoryGap, 0.2);
        FR.Chart.Donut2DPlotGlyph.superclass.initConstants.apply(this, [js]);

        this.MARGIN_GAP = 3.0; //上下间隔
        this.arcWidth = 0.0;
        this.middleX = 0.0;
        this.middleY = 0.0;
        this.innerRadius = 0.0;
        this.radius = 0.0;
    },

    configSeries:function(seriesConfig){
        this.innerRadiusPercent = FR.pick(seriesConfig.innerRadiusPercent, 0.5);
        this.seriesGap = FR.pick(seriesConfig.seriesGap, 0.05);
        this.categoryGap = FR.pick(seriesConfig.categoryGap, 0.2);
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        // do nothing;
    },

    layoutDataSeriesGlyph: function () {
        if(this.dataSeries.length == 0){
            return;
        }
        this.initArcValues();
        var boundsManager = new FR.Chart.BoundsManager();
        for (var categoryIndex = 0; categoryIndex < this.dataSeries[0].points.length; categoryIndex++) {
            this.dealWithDataSeries(categoryIndex, boundsManager);
        }
    },

    /**
     * 二维 三维 圆环图 初始化半径，圆心等值
     */
    initArcValues: function () {
        this.radius = Math.min(this.bounds.width, this.bounds.height) / 2 - this.MARGIN_GAP;


        if (this.plotStyle == FR.Chart.Constants.STYLE_OUTER) {
            this.outerArcRadius = this.radius;
            this.radius = this.outerArcRadius * 0.9;
        }
        var totalArcWidth = this.radius * (1.0 - this.innerRadiusPercent);
        this.arcWidth = totalArcWidth / this.dataSeries[0].points.length;
        this.innerRadius = this.radius * this.innerRadiusPercent;
        this.middleX = this.bounds.x + this.bounds.width / 2.0;
        this.middleY = this.bounds.y + this.bounds.height / 2.0;
    },

    //布局某个分类上的所有系列,包括标签
    dealWithDataSeries: function (categoryIndex, boundsManager) {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = this.getLastDataPointGlyph(dataPoint);
            if (dataPoint.value <= 0 || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataPoint.glyph = null;
                dataPoint.lastGlyph = null;
                continue;
            }
            var shapeGlyph = new FR.Chart.ShapeGlyph();
            shapeGlyph.shape = this._getRing4DataPoint(dataPoint);
            shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();
            dataPoint.glyph = shapeGlyph;

            //加载风格
            dataPoint.dataPointStyle = null;
            if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
                this.addChartStyle4DataPoint(this.plotStyle, dataPoint, shapeGlyph);
                shapeGlyph.dataPointStyle = dataPoint.dataPointStyle;//shapeGlyph里存下一后面动画要用
            }

            this.dealWithLabelBounds(dataPoint, boundsManager);
        }
    },

    addChartStyle4DataPoint: function (plotStyle, dataPoint, shapeGlyph) {
        if (shapeGlyph.info.getBackground().color) {
            var seriesColor = shapeGlyph.info.getBackground().color;
            var arcShape = shapeGlyph.shape;
            var para = {'color': seriesColor, 'arcShape': arcShape, 'avoidOriginDraw': true, 'totalRadius': this.radius - this.arcWidth * this.categoryGap / 2, 'count': this.getCategoryCount()};
            if (plotStyle == FR.Chart.Constants.STYLE_3D) {
                if (!this.arc3DStyleBase) { //平面3d的风格时有底座
                    this.arc3DStyleBase = new FR.Chart.Arc3DStyleBase(this.get3DStyleBaseShape());
                }
                dataPoint.dataPointStyle = new FR.Chart.ArcPlane3DStyle(para);
            } else if (plotStyle == FR.Chart.Constants.STYLE_OUTER) {
                if(FR.Browser.isIE8Before()){
                    dataPoint.dataPointStyle = null;//产品:ie678屏蔽高光效果
                }else{
                    if (!this.arcOuterBase) { //高光的外环
                        this.arcOuterBase = new FR.Chart.ArcOuterBase(this.outerArcRadius - this.arcWidth * this.categoryGap / 2, new FR.Chart.Point2D(this.middleX, this.middleY), this.innerRadius + this.arcWidth * this.categoryGap / 2);
                    }
                    dataPoint.dataPointStyle = new FR.Chart.DonutOuterStyle(para);
                }
            } else if (plotStyle == FR.Chart.Constants.STYLE_SHADE) {
                dataPoint.dataPointStyle = new FR.Chart.ArcTopDownShadeStyle(para);
            } else if (plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
                dataPoint.dataPointStyle = new FR.Chart.ArcTransparentChartStyle(para);
            }
        }
    },

    /**
     * 系列的动态 画法. 画法顺序可能不同. 画法结束 也可能不同.
     * @param ctx 画布
     * @param plotGlyph 绘图区Glyph
     * @param count 步骤count Index
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);
        }

        //平面3D的时候再画一遍
        if(this.plotStyle == FR.Chart.Constants.STYLE_3D){
            for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
                plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);
            }
        }
    },

    get3DStyleBaseShape: function () {
        var outerRadius = this.radius * (1 - this.innerRadiusPercent / 7);//7是产品规定的
        var innerRadius = outerRadius - this.radius * (1 - this.innerRadiusPercent);
        return new FR.Chart.Ring(this.middleX, this.middleY, outerRadius, 0, 360, innerRadius, 0, 360);
    },

    /**
     * 布局标签位置
     * @param dataPoint 系列点
     * @param boundsManager 标签位置管理器.
     */
    dealWithLabelBounds: function (dataPoint, boundsManager) {
        var textGlyph = dataPoint.dataLabel;
        if (textGlyph == null || textGlyph.text == null || textGlyph.text.length === 0) {
            return;
        }

        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;
        var dim = this.getLabelDim(dataPoint);
        var startAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex, false) % 360;
        var extent = this.getPercent(seriesIndex, categoryIndex) * (360 - this._getSeriesGapAngle(categoryIndex, seriesIndex, false));
        var endAngle = extent / 2 + startAngle;
        var midRadius = this.getArcInnerRadius(categoryIndex) + this.arcWidth / 2;
        var pointX = this.middleX - midRadius * Math.cos((endAngle / 180) * Math.PI);
        var pointY = this.middleY - midRadius * Math.sin((endAngle / 180) * Math.PI);
        var bounds = new FR.Chart.Bounds(pointX - dim.width / 2, pointY - dim.height / 2, dim.width, dim.height);
        textGlyph.bounds = bounds;

        //对标签重叠的情况处理进行处理
        if (boundsManager.isInteract(bounds)) {
            for (var time = 0; time < 6; time++) {
                var coeff = parseInt(time / 2) + 1;
                if (time % 2 === 0) {
                    coeff = 4 - coeff;
                } else {
                    coeff = 4 + coeff;
                }
                endAngle = (startAngle + coeff * extent / 8) % 360;
                pointX = pointX = this.middleX - midRadius * Math.cos((endAngle / 180) * Math.PI);
                pointY = this.middleY - midRadius * Math.sin((endAngle / 180) * Math.PI);
                bounds = bounds = new FR.Chart.Bounds(pointX - dim.width / 2, pointY - dim.height / 2, dim.width, dim.height);
                if (!boundsManager.isInteract(bounds)) {
                    textGlyph.bounds = bounds;
                    boundsManager.addBounds(bounds);
                    break;
                }
            }
            if (time === 6) {
                textGlyph.bounds = null;
            }
        } else {
            textGlyph.bounds = bounds;
            boundsManager.addBounds(bounds);
        }
    },

    getLabelDim: function (dataPoint) {
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        var textAttr = attrContents.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        return FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(dataPoint.dataLabel.text, textAttr);
    },

    /**
     * 返回系列起始的角度
     * @param categoryIndex 分类序号
     * @param seriesIndex 系列序号
     * @param isInner 是否为内系列
     * @returns {number} 角度值
     */
    getSeriesStartAngle: function (categoryIndex, seriesIndex, isInner) {
        var startAngle = 90;
        var gapAngle = this._getSeriesGapAngle(categoryIndex, seriesIndex, isInner);
        var seriesToSee = 0;
        var coSeries = [];
        for (var index = 0; index < this.dataSeries.length; index++) {
            var series = this.dataSeries[index];
            if (series.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                coSeries[index] = seriesToSee++;
            }
        }
        var eachGap = gapAngle / (this.getSeriesViewCount());
        var seriesAngle = 0;
        for (var i = 0; i < seriesIndex; i++) {
            seriesAngle += this.getPercent(i, categoryIndex) * (360 - gapAngle);
        }
        startAngle += seriesAngle + eachGap * coSeries[seriesIndex] + eachGap / 2.0;
        return startAngle;
    },

    _getSeriesGapAngle: function (categoryIndex, seriesIndex, isInner) {
        var seriesGap = this.seriesGap;
        if (this._getNonzeroSeriesCount(categoryIndex) <= 1) {
            seriesGap = 0;
        }
        var totalSeriesGap = seriesGap * 2 * Math.PI * this.innerRadius; //总弧长
        var arcRadius = isInner ? this.getArcInnerRadius(categoryIndex) : this.getActOuterRadius(categoryIndex);
        return (totalSeriesGap / arcRadius) * 360 / (2 * Math.PI); //总的间隔的角度
    },

    _getNonzeroSeriesCount:function(categoryIndex){
        var count = 0;
        for(var i = 0, seriesSize = this.dataSeries.length; i < seriesSize; i++){
            var series = this.dataSeries[i];
            if(series.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                continue;
            }

            var dataPoint = series.points[categoryIndex];
            if(dataPoint && dataPoint.value > 0){
                count++;
            }
        }
        return count;
    },

    getPercent: function (seriesIndex, categoryIndex) {
        if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return 0;
        }
        var totalValue = this.getTotalValue(categoryIndex);
        if (totalValue <= 0) {
            return 0;
        }
        var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
        return Math.abs(dataPoint.value) / totalValue;
    },

    getTotalValue: function (categoryIndex) {
        var totalValue = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if (!dataPoint || dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            totalValue += Math.abs(dataPoint.value);
        }
        return totalValue;
    },

    /**
     * 返回系列点的圆环 某个系列点的图形对象
     * @param dataPoint 系列点
     * @returns {FR.Chart.Ring}
     */
    _getRing4DataPoint: function (dataPoint) {
        var categoryIndex = dataPoint.categoryIndex;
        var seriesIndex = dataPoint.seriesIndex;

        var outerRadius = this.getActOuterRadius(categoryIndex);
        var outerStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex, false) % 360;
        var outerExtent = this.getPercent(seriesIndex, categoryIndex) * (360 - this._getSeriesGapAngle(categoryIndex, seriesIndex, false));

        var innerRadius = this.getArcInnerRadius(categoryIndex);
        var innerStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex, true) % 360;
        var innerExtent = this.getPercent(seriesIndex, categoryIndex) * (360 - this._getSeriesGapAngle(categoryIndex, seriesIndex, true));

        return new FR.Chart.Ring(this.middleX, this.middleY, outerRadius, outerStartAngle, outerExtent, innerRadius, innerStartAngle, innerExtent);
    },

    getArcInnerRadius: function (categoryIndex) {
        var gap = this.arcWidth * this.categoryGap / 2;
        return this.innerRadius + this.arcWidth * categoryIndex + gap;
    },

    getActOuterRadius: function (categoryIndex) {
        var gap = this.arcWidth * this.categoryGap / 2;
        return this.innerRadius + this.arcWidth * (categoryIndex + 1) - gap;
    },

    draw: function (ctxs, time, actionModel) {
        this.drawInfo(ctxs);

        FR.Chart.Donut2DPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    }
});

/**
 * Donut3DPlotGlyph.start
 * 三维圆环图绘图区
 *
 *      @example
 *      var 3ddonut = new FR.Chart.Donut3DPlotGlyph();
 *
 * @class FR.Chart.Donut3DPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.innerRadiusPercent = 0.2] 内半径比例
 * @param {Number} [js.seriesGap = 0] 系列间隔
 * @param {Number} [js.categoryGap = 0.1] 分类间隔
 * @param {Number} [js.deep = 0] 默认三维深度
 * @param {FR.Chart.Project} [js.projection] 三维属性
 */
FR.Chart.Donut3DPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.innerRadiusPercent = FR.pick(js.innerRadiusPercent, 0.5);
        this.seriesGap = FR.pick(js.seriesGap, 0.05);
        this.categoryGap = FR.pick(js.categoryGap, 0.2);
        this.projection = new FR.Chart.Projection(js.projection || {elevation:33, rotation:0});

        FR.Chart.Donut3DPlotGlyph.superclass.initConstants.apply(this, [js]);

        this.ring4cate = [];
        this.MARGIN_GAP = 3.0; //上下间隔
        this.arcWidth = 0.0;
        this.middleX = 0.0;
        this.middleY = 0.0;
        this.innerRadius = 0.0;
        this.radius = 0.0;
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        // do nothing;
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        this.initArcValues();
        this.ring4cate = [];
        for (var categoryIndex = 0, categoryCount = this.getCategoryCount(); categoryIndex < categoryCount; categoryIndex++) {
            var ring3D = new FR.Chart.Ring3D(this.projection);
            this.dealWithDataSeries(categoryIndex, ring3D);
            this.ring4cate.push(ring3D);
        }
    },

    initArcValues: function () {
        this.radius = Math.min(this.bounds.width, this.bounds.height) / 2 - this.MARGIN_GAP;
        var totalArcWidth = this.radius * (1.0 - this.innerRadiusPercent);
        this.arcWidth = totalArcWidth / this.dataSeries[0].points.length;
        this.innerRadius = this.radius * this.innerRadiusPercent;
        this.middleX = this.bounds.x + this.bounds.width / 2.0;
        this.middleY = this.bounds.y + this.bounds.height / 2.0;
    },

    dealWithDataSeries: function (categoryIndex, ring3D) {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = dataPoint.glyph;
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {

                continue;
            }
            this.calculateShapeGlyph4DifferentSub(dataPoint, ring3D, categoryIndex, seriesIndex);
        }
    },

    calculateShapeGlyph4DifferentSub: function (dataPoint, ring3D, categoryIndex, seriesIndex) {
        var innerRadius = this.getArcInnerRadius(categoryIndex);
        var outerRadius = this.getActOuterRadius(categoryIndex);

        var startAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var extent = this.getPercent(seriesIndex, categoryIndex) * 360 * (1 - this.seriesGap);

        if(this.getSeriesViewCount() === 1 || this.getPercent(seriesIndex, categoryIndex) === 1){
            startAngle = 0;
            extent = 360;
        }

        var depth = this.radius * 0.16;
        var outerCylinder = new FR.Chart.Pie3D(0, depth, outerRadius);
        outerCylinder.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
        var innerCylinder = new FR.Chart.Pie3D(0, depth, innerRadius);
        innerCylinder.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());

        outerCylinder.setAttribute(this.middleX, this.middleY, startAngle, extent, this.projection);
        innerCylinder.setAttribute(this.middleX, this.middleY, startAngle, extent, this.projection);

        var ring = new FR.Chart.Ring3D(this.projection);
        ring.addCylinder(outerCylinder, innerCylinder);
        ring3D.addCylinder(outerCylinder, innerCylinder);

        var shapeGlyph = new FR.Chart.ShapeGlyph();
        shapeGlyph.shape = ring;
        shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
        shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();
        dataPoint.glyph = shapeGlyph;
        this.dealDataPointLabel(dataPoint)
    },

    /**
     * 处理系列点标签
     * @param dataPoint 系列点
     */
    dealDataPointLabel: function (dataPoint) {
        var labelGlyph = dataPoint.dataLabel;
        if (!labelGlyph) {
            return;
        }

        var centerPoint = dataPoint.getShape().getTopArcCenterPoint();
        if (centerPoint != null) {
            var dim = this.getLabelDim(dataPoint);
            var tg_width = dim.width;
            var tg_height = dim.height;
            var labelX = centerPoint.x - tg_width / 2.0;
            var labelY = centerPoint.y - tg_height / 2.0;
            labelGlyph.bounds = new FR.Chart.Bounds(labelX, labelY, tg_width, tg_height);
        } else {
            labelGlyph.bounds = null;
        }
    },

    getLabelDim: function (dataPoint) {
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        var textAttr = attrContents.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        return FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(dataPoint.dataLabel.text, textAttr);
    },

    getArcInnerRadius: function (categoryIndex) {
        var gap = this.arcWidth * this.categoryGap / 2;
        return this.innerRadius + this.arcWidth * categoryIndex + gap;
    },

    getActOuterRadius: function (categoryIndex) {
        var gap = this.arcWidth * this.categoryGap / 2;
        return this.innerRadius + this.arcWidth * (categoryIndex + 1) - gap;
    },

    getPercent: function (seriesIndex, categoryIndex) {
        if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return 0;
        }
        var totalValue = this.getTotalValue(categoryIndex);
        if (totalValue <= 0) {
            return 0;
        }
        var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
        return Math.abs(dataPoint.value) / totalValue;
    },

    getTotalValue: function (categoryIndex) {
        var totalValue = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            totalValue += Math.abs(dataPoint.value);
        }
        return totalValue;
    },

    getSeriesStartAngle: function (categoryIndex, seriesIndex) {
        var startAngle = 90;
        var gapAngle = 360 * this.seriesGap;
        var seriesToSee = 0;
        var coSeries = [];
        for (var index = 0; index < this.dataSeries.length; index++) {
            var series = this.dataSeries[index];
            if (series.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                coSeries[index] = seriesToSee++;
            }
        }
        var eachGap = gapAngle / (seriesToSee);
        var seriesAngle = 0;
        for (var i = 0; i < seriesIndex; i++) {
            seriesAngle += this.getPercent(i, categoryIndex) * (360 - gapAngle);
        }
        startAngle += seriesAngle + eachGap * coSeries[seriesIndex] + eachGap / 2.0;
        return startAngle;
    },

    draw: function (ctxs, time, actionModel) {
        //3D圆环图应该在normalctx中绘画，而不是animatiectx
        var ctx = ctxs[1];

        this.drawInfo(ctxs);

        for (var i = this.ring4cate.length - 1; i >= 0; i--) {
            this._drawBackHalf(i, ctx);
        }

        for (var i = 0; i < this.ring4cate.length; i++) {
            this._drawFrontHalf(i, ctx);
        }

        var currentDivKey = FR.Chart.currentDivKey;
        this.drawMarkersLabelAfterSeriesAnimateDraw(ctx, currentDivKey);

        FR.Chart.GraphHelper.isDrawingEnd = true;
    },

    _drawBackHalf: function (categoryIndex, ctx) {
        var backRing = this.ring4cate[categoryIndex].calculateBackHalf();
        backRing.drawSide(ctx);
        backRing.drawBack(ctx);
        backRing.drawPositive(ctx);
        backRing.drawTop(ctx);
    },

    _drawFrontHalf: function (categoryIndex, ctx) {
        var frontRing = this.ring4cate[categoryIndex].calculateFrontHalf();
        frontRing.drawSide(ctx);
        frontRing.drawBack(ctx);
        frontRing.drawPositive(ctx);
        frontRing.drawTop(ctx);
    }
});

/**
 * PiePlotGlyph.start
 * 饼图绘图区
 *
 *      @example
 *      var pie = new FR.Chart.PiePlotGlyph();
 *
 * @class FR.Chart.PiePlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 * @property {Number}[this.separatePercent = 10] 默认扇形弹出百分比
 * @property {Number}[this.radius = 0] 饼图圆形的半径
 * @property {Number}{this.maxLabelWidth = 0} 最大的标签宽度
 * @property {Array}[this.leftUp = []] 密集情况时, 左上角标签集合.
 * @property {Array}[this.leftDown =[]] 密集情况时, 左下角标签集合.
 * @property {Array}[this.rightUp = []] 密集情况时, 右上角标签集合.
 * @property {Array}[this.rightDown=[]] 密集情况时, 右下角标签集合.
 *
 *
 */

FR.Chart.PiePlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.PiePlotGlyph.superclass.initConstants.apply(this, [js]);

        this.MAX_SERIES_MODE = 5;
        this.OUTSIDE_LABEL_RADIUS_COEFFICIENT = 1.3;

        this.separatePercent = 10;// 默认扇形弹出百分比.

        //以下是计算中用到的变量
        this.radius = 0;
        this.maxLabelWidth = 0;
        this.radiusPercent = 0.9;

        this.leftUp = [];
        this.leftDown = [];
        this.rightUp = [];
        this.rightDown = [];

        this.adjustAngle = 0.1;
    },


    seriesLabelsInSide:function(dataPoint){
        if(!this.conditionCollection || !dataPoint){
            return true;
        }
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(!attrContents){
            return true;
        }
        return attrContents.position === FR.Chart.Constants.INSIDE;
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        for(var i = 0; i < data.length; i++){
            var singleData = data[i];
            var sery = new FR.Chart[plotTypes.dataSeries]();
            sery.points = [];
            sery.seriesIndex = i;
            sery.seriesName = singleData[0];

            var dataPoint = new FR.Chart[plotTypes.dataPoint]();
            dataPoint.categoryIndex = 0;
            dataPoint.seriesIndex = i;
            dataPoint.seriesName = singleData[0];
            dataPoint.value = singleData[1];
            sery.points[0] = dataPoint;
            this.dataSeries[i] = sery;
        }
    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        var seriesName = para[0];
        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            if(FR.equals(seriesName, series.seriesName) && series.points[0]){
                return series.points[0];
            }
        }
    },

    /**
     * 坐标轴布局
     */
    layoutAxisGlyph: function () {
        // do nothing;
    },

    isExistOuterLabels : function (){
        return this.leftUp.length>0 || this.leftDown.length>0 || this.rightUp.length>0 || this.rightDown.length>0;
    },


    /**
     * 系列的动态 画法. 画法顺序可能不同. 画法结束 也可能不同.
     * @param ctx 画布
     * @param plotGlyph 绘图区Glyph
     * @param count 步骤count Index
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);
        }

        //先画默认的边框
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            var dataSeries = plotGlyph.dataSeries[seq[i]];
            var dataPoints = dataSeries.points;
            for (var j = 0; j < dataSeries.getDataPointCount(); j++) {
                if(dataPoints[j].isBorderDefault(count)){
                    dataPoints[j].drawOnlyBorder(ctx, count);
                }
            }

        }

        //再画条件属性设置的边框
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            var dataSeries = plotGlyph.dataSeries[seq[i]];
            var dataPoints = dataSeries.points;
            for (var j = 0; j < dataSeries.getDataPointCount(); j++) {
                if(!dataPoints[j].isBorderDefault(count)){
                    dataPoints[j].drawOnlyBorder(ctx, count);
                }
            }

        }

    },

    /**
     * 系列布局
     * 包括: 重新生成计算之后的百分比, 只是标签.
     */
    layoutDataSeriesGlyph: function () {
        for (var categoryIndex = 0; this.dataSeries[0] && categoryIndex < this.dataSeries[0].points.length; categoryIndex++) {
            this.dealEachCategory(categoryIndex);
        }
        this.dealLabelBoundsInOrder();// 主要针对数据标签内的标签
    },

    dealLabelBoundsOutPlotBounds:function(dataPoint4Pie,labelBounds,plotBounds,orderList,orderListIndex,boundsManager){
        if(!dataPoint4Pie.isShowLabel || dataPoint4Pie.dataLabel === null ||
            dataPoint4Pie.dataLabel.text ===null || dataPoint4Pie.dataLabel.text.length <=0){
            return ;
        }
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint4Pie);
        if(!attrContents.isShowGuidLine){//内部显示的
             FR.Chart.PiePlotGlyph.superclass.dealLabelBoundsOutPlotBounds.apply(this, [dataPoint4Pie,labelBounds,plotBounds,orderList,orderListIndex,boundsManager]);
        }
        if(labelBounds.x>=plotBounds.x && labelBounds.x + labelBounds.width <= plotBounds.x+plotBounds.width) {
            return ;
        }

        var width = labelBounds.width;
        if(labelBounds.x<plotBounds.x){
            width = labelBounds.x+labelBounds.width - plotBounds.x;
        }else if(labelBounds.x + labelBounds.width > plotBounds.x+plotBounds.width){
            width =(plotBounds.x+plotBounds.width) - labelBounds.x;
        }

        var text =FR.Chart.TextUtils.changeAllReline2Normal(dataPoint4Pie.dataLabel.text);
        text = text.split('\n') ;
        var textAttr = attrContents.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        var result = [];
        for (var relineIndex = 0; relineIndex < text.length; relineIndex++) {
            var singleText = text[relineIndex];
            var index = 0;
            for(; index < singleText.length ;index ++){
                var tmp = singleText.substring(0,index)+"...";
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(tmp, textAttr);
                if(dim.width>width){
                    break;
                }
            }
            if(index === 0){
                result[relineIndex] = "...";
            }else if(index == singleText.length){
                result[relineIndex] = singleText;
            }else{
                result[relineIndex] = singleText.substring(0,index-1)+"...";
            }
        }
        dataPoint4Pie.showLabel = "";
        for(var i = 0;i<result.length ;i++){
            if(i == result.length -1){
                dataPoint4Pie.showLabel +=result[i]
            }else{
                dataPoint4Pie.showLabel +=result[i]+'\n';
            }
        }

        var usedDim =  FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(dataPoint4Pie.showLabel, textAttr);
        if(labelBounds.x<plotBounds.x){
            labelBounds.setRect(plotBounds.x,labelBounds.y,usedDim.width,labelBounds.height);
        }else{
            labelBounds.setRect(labelBounds.x,labelBounds.y,usedDim.width,labelBounds.height);
        }

        dataPoint4Pie.dataLabel.bounds = labelBounds;
    },


    dealEachCategory:function(categoryIndex){
        this.initLabelData(categoryIndex);
        this.dealWithDataSeries(categoryIndex);
    },

    /**
     * 初始化标签 数据 最大的标签值.
     * @param categoryIndex
     */
    initLabelData: function (categoryIndex) {
        this.radius = this.getPieDimension(categoryIndex) / ( 2 + 2 * this.separatePercent * 0.01);
        if (this.plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
            this.radius -= 4;
        }
        this.maxLabelWidth = this.getMaxLabelWidth(categoryIndex);
        this.getSeriesCountWithQuadrant(categoryIndex);
    },

    /**
     * 根据分类 处理系列
     * @param categoryIndex 分类序号
     */
    dealWithDataSeries: function (categoryIndex) {
        this.arc3DStyleBase = null;
        this.arcOuterBase = null;
        var boundsPath = new FR.Chart.BoundsManager();
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = this.getLastDataPointGlyph(dataPoint);
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataPoint.glyph = null;
                dataPoint.lastGlyph = null;
                continue;
            }
            dataPoint.glyph = this.calculateShapeGlyph4DifferentSub(dataPoint);

            if (!this.subType) {
                // 判断加载何种风格
                dataPoint.dataPointStyle = null;
                if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
                    var shapeGlyph = dataPoint.glyph;
                    this.addChartStyle4DataPoint(this.plotStyle, dataPoint, shapeGlyph);
                    shapeGlyph.dataPointStyle = dataPoint.dataPointStyle;//shapeGlyph里存下一后面动画要用
                }
            }
        }
        this.dealWithCategoryLabels(categoryIndex,boundsPath);
    },

    dealWithCategoryLabels:function(categoryIndex,boundsPath){
        if (this.isExistOuterLabels()) {
            this.dealOutSideLabels(categoryIndex);
            this.dealLabelBoundsOverLay(this.leftUp,true,90,180);
            this.dealLabelBoundsOverLay(this.leftDown,true,180,270);
            this.dealLabelBoundsOverLay(this.rightUp,false,0,90);
            this.dealLabelBoundsOverLay(this.rightDown,false,270,360);
        }
        this.dealInsideLabels(categoryIndex,boundsPath);
    },


    dealOutSideLabels:function(categoryIndex){
        var maxDiff = 0;
        maxDiff = Math.max(maxDiff,this.dealEachArea(this.leftUp));
        maxDiff = Math.max(maxDiff,this.dealEachArea(this.leftDown));
        maxDiff = Math.max(maxDiff,this.dealEachArea(this.rightUp));
        maxDiff = Math.max(maxDiff,this.dealEachArea(this.rightDown));
        if(maxDiff === 0){
            return;
        }
        this.radiusPercent -= 0.05;
        if(this.radiusPercent >= 0.5){
             this.dealEachCategory(categoryIndex);
        }else{
             this.radiusPercent += 0.05;
        }

    } ,

    isLabelBoundsOutPlotBounds:function(labelBounds,minAngle,maxAngle){
        if(labelBounds == null){
            return false;
        }
        var gap = 3;
        var centerY = this.getBounds().height / 2 + this.getBounds().y;
        var maxY = this.getBounds().height + this.getBounds().y;
        if(minAngle >= 180 && maxAngle <= 360){
            return  labelBounds.y < centerY+gap || labelBounds.y+labelBounds.height +gap > this.bounds.y+this.bounds.height;
        }else{
            return labelBounds.y < this.bounds.y-gap || labelBounds.y+labelBounds.height> centerY;
        }
    },

    //处理标签重叠
    dealLabelBoundsOverLay:function(pointsArray,isLeft,minAngle,maxAngle){
        var boundsPath = new FR.Chart.GeneralPath();
        var isSuccess = true;
        for(var i = 0;i<pointsArray.length ; i++){
            var dataPoint =  isLeft ?pointsArray[i]:pointsArray[pointsArray.length-1-i];
            var textGlyph = dataPoint.dataLabel;
            if(textGlyph == null || textGlyph.text == null || textGlyph.text.length === 0){
                continue;
            }
            if(!dataPoint.isShowLabel){
                continue;
            }
            var labelBounds = new FR.Chart.Bounds(textGlyph.bounds.x,
                textGlyph.bounds.y,textGlyph.bounds.width,textGlyph.bounds.height);
            if(boundsPath.verticalIntersects(labelBounds.y,labelBounds.height)
                || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)) {
                if(!this.adjustLabelBoundsY(isLeft,boundsPath,dataPoint,labelBounds,minAngle,maxAngle)){
                    isSuccess = false;
                }
            } else{
                boundsPath.append(labelBounds,false);
            }
        }
        if(isSuccess){
            return ;
        }
        //没有成功说明往下调了之后超出角度范围，则需要往上调
        this.moveLabelUp(pointsArray,isLeft,minAngle,maxAngle);
    },


    moveLabelUp:function(pointsArray,isLeft,minAngle,maxAngle){
        var boundsPath = new FR.Chart.GeneralPath();
        var centerAngle = isLeft ? maxAngle : minAngle;
        for(var i = 0;i<pointsArray.length ; i++){
            var dataPoint =  isLeft ?pointsArray[pointsArray.length-1-i]:pointsArray[i];
            var textGlyph = dataPoint.dataLabel;
            if(textGlyph == null || textGlyph.text == null || textGlyph.text.length === 0){
                continue;
            }
            if(!dataPoint.isShowLabel){
                continue;
            }
            var labelBounds =dataPoint.dataLabel.bounds;
            var isNeedMove =!this.isLabelBoundsInRange(dataPoint,labelBounds,minAngle,maxAngle)
                   || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle);
            while(!this.isLabelBoundsInRange(dataPoint,labelBounds,minAngle,maxAngle)
                   || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)){
                centerAngle += (isLeft ? -1 : 1)*this.adjustAngle;
                this.calculateLabelBoundsWithAngle(centerAngle,dataPoint,labelBounds);
                if(centerAngle < minAngle || centerAngle > maxAngle){
                    break;
                }
            }
            if(isNeedMove){
                dataPoint.dataLabel.bounds = labelBounds;
                var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
                if(attrContents.isShowGuidLine){
                    this.relayLabelLine(dataPoint,labelBounds,centerAngle);
                }
            }

            if(boundsPath.verticalIntersects(labelBounds.y,labelBounds.height)) {
                centerAngle = this.adjustLabel4MoveUp(centerAngle,!isLeft,boundsPath,dataPoint,labelBounds,minAngle,maxAngle)
            } else{
                boundsPath.append(labelBounds,false);
            }
        }
    } ,

       //标签是否再各自范围内
   isLabelBoundsInRange:function(dataPoint,labelBounds,minAngle,maxAngle){
       var gap = 3;
        var centerPoint = this.getCenterPoint(dataPoint.seriesIndex,dataPoint.categoryIndex);
       var lessThanMaxY = labelBounds.y + labelBounds.height <= this.bounds.y+this.bounds.height;
       var lessThanCenterY = labelBounds.y+labelBounds.height<=centerPoint.y-gap;
       var greaterThanMinY = labelBounds.y + gap >= this.bounds.y;
       var greaterThanCenterY = labelBounds.y >= centerPoint.y+ gap;

        if(minAngle === 0 && maxAngle === 90){
            return greaterThanMinY && lessThanCenterY && labelBounds.x >= centerPoint.x;
        }else if(minAngle === 90 && maxAngle === 180){
            return labelBounds.x +labelBounds.width <= centerPoint.x &&
                lessThanCenterY && greaterThanMinY ;
        }else if(minAngle === 180 && maxAngle === 270){
            return greaterThanCenterY &&  lessThanMaxY && labelBounds.x +labelBounds.width <= centerPoint.x ;
        }else{
            return greaterThanCenterY && lessThanMaxY &&labelBounds.x >= centerPoint.x;
        }

    },

    adjustLabel4MoveUp:function(centerAngle,isLeft,boundsPath,dataPoint,labelBounds,minAngle,maxAngle){
        while(boundsPath.verticalIntersects(labelBounds.y,labelBounds.height)
           || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)){
            centerAngle += (isLeft ? 1 : -1)*this.adjustAngle;
            this.calculateLabelBoundsWithAngle(centerAngle,dataPoint,labelBounds);
            if(centerAngle < minAngle || centerAngle > maxAngle){
                break;
            }
        }
        dataPoint.dataLabel.bounds = labelBounds;
        boundsPath.append(labelBounds,false);
        if(centerAngle<minAngle || centerAngle>maxAngle || !this.isLabelBoundsInRange(dataPoint,labelBounds,minAngle,maxAngle)
            || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)){
            dataPoint.leadLine = null ;
            return centerAngle;
        }
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(!attrContents.isShowGuidLine || !dataPoint.isShowLabel){
           return centerAngle;
        }

        this.relayLabelLine(dataPoint,labelBounds,centerAngle);
        return centerAngle;
    },


    adjustLabelBoundsY:function(isLeft,boundsPath,dataPoint,labelBounds,minAngle,maxAngle){
        var centerAngle = this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex);
        while(boundsPath.verticalIntersects(labelBounds.y,labelBounds.height)
                 || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)){
            centerAngle += (isLeft ? 1 : -1)*this.adjustAngle;
            this.calculateLabelBoundsWithAngle(centerAngle,dataPoint,labelBounds);
            if(centerAngle < minAngle || centerAngle > maxAngle){
                break;
            }
        }
        dataPoint.dataLabel.bounds = labelBounds;
        boundsPath.append(labelBounds,false);
        if(centerAngle<minAngle || centerAngle>maxAngle || !this.isLabelBoundsInRange(dataPoint,labelBounds,minAngle,maxAngle)
            || this.isLabelBoundsOutPlotBounds(labelBounds,minAngle,maxAngle)){
            dataPoint.leadLine = null ;
            return false;
        }
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(!attrContents.isShowGuidLine || !dataPoint.isShowLabel){
           return true;
        }

        this.relayLabelLine(dataPoint,labelBounds,centerAngle);
        return true;
    },

    relayLabelLine:function(dataPoint,labelBounds,centerAngle){
        var linePath = new FR.Chart.GeneralPath();
        var pointX = 0;
        var pointY = 0;
        centerAngle = this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex);
        if(centerAngle >=90 && centerAngle < 270){
            pointX = labelBounds.x+labelBounds.width +2;
        }else{
            pointX = labelBounds.x - 2;
        }
        pointY = labelBounds.y+labelBounds.height/2+3;
        linePath.moveTo(pointX,pointY);
        if(centerAngle >=90 && centerAngle < 270){
            pointX +=0.1*this.radius;
        }else{
            pointX -= 0.1 * this.radius;
        }
        linePath.lineTo(pointX,pointY);
        var centerPoint = this.getCenterPoint(dataPoint.seriesIndex,dataPoint.categoryIndex);
        var R = this.radius+2;
        pointX = centerPoint.x + R * Math.cos((centerAngle / 180) * Math.PI);
        pointY = centerPoint.y - R * Math.sin((centerAngle / 180) * Math.PI);
        linePath.lineTo(pointX,pointY);
        var foldLine = new FR.Chart.FoldLine(linePath);
        dataPoint.leadLine = foldLine;

        var lineStyle = new FR.Chart.AttrLineStyle();
        lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
        foldLine.lineStyleInfo.attrLineStyle = lineStyle;
    },

    dealEachArea:function(pointsArray){
        var maxX = this.bounds.x + this.bounds.width;
        var minX = this.bounds.x;
        var diff = 0;
        this.checkPointsShowLabel(pointsArray);
        for(var i = 0;i<pointsArray.length ; i++){
            var textGlyph = pointsArray[i].dataLabel;
            if(pointsArray[i].glyph == null  || textGlyph == null || textGlyph.text == null || textGlyph.text.length === 0){
                continue;
            }
            var labelBounds = new FR.Chart.Bounds();
            this.dealOutsideLabelBoundsWhenNormal(labelBounds,pointsArray[i]);
            if(labelBounds.x < minX){
                diff = minX - labelBounds.x;
            }
            if(labelBounds.x + labelBounds.width > maxX){
                diff = labelBounds.x + labelBounds.width - maxX;
            }
        }
        return diff;
    },

    dealOutsideLabelBoundsWhenNormal:function(labelBounds,dataPoint){
        this.dealWithLabelLine(dataPoint);
        var centerAngle = this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex);
        this.calculateLabelBoundsWithAngle(centerAngle, dataPoint,labelBounds) ;
        dataPoint.dataLabel.bounds = labelBounds;
    },

    //返回牵引线指向标签的端点位置
    dealWithLabelLine:function(dataPoint){
        var centerPoint = this.getCenterPoint(dataPoint.seriesIndex,dataPoint.categoryIndex);
        var centerAngle = this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex);
        var path = new FR.Chart.GeneralPath();
        var R = this.radius+3;
        var rx = centerPoint.x + R * Math.cos((centerAngle / 180) * Math.PI);
        var ry = centerPoint.y - R * Math.sin((centerAngle / 180) * Math.PI);
        path.moveTo(rx,ry);
        R = this.radius * 1.2;
        rx = centerPoint.x + R * Math.cos((centerAngle / 180) * Math.PI);
        ry = centerPoint.y - R * Math.sin((centerAngle / 180) * Math.PI);
        ry += 3;
        path.lineTo(rx,ry);
        var centerAngle =this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex);
        if(centerAngle >=90 && centerAngle <270){
            rx -= this.radius * 0.1;
        } else{
            rx += this.radius * 0.1;
        }
        path.lineTo(rx,ry);
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(attrContents.isShowGuidLine && dataPoint.isShowLabel){
            var foldLine = new FR.Chart.FoldLine(path);
            dataPoint.leadLine = foldLine;

            var lineStyle = new FR.Chart.AttrLineStyle();
            lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
            foldLine.lineStyleInfo.attrLineStyle = lineStyle;
        }
    },

    calculateLabelBoundsWithAngle:function(centerAngle,dataPoint,labelBounds){
        var centerPoint = this.getCenterPoint(dataPoint.seriesIndex,dataPoint.categoryIndex);
        var dim = this.getLabelDim(dataPoint);
        var R = this.radius * 1.2;
        var rx = centerPoint.x + R * Math.cos(Math.PI * centerAngle / 180);
        var ry = centerPoint.y - R * Math.sin(Math.PI * centerAngle / 180);
        var littleLineLength = 0.1 * this.radius +2;
        // 垂直角度 左右分开
        if(centerAngle >=90 && centerAngle < 270){
            labelBounds.setRect(rx-littleLineLength-dim.width,ry - dim.height/2,dim.width,dim.height);
        }else{
            labelBounds.setRect(rx + littleLineLength,ry - dim.height/2,dim.width,dim.height);
        }
    } ,

    //标签支持条件显示的设置
    createDataLabel4DataPointWithCondition:function(dataPoint, attrContents, separator){
        //开始构造标签
        var label = dataPoint.getDataPointLabelString(attrContents, separator);

        //如果是自定义的标签就不是根据参数创建
        if(dataPoint.isCustomLabel && dataPoint.dataLabel){
            label = dataPoint.dataLabel.text;
        }

        var textGlyph = new FR.Chart.TextGlyph();
        textGlyph.text = label;
        textGlyph.textAttr = attrContents.textAttr;
        dataPoint.dataLabel = textGlyph;
        // 饼图这边去掉isShowLabel判断的原因是：
        // 点击图例使系列1消失，会调用checkPointsShowLabel，这个dataPoint的isShowLabel=false
        // 再次点击图例调用此处，dataPoint.showLabel不更新。
        dataPoint.showLabel = label;
    },

    checkPointsShowLabel:function(pointsArray){
        var allHeight = 0;
        var drawDataPoint ;
        for(var i = 0;i<pointsArray.length ; i++){
            if(pointsArray[i].glyph == null){
                pointsArray[i].isShowLabel = false;
                continue;
            }
            var drawDataPoint = pointsArray[i];
            var textGlyph = pointsArray[i].dataLabel;
            if(textGlyph != null && textGlyph.text != null && textGlyph.text.length>0){
                allHeight += this.getLabelDim(pointsArray[i]).height;
            }
        }
        if(allHeight > this.halfHeight4Shape(drawDataPoint)){
            //说明就算所有的都调整位置了，但是在这个1/4区域，标签还是不能都放下，所以要隐藏角度小的标签b
            this.removeAngleExtendMinPoint(pointsArray);
            this.checkPointsShowLabel(pointsArray);
        }
    },

    halfHeight4Shape:function(){
        return this.radius*1.2;
    },


    removeAngleExtendMinPoint:function(pointsArray){
        if(pointsArray == null || pointsArray.length <=0){
            return ;
        }
        var minIndex = -1;
        for(var i = 0;i<pointsArray.length ; i++){
            var beforeAngle = 0;
            var afterAngle = 0;
            if(pointsArray[i].glyph == null){
                pointsArray[i].isShowLabel = false;
                continue ;
            }
            minIndex = minIndex == -1?i:minIndex;
            if (pointsArray[i].glyph.shape instanceof FR.Chart.Arc2D) {
                 beforeAngle =pointsArray[i].glyph.shape.getExtent();
            }
            if(pointsArray[minIndex].glyph.shape instanceof FR.Chart.Arc2D) {
                 afterAngle = pointsArray[minIndex].glyph.shape.getExtent();
            }
            if(beforeAngle < afterAngle){
                minIndex = i;
            }
        }

        if(pointsArray[minIndex]){
            pointsArray[minIndex].isShowLabel = false;
            pointsArray.splice(minIndex,1);
        }
    },

    dealInsideLabels:function(categoryIndex,boundsPath){
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataPoint.glyph = null;
                continue;
            }

            if(this.isOutsideLabel(dataPoint)){
                continue ;
            }

            var textGlyph = dataPoint.dataLabel;
            if (textGlyph == null) {
                continue;
            }
            var text = textGlyph.text;
            if (text != null && text.length > 0) {
                this.dealInsideLabelBounds(dataPoint, boundsPath);
            }
        }
    },

    /**
     * 计算不同的shapeGlyph
     * @private
     * @param dataPoint
     * @returns {FR.Chart.ShapeGlyph}
     */
    calculateShapeGlyph4DifferentSub: function (dataPoint) {
        var shapeGlyph = new FR.Chart.ShapeGlyph();
        shapeGlyph.shape = this.getArc2D(dataPoint);
        shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());

        shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();

        return shapeGlyph;
    },

    /**
     * 返回扇形的Arc结果.
     * @param dataPoint 系列点
     * @returns {FR.Chart.Arc2D}
     */
    getArc2D: function (dataPoint) {
        var categoryIndex = dataPoint.categoryIndex;
        var seriesIndex = dataPoint.seriesIndex;
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var centerPoint = this.getCenterPoint(seriesIndex, categoryIndex);
        return this.getDataPointShape(centerPoint, seriesIndex, categoryIndex, seriesStartAngle, this.radius, dataPoint.isSeparateOut);
    },

    /**
     * 返回 绘图区的中心点位置.
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {FR.Chart.Point2D} 中心点.
     */
    getCenterPoint: function (seriesIndex, categoryIndex) {
        var eachWidth = this.getBounds().width / this.dataSeries[0].points.length;
        var normalX = categoryIndex * eachWidth + eachWidth / 2 + this.getBounds().x;
        return new FR.Chart.Point2D(normalX, this.getBounds().height / 2 + this.getBounds().y);
    },

    /**
     * 给系列点添加图表风格.
     * @param plotStyle 图表风格
     * @param dataPoint 系列点
     * @param shapeGlyph 系列形状
     * @param baseArc 基础的弧形.
     */
    addChartStyle4DataPoint: function (plotStyle, dataPoint, shapeGlyph, baseArc) {
        if (shapeGlyph.info.getBackground().color) {
            var seriesColor = shapeGlyph.info.getBackground().color;
            var arcShape = shapeGlyph.shape;
            var para = {'color': seriesColor, 'arcShape': arcShape, 'avoidOriginDraw': true, 'alpha':shapeGlyph.info.getAlpha()};
            if (plotStyle == FR.Chart.Constants.STYLE_3D) {
                if (!this.arc3DStyleBase) { //平面3d的风格时有底座
                    this.arc3DStyleBase = new FR.Chart.Arc3DStyleBase(this.get3DStyleBaseShape());
                }
                dataPoint.dataPointStyle = new FR.Chart.ArcPlane3DStyle(para);
                if (!dataPoint.isSeparateOut) {
                    var distance = (this.separatePercent / 100.0 * this.radius) / 6;
                    var startAngle = this.getSeriesStartAngle(dataPoint.categoryIndex, dataPoint.seriesIndex);
                    var halfAngle = this.getHalfAngle(startAngle, dataPoint.seriesIndex, dataPoint.categoryIndex);
                    shapeGlyph.shape.x += distance * Math.cos((360 - halfAngle) * Math.PI / 180);
                    shapeGlyph.shape.y += distance * Math.sin((360 - halfAngle) * Math.PI / 180);
                }
            } else if (plotStyle == FR.Chart.Constants.STYLE_OUTER) {
                if(FR.Browser.isIE8Before()){
                    dataPoint.dataPointStyle = null;//产品:ie678屏蔽高光效果
                }else{
                    if (!this.arcOuterBase) { //高光的外环
                        this.arcOuterBase = new FR.Chart.ArcOuterBase(this.radius, this.getCenterPoint(0, 0));
                    }
                    dataPoint.dataPointStyle = new FR.Chart.ArcTopDownShadeStyle(para);
                    //重新设置arc的半径
                    shapeGlyph.shape.radius = this.radius * 0.85;
                }
            } else if (plotStyle == FR.Chart.Constants.STYLE_SHADE) {
                dataPoint.dataPointStyle = new FR.Chart.ArcTopDownShadeStyle(para);
            } else if (plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
                dataPoint.dataPointStyle = new FR.Chart.ArcTransparentChartStyle(para);
            }
        }
    },

    /**
     * 3D风格的基础shape
     * @returns {FR.Chart.Arc2D}
     */
    get3DStyleBaseShape: function () {
        var radius = this.radius * 11 / 12;
        var centerPoint = this.getCenterPoint(0, 0);
        return new FR.Chart.Arc2D(centerPoint.x, centerPoint.y, radius, 0, 360);
    },

    /**
     * 是否弹出 扇形
     * @param categoryIndex 分类序号
     * @returns {boolean}
     */
    isPieSeperateOut: function (categoryIndex) {
        var isOut = false;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.isSeparateOut) {
                isOut = true;
                break;
            }
        }
        return isOut;
    },

    /**
     * 为系列处理牵引线
     */
    dealLeadLineSeries: function () {
        for (var categoryIndex = 0; this.dataSeries[0] && categoryIndex < this.dataSeries[0].points.length; categoryIndex++) {
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataSeries = this.dataSeries[seriesIndex];
                var dataPoint = dataSeries.points[categoryIndex];

                if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    dataPoint.glyph = null;
                    continue;
                }

                var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex);
                var centerAngle = this.getHalfAngle(seriesStartAngle, seriesIndex, categoryIndex);

                if (dataPoint.glyph && dataPoint.glyph.shape) {
                    if (dataPoint.glyph.shape instanceof FR.Chart.Arc2D) {
                        this.dealLeadLine(dataPoint.glyph.shape, dataPoint, centerAngle % 360);
                    }
                }
            }
        }
    },

    /**
     * 返回饼图直径
     * @param categoryIndex 分类序号
     * @returns {Number} 饼图直径
     */
    getPieDimension: function (categoryIndex) {
        var plotBounds = this.getBounds();
        var eachWidth = plotBounds.width / this.dataSeries[0].points.length;
        var resultD = 0;
        if (eachWidth > plotBounds.height) {
            resultD = plotBounds.height;
        } else {
            resultD = eachWidth;
        }

        resultD = this.deal4BestRad(resultD, categoryIndex);

        return resultD;
    },

    /**
     * 改变饼图半径
     * @param resultD 饼图直径
     * @param categoryIndex 分类序号
     * @returns {Number} 饼图直径
     */
    deal4BestRad: function (resultD, categoryIndex) {
        //饼图标签在外
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), this.dataSeries[0].points[categoryIndex]);
        if (attrContents.position == FR.Chart.Constants.OUTSIDE) {
            resultD *= this.radiusPercent;
            if (this.getCategoryCount() > 1) {
                resultD *= 0.8;
            }
        }
        return resultD;
    },

    /**
     * 返回系列点shape
     * @param centerPoint 中心点
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @param start 开始角度
     * @param radius 半径
     * @param isPointOut 饼图是否弹出
     * @returns {FR.Chart.Arc2D} 弧形shape
     */
    getDataPointShape: function (centerPoint, seriesIndex, categoryIndex, start, radius, isPointOut) {
        var distance = this.separatePercent / 100.0 * radius;
        if (!isPointOut) {
            distance = 0;
        }
        if (this.plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
            distance += 2;
        }
        var halfAngle = this.getHalfAngle(start, seriesIndex, categoryIndex);

        // canvas的arc角度计算是顺时针,这里记录的是逆时针的数据,只是在Arc2D中draw和paint时候再转化
        var startAngle = start % 360;
        var extendAngle = this.getPercent(seriesIndex, categoryIndex) * 360;

        var resultShape = new FR.Chart.Arc2D(
            centerPoint.x + distance * Math.cos((360 - halfAngle) * Math.PI / 180),
            centerPoint.y + distance * Math.sin((360 - halfAngle) * Math.PI / 180),
            radius, startAngle, extendAngle
        );

        return resultShape;
    },

    /**
     * 处理牵引线
     * @private
     * @param arc2D 弧形的arc2d
     * @param dataPoint 系列点
     * @param centerAngle 中心角度.
     */
    dealLeadLine: function (arc2D, dataPoint, centerAngle) {
        var textGlyph = dataPoint.dataLabel;
        if (textGlyph == null) {
            return;
        }

        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);

        var pieLabel = textGlyph.text;
        // 只要显示了牵引线, 就是默认位置在数据标签外了(饼图外)
        if (pieLabel != null && textGlyph.bounds != null && attrContents.isShowGuidLine) {

            var bounds = textGlyph.bounds;

            if (arc2D instanceof  FR.Chart.Arc2D) {
                var startPoint = arc2D.getArcPoint(-centerAngle);
            } else {
                var startPoint = arc2D.top().getMiddlePoint();
            }
            var startX = startPoint.x;
            var startY = startPoint.y;

            var endX = bounds.x;
            var endY = bounds.y;

            var zX = bounds.x;
            var zY = bounds.y;
            var tickLength = this.bounds.height * 0.01;
            centerAngle = centerAngle % 360;
            var isIntersected = arc2D.containsPoint(bounds.x, bounds.y) || arc2D.containsPoint(bounds.x + bounds.width, bounds.y) ||
                arc2D.containsPoint(bounds.x, bounds.y + bounds.height) || arc2D.containsPoint(bounds.x + bounds.width, bounds.y + bounds.height);

            if (!isIntersected) {
                if (startX < endX - tickLength) {
                    zX = bounds.x - tickLength;
                    zY = bounds.y + bounds.height / 2;
                    endX = bounds.x;
                    endY = bounds.y + bounds.height / 2;
                } else if (startX > endX + bounds.width + tickLength) {
                    zX = bounds.x + bounds.width;
                    zY = bounds.y + bounds.height / 2;
                    endX = bounds.x + bounds.width - tickLength;
                    endY = bounds.y + bounds.height / 2;
                } else {
                    if (centerAngle > 80 && centerAngle < 100 || centerAngle > 260 && centerAngle < 280) {
                        if (startY > bounds.y + bounds.height) {
                            zX = bounds.x + bounds.width / 2;
                            zY = bounds.y + bounds.height + tickLength;
                            endX = bounds.x + bounds.width / 2;
                            endY = bounds.y + bounds.height;
                        } else if (startY < bounds.y) {
                            zX = bounds.x + bounds.width / 2;
                            zY = bounds.y - tickLength;
                            endX = bounds.x + bounds.width / 2;
                            endY = bounds.y;
                        } else {// 标签压起始点, 不处理牵引线
                            return;
                        }
                    } else if (centerAngle >= 100 && centerAngle <= 260) {
                        zX = bounds.x + bounds.width;
                        zY = bounds.y + bounds.height / 2;
                        endX = bounds.x + bounds.width - tickLength;
                        endY = bounds.y + bounds.height / 2;
                    } else {
                        zX = bounds.x + tickLength;
                        zY = bounds.y + bounds.height / 2;
                        endX = bounds.x + tickLength * 2;
                        endY = bounds.y + bounds.height / 2;
                    }
                }
                var path = new FR.Chart.GeneralPath();
                path.moveTo(startX, startY);
                path.curveTo(zX, zY, (startX + endX) * 0.5, (startY + endY) * 0.5, endX, endY);
                var foldLine = new FR.Chart.FoldLine(path);
                dataPoint.leadLine = foldLine;

                var lineStyle = new FR.Chart.AttrLineStyle();
                lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
                foldLine.lineStyleInfo.attrLineStyle = lineStyle;
            } else {// 相交 清空线
                dataPoint.leadLine = null;
            }
        }
    },

    //返回标签所在圆的半径的点
    getLabelRadius: function(centerX, centerY, centerAngle, labelBounds){
        var y = labelBounds.y + labelBounds.height / 2.0;
        var divide = Math.sin(Math.PI * centerAngle / 180);
        var labelR = 0;
        if(divide === 0){
            labelR = centerY - y;
        }else{
            labelR = (centerY - y) / divide;
        }

        labelR = Math.abs(labelR);

        var rx = centerX + labelR * Math.cos(Math.PI * centerAngle / 180);
        var ry = centerY - labelR * Math.sin(Math.PI * centerAngle / 180);

        //防止小角度计算的时候值太大了。。。
        if(centerAngle >= 90 && centerAngle <= 270){
            if(rx <= labelBounds.x + labelBounds.width){
                rx = labelBounds.x + labelBounds.width;
                ry = labelBounds.y + labelBounds.height / 2;
            }
        }else{
            if(rx >= labelBounds.x){
                rx = labelBounds.x;
                ry = labelBounds.y + labelBounds.height / 2;
            }
        }

        return new FR.Chart.Point2D(rx, ry);
    },

    /**
     * 防止标签重叠判断算法.
     * @param boundsManager  标签管理器
     * @param bounds 标签北街.
     * @param angle   标签角度
     * @param R 半径
     * @param mx 绘图区的xy x
     * @param my 绘图区的my y位置.
     * @returns {FR.Chart.Recatangle2D} 返回新的标签位置.
     */
    avoidCoverLabelBounds: function (boundsManager, bounds, angle, R, mx, my) {
        if (bounds == null) {
            return null;
        }
        if (angle == null || R == null || mx == null || my == null) {
            return FR.Chart.PiePlotGlyph.superclass.avoidCoverLabelBounds.apply(this, [boundsManager, bounds]);
        }

        var tmpAngle = angle;
        this.whileTmp(boundsManager, bounds, tmpAngle, mx, my, R, 0.5);

        if (!boundsManager.isInteract(bounds)) {
            var oldBounds = new FR.Chart.Bounds(bounds.x, bounds.y, bounds.width + 2, bounds.height);
            boundsManager.addBounds(oldBounds);
            return oldBounds;
        }

        return null;
    },

    /**
     * 按角度循环 重叠了就 按角度移动
     * @param path 标签管理.
     * @param bounds 标签在回绘图区 南部
     * @param tmpAngle 临时 角度.
     * @param mx x位置.
     * @param my y位置
     * @param R 临时饼图baj9
     * @param incAngle 弧形内部角度
     *
     */
    whileTmp: function (path, bounds, tmpAngle, mx, my, R, incAngle) {
        if (R == null || incAngle == null) {// TODO kunsnat: 这边为什么要去 父类计算呢.  父类这个方法最好能私有.
            return FR.Chart.PiePlotGlyph.superclass.whileTmp.apply(this, [path, bounds, tmpAngle, mx, my]);
        }

        var backup = bounds.clone();
        var tt = 0;
        while (path.isInteract(bounds)) {
            tmpAngle += incAngle;

            if (tt > 360) {
                bounds.setRect(backup.x, backup.y, backup.width, backup.height);
                break;
            }

            var rx = mx + R * Math.cos(Math.PI * tmpAngle / 180);
            var ry = my - R * Math.sin(Math.PI * tmpAngle / 180);

            // 垂直角度 左右分开
            if (tmpAngle > 90 && tmpAngle < 225) { // 90度的跳跃点(差别大.)
                bounds.setRect(rx - bounds.width, ry - bounds.height / 2, bounds.width, bounds.height);
            } else if (tmpAngle >= 225 && tmpAngle < 270) {
                bounds.setRect(rx - bounds.width, ry - bounds.height, bounds.width, bounds.height);
            } else if (tmpAngle >= 270 && tmpAngle < 315) {
                bounds.setRect(rx, ry - bounds.height, bounds.width, bounds.height);
            } else {
                bounds.setRect(rx, ry - bounds.height / 2, bounds.width, bounds.height);
            }
            tt += incAngle;
        }
    },

    /**
      标签是否在弧形外.
     * @param dataPoint 系列点
     * @returns {Boolean|FR.Chart.AttrContents.isShowGuidLine|*}
     */
    isOutsideLabel: function (dataPoint) {
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        return attrContents.position == FR.Chart.Constants.OUTSIDE || attrContents.isShowGuidLine;
    },

    /**
     * 处理标签在外时的标签位置.
     * @param dataPoint 系列点
     * @param boundsPath 标签路径
     */
    dealOutsideLabelBounds: function (dataPoint, boundsPath) {
        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;

        var labelBounds = new FR.Chart.Bounds();
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex);
        var centerAngle = this.getHalfAngle(seriesStartAngle, seriesIndex, categoryIndex) % 360;

        if (this instanceof FR.Chart.Pie3DPlotGlyph) {
            centerAngle = dataPoint.getShape().top().getMiddleAngle();
            centerAngle = (180 - centerAngle + 360) % 360;
        }

        var count = 0;
        if (centerAngle >= 90 && centerAngle < 180) {
            count = this.leftUp.length;
        } else if (centerAngle >= 180 && centerAngle < 270) {
            count = this.leftDown.length;
        } else if (centerAngle >= 270 && centerAngle <= 360) {
            count = this.rightDown.length;
        } else {
            count = this.rightUp.length;
        }
        if (count >= this.MAX_SERIES_MODE) {
            this.dealOutsideLabelBoundsWithLargeSeriesSize(labelBounds, centerAngle, dataPoint);

        } else {
            this.dealOutsideLabelBoundsWithLittleSeriesSize(labelBounds, centerAngle, dataPoint, boundsPath);
        }
        dataPoint.dataLabel.bounds = labelBounds;
    },

    /**
     * 处理系列的标签位置
     * @param dataPoint 系列点
     * @param boundsPath 标签路径管理.
     */
    dealLabelBounds: function (dataPoint, boundsPath) {
        var textGlyph = dataPoint.dataLabel;
        if (textGlyph == null) {
            return;
        }
        var text = textGlyph.text;
        if (text != null && text.length > 0) {
            if (this.isOutsideLabel(dataPoint)) {
                this.dealOutsideLabelBounds(dataPoint, boundsPath);
            } else {
                this.dealInsideLabelBounds(dataPoint, boundsPath);
            }
        }
    },

    /**
     * 处理标签在系列内时
     * @param dataPoint 系列点
     * @param boundsPath 标签管理.
     */
    dealInsideLabelBounds: function (dataPoint, boundsPath) {
        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;
        var centerPoint = this.getCenterPoint(seriesIndex, categoryIndex);
        var dim = this.getLabelDim(dataPoint);
        var tg_width = dim.width;
        var tg_height = dim.height;
        var time = 0.5;
        var radius = this.changeRadius(categoryIndex, seriesIndex);
        var textGlyph = dataPoint.dataLabel;
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var centerAngle = this.getHalfAngle(seriesStartAngle, seriesIndex, categoryIndex) % 360;

        if (centerAngle == seriesStartAngle) {// 如果为0值的时候, 直接在饼图外处理
            this.dealOutsideLabelBounds(dataPoint, boundsPath);
            return;
        }

        var pie = this.getDataPointShape(centerPoint, seriesIndex, categoryIndex, seriesStartAngle, radius, false);
        var tmpX = radius * Math.cos(Math.PI * centerAngle / 180);
        var tmpY = radius * Math.sin(Math.PI * centerAngle / 180);
        for (time; time < 1; time += 0.1) {
            var rx = centerPoint.x + tmpX * time;
            var ry = centerPoint.y - tmpY * time;
            var centerLabelx = rx - tg_width / 2,
                centerLabely = ry - tg_height / 2;
            var bounds = new FR.Chart.Bounds(centerLabelx, centerLabely, dim.width, dim.height);
            if (pie.containsPoint(centerLabelx, centerLabely) && pie.containsPoint(centerLabelx + dim.width, centerLabely)
                && pie.containsPoint(centerLabelx, centerLabely + dim.height) && pie.containsPoint(centerLabelx + dim.width, centerLabely + dim.height)) {
                textGlyph.bounds = bounds;
                break;
            }
        }
        if (time >= 0.9) {
            textGlyph.bounds = null;
        }
    },

    /**
     * 在系列标签较少时, 处理标签在外时的标签位置.
     * @param labelBounds 标签位置
     * @param centerAngle 中心角度
     * @param dataPoint 系列点
     * @param boundsPath 标签管理.
     */
    dealOutsideLabelBoundsWithLittleSeriesSize: function (labelBounds, centerAngle, dataPoint, boundsPath) {
        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;
        var centerPoint = this.getCenterPoint(seriesIndex, categoryIndex);
        var dim = this.getLabelDim(dataPoint);
        var tg_width = dim.width;
        var tg_height = dim.height;
        var mx = centerPoint.x;
        var my = centerPoint.y;
        var R = this.changeRadius(categoryIndex, seriesIndex);
        var rx = mx + R * Math.cos(Math.PI * centerAngle / 180);
        var ry = my - R * Math.sin(Math.PI * centerAngle / 180);

        var minX = this.getBounds().x;
        var minY = this.getBounds().y;

        var textAlign = (centerAngle >= 90 && centerAngle <= 270) ? 'right' : 'left';
        rx += textAlign == 'left' ? 20 : -20;

//        data.__labelX = x - (textAlign == 'left' ? 5 : -5);

        // 垂直角度 左右分开
        if (centerAngle > 90 && centerAngle < 225) {
            labelBounds.setRect(Math.max(minX, rx - tg_width), Math.max(minY, ry - tg_height / 2), tg_width, tg_height);
        } else if (centerAngle >= 225 && centerAngle < 270) {
            labelBounds.setRect(Math.max(minX, rx - tg_width), Math.max(minY, ry - tg_height / 2), tg_width, tg_height);
        } else if (centerAngle >= 270 && centerAngle < 315) {
            labelBounds.setRect(rx, Math.max(minY, ry - tg_height / 2), tg_width, tg_height);
        } else {
            labelBounds.setRect(rx, Math.max(minY, ry), tg_width, tg_height);
        }
        labelBounds = this.avoidCoverLabelBounds(boundsPath, labelBounds, centerAngle, R, mx, my);
    },

    /**
     * 调整半径
     * @param categoryIndex 分类序号
     * @param seriesIndex 系列序号
     * @returns {Number} 半径大小.
     */
    changeRadius: function (categoryIndex, seriesIndex) {
        return this.radius * this.OUTSIDE_LABEL_RADIUS_COEFFICIENT + this.separatePercent * this.radius / 100;
    },

    /**
     * 返回系列点的标签占用大小.
     * @param dataPoint 系列点
     * @returns {FR.Chart.Dimension2D} 占用大小.
     */
    getLabelDim: function (dataPoint) {
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        var textAttr = attrContents.textAttr;
        if (textAttr == null) {
            textAttr = new FR.Chart.TextAttr();
        }
        return FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(dataPoint.dataLabel.text, textAttr);
    },

    /**
     * 返回合适的标签间隔大小
     * @param seriesCount 系列个数
     * @param tg_height 标签高度
     * @returns {Number} 标签间隔大小
     */
    getLabelMargin: function (seriesCount, tg_height) {
        return Math.max((this.bounds.height / 2 - tg_height * seriesCount) / (seriesCount + 1), 1);
    },

    /**
     * 处理 数据标签外 大量标签时的位置
     * @param labelBounds 标签位置
     * @param centerAngle 中心角度
     * @param dataPoint 系列点
     */
    dealOutsideLabelBoundsWithLargeSeriesSize: function (labelBounds, centerAngle, dataPoint) {
        var categoryIndex = dataPoint.categoryIndex;
        var dim = this.getLabelDim(dataPoint);

        var tg_width = dim.width;
        var tg_height = dim.height;
        var eachWidth = this.bounds.width / this.getCategoryCount();
        var tangent = (eachWidth - this.getPieDimension(categoryIndex)) / 2;
        var margin = (tangent - this.maxLabelWidth) / 2;

        //各象限每个标签的上下间距
        var quadrantMargin = 0;
        var boundsX = 0;
        var boundsY = 0;

        if (centerAngle >= 90 && centerAngle < 180) {
            quadrantMargin = this.getLabelMargin(this.leftUp.length, tg_height);
            boundsX = (margin > 0 ? (tangent - margin - tg_width) : (eachWidth * 0.01) ) + eachWidth * categoryIndex;

            var tmpIndex = this.leftUp.indexOf(centerAngle);
            boundsY = tmpIndex * (quadrantMargin + tg_height) + quadrantMargin;
            if (tmpIndex >= 0 && tmpIndex < this.leftUp.length) {
                this.leftUp[tmpIndex] = Number.MAX_VALUE;// kunsnat: 标记已经使用过, 防止重复的 角度只计算第一个.
            }
        } else if (centerAngle >= 180 && centerAngle < 270) {
            quadrantMargin = this.getLabelMargin(this.leftDown.length, tg_height);
            boundsX = (margin > 0 ? (tangent - margin - tg_width) : (eachWidth * 0.01) ) + eachWidth * categoryIndex;

            var tmpIndex = this.leftDown.indexOf(centerAngle);
            boundsY = tmpIndex * (quadrantMargin + tg_height) + quadrantMargin + this.bounds.height / 2;
            if (tmpIndex >= 0 && tmpIndex < this.leftDown.length) {
                this.leftDown[tmpIndex] = Number.MAX_VALUE;
            }
        } else if (centerAngle >= 270 && centerAngle < 360) {
            quadrantMargin = this.getLabelMargin(this.rightDown.length, tg_height);
            boundsX = (margin > 0 ? (tangent + this.getPieDimension(categoryIndex) + margin) : (eachWidth * 0.99 - tg_width) ) + eachWidth * categoryIndex;

            var tmpIndex = this.rightDown.indexOf(centerAngle);
            boundsY = this.bounds.height - tmpIndex * (tg_height + quadrantMargin);
            if (tmpIndex >= 0 && tmpIndex < this.rightDown.length) {
                this.rightDown[tmpIndex] = Number.MAX_VALUE;
            }
        } else {
            quadrantMargin = this.getLabelMargin(this.rightUp.length, tg_height);
            boundsX = (margin > 0 ? (tangent + this.getPieDimension(categoryIndex) + margin) : (eachWidth * 0.99 - tg_width) ) + eachWidth * categoryIndex;

            var tmpIndex = this.rightUp.indexOf(centerAngle);
            boundsY = this.bounds.height / 2 - tmpIndex * (tg_height + quadrantMargin);
            if (tmpIndex >= 0 && tmpIndex < this.rightUp.length) {
                this.rightUp[tmpIndex] = Number.MAX_VALUE;
            }
        }
        labelBounds.setRect(boundsX + this.getBounds().x, boundsY + this.getBounds().y, tg_width, tg_height);
    },

    /**
     * 根据系列点的角度 对标签等 进行分区, 四象限.
     * @param categoryIndex 分类序号.
     */
    getSeriesCountWithQuadrant: function (categoryIndex) {
        this.leftUp.length = 0;
        this.leftDown.length = 0;
        this.rightDown.length = 0;
        this.rightUp.length = 0;

        for (var i = 0; i < this.dataSeries.length; i++) {
            var centerAngle =  this.getCenterAngle4DataPoint(categoryIndex,i);
            if (this.dataSeries[i].points[categoryIndex].valueIsNull) {
                continue;
            }
            var dataPoint = this.dataSeries[i].points[categoryIndex];

            if(!this.isOutsideLabel(dataPoint)){
                dataPoint.leadLine = null;
                continue;
            }
            this.dataSeries[i].points[categoryIndex].isShowLabel = true;
            var dataPoint = this.dataSeries[i].points[categoryIndex];
            if (centerAngle >= 90 && centerAngle < 180) {
                this.leftUp.push(dataPoint);
            } else if (centerAngle >= 180 && centerAngle < 270) {
                this.leftDown.push(dataPoint);
            } else if (centerAngle >= 270 && centerAngle < 360) {
                this.rightDown.push(dataPoint);
            } else {
                this.rightUp.push(dataPoint);
            }
        }
        this._sortList();
    },

    getCenterAngle4DataPoint:function(categoryIndex,seriesIndex){
        return this.getHalfAngle(this.getSeriesStartAngle(categoryIndex, seriesIndex), seriesIndex, categoryIndex) % 360;
    },

    _sortList: function () {
        this.bubbleSort(this.leftUp);
        this.bubbleSort(this.rightUp);
        this.bubbleSort(this.leftDown);
        this.bubbleSort(this.rightDown);
        //this.leftUp.sort(function compare(a, b) {
        //    return this.getCenterAngle4DataPoint(a.ca) - b;
        //});
        //this.leftDown.sort(function compare(a, b) {
        //    return a - b;
        //});
        //this.rightDown.sort(function compare(a, b) {
        //    return a - b;
        //});
        //this.rightUp.sort(function compare(a, b) {
        //    return a - b;
        //});
    },


    bubbleSort:function(array){
        var low = 0;
        var high = array.length - 1; //设置变量的初始值
        var tmp = 0;
        var j=0 ;
        while (low < high) {
            for (j = low; j < high; ++j) { //正向冒泡,找到最大者
                if (this.getCenterAngle4DataPoint(array[j].categoryIndex,array[j].seriesIndex)>
                    this.getCenterAngle4DataPoint(array[j+1].categoryIndex,array[j+1].seriesIndex)) {
                    tmp = array[j];
                    array[j] = array[j + 1];
                    array[j + 1] = tmp;
                }
            }
            --high; //修改high值, 前移一位
            for (j = high; j > low; --j) { //反向冒泡,找到最小者
                if (this.getCenterAngle4DataPoint(array[j].categoryIndex,array[j].seriesIndex) <
                    this.getCenterAngle4DataPoint(array[j-1].categoryIndex,array[j-1].seriesIndex)) {
                    tmp = array[j];
                    array[j] = array[j - 1];
                    array[j - 1] = tmp;
                }
            }
            ++low;                  //修改low值,后移一位
        }
    },

    /**
     * 最大标签宽度.
     * @param categoryIndex 分类序号
     * @returns {Number} 标签宽度
     */
    getMaxLabelWidth: function (categoryIndex) {
        var maxLabelWidth = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
            if (dataPoint == null || dataPoint.dataLabel == null) {
                continue;
            }
            var text = dataPoint.dataLabel.text;
            if (text != null && text.length > 0) {
                var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
                if (attrContents != null) {
                    var textAttr = attrContents.textAttr;
                    if (textAttr == null) {
                        textAttr = new FR.Chart.TextAttr();
                    }
                    var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(text, textAttr);
                    maxLabelWidth = Math.max(maxLabelWidth, dim.width);
                }
            }
        }
        return maxLabelWidth;
    },

    /**
     * 返回系列点的起始角度
     * @param categoryIndex 分类序号
     * @param seriesIndex 系列序号
     * @returns {Number} 起始角度
     */
    getSeriesStartAngle: function (categoryIndex, seriesIndex) {
        var startAngle = 90;
        for (var i = 0; i < seriesIndex; i++) {
            startAngle = startAngle + this.getPercent(i, categoryIndex) * 360;
        }
        return startAngle;
    },

    /**
     * 返回扇形的中间角度
     * @param seriesStartAngle 系列起始角度
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {Number} 扇形中间角度.
     */
    getHalfAngle: function (seriesStartAngle, seriesIndex, categoryIndex) {
        return seriesStartAngle + 0.5 * this.getPercent(seriesIndex, categoryIndex) * 360;
    },

    /**
     * 返回系列点的百分比.
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {Number} 百分比值.
     */
    getPercent: function (seriesIndex, categoryIndex) {
        if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return 0;
        }
        var totalValue = this.getTotalValue(categoryIndex);
        if (totalValue <= 0) {
            return 0;
        }
        var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
        return Math.abs(dataPoint.value) / totalValue;
    },

    /**
     * 返回分类的饼图总值.
     * @param categoryIndex 分类
     * @returns {Number} 总值
     */
    getTotalValue: function (categoryIndex) {
        var totalValue = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if(dataPoint == null){
                continue;
            }
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            totalValue += Math.abs(dataPoint.value);
        }
        return totalValue;
    },

    /**
     * 画出饼图
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        this.drawInfo(ctxs);

        FR.Chart.PiePlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    /**
     * 在系列扇形等 动态结束之后 加上标记 标签等.
     * @param ctx 画布
     * @param currentDivKey 当前的div key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {// kunsnat: 只是为了组合饼图的继承?
        FR.Chart.PiePlotGlyph.superclass.drawMarkersLabelAfterSeriesAnimateDraw.apply(this, [ctx, currentDivKey]);
    }
});

/**
 * FunnelPlotGlyph.start
 * 饼图绘图区
 *
 *      @example
 *      var funnel = new FR.Chart.FunnelPlotGlyph();
 *
 * @class FR.Chart.FunnelPlotGlyph
 * @extends FR.Chart.PlotGlyph
 */
FR.Chart.FunnelPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        FR.Chart.FunnelPlotGlyph.superclass.initConstants.apply(this, [js]);

        this.TOP_BOTTOM_GAP = 0.1;
        this.LEFT_RIGHT_GAP = 0.05;
        this.LABEL_GAP = 0.06;
        this.LEAD_LINE_GAP = 0.01;
        this.MAX_LABEL_WIDTH = 0.46;
        this.LABEL_VALUE_GAP = 0.02;
        this.SERIES_GAP = 1;
        this.seriesX = [];
        this.seriesY = [];
        this.seriesHeight = [];
        this.seriesWidth = [];
        this.labelWidth = 0;
        this.labelX = 0;
        this.showLabel = true;
        this.originalPoint;
    },

    /**
     * 坐标轴布局
     */
    layoutAxisGlyph: function () {
        // do nothing;
    },

    isSupportLegend:function(){
        return false;
    },

    getTotalHeight: function(categoryIndex) {
        var totalHeight = 0;
        for (var i = 1; i < this.dataSeries.length; i ++) {
            var dataPointFrom = this.dataSeries[i - 1].points[categoryIndex];
            var dataPointTo = this.dataSeries[i].points[categoryIndex];
            totalHeight += Math.abs(Math.abs(dataPointFrom.value) - Math.abs(dataPointTo.value));
        }
        return totalHeight;
    },

    initSeriesHeight: function(categoryIndex) {
        var totalHeight = this.getTotalHeight(categoryIndex);
        var realHeight = this.bounds.height * (1 - this.TOP_BOTTOM_GAP * 2);
        this.seriesHeight[0] = 0;
        for (var j = 1; j < this.dataSeries.length; j++) {
            var dataPointFrom = this.dataSeries[j - 1].points[categoryIndex];
            var dataPointTo = this.dataSeries[j].points[categoryIndex];
            this.seriesHeight[j] = Math.abs(Math.abs(dataPointFrom.value) - Math.abs(dataPointTo.value)) / totalHeight * realHeight;
            if(dataPointTo.valueIsNull){
                this.seriesHeight[j] = 0;
            }
        }
    },

    initSeriesWidth:function(categoryIndex){
        var realWidth = this.bounds.width * (1 - this.LEFT_RIGHT_GAP * 2 - this.LEAD_LINE_GAP * 2 - this.LABEL_GAP) - this.labelWidth;
        if (!this.showLabel) {
            realWidth = this.bounds.width * (1 - this.LEFT_RIGHT_GAP * 2 - this.LEAD_LINE_GAP * 2);
        }
        var maxWidth = this._getMaxValue(categoryIndex);
        for (var j = 0; j < this.dataSeries.length; j ++) {
            var dataPointTo = this.dataSeries[j].points[categoryIndex];
            this.seriesWidth[j] = Math.abs(dataPointTo.value) / maxWidth * realWidth;
        }
    },

    _getMaxValue:function(categoryIndex){
        var maxValue = Number.MIN_VALUE;
        for(var i = 0; i < this.dataSeries.length; i++){
            if(this.dataSeries[i].points[categoryIndex]){
                var point = this.dataSeries[i].points[categoryIndex];
                maxValue = Math.max(maxValue, Math.abs(point.value));
            }
        }
        return maxValue;
    },

    _getMaxSeriesWidth:function(){
        if(this.seriesWidth && this.seriesWidth[0]){
            var maxWidth = this.seriesWidth[0];
            for(var i = 0; i < this.seriesWidth.length; i++){
                maxWidth = Math.max(maxWidth, this.seriesWidth[i]);
            }
            return maxWidth;
        }
        return 0;
    },

   calculateOriginalPoint: function() {
        var x = this.bounds.x + this.bounds.width * this.LEFT_RIGHT_GAP + this._getMaxSeriesWidth() / 2;
        var y = this.bounds.y + this.bounds.height * this.TOP_BOTTOM_GAP;
        this.originalPoint = new FR.Chart.Point2D(x, y);
   },

   calculateSeriesXY: function() {
        this.calculateOriginalPoint();
        var height = 0;
        for (var i = 0; i < this.dataSeries.length; i ++) {
            this.seriesX[i] = this.originalPoint.x - this.seriesWidth[i] / 2;
            height += this.seriesHeight[i];
            this.seriesY[i] = this.originalPoint.y + height;
        }
   },

   initSeriesBounds: function(categoryIndex) {
        this.initSeriesHeight(categoryIndex);
        this.initSeriesWidth(categoryIndex);
        this.calculateSeriesXY();
   },

   calculateLabelWidth: function(categoryIndex) {
        var seriesWidth = 0;
        var valueWidth = 0;
        var maxWidth = this.bounds.width * this.MAX_LABEL_WIDTH;
        var valueGap = this.bounds.width * this.LABEL_VALUE_GAP;
        for (var j = 0; j < this.dataSeries.length; j ++) {
            var dataPoint = this.dataSeries[j].points[categoryIndex];
            var label = dataPoint.dataLabel;
            if (label == null) {
                this.labelWidth = 0;
                this.showLabel = false;
                return;
            }
            dataPoint.valueGap = valueGap;
            var text = dataPoint.getSeriesValueString();
            var textAttr = label.textAttr;
            var dim1 = null;
            var dim2 = null;
            if (text != null) {
                if (!FR.isEmpty(text[0])) {
                    dim1 = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(text[0], textAttr);
                }
                if (!FR.isEmpty(text[1])) {
                    dim2 = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(text[1], textAttr);
                }
            }
            if (dim1 != null && dim1.width > seriesWidth) {
                seriesWidth = dim1.width;
            }
            if (dim2 != null && dim2.width > valueWidth) {
                valueWidth = dim2.width;
            }
        }
        for (var j = 0; j < this.dataSeries.length; j ++) {
            var dataPoint = this.dataSeries[j].points[categoryIndex];
            var label = dataPoint.dataLabel;
            if (label == null) {
                continue;
            }
            dataPoint.valueWidth = valueWidth;
        }
        this.labelWidth = Math.min(seriesWidth + valueWidth + valueGap, maxWidth);
        this.labelX = this.bounds.x + this.bounds.width * (1 - this.LEFT_RIGHT_GAP) - this.labelWidth;
    },

   initLabelData: function(categoryIndex) {
        this.calculateLabelWidth(categoryIndex);
    },

    /**
     * 漏斗图只有系列
     */
   layoutDataSeriesGlyph: function() {
        //过滤空的系列
        var seriesSize = this.dataSeries.length, validSeries = [];
        for (var seriesIndex = 0; seriesIndex < seriesSize; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPointTo = dataSeries.points[0];
            if (dataPointTo && !dataPointTo.valueIsNull) {
                validSeries.push(dataSeries)
            }
        }
        this.dataSeries = validSeries;

        for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
            this.deal4EachCategory(categoryIndex);
        }
        this.dealLabelBoundsInOrder();
    },

    _indexOfDataPoint:function(dataPoint){
        var seriesSize = this.dataSeries.length;
        for (var seriesIndex = 0; seriesIndex < seriesSize; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if(dataSeries.points[0] == dataPoint){
                return seriesIndex;
            }
        }
        return 0;
    },

   deal4EachCategory: function(categoryIndex) {
        this.initLabelData(categoryIndex);
        this.initSeriesBounds(categoryIndex);
        this.dealLabelBounds(categoryIndex);
        this.dealWithDataSeries(categoryIndex);
    },

   dealWithDataSeries: function(categoryIndex) {
        var seriesSize = this.dataSeries.length;
        for (var seriesIndex = 0; seriesIndex < seriesSize; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPointTo = dataSeries.points[categoryIndex];
            var dataPointFrom;
            if (seriesIndex > 0) {
                dataPointFrom =this.dataSeries[seriesIndex - 1].points[categoryIndex];
            } else {
                dataPointFrom = dataPointTo;
            }
            if (dataPointTo.valueIsNull) {
                continue;
            }
            var shapeGlyph = this.calculateShapeGlyph4DifferentSub(dataPointFrom, dataPointTo);
            dataPointTo.glyph = (shapeGlyph);
        }
    },

   calculateShapeGlyph4DifferentSub: function(dataPointFrom, dataPointTo) {
        var path = new FR.Chart.GeneralPath();
        var fromIndex = this._indexOfDataPoint(dataPointFrom);
        var toIndex = this._indexOfDataPoint(dataPointTo);
        if (fromIndex != toIndex) {
            path.moveTo(this.seriesX[fromIndex], this.seriesY[fromIndex] + this.SERIES_GAP);
            path.lineTo(this.seriesX[fromIndex] + this.seriesWidth[fromIndex], this.seriesY[fromIndex] + this.SERIES_GAP);
            path.lineTo(this.seriesX[toIndex] + this.seriesWidth[toIndex], this.seriesY[toIndex] - this.SERIES_GAP);
            path.lineTo(this.seriesX[toIndex], this.seriesY[toIndex] - this.SERIES_GAP);
            path.closePath();
        }
        var shapeGlyph = new FR.Chart.ShapeGlyph();
        shapeGlyph.setShape(path);
        shapeGlyph.info.dealCondition(this.conditionCollection, dataPointTo, this.createColors4Series());
        shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();
        return shapeGlyph;
    },

   isGuideLineShow: function(dataPoint) {
        var textGlyph = dataPoint.dataLabel;
        if (textGlyph == null) {
            return false;
        }

        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if (attrContents) {
            var label = textGlyph.text;
            return label != null && attrContents.isShowGuidLine && textGlyph.bounds != null;
        }
        return false;
   },

   isCoverLabelBounds: function(path, bounds) {
        if (bounds == null) {
            return false;
        }
        if (!path.verticalIntersects(bounds.y, bounds.height)) {
            var oldBounds = new FR.Chart.Bounds(bounds.x, bounds.y, bounds.width + 2, bounds.height);
            path.append(oldBounds, false);
            return false;
        }
        return true;
    },

   checkCoverLabelBounds: function(categoryIndex) {
        var path = new FR.Chart.GeneralPath();
        for (var i = 0; i < this.dataSeries.length; i ++) {
            var dataPoint4Funnel =this.dataSeries[i].points[categoryIndex];
            if (dataPoint4Funnel == null) {
                continue;
            }
            var textGlyph = dataPoint4Funnel.dataLabel;
            if (textGlyph == null) {
                continue;
            }
            var label = textGlyph.text;
            if (label != null && label.length > 0) {
                if (this.isCoverLabelBounds(path, textGlyph.getBounds())) {
                    return true;
                }
            }
        }
        return false;
    },

   adjustCoverLabelBounds: function(categoryIndex) {
        var path = new FR.Chart.GeneralPath();
        for (var i = 0; i < this.dataSeries.length; i ++) {
            var dataPoint4Funnel =this.dataSeries[i].points[categoryIndex];
            if (dataPoint4Funnel == null) {
                continue;
            }
            var textGlyph = dataPoint4Funnel.dataLabel;
            if (textGlyph == null) {
                continue;
            }
            var label = textGlyph.text;
            if (label != null && label.length > 0) {
                if (this.isCoverLabelBounds(path, textGlyph.getBounds())) {
                    textGlyph.bounds = null;
                    dataPoint4Funnel.leadLine = null;
                }
            }
        }
    },

    /**
     * 处理标签边界
     */
   dealLabelBounds: function(categoryIndex) {
        for (var i = 0; i < this.dataSeries.length; i ++) {
            var dataPoint4Funnel =this.dataSeries[i].points[categoryIndex];
            if (dataPoint4Funnel == null) {
                continue;
            }
            var textGlyph = dataPoint4Funnel.dataLabel;
            if (textGlyph == null) {
                return;
            }
            var label = textGlyph.text;
            if (label != null && label.length > 0) {
                this.dealOutsideLabelBounds(textGlyph, dataPoint4Funnel, this._indexOfDataPoint(dataPoint4Funnel));
            }
        }
        if (this.checkCoverLabelBounds(categoryIndex)) {
            this.dealLabelBoundsWithCover(categoryIndex);
        }
        for (var i = 0; i < this.dataSeries.length; i ++) {
            var dataPoint4Funnel =this.dataSeries[i].points[categoryIndex];
            if (dataPoint4Funnel == null) {
                continue;
            }
            var textGlyph = dataPoint4Funnel.dataLabel;
            if (textGlyph == null) {
                return;
            }
            dataPoint4Funnel.initLabelBounds();
        }
    },

   dealLabelBoundsWithCover: function(categoryIndex) {
        for (var i = 0; i < this.dataSeries.length; i ++) {
            var dataPoint4Funnel =this.dataSeries[i].points[categoryIndex];
            if (dataPoint4Funnel == null) {
                continue;
            }
            var textGlyph = dataPoint4Funnel.dataLabel;
            if (textGlyph == null) {
                return;
            }
            var label = textGlyph.text;
            if (label != null && label.length > 0) {
                this.dealOutsideLabelBoundsWithCover(textGlyph, dataPoint4Funnel, this._indexOfDataPoint(dataPoint4Funnel));
            }
        }
        this.adjustCoverLabelBounds(categoryIndex);
    },

   dealOutsideLabelBounds: function(textGlyph, dataPoint, index) {
        var labelBounds = new FR.Chart.Bounds();
        var dimension = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(textGlyph.text, textGlyph.textAttr);
        var y = this.seriesY[index] - dimension.height / 2;
        labelBounds.setRect(this.labelX, y, this.labelWidth, dimension.height);
        this.dealOutsideLabelBoundsWhenNormal(labelBounds, dataPoint, index);
    },

   dealOutsideLabelBoundsWithCover: function(textGlyph, dataPoint, index) {
        var labelBounds = new FR.Chart.Bounds();
        var dimension = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(textGlyph.text, textGlyph.textAttr);
        var gap = (this.bounds.height * (1 - this.TOP_BOTTOM_GAP * 2)) / (this.seriesY.length - 1);
        var y = this.seriesY[0] + gap * index - dimension.height / 2;
        labelBounds.setRect(this.labelX, y, this.labelWidth, dimension.height);
        this.dealOutsideLabelBoundsWhenNormal(labelBounds, dataPoint);
    },

    /**
     * 设置系列标签和牵引线位置
     */
   dealOutsideLabelBoundsWhenNormal: function(labelBounds, dataPoint, index) {
        dataPoint.dataLabel.bounds = labelBounds;
        if (this.isGuideLineShow(dataPoint)) {
            this.dealWidthLine(dataPoint, index);
        }
    },

   dealWidthLine: function(dataPoint, index) {
        var path = new FR.Chart.GeneralPath();
        var startX = this.seriesX[index] + this.seriesWidth[index] + this.bounds.width * this.LEAD_LINE_GAP;
        var startY = this.seriesY[index];
        var midX = this.labelX - this.bounds.width * (this.LEAD_LINE_GAP + this.LABEL_GAP);
        var endX = this.labelX - this.bounds.width * this.LEAD_LINE_GAP;
        var endY = startY;
        if (dataPoint.dataLabel) {
            var bounds = dataPoint.dataLabel.bounds;
            if (bounds) {
                endY = bounds.y + bounds.height / 2;
            }
        }
        path.moveTo(startX, startY);
        path.lineTo(midX, endY);
        path.lineTo(endX, endY);
        var foldLine = new FR.Chart.FoldLine(path);
        dataPoint.leadLine = foldLine;
        var lineStyle = new FR.Chart.AttrLineStyle();
        lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
        foldLine.lineStyleInfo.attrLineStyle = lineStyle;
    },

    /**
     * draw方法.画出漏斗图对象.
     */
    draw: function (ctxs, time, actionModel) {
        this.drawInfo(ctxs);

        FR.Chart.PiePlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    }
});

/**
 * 三维饼图
 * Pie3DPlotGlyph.start
 *
 *      @example
 *      var 3dpie = new FR.Chart.Pie3DPlotGlyph();
 *
 * @class FR.Chart.Pie3DPlotGlyph
 * @extends FR.Chart.PiePlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.depth = 0] 三维深度.
 * @param {FR.Chart.Projection} [js.projection] 三维属性.
 */
FR.Chart.Pie3DPlotGlyph = FR.extend(FR.Chart.PiePlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.depth = FR.pick(js.depth, 0);
        this.projection = new FR.Chart.Projection(js.projection || {rotation:0, elevation:25});

        FR.Chart.Pie3DPlotGlyph.superclass.initConstants.apply(this, [js]);

        this.pie3DList = []; //存放所有的side
    },


    dealEachCategory:function(categoryIndex){
        var eachWidth = this.bounds.width / this.getCategoryCount();
        this.initLabelData(categoryIndex);
        var boundsPath = new FR.Chart.BoundsManager();

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            dataPoint.lastGlyph = dataPoint.glyph;
            dataPoint.leadLine = null;
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataPoint.glyph = null;
                continue;
            }

            // 从90度开始    顺时针调整旋转
            var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;

            var centerX = categoryIndex * eachWidth + eachWidth / 2 + this.bounds.x;
            var cateCenterPoint = new FR.Chart.Point2D(centerX, this.bounds.height / 2 + this.bounds.y);

            var depth = this.radius / 5; // kunsnat: bug: 4304 默认厚度(666之后删掉厚度属性)
            var pie3D = new FR.Chart.Pie3D(0, depth, this.radius); // 此处的depth根据饼图大小决定
            var extent = this.getPercent(seriesIndex, categoryIndex) * 360;

            var ellipse = this.projection.projecteeArc(0, this.radius, seriesStartAngle, extent);
            var distanceX = this.separatePercent / 100.0 * ellipse.width / 2;
            var distanceY = this.separatePercent / 100.0 * ellipse.height / 2;
            if (!dataPoint.isSeparateOut) {
                distanceX = 0;
                distanceY = 0;
            }
            var halfAngle = ellipse.getMiddleAngle();
            halfAngle = (halfAngle / 180) * Math.PI + Math.PI;
            pie3D.x = cateCenterPoint.x + distanceX * Math.cos(halfAngle);
            pie3D.y = cateCenterPoint.y + distanceY * Math.sin(halfAngle);
            pie3D.start = seriesStartAngle;
            pie3D.extent = extent;
            pie3D.projection = this.projection;

            var shapeGlyph = new FR.Chart.ShapeGlyph();
            shapeGlyph.shape = pie3D;
            shapeGlyph.info.dealColorCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();
            dataPoint.glyph = shapeGlyph;

            this.pie3DList.push(new FR.Chart.Pie3DSide(shapeGlyph, true));
            this.pie3DList.push(new FR.Chart.Pie3DSide(shapeGlyph, false));

            this.dealLabelBounds(dataPoint, boundsPath);
        }
       this.dealWithCategoryLabels(categoryIndex,boundsPath);
    },


    relayLabelLine:function(dataPoint,labelBounds,labelAngle){
        var path = new FR.Chart.GeneralPath();
        var pie3D =dataPoint.glyph.shape;
        var startPoint = pie3D.bottom().getMiddlePoint();
        var centerAngle = this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex) ;
        if(centerAngle>=0 && centerAngle <=180){
            startPoint = pie3D.top().getMiddlePoint();
        }
        var rx = startPoint.x;
        var ry = startPoint.y;
        path.moveTo(rx,ry);
        var topEllipse =pie3D.top();
        var bigEllipse = new FR.Chart.Ellipse(topEllipse.x,topEllipse.y,topEllipse.width*1.2,topEllipse.width*0.93,topEllipse.start,topEllipse.extent,topEllipse.type);
        startPoint = bigEllipse.getPointWithAngle((180-labelAngle+360)%360);
        rx = startPoint.x;
        ry = startPoint.y+3;
        path.lineTo(rx,ry);
        if(centerAngle >=90 && centerAngle <270){
            rx -= this.radius * 0.1;
        } else{
            rx += this.radius * 0.1;
        }
        path.lineTo(rx,ry);
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        if(attrContents.isShowGuidLine && dataPoint.isShowLabel){
            var foldLine = new FR.Chart.FoldLine(path);
            dataPoint.leadLine = foldLine;

            var lineStyle = new FR.Chart.AttrLineStyle();
            lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
            foldLine.lineStyleInfo.attrLineStyle = lineStyle;
        }
    },

    calculateLabelBoundsWithAngle:function(centerAngle,dataPoint,labelBounds){
        var pie3D =dataPoint.glyph.shape;
        var dim = this.getLabelDim(dataPoint);
        var topEllipse =pie3D.top();
        var bigEllipse = new FR.Chart.Ellipse(topEllipse.x,topEllipse.y,topEllipse.width*1.2,topEllipse.width*0.93,topEllipse.start,topEllipse.extent,topEllipse.type);
        var startPoint = bigEllipse.getPointWithAngle((180-centerAngle+360)%360);
        var rx = startPoint.x;
        var ry = startPoint.y;
        if(centerAngle >=90 && centerAngle <270){
            rx -= this.radius * 0.1+2;
        } else{
            rx += this.radius * 0.1+2;
        }
        // 垂直角度 左右分开
        if(centerAngle >=90 && centerAngle < 270){
            labelBounds.setRect(rx-dim.width,ry - dim.height/2,dim.width,dim.height);
        }else{
            labelBounds.setRect(rx,ry - dim.height/2,dim.width,dim.height);
        }
    },


    dealWithLabelLine:function(dataPoint){
        this.relayLabelLine(dataPoint,dataPoint.dataLabel.bounds,this.getCenterAngle4DataPoint(dataPoint.categoryIndex,dataPoint.seriesIndex));
    },

    halfHeight4Shape:function(drawDataPoint){
        if(drawDataPoint && drawDataPoint.glyph && drawDataPoint.glyph.shape){
            var pie3D = drawDataPoint.glyph.shape;
            var topEllipse = pie3D.top();
            return topEllipse.width * 0.93/2 - 3;
        }
        return 0;
     },

    removeAngleExtendMinPoint:function(pointsArray){
        if(pointsArray == null || pointsArray.length <=0){
            return ;
        }
        var minIndex = -1;
        for(var i = 1;i<pointsArray.length ; i++){
            var beforeAngle = 0;
            var afterAngle = 0;
            if(pointsArray[i].glyph == null){
                pointsArray[i].isShowLabel = false;
                continue ;
            }
            minIndex = minIndex == -1?i:minIndex;
            if (pointsArray[i].glyph.shape instanceof FR.Chart.Pie3D) {
                 beforeAngle =pointsArray[i].glyph.shape.extent;
            }
            if(pointsArray[minIndex].glyph.shape instanceof FR.Chart.Pie3D) {
                 afterAngle = pointsArray[minIndex].glyph.shape.extent;
            }
            if(beforeAngle < afterAngle){
                minIndex = i;
            }
        }

        if(pointsArray[minIndex]){
            pointsArray[minIndex].isShowLabel = false;
            pointsArray.splice(minIndex,1);
        }
    },

    // 三围饼图和二维的角度不太一样，重写
    whileTmp: function (path, bounds, tmpAngle, mx, my, R, incAngle) {
        if (R == null || incAngle == null) {// TODO kunsnat: 这边为什么要去 父类计算呢.  父类这个方法最好能私有.
            return FR.Chart.Pie3DPlotGlyph.superclass.whileTmp.apply(this, [path, bounds, tmpAngle, mx, my]);
        }

        tmpAngle = (180 - tmpAngle + 360) % 360;

        var backup = bounds.clone();
        var tt = 0;
        while (path.isInteract(bounds)) {
            tmpAngle += incAngle;

            if (tt > 360) {
                bounds.setRect(backup.x, backup.y, backup.width, backup.height);
                break;
            }

            var rx = mx - R * Math.cos(Math.PI * tmpAngle / 180);
            var ry = my - R * Math.sin(Math.PI * tmpAngle / 180);

            // 垂直角度 左右分开
            if (tmpAngle >= 315 || tmpAngle < 90) { // 90度的跳跃点(差别大.)
                bounds.setRect(rx - bounds.width, ry - bounds.height / 2, bounds.width, bounds.height);
            } else if (tmpAngle >= 270 && tmpAngle < 315) {
                bounds.setRect(rx - bounds.width, ry - bounds.height, bounds.width, bounds.height);
            } else if (tmpAngle >= 225 && tmpAngle < 270) {
                bounds.setRect(rx, ry - bounds.height, bounds.width, bounds.height);
            } else {
                bounds.setRect(rx, ry - bounds.height / 2, bounds.width, bounds.height);
            }
            tt += incAngle;
        }
    },

    getCenterAngle4DataPoint:function(categoryIndex,seriesIndex){
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var extent = this.getPercent(seriesIndex, categoryIndex) * 360;
        var ellipse = this.projection.projecteeArc(0, this.radius, seriesStartAngle, extent);
        var centerAngle =  ellipse.getMiddleAngle();
        return (180 - centerAngle + 360) % 360;
    },

    getSeriesCountWithQuadrant: function (categoryIndex) {
        this.leftUp.length = 0;
        this.leftDown.length = 0;
        this.rightDown.length = 0;
        this.rightUp.length = 0;
        for (var i = 0; i < this.dataSeries.length; i++) {

            if (this.dataSeries[i].points[categoryIndex].valueIsNull) {
                continue;
            }
            this.dataSeries[i].points[categoryIndex].isShowLabel = true;
            var centerAngle = this.getCenterAngle4DataPoint(categoryIndex,i);
            var dataPoint = this.dataSeries[i].points[categoryIndex];
            if (centerAngle >= 90 && centerAngle < 180) {
                this.leftUp.push(dataPoint);
            } else if (centerAngle >= 180 && centerAngle < 270) {
                this.leftDown.push(dataPoint);
            } else if (centerAngle >= 270 && centerAngle < 360) {
                this.rightDown.push(dataPoint);
            } else {
                this.rightUp.push(dataPoint);
            }
        }
        this._sortList();
    },

    dealInsideLabelBounds: function (dataPoint, boundsPath) {
        var seriesIndex = dataPoint.seriesIndex;
        var categoryIndex = dataPoint.categoryIndex;
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var centerAngle = this.getHalfAngle(seriesStartAngle, seriesIndex, categoryIndex) % 360;
        if (centerAngle == seriesStartAngle) {// 如果为0值的时候, 直接在饼图外处理
            this.dealOutsideLabelBounds(dataPoint, boundsPath);
            return;
        }
        var textGlyph = dataPoint.dataLabel;
        var centerPoint = dataPoint.getShape().getTopArcCenterPoint();
        if (centerPoint != null) {
            var dim = this.getLabelDim(dataPoint);
            var tg_width = dim.width;
            var tg_height = dim.height;
            var labelX = centerPoint.x - tg_width / 2.0;
            var labelY = centerPoint.y - tg_height / 2.0;
            textGlyph.bounds = new FR.Chart.Bounds(labelX, labelY, tg_width, tg_height);
        } else {
            textGlyph.bounds = null;
        }
    },

    dealLeadLineSeries: function () {
        for (var categoryIndex = 0; this.dataSeries[0] && categoryIndex < this.dataSeries[0].points.length; categoryIndex++) {
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataSeries = this.dataSeries[seriesIndex];
                var dataPoint = dataSeries.points[categoryIndex];

                if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    dataPoint.glyph = null;
                    continue;
                }

                var centerAngle = dataPoint.getShape().top().getMiddleAngle();

                if (dataPoint.glyph && dataPoint.glyph.shape) {
                    this.dealLeadLine(dataPoint.glyph.shape, dataPoint, centerAngle % 360);
                }
            }
        }
    },

    /**
     * 动态系列 画法 (考虑系列 画法顺序等)
     * @param ctx
     * @param plotGlyph
     * @param count
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        this.pie3DList = [];
        for (var index = 0; index < plotGlyph.dataSeries.length; index++) {
            var dataSeries = plotGlyph.dataSeries[seq[index]];

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var dataPoints = dataSeries.points;
            for (var i = 0; i < dataSeries.getDataPointCount(); i++) {
                if(dataPoints[i].valueIsNull){
                    continue;
                }
                if( dataPoints[i].animatesGlyph && dataPoints[i].animatesGlyph.length > count){
                    this.pie3DList.push(new FR.Chart.Pie3DSide(dataPoints[i].animateGlyph[count], true));
                    this.pie3DList.push(new FR.Chart.Pie3DSide(dataPoints[i].animateGlyph[count], false));
                }
            }
        }

        this.pie3DList.sort(FR.Chart.ChartUtils.sortPie3DSide);
        this.drawSide(ctx);

        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);
        }
    },

    /**
     * 返回三维饼图 系列的顺序
     * @returns {Array}  返回顺序.
     */
    getDrawSequence: function () {
        var seq = [];
        var points = [];
        for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
            seq.push(i);
            points.push(this.dataSeries[i].points[0]);
        }
        return seq.sort(function (x, y) {
            var glyphX = points[x].glyph;
            var glyphY = points[y].glyph;
            if (!glyphX || !glyphY) {
                return 0;
            }
            var angleX = Math.abs(270 - glyphX.shape.top().start);
            var angleY = Math.abs(270 - glyphY.shape.top().start);
            return angleX > angleY ? -1 : (angleX < angleY ? 1 : 0);
        });
    },

    drawSide: function (ctx) {

        for (var i = 0; i < this.pie3DList.length; i++) {
            if(!ctx){     return; } ctx.save();
            var cylinder = this.pie3DList[i].cylinder;
            if (!!cylinder) {
                if (cylinder.extent === 0) {
                    continue;
                }
                ctx.fillStyle = this.pie3DList[i].shapeGlyph.info.seriesAttrBackground.seriesBackground.color;
                if (this.pie3DList[i].isStart) {
                    cylinder.paintStartSide(ctx);
                } else {
                    cylinder.paintEndSide(ctx);
                }
            }
            ctx.restore();
        }
    }

});

/**
 * 组合的饼图(柱形, 饼图)
 *
 * @class FR.Chart.CompositePiePlotGlyph
 * @extends FR.Chart.PiePlotGlyph
 *
 * @param {JSON} js 属性配置
 *
 * @param {Number} [js.subType = 1] 组合类型
 * @param {Number} [js.smallPercent = 0.05] 默认第二组合中的百分比
 */
FR.Chart.CompositePiePlotGlyph = FR.extend(FR.Chart.PiePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.subType = FR.pick(js.subType, 2);
        this.smallPercent = FR.pick(js.smallPercent, 0.05);
        FR.Chart.CompositePiePlotGlyph.superclass.initConstants.apply(this, [js]);

        this.Pie_Normal = 1,
            this.Pie_Pie = 2,
            this.Pie_Bar = 3,

            this.SMALLPERCENT_LABEL_RADIUS_COEFFICIENT = 0.7;

        this.list = [];
    },


    //计算百分比的值
    //组合饼图计算百分比需考虑第二绘图区的合并值
    calculatePercentValue:function(){
        //计算下percentValue
        for(var i = 0; i < this.getCategoryCount(); i++){
            var total = 0;
            var smallTotal = 0;
            //主饼的len
            var mainlLen = this.dataSeries.length - 1;
            for(var j = 0; j < mainlLen; j++){
                if(this.dataSeries[j].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                    continue;
                }
                total += Math.abs(this.dataSeries[j].points[i].value);
            }

            for(var j = 0; j < mainlLen; j++){
                if(this.dataSeries[j].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                    continue;
                }
                var dataPoint = this.dataSeries[j].points[i];
                var value = Math.abs(dataPoint.value);
                dataPoint.percentValue = value / total;
                if (dataPoint.percentValue < this.smallPercent){
                    smallTotal += value;
                }
            }

            if(mainlLen > 0){
                var dataPoint = this.dataSeries[mainlLen].points[i];
                dataPoint.value = smallTotal;
                dataPoint.percentValue = smallTotal / total;
            }
        }
    },

    /**
     *获取数据点的lastGlyph
     * 因为组合饼图有可能出现。原来是bar，现在是arc。计算animationglyph会出问题。
     * 所以此处返回null
     * @param dataPoint
     */
    getLastDataPointGlyph:function(dataPoint){
        return null;
    },

    configSeries:function(seriesConfig){
        this.subType = 2;
        if(seriesConfig.subType && seriesConfig.subType == "bar"){
            this.subType = 3;
        }
        this.smallPercent = FR.pick(seriesConfig.smallPercent, 0.05);
    },

    /**
     * 系列布局.
     * 重新生成计算之后的百分比, 只是标签.
     */
    layoutDataSeriesGlyph: function () {
        this.list.length = 0;
        for (var categoryIndex = 0; this.dataSeries[0] && categoryIndex < this.dataSeries[0].points.length; categoryIndex++) {
            this.initLabelData(categoryIndex);
            this.dealWithDataSeries(categoryIndex);
            var linePath = this.getLinePath(categoryIndex);
            if (linePath != null) {
                this.list.push(linePath);
            }
        }
        this.dealLabelBoundsInOrder();// 主要针对数据标签内的标签
    },

    /**
     * 是否为 扇面外的标签
     * @param dataPoint 系列点
     * @returns {Boolean}
     */
    isOutsideLabel: function (dataPoint) {
        var attrContents = this.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
        return (attrContents.position == FR.Chart.Constants.OUTSIDE || attrContents.isShowGuidLine)
            && this._isMainPie(dataPoint.seriesIndex, dataPoint.categoryIndex);
    },

    /**
     * 返回系列 分类所占百分比
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {number} 返回百分比
     */
    getPercent: function (seriesIndex, categoryIndex) {
        if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return 0;
        }
        var isMainPie = this._isMainPie(seriesIndex, categoryIndex);
        var totalValue = this.getTotalValue(categoryIndex, isMainPie);
        if (totalValue <= 0) {
            return 0;
        }
        var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
        return Math.abs(dataPoint.value) / totalValue;
    },

    /**
     * 返回 分类的总值 (组合和非组合的分开汇总)
     * @param categoryIndex 分类序号 (其实就是单独的整个饼图)
     * @param isMore 是否为more
     * @returns {Number} 返回分类总值.
     */
    getTotalValue: function (categoryIndex, isMore) {
        var totalValue = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
            var percentValue = dataPoint.percentValue;
            if (isMore) {
                if (percentValue >= this.smallPercent || seriesIndex == this.dataSeries.length - 1) {
                    totalValue += Math.abs(dataPoint.value);
                }
            } else {
                if (percentValue < this.smallPercent && seriesIndex != this.dataSeries.length - 1) {
                    totalValue += Math.abs(dataPoint.value);
                }
            }
        }
        return totalValue;
    },

    /**
     * 返回 中心点
     * @param seriesIndex
     * @param categoryIndex
     * @returns {*}
     */
    getCenterPoint: function (seriesIndex, categoryIndex) {
        var dataSeries = this.dataSeries[seriesIndex];
        var dataPoint = dataSeries.points[categoryIndex];
        if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return null;
        }

        var plotBounds = this.getBounds();
        var eachWidth = plotBounds.width / this.dataSeries[0].points.length;
        if (this.subType == this.Pie_Pie || this.subType == this.Pie_Bar) {
            var centerY = plotBounds.y + plotBounds.height / 2;
            if (dataPoint.percentValue < this.smallPercent && seriesIndex != this.dataSeries.length - 1) {
                var centerBX = categoryIndex * eachWidth + eachWidth * 0.8 + plotBounds.x;
                return new FR.Chart.Point2D(centerBX, centerY);
            } else {
                var centerX = categoryIndex * eachWidth + eachWidth * 0.3 + plotBounds.x;
                return new FR.Chart.Point2D(centerX, centerY);
            }
        }
        return null;
    },

    /**
     *返回 两个点之间的角度差.
     * @param radius 半径
     * @param centerPoint 中心点
     * @param valuePoint 值点.
     * @returns {number} 角度值.
     */
    getAngle4Pie: function (radius, centerPoint, valuePoint) {
        var xLength = Math.abs(centerPoint.x - valuePoint.x);
        var yLength = Math.abs(centerPoint.y - valuePoint.y);

        var xAngle = Math.atan(yLength / xLength); // 0.27829965900511133
        var yAngle = Math.acos(radius * Math.sin(xAngle) / yLength);// 0.9890997676289591

        return xAngle + yAngle;
    },

    /**
     * 返回 组合饼图中的相关牵引线
     * @param categoryIndex 分类序号.
     * @returns {FR.Chart.GeneralPath} 返回牵引线.
     */
    getLinePath: function (categoryIndex) {
        if (this.subType == this.Pie_Pie && this.getPercent(this.dataSeries.length - 1, categoryIndex) > 0) {
            return this._getLinePath4PieSub(categoryIndex);
        } else if (this.subType == this.Pie_Bar && this.getPercent(this.dataSeries.length - 1, categoryIndex) > 0) {
            return this._getLinePath4BarSub(categoryIndex);
        }
        return null;
    },

    _getLinePath4BarSub: function (categoryIndex) {
        var linePath = new FR.Chart.GeneralPath();

        var pie = this.dataSeries[this.dataSeries.length - 1].points[categoryIndex];
        var arc = pie.glyph.shape;

        var highPoint = arc.getEndPoint().y > arc.getStartPoint().y ? arc.getStartPoint() : arc.getEndPoint();
        var lowPoint = arc.getEndPoint().y > arc.getStartPoint().y ? arc.getEndPoint() : arc.getStartPoint();
        var eachWidth = this.getBounds().width / this.dataSeries[0].points.length;
        var centerBX = categoryIndex * eachWidth + eachWidth * 0.8 + this.getBounds().x;
        var bCenterPoint = new FR.Chart.Point2D(centerBX, this.getBounds().height / 2 + this.getBounds().y);
        if (highPoint) {
            if (arc.extent >= 180) {
                var arcCenterPoint = new FR.Chart.Point2D(arc.x, arc.y);
                var angle = this.getAngle4Pie(this.radius, arcCenterPoint,
                    new FR.Chart.Point2D(bCenterPoint.x - this.radius / 4, bCenterPoint.y - this.radius / 2));
                highPoint = new FR.Chart.Point2D(arcCenterPoint.x + Math.cos(angle) * this.radius,
                    arcCenterPoint.y - Math.sin(angle) * this.radius + 1);
            }

            linePath.moveTo(bCenterPoint.x - this.radius / 4, bCenterPoint.y - this.radius / 2);
            linePath.lineTo(highPoint.x, highPoint.y);
        }

        if (lowPoint) {
            if (arc.extent >= 180) {
                var arcCenterPoint = new FR.Chart.Point2D(arc.x, arc.y);
                var angle = this.getAngle4Pie(this.radius, arcCenterPoint,
                    new FR.Chart.Point2D(bCenterPoint.x - this.radius / 4, bCenterPoint.y - this.radius / 2));
                lowPoint = new FR.Chart.Point2D(arcCenterPoint.x + Math.cos(angle) * this.radius,
                    arcCenterPoint.y + Math.sin(angle) * this.radius - 1);
            }

            linePath.moveTo(lowPoint.x, lowPoint.y);
            linePath.lineTo(bCenterPoint.x - this.radius / 4, bCenterPoint.y + this.radius / 2);
        }
        return linePath;
    },

    _getLinePath4PieSub: function (categoryIndex) {
        var linePath = new FR.Chart.GeneralPath();

        var pie = this.dataSeries[this.dataSeries.length - 1].points[categoryIndex];
        var arc = pie.glyph.shape;

        var highPoint = arc.getEndPoint().y > arc.getStartPoint().y ? arc.getStartPoint() : arc.getEndPoint();
        var lowPoint = arc.getEndPoint().y > arc.getStartPoint().y ? arc.getEndPoint() : arc.getStartPoint();

        var topPoint = this._getSubLineFrom(categoryIndex, true);
        var bottomPoint = this._getSubLineFrom(categoryIndex, false);

        if (topPoint) {
            if (arc.extent >= 180) {
                var arcCenterPoint = new FR.Chart.Point2D(arc.x, arc.y);
                var topAngle = this.getAngle4Pie(this.radius, arcCenterPoint, topPoint);
                highPoint = new FR.Chart.Point2D(arcCenterPoint.x + Math.cos(topAngle) * this.radius,
                    arcCenterPoint.y - Math.sin(topAngle) * this.radius);
            }

            linePath.moveTo(topPoint.x, topPoint.y);
            linePath.lineTo(highPoint.x, highPoint.y);
        }

        if (bottomPoint) {
            if (arc.extent >= 180) {
                var arcCenterPoint = new FR.Chart.Point2D(arc.x, arc.y);
                var bottomAngle = this.getAngle4Pie(this.radius, arcCenterPoint, bottomPoint);
                lowPoint = new FR.Chart.Point2D(arcCenterPoint.x + Math.cos(bottomAngle) * this.radius,
                    arcCenterPoint.y + Math.sin(bottomAngle) * this.radius);
            }

            linePath.moveTo(lowPoint.x, lowPoint.y);
            linePath.lineTo(bottomPoint.x, bottomPoint.y);
        }

        return linePath;
    },

    /**
     * 返回系列起始的角度
     * @param categoryIndex 分类序号
     * @param seriesIndex 系列序号
     * @returns {*}
     */
    getSeriesStartAngle: function (categoryIndex, seriesIndex) {
        var dataPoint4Pie = this.dataSeries[seriesIndex].points[categoryIndex];
        var isTest = dataPoint4Pie.percentValue >= this.smallPercent || seriesIndex == this.dataSeries.length - 1;
        var startAngle = this.getStartAngle(seriesIndex, categoryIndex);
        for (var i = 0; i < seriesIndex; i++) {
            var tmp = this.dataSeries[i].points[categoryIndex];
            if (isTest) {
                if (tmp.percentValue >= this.smallPercent || i == this.dataSeries.length - 1) {
                    startAngle = startAngle + this.getPercent(i, categoryIndex) * 360;
                }
            } else {
                if (tmp.percentValue < this.smallPercent && i != this.dataSeries.length - 1) {
                    startAngle = startAngle + this.getPercent(i, categoryIndex) * 360;
                }
            }
        }
        return startAngle;
    },

    /**
     * 返回系列点 对应的shape
     * @param dataPoint 系列点
     * @returns {Shape} 系列点图形
     */
    getArc2D: function (dataPoint) {
        var categoryIndex = dataPoint.categoryIndex;
        var seriesIndex = dataPoint.seriesIndex;
        var seriesStartAngle = this.getSeriesStartAngle(categoryIndex, seriesIndex) % 360;
        var centerPoint = this.getCenterPoint(seriesIndex, categoryIndex);
        if (!(dataPoint.percentValue < this.smallPercent && seriesIndex != this.dataSeries.length - 1)) {
            return this.getDataPointShape(centerPoint, seriesIndex, categoryIndex, seriesStartAngle, this.radius, dataPoint.isSeparateOut);
        } else if (this.subType == this.Pie_Pie) {
            return this.getDataPointShape(centerPoint, seriesIndex, categoryIndex, seriesStartAngle, this.radius / 2, dataPoint.isSeparateOut);
        } else if (this.subType == this.Pie_Bar) {
            return this._get4PieBar(seriesIndex, categoryIndex, centerPoint, this.radius / 2);
        }
        return null;
    },

    _isMainPie: function (seriesIndex, categoryIndex) {
        var dataPoint = this.dataSeries[seriesIndex].points[categoryIndex];
        return dataPoint.percentValue >= this.smallPercent || seriesIndex == this.dataSeries.length - 1;
    },

    _get4PieBar: function (seriesIndex, categoryIndex, centerPoint, radius) {
        var preSumPercent = 0;
        var isMainPie = this._isMainPie(seriesIndex, categoryIndex);
        for (var i = 0; i < seriesIndex; i++) {
            var tmpIsMain = this._isMainPie(i, categoryIndex);
            if (tmpIsMain == isMainPie) {//非主饼才加入堆积
                var percent = this.getPercent(i, categoryIndex);
                preSumPercent += percent;
            }
        }

        var dataPercent = this.getPercent(seriesIndex, categoryIndex);
        var bounds = new FR.Chart.Bounds(centerPoint.x - radius / 2, centerPoint.y - radius + preSumPercent * radius * 2,
            radius, radius * 2 * dataPercent);

        return bounds;
    },

    /**
     * 调整 半径
     * @param categoryIndex 分类序号
     * @param seriesIndex 系列序号
     * @returns {number} 调整之后的半径
     */
    changeRadius: function (categoryIndex, seriesIndex) {
        var isMainPie = this._isMainPie(seriesIndex, categoryIndex);
        return (isMainPie ? this.radius * this.OUTSIDE_LABEL_RADIUS_COEFFICIENT
            : this.radius * this.SMALLPERCENT_LABEL_RADIUS_COEFFICIENT) + this.separatePercent * this.radius / 100;
    },

    /**
     * 返回开始的角度
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号.
     * @returns {number} 开始的角度.
     */
    getStartAngle: function (seriesIndex, categoryIndex) {
        var dataPie = this.dataSeries[seriesIndex].points[categoryIndex];
        if (dataPie.percentValue >= this.smallPercent || seriesIndex == this.dataSeries.length - 1) {
            var angle = this.getPercent(this.dataSeries.length - 1, categoryIndex) * 180;
            return angle;
        }
        return 90;
    },

    /**
     * 返回 牵引线的点位置
     * @param categoryIndex 分类序号
     * @param isTop 是否为上 (对应下) 的起始点
     * @returns {FR.Chart.Point2D} 点位置.
     */
    _getSubLineFrom: function (categoryIndex, isTop) {
        var point = null;

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var dataPoint = dataSeries.points[categoryIndex];
            if (dataPoint.valueIsNull || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            var pie = dataPoint;
            var arc = pie.glyph.shape;

            if (pie.percentValue < this.smallPercent && seriesIndex != this.dataSeries.length - 1) {
                var bounds = arc.getBounds2D(); // 得到最低的y --> x. Point2D

                var y = bounds.y;
                if (!isTop) {
                    y = bounds.y + bounds.height;
                }

                var R = arc.radius;
                var my = arc.y;
                var mx = arc.x;

                var sina = (my - y) / R;

                sina = Math.min(1, sina);// 范围-1 到1
                sina = Math.max(-1, sina);

                var tmpAngle = Math.asin(sina) * 180 / Math.PI;

                var x = mx + R * Math.cos(Math.PI * tmpAngle / 180);

                if (point == null || (isTop && y <= point.y) || (!isTop && y >= point.y)) {
                    point = new FR.Chart.Point2D(x, y);
                }
            }
        }

        return point;
    },

    /**
     * 处理调整 合适的半径
     * @param resultD 结果直径
     * @param categoryIndex 分类序号
     * @returns {number} 返回合适的半径
     */
    deal4BestRad: function (resultD, categoryIndex) {
        return FR.Chart.CompositePiePlotGlyph.superclass.deal4BestRad.apply(this, [resultD, categoryIndex]) * 0.8;
    },

    /**
     * 在组合饼图 动态结束之后 画上标记 标签 牵引线等.
     * @param ctx 画布
     * @param currentDivKey 当前div的key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        FR.Chart.CompositePiePlotGlyph.superclass.drawMarkersLabelAfterSeriesAnimateDraw.apply(this, [ctx, currentDivKey]);
        if(!ctx){     return; } ctx.save();
        ctx.strokeStyle = "rgb(125, 125, 125)";
        for (var i = 0; i < this.list.length; i++) {
            var path = this.list[i];
            path.draw(ctx);
        }
        ctx.restore();
    }

});

/**
 * AreaPlotGlyph.start
 * 面积图
 *
 *      @example
 *      var area = new FR.Chart.AreaPlotGlyph();
 *
 * @class FR.Chart.AreaPlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Boolean} [js.isCurve = false] 是否曲线连接
 *
 */
FR.Chart.AreaPlotGlyph = FR.extend(FR.Chart.CategoryPlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isCurve = js.isCurve;

        FR.Chart.AreaPlotGlyph.superclass.initConstants.apply(this, [js]);

    },

    configSeries:function(seriesConfig){
        this.isCurve = FR.pick(seriesConfig.isCurve, false);
        var defaultAttr = this.conditionCollection.defaultAttr;
        if(seriesConfig.markerType){
            var attrMarker = new FR.Chart.AttrMarkerType();
            attrMarker.markerType = seriesConfig.markerType;
            defaultAttr.addAttr(attrMarker);
        }
    },

    isSupportLabelWrap:function(){
        return true;
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries == null) {
                continue;
            }

            dataSeries.lastGlyph = this.getLastSeriesGlyph(dataSeries);

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries.glyph = null;
                continue;
            }

            var adjustType =this.dealAreaShape4PlotGlyph(dataSeries, this, this.xAxisGlyph, this.getSeriesAxisGlyph(seriesIndex));
            if(adjustType !== -1){
                this.dealAreaLargeData4StockChart(dataSeries, this, this.xAxisGlyph, this.getSeriesAxisGlyph(seriesIndex),adjustType);
            }

            if(this.isStockChart){
                this.refreshDataPointLabel();
            }
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     * 为面积图的大数据量的stock charts 数据重叠调整
     * @param dataSeries
     * @param categoryPlotGlyph
     * @param xAxisGlyph
     * @param yAxisGlyph
     * @param adjustType 调整类型
     */
    dealAreaLargeData4StockChart : function(dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph,adjustType){
        if (dataSeries == null) {
            return;
        }

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            dataSeries.points[categoryIndex].revert2Original();
        }

        var currentCombineDate = null;
        var combineDataPoint = null;
        var currentCombineX = -1;

        var stockDataPoints = [];

        //调整合并
        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];
            if(dataPoint.valueIsNull){
                dataPoint.glyph = null;
                continue ;
            }

            var dataPointX = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex)).x;
            if (this.isDataPointXNotInPlotBounds(dataPointX)) {
                dataPoint.glyph = null;
                continue;
            }

            var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
            if (stockDataPoints.length === 0) {
                currentCombineDate = cateDate;
                stockDataPoints[0] = dataPoint;
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                combineDataPoint = dataPoint;
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
                continue;
            } else if (FR.Chart.ChartUtils.isBelongToSameUnit(cateDate, currentCombineDate, adjustType)) {//同一范围数值累加
                combineDataPoint.addDataPoint4StockCategory(dataPoint);
                combineDataPoint.stockDataPointCount++;
                dataPoint.valueIsNull = true;
                continue;
            }


            var stockIndex = this.getStockDateRangeIndex(stockDataPoints, cateDate, xAxisGlyph, adjustType);
            //是否属于之前的一个统计区间
            if (stockIndex !== -1) {
                stockDataPoints[stockIndex].addDataPoint4StockCategory(dataPoint);
                dataPoint.valueIsNull = true;
                combineDataPoint.stockDataPointCount++;
            } else { //属于新的统计区间

                currentCombineX = parseInt(dataPointX);
                dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustType);
                stockDataPoints[stockDataPoints.length] = dataPoint;
                currentCombineDate = cateDate;
                combineDataPoint = dataPoint;
                combineDataPoint.stockDataPointCount = 1;
                currentCombineX = parseInt(dataPointX);
            }
        }


        //折线图面积图stock chart调整区间内是以平均值来展示的
        var stockCount = stockDataPoints.length;
        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];
            stockPoint.value = stockPoint.value / stockPoint.stockDataPointCount;
            stockPoint.percentValue = stockPoint.percentValue / stockPoint.stockDataPointCount;
        }

        //刷新值轴
        yAxisGlyph.oldMin = yAxisGlyph.minValue;
        yAxisGlyph.oldMax = yAxisGlyph.maxValue;
        yAxisGlyph.oldTick = yAxisGlyph.tickLength;

        yAxisGlyph.initMinMaxValue(this.getMinValueFromData(yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));
        yAxisGlyph.calculateAxisGlyph(this.bounds);
        xAxisGlyph.stockChartMainType = adjustType;


        //布局图形

        var isSeriesCurve = this.isSeriesCurve(dataSeries);
        var topLinePath = new FR.Chart.GeneralPath();
        var topFoldLine = new FR.Chart.FoldLine(topLinePath);
        dataSeries.topFoldLine = topFoldLine;

        var areaPaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        var sg = new FR.Chart.ShapeGlyph();
        sg.shape = areaPaths;
        dataSeries.glyph = sg;
        sg.info.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());
        if(!this.conditionCollection.isContainsAttr(dataSeries,new FR.Chart.AttrAlpha())){
            sg.info.seriesAttrAlpha.alpha = 0.5;
        }

        sg.toolTipBackground = sg.info.getBackground();

        // 上层折线  背景统一
        var color = 'rgb(0,0,255)';
        if (sg.info.seriesAttrBackground.seriesBackground instanceof FR.Chart.ColorBackground) {
            color = sg.info.seriesAttrBackground.seriesBackground.color;
        }
        var lineColor = new FR.Chart.AttrColor();
        lineColor.seriesColor = color;

        var lineStyle = new FR.Chart.AttrLineStyle();
        lineStyle.lineStyle = FR.Chart.Constants.LINE_THICK;

        topFoldLine.lineStyleInfo.attrLineColor = lineColor;
        topFoldLine.lineStyleInfo.attrLineStyle = lineStyle;

        // 准备数组
        var pre_value = [];

        for (var stockIndex = 0; stockIndex < stockCount; stockIndex++) {
            var stockPoint = stockDataPoints[stockIndex];
            var point_x = this.getStockChartDatePointX(xAxisGlyph,stockPoint,adjustType,this.getBounds());

            var pre_sum = categoryPlotGlyph.getPreSum4Area(dataSeries.seriesIndex, stockPoint.categoryIndex);
            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, stockPoint.categoryIndex);

            var point_y = yAxisGlyph.getPoint2D(pre_sum + value).y; // 系列堆积的值,
            pre_value[stockIndex] = yAxisGlyph.getPoint2D(pre_sum).y;   // 上个系列堆积的值
            this.dealLinePath4Area(isSeriesCurve,curvePaths,areaPaths,point_x,point_y,dataPoint);
        }


        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(areaPaths, curvePaths);
            curvePaths.clear();
        }

        if (isSeriesCurve) {
            for (var i = pre_value.length - 1; i >= 0; i--) {
                var dataPoint = stockDataPoints[i];

                if(dataPoint.valueIsNull){
                    continue;
                }

                var xValue = this.getStockChartDatePointX(xAxisGlyph,dataPoint,adjustType,this.getBounds());
                if (this.isDataPointXNotInPlotBounds(point_x)) {
                    continue;
                }
                if (i == pre_value.length - 1) {
                    curvePaths.moveTo(xValue, pre_value[i]);
                } else {
                    curvePaths.lineTo(xValue, pre_value[i]);
                }
            }
            FR.Chart.FoldLine.curveTo(areaPaths, curvePaths);
            curvePaths.clear();
        } else {
            for (var i = pre_value.length - 1; i >= 0; i--) {
                var dataPoint = stockDataPoints[i];

                if(dataPoint.valueIsNull){
                    continue;
                }

                var xValue = this.getStockChartDatePointX(xAxisGlyph,dataPoint,adjustType,this.getBounds());
                if (this.isDataPointXNotInPlotBounds(point_x)) {
                    continue;
                }
                areaPaths.lineTo(xValue, pre_value[i]);
            }
        }
        areaPaths.closePath();

        if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
            this.addChartStyle4DataSeries(this.plotStyle, dataSeries, sg, areaPaths);
            sg.dataPointStyle = dataSeries.dataSeriesStyle;
        }
    },


    dealLinePath4Area:function(isSeriesCurve,curvePaths,areaPaths,point_x,point_y,dataPoint){
        if (isSeriesCurve) {
            if (curvePaths.x.length === 0) {
                curvePaths.moveTo(point_x, point_y);
            } else {
                curvePaths.lineTo(point_x, point_y);
            }
        } else {
            if (areaPaths.x.length === 0) {
                areaPaths.moveTo(point_x, point_y);
            } else {
                areaPaths.lineTo(point_x, point_y);
            }
        }

        if(!this.isDataPointXNotInPlotBounds(point_x)){
            var markerGlyph = new FR.Chart.MarkerGlyph();
            markerGlyph.dealCondition4Area(this.conditionCollection, dataPoint, this.createColors4Series());
            markerGlyph.setShape(new FR.Chart.Bounds(point_x - FR.Chart.Constants.MarkerDefaultSize/2, point_y - FR.Chart.Constants.MarkerDefaultSize/2, FR.Chart.Constants.MarkerDefaultSize, FR.Chart.Constants.MarkerDefaultSize));
            markerGlyph.background = this.getBackground() !== null ? this.getBackground():this.wholeChartBackground;
            dataPoint.glyph = markerGlyph;
            markerGlyph.toolTipBackground = markerGlyph.background;
            this.dealDataPointLabel(dataPoint);
        }
    } ,


    /**
     * 处理 面积图的 ,返回-1表示布局成功，返回其他的表示stock charts情况下，有数据重叠，需要重新布局
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类绘图类型
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴.
     */
    dealAreaShape4PlotGlyph: function (dataSeries, categoryPlotGlyph, xAxisGlyph, yAxisGlyph) {
        if (dataSeries == null) {
            return;
        }

        xAxisGlyph.stockChartMinDateInt = null;
        xAxisGlyph.stockChartMainType = null;
        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
        }

        var topLinePath = new FR.Chart.GeneralPath();
        var topFoldLine = new FR.Chart.FoldLine(topLinePath);
        dataSeries.topFoldLine = topFoldLine;

        var areaPaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        var sg = new FR.Chart.ShapeGlyph();
        sg.shape = areaPaths;
        dataSeries.glyph = sg;
        sg.info.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());
        if(!this.conditionCollection.isContainsAttr(dataSeries,new FR.Chart.AttrAlpha())){
            sg.info.seriesAttrAlpha.alpha = 0.5;
        }

        sg.toolTipBackground = sg.info.getBackground();

        // 上层折线  背景统一
        var color = 'rgb(0,0,255)';
        if (sg.info.seriesAttrBackground.seriesBackground instanceof FR.Chart.ColorBackground) {
            color = sg.info.seriesAttrBackground.seriesBackground.color;
        }
        var lineColor = new FR.Chart.AttrColor();
        lineColor.seriesColor = color;

        var lineStyle = new FR.Chart.AttrLineStyle();
        lineStyle.lineStyle = FR.Chart.Constants.LINE_THICK;

        topFoldLine.lineStyleInfo.attrLineColor = lineColor;
        topFoldLine.lineStyleInfo.attrLineStyle = lineStyle;

        // 准备数组
        var pre_value = [];

        var crossValue = 0;
        if (yAxisGlyph.isLog) {
            crossValue = 1;
        }

        var isSeriesCurve = this.isSeriesCurve(dataSeries);
        var point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getCrossValue()).x;
        var point_y = yAxisGlyph.getPoint2D(crossValue).y;

        var lastPointX = -1;
        var lastDate = null;

        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];

            if(dataPoint.valueIsNull){
                continue ;
            }

            var isZoom = xAxisGlyph.zoomBackward !== 0 || xAxisGlyph.zoomForward !== 0 ;
            point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex)).x;
            if (this.isDataPointXNotInPlotBounds(point_x)) {
                dataPoint.glyph = null;
                if(!isZoom || (isZoom && dataPoint.isOutBounds)){
                    dataPoint.isOutBounds  = true;
                    continue;
                }
            }


            if(this.isStockChart && lastPointX === parseInt(point_x)){
                //纪录一下是在年月日还是时分秒重叠,返回下面要用的时间单位
                var cateDate =xAxisGlyph.getDate4Value(categoryIndex);
                return this.getStockChartAdjustDateType(cateDate,lastDate);
            }else{
                lastPointX = parseInt(point_x);
            }

            if(this.isStockChart){
                lastDate = xAxisGlyph.getDate4Value(categoryIndex);
            }

            var pre_sum = categoryPlotGlyph.getPreSum4Area(dataSeries.seriesIndex, categoryIndex);
            var value = categoryPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);
            point_y = yAxisGlyph.getPoint2D(pre_sum + value).y; // 系列堆积的值,
            pre_value[categoryIndex] = yAxisGlyph.getPoint2D(pre_sum).y;   // 上个系列堆积的值

            this.dealLinePath4Area(isSeriesCurve,curvePaths,areaPaths,point_x,point_y,dataPoint);
        }

        if (isSeriesCurve) {
            FR.Chart.FoldLine.curveTo(areaPaths, curvePaths);
            curvePaths.clear();
        }

        if (isSeriesCurve) {
            for (var i = pre_value.length - 1; i >= 0; i--) {
                var dataPoint = dataSeries.points[i];

                if(dataPoint.valueIsNull){
                    continue;
                }

                var xValue = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(i)).x;
                if (this.isDataPointXNotInPlotBounds(xValue) && dataPoint.isOutBounds) {
                    continue;
                }
                if (i == pre_value.length - 1) {
                    curvePaths.moveTo(xValue, pre_value[i]);
                } else {
                    curvePaths.lineTo(xValue, pre_value[i]);
                }
            }
            FR.Chart.FoldLine.curveTo(areaPaths, curvePaths);
            curvePaths.clear();
        } else {
            for (var i = pre_value.length - 1; i >= 0; i--) {
                var xValue = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(i)).x;
                var dataPoint = dataSeries.points[i];

                if(dataPoint.valueIsNull){
                    continue;
                }

                if (this.isDataPointXNotInPlotBounds(xValue) && dataPoint.isOutBounds) {
                    continue;
                }
                areaPaths.lineTo(xValue, pre_value[i]);
            }
        }
        areaPaths.closePath();

        if (this.plotStyle !== FR.Chart.Constants.STYLE_NONE) {
            this.addChartStyle4DataSeries(this.plotStyle, dataSeries, sg, areaPaths);
            sg.dataPointStyle = dataSeries.dataSeriesStyle;
        }

        return -1;
    },

    /**
     * 添加图表系列风格.
     * @param plotStyle 系列风格
     * @param dataSeries 系列
     * @param shapeGlyph 图形shape
     * @param areaPath 面积路径.
     */
    addChartStyle4DataSeries: function (plotStyle, dataSeries, shapeGlyph, areaPath) {
        if (shapeGlyph.info.getBackground().color) {
            var seriesColor = shapeGlyph.info.getBackground().color;
            var para = {'color': seriesColor, 'avoidOriginDraw': true, 'areaPath': areaPath, 'axisReversed': this.yAxisGlyph.axisReversed};
            if (plotStyle == FR.Chart.Constants.STYLE_3D) {
                shapeGlyph.info.seriesAttrAlpha.alpha = 1;
                dataSeries.dataSeriesStyle = new FR.Chart.AreaPlane3DStyle(para);
                dataSeries.dataSeriesStyle.animateGlyph = shapeGlyph;
            } else if (plotStyle == FR.Chart.Constants.STYLE_OUTER) {
                dataSeries.dataSeriesStyle = new FR.Chart.AreaHighLightStyle(para);
            } else if (plotStyle == FR.Chart.Constants.STYLE_SHADE) {
                dataSeries.dataSeriesStyle = new FR.Chart.AreaTopDownShadeStyle(para);
            } else if (plotStyle == FR.Chart.Constants.STYLE_TRANSPARENT) {
                dataSeries.dataSeriesStyle = new FR.Chart.AreaTransparentChartStyle(para);
            }
        }
    },

    /**
     * 根据系列标签位置 返回系列点 标签边界.
     * @param labelPreDim 标签占用的文本大小
     * @param dataPointRect 系列点占用的边界
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签位置.
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return this.getDataPointLabelBoundsWithPosition4Line(labelPreDim, dataPointRect, position);
    },

    /**
     * 返回面积图的 系列画 顺序
     * @returns {Array} 画顺序
     */
    getDrawSequence: function () {
        if (this.isStacked) {
            var seq = [];
            for (var i = 0; this.dataSeries && i < this.dataSeries.length; i++) {
                seq[i] = this.dataSeries.length - i - 1;
            }
            return seq;
        }

        var order = [];
        var value = [];

        for (var i = 0; i < this.dataSeries.length; i++) {
            order[i] = i;
            value[i] = -1;

            var dataSeries = this.dataSeries[i];
            for (var j = 0; j < dataSeries.getDataPointCount(); j++) {
                if ((!dataSeries.points[j].valueIsNull)
                    && Math.abs(dataSeries.points[j].value) > value[i]) {
                    value[i] = Math.abs(dataSeries.points[j].value);
                }
            }
        }

        order.sort(function (x, y) {
            return value[x] > value[y] ? -1 :
                value[x] < value[y] ? 1 : 0;
        });

        return order;
    },

    // AreaPlot 动态之后补画折线Marker
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        this.drawAfterAnimate4LineMarker(ctx, currentDivKey);
        this.drawAlertLinesAfterSeriesAnimateDraw(ctx);
    },

    /**
     * 判断数据点对应的shape的部分是否在鼠标画出来的矩形中
     * @param dataPoint{DataPoint} 数据点对象，里面存有该数据点对应的图形对象
     * @param drawBounds{Bounds} 鼠标画出来的矩形
     * @returns {boolean} 返回矩形是否包含图形
     */
    categoryInBounds: function (dataPoint, drawBounds) {
        if(!dataPoint.glyph){
            return false;
        }
        var pointBounds = dataPoint.glyph.getShape();
        if (pointBounds != null) {
            return drawBounds.containsPoint(pointBounds.x, pointBounds.y);
        }
    },

    /**
     * 寻找鼠标移动 所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件
     * @param ctx4OnMove 最上层触发的ctx图层.
     */
    findOnMoveGlyph:function(mousePos, event, ctx4OnMove) {
        this._findOnMoveGlyph4Line(mousePos, event, ctx4OnMove);
    }
});

/**
 * Area3DPlotGlyph.start
 * 3D面积图
 *
 *      @example
 *      var area3D = new FR.Chart.Area3DPlotGlyph();
 *
 * @class FR.Chart.Area3DPlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.seriesIntervalPercent] 系列间隔
 * @param {Number} [js.deep] 三维深度
 * @param {FR.Chart.Projection} [js.projection] 三维属性
 */
FR.Chart.Area3DPlotGlyph = FR.extend(FR.Chart.CategoryPlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.seriesIntervalPercent = FR.pick(js.seriesIntervalPercent, 1.0);
        this.deep = FR.pick(js.deep, -1);
        this.projection = new FR.Chart.Projection(js.projection);

        FR.Chart.Area3DPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    isSupportLabelWrap:function(){
        return true;
    },

    get3DDeep: function () {
        return 50;
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        this.layoutAxisGlyph4ThreeD(this.xAxisGlyph, this.yAxisGlyph);
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        var series_deep = this.get3DDeep();
        var z = 0;

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries == null) {
                continue;
            }
            dataSeries.lastGlyph = dataSeries.glyph;
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries.glyph = null;
                continue;
            }

            var area3D = new FR.Chart.Area3D(z, series_deep);
            area3D.projection = this.projection;
            area3D.topSize = dataSeries.getDataPointCount();
            var shapeGlyph = new FR.Chart.ShapeGlyph();
            shapeGlyph.shape = area3D;
            dataSeries.glyph = shapeGlyph;
            shapeGlyph.info.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());
            shapeGlyph.info.seriesAttrAlpha.alpha = 0.8;
            shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();

            var pre_value = [];

            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                this._calculatePreValueWithStacked(area3D, dataPoint, pre_value, z,
                    categoryIndex, seriesIndex);
            }

            for (var i = pre_value.length - 1; i >= 0; i--) {
                area3D.lineTo(this.xAxisGlyph.getPoint2D(i).x, pre_value[i]);
            }

            area3D.closePath();
            area3D.preSize = pre_value.length;

            if (seriesIndex == this._getFirstSeriesIndexToDrawAreaSide()) {
                area3D.isFirst = true;
            }
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     * 返回某个分类下0..endIndex下所有数据点的值的和占分类下所有值的百分比
     * 由于效果的原因，不能叠加占比的值，每次都重新计算，这样只用了一次除法
     */
    _getPrePercentValue: function (endSeriesIndex, categoryIndex) {
        var pre_sum = 0;
        var cateSum = 0;

        for (var series = 0; series < this.dataSeries.length; series++) {
            var dataSeries = this.dataSeries[series];
            var pointValue = Math.abs(dataSeries.points[categoryIndex].value);
            cateSum = cateSum + pointValue;
            if(series <= endSeriesIndex && dataSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                pre_sum += pointValue;
            }
        }

        if (!this.yAxisGlyph.isPercentage) {
            return pre_sum;
        }

        if (cateSum === 0) {
            return 0;
        }

        return pre_sum / cateSum;
    },

    _getFirstSeriesIndexToDrawAreaSide: function () {//第一个系列需要画底
        for (var i = 0; i < this.dataSeries.length; i++) {
            if (this.dataSeries[i].paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                return i;
            }
        }
    },

    _calculatePreValueWithStacked: function (area3D, dataPoint, pre_value, z, categoryIndex, seriesIndex) {
        var point_x, point_y;
        var value = this.getDataPointPercentValue(seriesIndex, categoryIndex);

        point_x = this.xAxisGlyph.getPointInBounds(categoryIndex).x;

        var total_sum = this._getPrePercentValue(seriesIndex, categoryIndex)
        var pre_sum = this._getPrePercentValue(seriesIndex - 1, categoryIndex);

        point_y = this.threeDBase.getGapHeight()
            + this.getSeriesAxisGlyph(seriesIndex).getPointInBounds(total_sum).y;

        pre_value[categoryIndex] = this.threeDBase.getGapHeight()
            + this.getSeriesAxisGlyph(seriesIndex).getPointInBounds(pre_sum).y;

        this.dealPointPath(dataPoint, area3D, point_x, point_y, z, categoryIndex);
        this.dealDataPointLabel(dataPoint);

        var markerGlyph = new FR.Chart.MarkerGlyph();
        markerGlyph.dealCondition4Line(this.conditionCollection, dataPoint, this.createColors4Series());
        markerGlyph.setShape(new FR.Chart.Bounds(point_x - FR.Chart.Constants.MarkerDefaultSize/2, point_y - FR.Chart.Constants.MarkerDefaultSize/2, FR.Chart.Constants.MarkerDefaultSize, FR.Chart.Constants.MarkerDefaultSize));
        markerGlyph.background = this.getBackground() !== null ? this.getBackground():this.wholeChartBackground;
        dataPoint.glyph = markerGlyph;
        markerGlyph.toolTipBackground = markerGlyph.background;

    },

    dealPointPath: function (dataPoint, area3D, point_x, point_y, z, categoryIndex) {
        if (categoryIndex === 0) {
            area3D.moveTo(point_x, point_y);
        } else {
            area3D.lineTo(point_x, point_y);
        }

        var p = this.projection.projectee(point_x, point_y, z);

        dataPoint.setShape(
            new FR.Chart.Arc2D(p.x - 3, p.y - 3, 3, 0, 360, FR.Chart.Arc2D.PIE)
        );
    },

    /**
     * 三维面积图 画出.
     * @param ctxs 画布层
     * @param time 动态时间
     * @param actionModel 动态方式.
     */
    draw: function (ctxs, time, actionModel) {
        this.draw3DInfo(ctxs);

        if (this.xAxisGlyph != null) {
            this.xAxisGlyph.draw4ThreeD(ctxs[1]);
        }
        if (this.yAxisGlyph != null) {
            this.yAxisGlyph.draw4ThreeD(ctxs[1]);
        }

        FR.Chart.Bar3DPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    /**
     * 画出背景信息
     * @param ctxs 画布层级
     */
    drawInfo: function (ctxs) {
        this.info.paint(ctxs[0], this.getOutline4Fill());
    },

    getOutline4Fill: function () {
        var basePoint = this.threeDBase.getPoints()[4];
        var axisWidth = this.yAxisGlyph.bounds.width;
        var width = this.threeDBase.width;
        var height = this.threeDBase.height;
        return new FR.Chart.Bounds(basePoint.x + axisWidth, basePoint.y, width - axisWidth, height)
    },

    /**
     * 根据系列标签位置 返回系列点 标签边界.
     * @param labelPreDim 标签占用的文本大小
     * @param dataPointRect 系列点占用的边界
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签位置.
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return this.getDataPointLabelBoundsWithPosition4Line(labelPreDim, dataPointRect, position);
    }

});

/**
 * 雷达图
 *    @example
 *    var op = {
 *      isFilled : false,
 *      intervalColor : null,
 *      radarAxisGlyph : {
 *          categoryCount : 4,
  *         labelList : [a, b, c, d]
 *      }
 *    };
 *    var radar = new FR.Chart.RadarPlotGlyph(op);
 *
 *
 * @class FR.Chart.RadarPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Boolean} [js.isFilled = true] 是否填充
 * @param {Color} [js.intervalColor] 间隔颜色
 * @param {FR.Chart.RadarAxisGlyph}[js.radarAxisGlyph] 雷达坐标轴
 *
 */
FR.Chart.RadarPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isFilled = FR.pick(js.isFilled, false);
        this.intervalColor = js.intervalColor;
        this.radarAxisGlyph = new FR.Chart.RadarAxisGlyph(js.radarAxisGlyph || js.radarAxis);

        FR.Chart.RadarPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    configSeries:function(seriesConfig){
        this.isFilled = FR.pick(seriesConfig.isFilled, false);
        this.isNullValueBreak = FR.pick(seriesConfig.isNullValueBreak, true);
        var defaultAttr = this.conditionCollection.defaultAttr;

        if(this.isFilled){
            var attrAlpha = new FR.Chart.AttrAlpha();
            attrAlpha.alpha = 0.7;
            defaultAttr.addAttr(attrAlpha);
        }

        if(seriesConfig.lineStyle != null && seriesConfig.lineStyle != undefined){
            var attrLine = new FR.Chart.AttrLineStyle();
            attrLine.lineStyle = seriesConfig.lineStyle;
            defaultAttr.addAttr(attrLine);
        }

        var markerType = seriesConfig.markerType || 'NullMarker';
        var attrMarker = new FR.Chart.AttrMarkerType();
        attrMarker.markerType = markerType;
        defaultAttr.addAttr(attrMarker);
    },

    configAxisLabelsWithOptionData:function(data){
        if(this.radarAxisGlyph && data && data.category){
            this.radarAxisGlyph.labelList = data.category;
            this.radarAxisGlyph.categoryCount = data.category.length;
        }
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        FR.Chart.RadarPlotGlyph.superclass.addSeries2PlotGlyph.apply(this, [chartData, plotTypes]);

        //处理下radarAxisGlyph,刷新应该就可以了
        this.radarAxisGlyph.categoryCount = this.getCategoryCount();
    },

    getLegendMarkerIcon:function(dataSeries, colors){
        return this.getLegendMarkerIcon4Line(dataSeries, colors);
    },

    /**
     * 是否支持两步的画图,散点，气泡，雷达，地图支持
     */
    isSupportTwoStepDraw:function(){
        return true;
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        var plotBounds = this.getBounds();

        var preGap4Label = 20;

        this.radarAxisGlyph.dealPlotBoundsWithAxisLabel(plotBounds);

        plotBounds.height = plotBounds.height - preGap4Label;
        plotBounds.y = plotBounds.y + preGap4Label / 2;

        this.radarAxisGlyph.init(plotBounds);
        this.radarAxisGlyph.calculateAxisLengthUnit();

    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            dataSeries.lastGlyph = dataSeries.glyph;

            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                dataSeries.glyph = null;
                continue;
            }

            var linePath = new FR.Chart.GeneralPath();

            linePath.centerPoint = this.radarAxisGlyph.getPointInBounds(0, this.radarAxisGlyph.getCrossValue());

            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                var value = dataPoint.value;
                var point = this.radarAxisGlyph.getPointInBounds(categoryIndex, value);

                if (dataPoint.valueIsNull) {
                    if (!this.isNullValueBreak) {
                        continue;  //空值跳过
                    }
                    point = this.radarAxisGlyph.originPoint;  //空值断开 画向原点
                }

                if (categoryIndex === 0) {
                    linePath.moveTo(point.x, point.y);
                } else {
                    linePath.lineTo(point.x, point.y);
                }

                var markerGlyph = new FR.Chart.MarkerGlyph();
                markerGlyph.dealCondition4Line(this.conditionCollection, dataPoint, this.createColors4Series());
                markerGlyph.setShape(new FR.Chart.Bounds(point.x - FR.Chart.Constants.MarkerDefaultSize/2, point.y - FR.Chart.Constants.MarkerDefaultSize/2, FR.Chart.Constants.MarkerDefaultSize, FR.Chart.Constants.MarkerDefaultSize));
                markerGlyph.background = this.getBackground() != null ? this.getBackground() : this.wholeChartBackground;;
                dataPoint.glyph = markerGlyph;
                markerGlyph.toolTipBackground = markerGlyph.background;

                this.dealDataPointLabel(dataPoint);
            }

            // 闭合
            linePath.closePath();

            // 这边 和 LinePlot 散点图一样 只有线型, 没有边框
            var foldLine = new FR.Chart.FoldLine(linePath);
            foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());
            dataSeries.glyph = foldLine;

            if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
                var tipBackground = new FR.Chart.ColorBackground();
                tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
                foldLine.toolTipBackground = tipBackground;
            }

            if (this.isFilled) {
                var shapeGlyph = new FR.Chart.ShapeGlyph();
                shapeGlyph.shape = linePath;

                var colorBackground = FR.Chart.ColorBackground.getInstance(foldLine.lineStyleInfo.attrLineColor.seriesColor);

                var attrBackground = new FR.Chart.AttrBackground();
                attrBackground.seriesBackground = colorBackground;

                var attrAlpha = new FR.Chart.AttrAlpha();
                attrAlpha.alpha = foldLine.lineStyleInfo.seriesAttrAlpha.alpha;

                shapeGlyph.info.seriesAttrBackground = attrBackground;
                shapeGlyph.info.seriesAttrAlpha = attrAlpha;

                shapeGlyph.toolTipBackground = colorBackground;

                dataSeries.glyph = shapeGlyph;
            }
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     * 根据标签的位置返回标签的边界
     * @param labelPreDim 标签占用大小
     * @param dataPointRect 系列点 范围
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签范围.
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return this.getDataPointLabelBoundsWithPosition4Line(labelPreDim, dataPointRect, position);
    },

    /**
     * 雷达 值轴 重新计算大小
     */
    refreshAxisGlyph: function () {
        this.radarAxisGlyph.oldMin = this.radarAxisGlyph.minValue;
        this.radarAxisGlyph.oldMax = this.radarAxisGlyph.maxValue;
        this.radarAxisGlyph.oldTick = this.radarAxisGlyph.tickLength;

        this.radarAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.radarAxisGlyph.isLog), this.getMaxValueFromData());

        // 如果分开的话. 非定义的 从分类对应数中取.
        var max = [];
        var min = [];
        if (this.radarAxisGlyph.isAllMaxMin) {
            for (var i = 0; i < this.getCategoryCount(); i++) {
                max[i] = this.radarAxisGlyph.maxValue;
                min[i] = this.radarAxisGlyph.minValue;
            }
        } else {
            for (var i = 0; i < this.getCategoryCount(); i++) {
                min[i] = this._getMinValueFromaCateData(i);
                var maxCate = this._getMaxValueFromCateData(i);
                if (maxCate <= min[i]) {
                    maxCate = maxCate + 10;
                }
                max[i] = maxCate;
            }
        }

        this.radarAxisGlyph.radarMax = max;
        this.radarAxisGlyph.radarMin = min;
    },

    _getMaxValueFromCateData: function (categoryIndex) {
        var maxValue = 0.0;
        for (var j = 0; j < this.dataSeries.length; j++) {
            if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            var number = this.dataSeries[j].points[categoryIndex];
            if (!number.valueIsNull) {
                maxValue = (number.value > maxValue) ? number.value : maxValue;
            }
        }

        if (maxValue == this.getMinValueFromData(false)) {
            return maxValue + 10;
        }

        return maxValue;
    },

    _getMinValueFromaCateData: function (categoryIndex) {
        var minValue = 0.0;
        for (var j = 0; j < this.dataSeries.length; j++) {
            if (this.dataSeries[j].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            var number = this.dataSeries[j].points[categoryIndex];
            if (!number.valueIsNull) {
                minValue = (number.value < minValue) ? number.value : minValue;
            }
        }
        return minValue;
    },

    /**
     * 返回雷达图 系列的画顺序
     * @returns {Array}
     */
    getDrawSequence: function () {
        var order = [];
        var value = [];

        for (var i = 0; i < this.dataSeries.length; i++) {
            order[i] = i;
            value[i] = -1;

            var dataSeries = this.dataSeries[i];
            for (var j = 0; j < dataSeries.getDataPointCount(); j++) {
                if ((!dataSeries.points[j].valueIsNull)
                    && Math.abs(dataSeries.points[j].value) > value[i]) {
                    value[i] = Math.abs(dataSeries.points[j].value);
                }
            }
        }

        order.sort(function (x, y) {
            return value[x] > value[y] ? -1 :
                value[x] < value[y] ? 1 : 0;
        });

        return order;
    },

    /**
     * 画出雷达图绘图区
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        this.drawInfo(ctxs);

        if (this.radarAxisGlyph) {
            this.radarAxisGlyph.draw(ctxs[1]);
        }

        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            var stepNumber = 6; // actionModel 动态变化的方式
            if (FR.Browser.isIE6()) {
                stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[1];
                time = FR.Chart.ChartPaintConstants.ACTION_TIME[1];
            } else {
                if (actionModel) {
                    stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[actionModel];
                    time = FR.Chart.ChartPaintConstants.ACTION_TIME[actionModel];
                }
            }
            this.dataSeries[i].getAnimateGlyph(stepNumber, actionModel);
        }

        var step = this.getAnimateStepNumber();
        if (step > 0) {
            this.animateGlyphDraw(ctxs[2], time, step);
        } else {
            this.animateGlyphDraw(ctxs[2], 1, 1);
        }
    },

    /**
     * 返回动态步骤的个数.
     * @returns {number} 返回步骤个数.
     */
    getAnimateStepNumber: function () {
        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var i = 0;
        while (this.dataSeries[i] && !(this.dataSeries[i].getAnimateStepNumber() > 0
            && this.dataSeries[i].paintState != FR.Chart.ChartPaintConstants.STATE_UNENABLE)) {
            i++;
        }
        if (this.dataSeries[i]) { // 暂时判断是否为空值, 还有其他空值情况..
            return this.dataSeries[i].getAnimateStepNumber();
        } else {
            return -1;
        }
    },

    /**
     * 画出背景信息.
     * @param ctxs 画布层级
     */
    drawInfo: function (ctxs) {
        this.info.paint(ctxs[0], this.getOutline4Fill());

        if (this.radarAxisGlyph != null) {
            this.radarAxisGlyph.drawAxisGrid(ctxs[1]);

            if (this.intervalColor) {
                this.radarAxisGlyph.paintRadarBackground(ctxs[1], this.intervalColor);
            }
        }
    },

    /**
     * radarplot 动态之后, 画标记
     * @param ctx 画布
     * @param currentDivKey 当前文本的key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        this.drawAfterAnimate4LineMarker(ctx, currentDivKey);
    }
});

//StockPlotGlyph.start
/**
 * 股价图
 *
 *      @example
 *      var stock = new FR.Chart.StockPlotGlyph();
 *
 * @class FR.Chart.StockPlotGlyph
 * @extends FR.Chart.RectanglePlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {String} [js.errorImageIESrc] ie报错图片src
 * @param {String} [js.errorImageSrc] 非ie报错图片src
 * @param {String} [js.errorToopTip]  报错热点提示.
 * @param {Number} [js.upbarBorderLineStyle = 1] 上升柱形边框线型
 * @param {Color} [js.upBarBorderLineBackground = 'red'] 默认上升柱形边框颜色
 * @param {Color} [js.upBarBackground = 'red'] 默认上升柱形颜色
 *
 * @param {Number} [js.downBarBorderLineStyle = 1] 默认下降柱形边框线型
 * @param {Color} [js.downBarBorderLineBackground = 'green'] 默认下降柱形边框也暗色
 * @param {Color} [js.downBarBackground = 'green'] 默认下降柱形颜色
 *
 * @param {Number} [js.highLowLineStyle = 1] 高低线线型
 * @param {Color} [js.highLowLineColor = 'black'] 高低线颜色
 */
FR.Chart.StockPlotGlyph = FR.extend(FR.Chart.RectanglePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.errorImageIESrc = js.iesrc;
        this.errorImageSrc = js.src;
        this.errorToopTip = js.errorToopTip;

        this.upBarBorderLineStyle = FR.pick(js.upBarBorderLineStyle, 1);
        this.upBarBorderLineBackground = FR.pick(js.upBarBorderLineBackground, 'red');
        this.upBarBackground = FR.pick(js.upBarBackground, 'red');

        this.downBarBorderLineStyle = FR.pick(js.downBarBorderLineStyle, 1);
        this.downBarBorderLineBackground = FR.pick(js.downBarBorderLineBackground, 'green');
        this.downBarBackground = FR.pick(js.downBarBackground, 'green');

        this.highLowLineStyle = FR.pick(js.highLowLineStyle, 1);
        this.highLowLineColor = FR.pick(js.highLowLineColor, 'black');

        FR.Chart.StockPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    //stockChartData
    configAxisLabels:function(chartData){
        var xAxisGlyph = this.xAxisGlyph;

        xAxisGlyph.categoryLabelList = chartData.cateTime || [];

        if (xAxisGlyph.categoryLabelList) {
            xAxisGlyph.maxValue = xAxisGlyph.categoryLabelList.length;
            xAxisGlyph.minValue = 0;
            xAxisGlyph.tickLength = xAxisGlyph.isCustomMainUnit ? xAxisGlyph.tickLength : 1;
            xAxisGlyph.smallTickLength = xAxisGlyph.isCustomSecUnit ? xAxisGlyph.smallTickLength : this.tickLength/5;
        }
    },

    isSupportLegend:function(){
        return false;
    },

    refreshAxisGlyph: function (chartData) {
        if(!chartData){
            this.refreshAxisGlyphFromData();
            return;
        }

        this.yAxisGlyph.oldMin = this.yAxisGlyph.minValue;
        this.yAxisGlyph.oldMax = this.yAxisGlyph.maxValue;
        this.yAxisGlyph.oldTick = this.yAxisGlyph.tickLength;

        this.yAxisGlyph.initMinMaxValue(this.getMinValueFromTableData(chartData, this.yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromTableData(chartData, FR.Chart.CustomAttr.Axis_LEFT));

        if (this.secondAxisGlyph) {
            this.secondAxisGlyph.oldMin = this.secondAxisGlyph.minValue;
            this.secondAxisGlyph.oldMax = this.secondAxisGlyph.maxValue;
            this.secondAxisGlyph.oldTick = this.secondAxisGlyph.tickLength;

            this.secondAxisGlyph.initMinMaxValue(this.getMinValueFromTableData(chartData, this.secondAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_RIGHT),
                this.getMaxValueFromTableData(chartData, FR.Chart.CustomAttr.Axis_RIGHT));
        }
    },

    refreshAxisGlyphFromData : function(){
        this.yAxisGlyph.oldMin = this.yAxisGlyph.minValue;
        this.yAxisGlyph.oldMax = this.yAxisGlyph.maxValue;
        this.yAxisGlyph.oldTick = this.yAxisGlyph.tickLength;

        this.yAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.yAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_LEFT),
            this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_LEFT));
        if (this.secondAxisGlyph) {
            this.secondAxisGlyph.oldMin = this.secondAxisGlyph.minValue;
            this.secondAxisGlyph.oldMax = this.secondAxisGlyph.maxValue;
            this.secondAxisGlyph.oldTick = this.secondAxisGlyph.tickLength;

            this.secondAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.secondAxisGlyph.isLog, FR.Chart.CustomAttr.Axis_RIGHT),
                this.getMaxValueFromData(FR.Chart.CustomAttr.Axis_RIGHT));
        }
    },


    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        if (this.xAxisGlyph && this.xAxisGlyph.isZoom && this.bounds.containsPoint(mousePos.x, mousePos.y)) {
            event._lastHover = this;
            event._draggingTarget = event._isMouseDown ? this : null;// kunsnat: 移动端不支持, 和页面拖拽冲突.
        }
        FR.Chart.StockPlotGlyph.superclass.findOnMoveGlyph.apply(this, [mousePos, event, ctx4OnMove]);
    },

    categoryInBounds: function (dataPoint, drawBounds) {
        if(!dataPoint.glyph){
            return false;
        }
        var pointBounds = dataPoint.glyph.getShape();
        if (pointBounds != null) {
            return FR.Chart.ChartUtils.rectangleOverlapped(pointBounds, drawBounds);
        }
    },
    /**
     * 拖拽结束时 响应事件
     * @param mouseStartPos 鼠标拖拽开始位置
     * @param mousePos 鼠标当前位置
     */
    onDragEnd: function (mouseStartPos, mousePos) {

        //鼠标按下或者释放的时候是否是在绘图区,不在则不支持拖拽缩放
        if(!this.bounds.containsPoint(mouseStartPos.x, mouseStartPos.y)
            || !this.bounds.containsPoint(mousePos.x, mousePos.y)){
            return ;
        }

        var pointX = Math.min(mouseStartPos.x, mousePos.x);
        var pointY = Math.min(mouseStartPos.y, mousePos.y);
        var width = Math.abs(mouseStartPos.x - mousePos.x);
        var height = Math.abs(mouseStartPos.y - mousePos.y);
        var drawBounds = new FR.Chart.Bounds(pointX, pointY, width, height);

        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var minCategory = this.getCategoryCount();
            var maxCategory = 0;
            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var series = this.dataSeries[seriesIndex];
                for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {
                    var dataPoint = series.points[categoryIndex];
                    if (dataPoint.glyph != null && this.categoryInBounds(dataPoint, drawBounds)) {
                        if(minCategory >series.points.length -1 ||
                            !this.categoryInBounds(series.points[minCategory], drawBounds)){
                            minCategory = categoryIndex;
                        }
                        if(!this.categoryInBounds(series.points[maxCategory], drawBounds)){
                            maxCategory = categoryIndex ;
                        }
                        minCategory = this.xAxisGlyph.getTickIndex4Value(minCategory) < this.xAxisGlyph.getTickIndex4Value(categoryIndex) ?minCategory: categoryIndex;
                        maxCategory = this.xAxisGlyph.getTickIndex4Value(maxCategory) > this.xAxisGlyph.getTickIndex4Value(categoryIndex) ?maxCategory: categoryIndex;
                    }
                }
            }

            if (minCategory < this.getCategoryCount()) {//有选中的分类
                this.xAxisGlyph.zoomBackward = this.xAxisGlyph.getOriginalCrossValue() -  this.xAxisGlyph.getTickIndex4Value(minCategory) ;
                this.xAxisGlyph.zoomForward =  this.xAxisGlyph.getTickIndex4Value(maxCategory)+1-this.xAxisGlyph.getOriginalArrowValue();
            }
        }
    },


    getMinValueFromData:function(isLog,leftOrRight){
        var minValue = 0.0;
        if (isLog) {
            minValue = Number.MAX_VALUE;
        }
        var dataSeries = this.dataSeries[0]; // 只有一个系列
        if (!dataSeries || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return minValue;
        }

        for (var index = 0; index < this.getCategoryCount(); index++) {
            var dataPoint = dataSeries.points[index];
            if (leftOrRight == FR.Chart.CustomAttr.Axis_LEFT) {
                var number = dataPoint.values[1];
                minValue = this.getMinValue(isLog, minValue, number);

                number = dataPoint.values[2];
                minValue = this.getMinValue(isLog, minValue, number);

                number = dataPoint.values[3];
                minValue = this.getMinValue(isLog, minValue, number);

                number = dataPoint.values[4];
                minValue = this.getMinValue(isLog, minValue, number);
            }else{ // 成交量
                var number = dataPoint.values[0];
                minValue = this.getMinValue(isLog, minValue, number);
            }
        }

        if (minValue == Number.MAX_VALUE) {
            return 1;
        }
        return minValue;

    },

    getMaxValueFromData:function(leftOrRight){
        var maxValue = 0.0;

        var dataSeries = this.dataSeries[0]; // 只有一个系列
        if (!dataSeries || dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
            return maxValue;
        }

        for (var index = 0; index < this.getCategoryCount(); index++) {
            var dataPoint = dataSeries.points[index];
            if (leftOrRight == FR.Chart.CustomAttr.Axis_LEFT) {
                var number = dataPoint.values[1];
                maxValue = this.getMaxValue( maxValue, number);

                number = dataPoint.values[2];
                maxValue = this.getMaxValue( maxValue, number);

                number = dataPoint.values[3];
                maxValue = this.getMaxValue( maxValue, number);

                number = dataPoint.values[4];
                maxValue = this.getMaxValue( maxValue, number);
            }else{ // 成交量
                var number = dataPoint.values[0];
                if(!number || isNaN(number)){
                    continue;
                }
                maxValue = this.getMaxValue( maxValue, number);
            }
        }

        if (FR.Chart.CustomAttr.Axis_RIGHT == leftOrRight) {
            maxValue = maxValue * 4;
        }

        if (maxValue == this.getMinValueFromData(false, leftOrRight)) {
            return maxValue + 10;
        }

        return maxValue;
    },


    getMinValueFromTableData:function(chartData,isLog,leftOrRight){
        var minValue = 0.0;
        if (isLog) {
            minValue = Number.MAX_VALUE;
        }

        for (var categoryIndex = 0; categoryIndex < chartData.getCategoryLabelCount(); categoryIndex++) {
            if (leftOrRight == FR.Chart.CustomAttr.Axis_LEFT) {
                var number = chartData.getOpenValue(categoryIndex);
                minValue = this.getMinValue(isLog, minValue, number);

                number = chartData.getHighValue(categoryIndex);
                minValue = this.getMinValue(isLog, minValue, number);

                number = chartData.getLowValue(categoryIndex);
                minValue = this.getMinValue(isLog, minValue, number);

                number = chartData.getCloseValue(categoryIndex);
                minValue = this.getMinValue(isLog, minValue, number);
            } else {// 成交量
                var number = chartData.getVolume(categoryIndex);
                minValue = this.getMinValue(isLog, minValue, number);
            }
        }
        if (minValue == Number.MAX_VALUE) {
            return 1;
        }
        return minValue;
    },

    getMinValue:function(isLog, minValue, number){
        if (number == null || number == undefined) {
            return minValue;
        }

        if (isLog) {
            var tmp = number;
            if(tmp < minValue) {
                if(tmp > 0 && tmp < 1) {
                    minValue = tmp;
                }
            }
        } else {
            minValue = (number < minValue) ? number : minValue;
        }

        return minValue;
    },


    getMaxValueFromTableData:function(chartData, leftOrRight){
        var maxValue = 0.0;
        for (var categoryIndex = 0; categoryIndex < chartData.getCategoryLabelCount(); categoryIndex++) {
            if (leftOrRight == FR.Chart.CustomAttr.Axis_LEFT) {
                var number = chartData.getOpenValue(categoryIndex);
                maxValue = this.getMaxValue(maxValue, number);

                number = chartData.getHighValue(categoryIndex);
                maxValue = this.getMaxValue(maxValue, number);

                number = chartData.getLowValue(categoryIndex);
                maxValue = this.getMaxValue(maxValue, number);

                number = chartData.getCloseValue(categoryIndex);
                maxValue = this.getMaxValue(maxValue, number);
            } else {// 成交量
                var number = chartData.getVolume(categoryIndex);
                maxValue = this.getMaxValue(maxValue, number);
            }
        }

        if (FR.Chart.CustomAttr.Axis_RIGHT == leftOrRight) {
            maxValue = maxValue * 4;
        }

        if (maxValue == this.getMinValueFromData(chartData, false, leftOrRight)) {
            return maxValue + 10;
        }

        return maxValue;
    },


    /**
     * 处理折线的图表缩放区系列.
     * @param dataSeries 系列
     * @param categoryPlotGlyph 分类绘图区.
     * @param xAxisGlyph x轴
     * @param yAxisGlyph y轴
     */
    dealLine4DemoGlyph: function (dataSeries,stockPlotGlyph, xAxisGlyph, yAxisGlyph) {
        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;

        var linePaths = new FR.Chart.GeneralPath();
        var curvePaths = new FR.Chart.GeneralPath();
        if (xAxisGlyph.axisReversed) {
            linePaths.axisReversed = true;
            curvePaths.axisReversed = true;
        }

        var isStartPoint = true;
        for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
            var dataPoint = dataSeries.points[categoryIndex];

            if (dataPoint.valueIsNull) {
                if (this.isNullValueBreak) {
                    isStartPoint = true;  //空值断开
                }
                continue;
            }

            var value = stockPlotGlyph.getDataPointPercentValue(dataSeries.seriesIndex, categoryIndex);
            var point_x = xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x;
            if (this.isDataPointXNotInPlotBounds(point_x)) {//kunsnat: 超出 左侧和右侧范围 不进入统计. 针对IE 但是上下两侧无法处理, 需要确认.
                continue;
            }
            var point_y = yAxisGlyph.getPoint2D(value).y;

            xVal[lineDataCount] = point_x;
            yVal[lineDataCount] = point_y;
            lineDataCount++;

            if (this.isCurve) {
                if (isStartPoint) {
                    FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
                    curvePaths.clear();

                    curvePaths.moveTo(point_x, point_y);
                    linePaths.moveTo(point_x, point_y);
                    isStartPoint = false;
                } else {
                    curvePaths.lineTo(point_x, point_y);
                }
            } else {
                if (isStartPoint) {
                    linePaths.moveTo(point_x, point_y);
                    isStartPoint = false;
                } else {
                    linePaths.lineTo(point_x, point_y);
                }
            }
        }

        if (this.isCurve) {
            FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
        }

        var foldLine = new FR.Chart.FoldLine(linePaths);
        dataSeries.glyph = foldLine;

        foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());

        if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
            var tipBackground = new FR.Chart.ColorBackground();
            tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
            foldLine.toolTipBackground = tipBackground;
        }
    },


    /**
     * 返回系列点的 百分比
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {*} 百分比
     */
    getDataPointPercentValue: function (seriesIndex, categoryIndex) {
        var dataPointValue = this.dataSeries[seriesIndex].points[categoryIndex].values[4]//收盘值;

        if (!this.yAxisGlyph.isPercentage) {
            return dataPointValue;
        }

        var cateSum = 0;
        for (var series = 0; series < this.dataSeries.length; series++) {
            var dataSeries = this.dataSeries[series];
            cateSum = cateSum + Math.abs(dataSeries.points[categoryIndex].values[4]);
        }

        if (cateSum === 0) {
            return 0;
        }

        return dataPointValue / cateSum;
    },

    getMaxValue:function(maxValue, number){
        return number ? Math.max(number, maxValue) : maxValue;
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.dataSeries = [];

        var series = new FR.Chart[plotTypes.dataSeries];
        series.seriesName = "stock";
        series.seriesIndex = 0;
        series.points = [];

        this.dataSeries.push(series);

        for (var categoryIndex = 0; categoryIndex < chartData.getCategoryLabelCount(); categoryIndex++) {
            var dataPoint = new FR.Chart[plotTypes.dataPoint];


            // 引入空值 便于Glyph进行再次判断
            var close = chartData.getCloseValue(categoryIndex);
            var low = chartData.getLowValue(categoryIndex);
            var high = chartData.getHighValue(categoryIndex);
            var volume = chartData.getVolume(categoryIndex);
            var open = chartData.getOpenValue(categoryIndex);

            dataPoint.values = [];
            dataPoint.values.push(volume);
            dataPoint.values.push(open);
            dataPoint.values.push(high);
            dataPoint.values.push(low);
            dataPoint.values.push(close);

            dataPoint.seriesIndex = 0;
            dataPoint.categoryIndex = categoryIndex;
            dataPoint.categoryName = chartData.getCategoryLabel(categoryIndex);
            dataPoint.seriesName = "stock";
            series.points[categoryIndex] = dataPoint;
        }

        this.refreshAxisGlyph(chartData);
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        if(data && data.category && data.value){
            var cateTime = data.category;
            var volume = [];
            var open = [];
            var close = [];
            var high = [];
            var low = [];
            for(var i = 0; i < data.value.length; i++){
                var singleValue = data.value[i];
                if(singleValue.length === 5){
                    volume.push(singleValue[0]);
                    open.push(singleValue[1]);
                    high.push(singleValue[2]);
                    low.push(singleValue[3]);
                    close.push(singleValue[4]);
                }
            }
            var chartData = new FR.Chart.StockChartData(cateTime,open,close,low,high);
            this.addSeries2PlotGlyph(chartData,plotTypes);
            this.refreshAxisGlyph(chartData);
        }
    },

    /**
     *  布局 坐标轴
     */
    layoutAxisGlyph: function () {
        FR.Chart.StockPlotGlyph.superclass.layoutDoubleAxisGlyph.apply(this, [this.xAxisGlyph,
            this.yAxisGlyph, this.secondAxisGlyph]);
    },


    /**
     * 调节的范围，是以月或者年为一个整体进行调整
     * @param adjustDataType 调整
     * @returns {*}
     */
    layoutLargeDataSeries4StockChart : function(adjustDataType){

        var dataSeries = this.dataSeries[0]; // 只有一个系列
        if (!dataSeries) {
            return;
        }
        this.xAxisGlyph.stockChartMinDateInt = null;

        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
            this.refreshDataPointLabel();
        }

        //布局分类合并
        var currentCombineDate = null;
        var combineDataPoint = null;

        var stockDataPoints = [];
        var plotBounds = this.getBounds();

         for (var categoryIndex = dataSeries.getDataPointCount()-1, categoryCount =dataSeries.getDataPointCount() ;
              categoryIndex >= 0; categoryIndex--) {
             var dataPoint = dataSeries.points[categoryIndex];
             dataPoint.glyph = null;
             dataPoint.highLowLine = null;
             dataPoint.volumeGlyph = null;

             if (dataPoint.valueIsNull) {
                 continue;
             }

             var xAxisGlyph = this.xAxisGlyph;
             var dataX = Math.round(xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x);

             if (dataX < plotBounds.x || dataX > plotBounds.x + plotBounds.width) {
                 continue;
             }

             var cateDate = xAxisGlyph.getDate4Value(categoryIndex);

             if(stockDataPoints.length === 0){
                 currentCombineDate = cateDate;
                 stockDataPoints[0] = dataPoint;
                 dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustDataType);
                 combineDataPoint = dataPoint;
                 combineDataPoint.stockDataPointCount = 1;
                 continue;
             }else if(FR.Chart.ChartUtils.isBelongToSameUnit(currentCombineDate, cateDate,adjustDataType)){
                 combineDataPoint.addDataPoint4StockCategory(dataPoint);
                 combineDataPoint.stockDataPointCount++;
                 dataPoint.valueIsNull = true;
                 continue;
             }

             var stockIndex = this.getStockDateRangeIndex(stockDataPoints,cateDate,xAxisGlyph,adjustDataType);
             //是否属于之前的一个统计区间
             if(stockIndex !== -1){
                 stockDataPoints[stockIndex].addDataPoint4StockCategory(dataPoint);
                 dataPoint.valueIsNull = true;
                 combineDataPoint.stockDataPointCount++;
             }else{ //属于新的统计区间
                 dataPoint.categoryName = FR.Chart.ChartUtils.calculateAdjustDateRange(cateDate,adjustDataType);
                 stockDataPoints[stockDataPoints.length] = dataPoint;
                 currentCombineDate = cateDate;
                 combineDataPoint = dataPoint;
                 combineDataPoint.stockDataPointCount = 1;
             }

         }

        this.xAxisGlyph.stockChartMainType = adjustDataType;
        this.refreshAxisGlyphFromData();
        this.yAxisGlyph.calculateAxisGlyph(this.bounds);
        this.secondAxisGlyph.calculateAxisGlyph(this.bounds);

        //布局合并后的数据点
        var crossValue = 0;
        if (this.secondAxisGlyph && this.secondAxisGlyph.isLog) {
            crossValue = 1;
        }


        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;
        var widthDiff = -1;



        for(var stockIndex = stockDataPoints.length-1; stockIndex>=0 ;stockIndex-- ) {
            var dataPoint = stockDataPoints[stockIndex];
            var stockValues = dataPoint.values;
            var volume = stockValues[0], open = stockValues[1], high = stockValues[2],
                low = stockValues[3], close = stockValues[4];

            var cateWidth = this.xAxisGlyph.unitLength* dataPoint.stockDataPointCount;
            var unitIncrement = parseInt(cateWidth * 0.5); // 间隔变化. (cateWidth范围.)
            var rectWidth = Math.max(cateWidth - unitIncrement, 1);

            var dataX = this.getStockChartDatePointX(xAxisGlyph,dataPoint,adjustDataType,plotBounds);

            if(widthDiff <0){
                widthDiff = Math.abs(dataX - this.getStockChartDatePointX(xAxisGlyph,stockDataPoints[stockIndex-1],adjustDataType,plotBounds));
            }

            this.dealDataPointGlyph(plotBounds,categoryCount,
                volume,high,low,open,close,dataPoint,dataX,widthDiff,rectWidth,crossValue,xVal,yVal,lineDataCount);
            lineDataCount++;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, this.yAxisGlyph);
        this.refreshDataPointLabel();
    },


    dealDataPointGlyph:function(plotBounds,categoryCount,volume,high,low,open,close,dataPoint,dataX,widthDiff,rectWidth,crossValue,xVal,yVal,lineDataCount){
        if (FR.Browser.isIE9()) {// 平移0.5 便于ie9下画出.
             dataX += 0.5;
         }
        var categoryIndex = dataPoint.categoryIndex;
         var choosedX =dataX - widthDiff/2;
         var chooseWidth = widthDiff;
         if(categoryIndex === 0 && choosedX>plotBounds.x){
             chooseWidth += choosedX-plotBounds.x;
             choosedX =plotBounds.x;
         }else if(categoryIndex === 0 && choosedX<plotBounds.x){
             chooseWidth -= plotBounds.x - choosedX;
             choosedX =plotBounds.x;
         }else if(categoryIndex === categoryCount-1 && (choosedX+chooseWidth)<(plotBounds.x+plotBounds.width)){
             chooseWidth += (plotBounds.x+plotBounds.width)- (choosedX+chooseWidth);
         }
         dataPoint.choosedBounds = new FR.Chart.Bounds(choosedX,plotBounds.y,chooseWidth,plotBounds.height);

        var highY = this._getValueInBounds(this.yAxisGlyph.getPoint2D(high).y);
        var lowY = this._getValueInBounds(this.yAxisGlyph.getPoint2D(low).y);
        var openY = this._getValueInBounds(this.yAxisGlyph.getPoint2D(open).y);
        var closeY = this._getValueInBounds(this.yAxisGlyph.getPoint2D(close).y);
        var volumeWidth = parseInt(rectWidth);

        var seriesDrawGlyph = new FR.Chart.ShapeGlyph();
        dataPoint.glyph = seriesDrawGlyph;
        dataPoint.glyph.setShape(new FR.Chart.Bounds(0, 0, 0, 0));

        var highLowLine = new FR.Chart.FoldLine(); // 高低线

        var path = new FR.Chart.GeneralPath();
        path.yZero = lowY;
        path.moveTo(dataX, highY);
        path.lineTo(dataX, lowY);

        highLowLine.generalPath = path;
        dataPoint.highLowLine = highLowLine;

        if (isNaN(volume) && isNaN(open)) {
            seriesDrawGlyph.setShape(new FR.Chart.Bounds(dataX - rectWidth, closeY, rectWidth, 1));
            seriesDrawGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());

            highLowLine.lineStyleInfo.attrLineColor.seriesColor = this.highLowLineColor;
            highLowLine.lineStyleInfo.attrLineStyle.lineStyle = this.highLowLineStyle;

            if (highLowLine.lineStyleInfo.attrLineColor.seriesColor) {
                var tipBackground = new FR.Chart.ColorBackground();
                tipBackground.color = highLowLine.lineStyleInfo.attrLineColor.seriesColor;
                highLowLine.toolTipBackground = tipBackground;
            }

        } else if (isNaN(volume)) {
            this._dealUpDown(openY, closeY, dataX, highLowLine, seriesDrawGlyph, rectWidth, null);
        } else if (isNaN(open)) {
            var volumeY = this.secondAxisGlyph.getPoint2D(volume).y;
            var volumenHeight = this.secondAxisGlyph.getPoint2D(crossValue).y - volumeY;
            var volumeGlyph = new FR.Chart.ShapeGlyph();
            volumeGlyph.setShape(new FR.Chart.Bounds(dataX - volumeWidth / 2, volumeY, volumeWidth, volumenHeight));
            dataPoint.volumeGlyph = volumeGlyph;
            volumeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            volumeGlyph.toolTipBackground = volumeGlyph.info.getBackground();

            // 收盘
            seriesDrawGlyph.setShape(new FR.Chart.Bounds(dataX - rectWidth, closeY, rectWidth, 1));
            seriesDrawGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            // 高低线
            highLowLine.lineStyleInfo.attrLineColor.seriesColor = this.highLowLineColor;
            highLowLine.lineStyleInfo.attrLineStyle.lineStyle = this.highLowLineStyle;

            if (highLowLine.lineStyleInfo.attrLineColor.seriesColor) {
                var tipBackground = new FR.Chart.ColorBackground();
                tipBackground.color = highLowLine.lineStyleInfo.attrLineColor.seriesColor;
                highLowLine.toolTipBackground = tipBackground;
            }
        } else {
            var volumeY = this.secondAxisGlyph.getPoint2D(volume).y;
            var volumenHeight = this.secondAxisGlyph.getPoint2D(crossValue).y - volumeY;
            var volumeGlyph = new FR.Chart.ShapeGlyph();
            volumeGlyph.setShape(new FR.Chart.Bounds(dataX - volumeWidth / 2, volumeY, volumeWidth, volumenHeight));
            dataPoint.volumeGlyph = volumeGlyph;

            volumeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
            volumeGlyph.toolTipBackground = volumeGlyph.info.getBackground();
            // 系列涨跌柱
            this._dealUpDown(openY, closeY, dataX, highLowLine, seriesDrawGlyph, rectWidth, volumeGlyph);
        }

        seriesDrawGlyph.toolTipBackground = seriesDrawGlyph.info.getBackground();
        xVal[lineDataCount] = dataX;
        yVal[lineDataCount] = closeY;
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        if (this.errorImageIESrc || this.errorImageSrc) {
            var newLine = '<br />';
            var resultHTML = '';
            resultHTML += FR.i18nText('Fine-Dec_Chart_Stock_Error') + newLine;
            resultHTML += this.errorToopTip + FR.i18nText('Fine-Dec_Chart_Use_Data') + newLine;
            resultHTML += FR.i18nText('Fine-Dec_Chart_Stock_Check_Data');
            this.errorDataTip = resultHTML;
            return;
        }

        this.xAxisGlyph.stockChartMinDateInt = null;
        this.xAxisGlyph.stockChartMainType = null;

        var plotBounds = this.getBounds();

        var cateWidth = this.xAxisGlyph.unitLength;

        var unitIncrement = parseInt(cateWidth * 0.5); // 间隔变化. (cateWidth范围.)
        var rectWidth = Math.max(cateWidth - unitIncrement, 1);
        var volumeWidth = parseInt(rectWidth);

        var crossValue = 0;
        if (this.secondAxisGlyph && this.secondAxisGlyph.isLog) {
            crossValue = 1;
        }

        var dataSeries = this.dataSeries[0]; // 只有一个系列
        if (!dataSeries) {
            return;
        }

        if(this.isStockChart){
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                dataSeries.points[categoryIndex].revert2Original();
            }
            this.refreshDataPointLabel();
        }

        var xVal = [];
        var yVal = [];
        var lineDataCount = 0;
        var widthDiff = -1;

        var lastDate = null;

        for (var categoryIndex = 0, categoryCount = dataSeries.getDataPointCount();
             categoryIndex < categoryCount; categoryIndex++) {

            var dataPoint = dataSeries.points[categoryIndex];

            if (dataPoint.valueIsNull) {
                dataPoint.glyph = null;
                dataPoint.highLowLine = null;
                dataPoint.volumeGlyph = null;
                continue;
            }

            var stockValues = dataPoint.values;
            var volume = stockValues[0], open = stockValues[1], high = stockValues[2],
                low = stockValues[3], close = stockValues[4];

            var xAxisGlyph = this.xAxisGlyph;
            var dataX = Math.round(xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex) + xAxisGlyph.labelOffset).x);

            if (dataX < plotBounds.x || dataX > plotBounds.x + plotBounds.width) {
                dataPoint.glyph = null;
                dataPoint.highLowLine = null;
                dataPoint.volumeGlyph = null;
                continue;
            }

            if(this.isStockChart){
                var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
                if(volumeWidth <2 && lastDate){
                    var adjustType = this.getStockChartAdjustDateType(cateDate,lastDate);
                    if(adjustType !== FR.Chart.Constants.YEAR || adjustType !== FR.Chart.Constants.MONTH){
                        adjustType = FR.Chart.Constants.MONTH;
                    }
                    //股价图最少是以日为单位，所以如果调节的话，最小的调节范围应该是月为单位
                    return this.layoutLargeDataSeries4StockChart(adjustType);
                }
                lastDate = cateDate;
            }

            if(widthDiff<0){
                var nextDataX = Math.round(xAxisGlyph.getPoint2D(xAxisGlyph.getTickIndex4Value(categoryIndex+1) + xAxisGlyph.labelOffset).x);
                widthDiff = nextDataX - dataX;
            }


            this.dealDataPointGlyph(plotBounds,categoryCount,
                volume,high,low,open,close,dataPoint,dataX,widthDiff,rectWidth,crossValue,xVal,yVal,lineDataCount);
            lineDataCount++;
        }

        this.trendLineFitting(xVal, yVal, dataSeries, this.yAxisGlyph);
    },

    _getValueInBounds: function (value) {
        if (value < 0) {
            value = 0;
        }
        if (value > this.bounds.y + this.bounds.height) {
            value = this.bounds.y + this.bounds.height;
        }
        return value;
    },

    _dealUpDown: function (openValue, closeValue, dataX, highLowLine, shapeGlyph, rectWidth, volumeGlyph) {
        if (openValue > closeValue) {	// 涨
            var openCloseRec = new FR.Chart.Bounds(
                dataX - rectWidth / 2, closeValue, rectWidth, openValue - closeValue);
            openCloseRec.upDown = 1;

            shapeGlyph.setBackground(FR.Chart.ColorBackground.getInstance(this.upBarBackground));
            shapeGlyph.setBorderColor(this.upBarBorderLineBackground);
            shapeGlyph.setBorderStyle(this.upBarBorderLineStyle);
            shapeGlyph.setShape(openCloseRec);

            highLowLine.lineStyleInfo.attrLineColor.seriesColor = this.upBarBackground;
            highLowLine.generalPath.upDown = 1;

            if (volumeGlyph) {
                volumeGlyph.setBackground(FR.Chart.ColorBackground.getInstance(this.upBarBackground));
            }
        } else {	// 跌
            var openCloseRec = new FR.Chart.Bounds(
                dataX - rectWidth / 2, openValue, rectWidth, closeValue - openValue);
            openCloseRec.upDown = -1;

            shapeGlyph.setBackground(FR.Chart.ColorBackground.getInstance(this.downBarBackground));
            shapeGlyph.setBorderColor(this.downBarBorderLineBackground);
            shapeGlyph.setBorderStyle(this.downBarBorderLineStyle);
            shapeGlyph.setShape(openCloseRec);

            highLowLine.lineStyleInfo.attrLineColor.seriesColor = this.downBarBackground;
            highLowLine.generalPath.upDown = -1;

            if (volumeGlyph) {
                volumeGlyph.setBackground(FR.Chart.ColorBackground.getInstance(this.downBarBackground));
            }
        }
    },

    /**
     *画 股价图的绘图区
     * @param ctxs
     * @param time
     * @param actionModel
     */
    draw: function (ctxs, time, actionModel) {

        if (this.errorImageSrc || this.errorImageIESrc) {
            var ctx = ctxs[2];
            if(!ctx){     return; } ctx.save();

            var image = new Image();

            var imageX = this.getBounds().x + this.getBounds().width / 2;
            var imageY = this.getBounds().y + this.getBounds().height / 2;

            image.onload = function () {
                // kunsnat: 预防IE9的概率性 加载失败, 但是现在仍然小概率发生, 没辙了.
                setTimeout(function () {
                        ctx.drawImage(image,
                            imageX - image.width / 2, imageY - image.height / 2,
                            image.width, image.height);
                    },
                    100);
            };

            if (FR.Browser.isIE() || FR.Chart.ChartUtils.isIE11()) {
                image.src = FR.servletURL + "?op=fr_attach&cmd=ah_image&id=" + this.errorImageIESrc;
            } else {
                image.src = this.errorImageSrc;
            }

            ctx.restore();
        }

        FR.Chart.StockPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    }
});
$.extend(FR.Chart.StockPlotGlyph, /**@class FR.Chart.StockPlotGlyph*/{
    /**
     * 盘高 盘低 收盘
     * @static
     * @property
     */
    HIGH_LOW_CLOSE: 0,

    /**
     * 开盘-盘高-盘低-收盘
     * @static
     * @property
     */
    OPEN_HIGH_LOW_CLOSE: 1,

    /**
     * 成交量-盘高-盘低-收盘
     * @static
     * @property
     */
    VOLUME_HIGH_LOW_CLOSE: 2,

    /**
     * 成交量-开盘-盘高-盘低-收盘.
     * @static
     * @property
     */
    VOLUME_OPEN_HIGH_LOW_CLOSE: 3
});

/**
 * 散点图
 *
 *      @example
 *      var xyplot = new FR.Chart.XYScatterPlotGlyph();
 *
 * @class FR.Chart.XYScatterPlotGlyph();
 * @extends FR.Chart.RectanglePlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Boolean} [js.isCurve = false] 曲线连接.
 */
FR.Chart.XYScatterPlotGlyph = FR.extend(FR.Chart.RectanglePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isCurve = js.isCurve;

        FR.Chart.XYScatterPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 获取第二个值轴
     */
    getSecondValueAxisGlyph:function(){
        return this.xAxisGlyph;
    },

    configSeries:function(seriesConfig){
        this.isCurve = FR.pick(seriesConfig.isCurve, false);
        this.isNullValueBreak = FR.pick(seriesConfig.isNullValueBreak, true);
        var defaultAttr = this.conditionCollection.defaultAttr;

        var attrLine = new FR.Chart.AttrLineStyle();
        defaultAttr.addAttr(attrLine);
        if(seriesConfig.lineStyle != null && seriesConfig.lineStyle != undefined){
            attrLine.lineStyle = seriesConfig.lineStyle;
        }else{
            attrLine.lineStyle = FR.Chart.Constants.LINE_NONE;
        }

        if(seriesConfig.markerType){
            var attrMarker = new FR.Chart.AttrMarkerType();
            attrMarker.markerType = seriesConfig.markerType;
            defaultAttr.addAttr(attrMarker);
        }
        FR.Chart.XYScatterPlotGlyph.superclass.configSeries.apply(this, [seriesConfig]);
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.dataSeries = [];
        for (var i = 0; i < chartData.getItemCount(); i++) {
            var series = new FR.Chart[plotTypes.dataSeries]();
            series.points = [];
            series.seriesIndex = i;
            series.seriesName = chartData.getOriginalItem(i);

            for (var categoryIndex = 0; categoryIndex < chartData.getCategoryLabelCount(); categoryIndex++) {
                var dataPoint = new FR.Chart[plotTypes.dataPoint]();
                dataPoint.categoryIndex = categoryIndex;
                dataPoint.seriesIndex = i;

                var xNumber = chartData.getX(i, categoryIndex);
                var yNumber = chartData.getY(i, categoryIndex);

                dataPoint.x = xNumber ? xNumber : 0;
                dataPoint.categoryOriginalName = dataPoint.x + "";
                dataPoint.value = yNumber ? yNumber : 0;

                if (xNumber == null || yNumber == null) {
                    dataPoint.valueIsNull = true;
                }

                dataPoint.seriesName = chartData.getOriginalItem(i);

                series.points.push(dataPoint);
            }

            this.dataSeries[i] = series;
        }
    },

    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        if(data){
            for(var i = 0; i < data.length; i++){
                var seryConfig = data[i];
                var sery = new FR.Chart[plotTypes.dataSeries]();
                sery.points = [];
                sery.seriesIndex = seryConfig.seriesIndex;
                sery.seriesName = seryConfig.seriesName;
                var value = seryConfig.value;
                for(var cateIndex = 0, count = value.length; cateIndex < count; cateIndex++){
                    var dataPoint = new FR.Chart[plotTypes.dataPoint]();
                    dataPoint.seriesIndex = seryConfig.seriesIndex;
                    dataPoint.seriesName = seryConfig.seriesName;
                    var valueCouple = value[cateIndex];
                    dataPoint.x = valueCouple[0];
                    dataPoint.value = valueCouple[1];
                    sery.points[cateIndex] = dataPoint;
                }
                this.dataSeries[i] = sery;
            }
        }
    },

    getLegendMarkerIcon:function(dataSeries, colors){
        return this.getLegendMarkerIcon4Line(dataSeries, colors);
    },

    /**
     *  重新计算图表的坐标轴 实际使用最大最小值.
     */
    refreshAxisGlyph: function () {
        this.refreshAxisGlyph4XY();
    },

    /**
     * 是否支持两步的画图,散点，气泡，雷达，地图支持
     */
    isSupportTwoStepDraw:function(){
        return true;
    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        return this._getDataPointByXY(para);
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            dataSeries.lastGlyph = dataSeries.glyph;
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var xVal = [];
            var yVal = [];
            var lineDataCount = 0;

            var linePaths = new FR.Chart.GeneralPath();
            var curvePaths = new FR.Chart.GeneralPath();
            linePaths.yZero = this.yAxisGlyph.bounds.y + this.yAxisGlyph.bounds.height / 2;
            var isStartPoint = true;

            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {

                var dataPoint = dataSeries.points[categoryIndex];
                dataPoint.lastGlyph = dataPoint.glyph;

                if (dataPoint.valueIsNull) {
                    if (this.isNullValueBreak) {
                        isStartPoint = true;  //空值断开
                    }
                    continue;
                }

                var point_x = this.xAxisGlyph.getPoint2D(dataPoint.x).x;
                var point_y = this.getStartY(dataPoint);

                if (this.isDataPointXNotInPlotBounds(point_x)) {
                    continue;
                }

                xVal[lineDataCount] = point_x;
                yVal[lineDataCount] = point_y;
                lineDataCount++;

                if (this.isCurve) {
                    if (isStartPoint) {
                        FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
                        curvePaths.clear();
                        curvePaths.moveTo(point_x, point_y);
                        linePaths.moveTo(point_x, point_y);
                        isStartPoint = false;
                    } else {
                        curvePaths.lineTo(point_x, point_y);
                    }
                } else {
                    if (isStartPoint) {
                        linePaths.moveTo(point_x, point_y);
                        isStartPoint = false;
                    } else {
                        linePaths.lineTo(point_x, point_y);
                    }
                }

                var markerGlyph = new FR.Chart.MarkerGlyph();
                markerGlyph.dealCondition4Line(this.conditionCollection, dataPoint, this.createColors4Series());
                markerGlyph.setShape(new FR.Chart.Bounds(point_x - FR.Chart.Constants.MarkerDefaultSize/2, point_y - FR.Chart.Constants.MarkerDefaultSize/2, FR.Chart.Constants.MarkerDefaultSize, FR.Chart.Constants.MarkerDefaultSize));
                markerGlyph.background = this.getBackground() !== null ? this.getBackground():this.wholeChartBackground;;
                dataPoint.glyph = markerGlyph;

                markerGlyph.toolTipBackground = markerGlyph.background;

                this.dealDataPointLabel(dataPoint);
            }

            if (this.isCurve) {
                FR.Chart.FoldLine.curveTo(linePaths, curvePaths);
            }
            var foldLine = new FR.Chart.FoldLine();
            dataSeries.glyph = foldLine;
            dataSeries.isPointAnimate = true;

            foldLine.generalPath = linePaths;
            foldLine.generalPath.isXYScatter = true;

            foldLine.lineStyleInfo.dealCondition(this.conditionCollection, dataSeries, this.createColors4Series());
            if (foldLine.lineStyleInfo.attrLineColor.seriesColor) {
                var tipBackground = new FR.Chart.ColorBackground();
                tipBackground.color = foldLine.lineStyleInfo.attrLineColor.seriesColor;
                foldLine.toolTipBackground = tipBackground;
            }
            if (foldLine.lineStyleInfo.attrLineStyle.lineStyle === FR.Chart.Constants.LINE_NONE) {
                dataSeries.glyph = null;
            }
            // 拟合计算
            this.trendLineFitting(xVal, yVal, dataSeries, this.getSeriesAxisGlyph(seriesIndex));
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     * 根据标签位置返回 系列点的标签边界
     * @param labelPreDim 标签占用大小
     * @param dataPointRect 系列点的图形矩形区域
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 返回标签边界
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        return this.getDataPointLabelBoundsWithPosition4Line(labelPreDim, dataPointRect, position);
    },

    /**
     * 返回系列的动态步骤数
     * @returns {Number} 返回动态数
     */
    getAnimateStepNumber: function () {
        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var i = 0;
        while (this.dataSeries[i] && !(this.dataSeries[i].getAnimateStepNumber() > 0
            && this.dataSeries[i].paintState != FR.Chart.ChartPaintConstants.STATE_UNENABLE)) {
            i++;
        }
        if (this.dataSeries[i]) { // 暂时判断是否为空值, 还有其他空值情况..
            return this.dataSeries[i].getAnimateStepNumber();
        } else {
            return -1;
        }
    }
});

/**
 * 气泡图绘图区
 * BubblePlotGlyph.start
 *
 *      @example
 *      var bubble = new FR.Chart.BubblePlotGlyph();
 * @class FR.Chart.BubblePlotGlyph
 * @extends FR.Chart.RectanglePlotGlyph
 *
 * @param {JSON} js配置属性
 *
 * @param {Number} [js.maxBubblePixed = 30] 默认气泡的大小
 * @param {Number} [js.seriesEqualsBubbleInWidthOrArea = FR.Chart.Constants.BubbleType_Width] {@link FR.Chart.Constants#BubbleType_Width} 气泡宽度匹配比例
 * @param {Number} [js.isShowNegativeBubble = true] 是否实现负值气泡.
 */
FR.Chart.BubblePlotGlyph = FR.extend(FR.Chart.RectanglePlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.maxBubblePixel = FR.pick(js.maxBubblePixel, 30);
        this.seriesEqualsBubbleInWidthOrArea = FR.pick(js.compareRule, FR.Chart.Constants.BubbleType_Width);
        this.isShowNegativeBubble = FR.pick(js.isShowNegativeBubble, true);
        this.customizedMaxValue = FR.pick(js.customizedMaxValue, -1);

        FR.Chart.BubblePlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 获取第二个值轴
     */
    getSecondValueAxisGlyph:function(){
        return this.xAxisGlyph;
    },

    configSeries:function(seriesConfig){
        this.maxBubblePixel = FR.pick(seriesConfig.maxBubblePixel, 30);
        this.seriesEqualsBubbleInWidthOrArea = FR.pick(seriesConfig.compareRule, FR.Chart.Constants.BubbleType_Width);
        this.isShowNegativeBubble = FR.pick(seriesConfig.isShowNegativeBubble, true);
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.dataSeries = [];
        for (var i = 0; i < chartData.getItemCount(); i++) {
            var series = new FR.Chart[plotTypes.dataSeries];
            series.seriesName = chartData.getOriginalItem(i);
            series.points = [];
            for (var categoryIndex = 0; categoryIndex < chartData.getCategoryLabelCount(); categoryIndex++) {
                var dataPoint = new FR.Chart[plotTypes.dataPoint];
                dataPoint.categoryIndex = categoryIndex;
                dataPoint.seriesIndex = i;

                var xNumber = chartData.getX(i, categoryIndex);
                var yNumber = chartData.getY(i, categoryIndex);
                var sizeNumber = chartData.getSize(i, categoryIndex);

                dataPoint.x = xNumber ? xNumber : 0;
                dataPoint.value = yNumber ? yNumber : 0;
                dataPoint.sizeValue = sizeNumber ? sizeNumber : 0;

                if(xNumber == null || yNumber == null || sizeNumber == null) {
                    dataPoint.valueIsNull = true;
                }

                dataPoint.categoryName = dataPoint.value + "";
                dataPoint.seriesName = chartData.getOriginalItem(i);
                series.points[categoryIndex] = dataPoint;
            }
            series.seriesIndex = i;
            this.dataSeries[i] = series;
        }
    },

    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        if(data){
            for(var i = 0; i < data.length; i++){
                var seryConfig = data[i];
                var sery = new FR.Chart[plotTypes.dataSeries]();
                sery.points = [];
                sery.seriesIndex = seryConfig.seriesIndex;
                sery.seriesName = seryConfig.seriesName;
                var value = seryConfig.value;
                for(var cateIndex = 0, count = value.length; cateIndex < count; cateIndex++){
                    var dataPoint = new FR.Chart[plotTypes.dataPoint]();
                    dataPoint.seriesIndex = seryConfig.seriesIndex;
                    dataPoint.seriesName = seryConfig.seriesName;
                    var valueCouple = value[cateIndex];
                    dataPoint.x = valueCouple[0];
                    dataPoint.value = valueCouple[1];
                    dataPoint.sizeValue = valueCouple[2];
                    sery.points[cateIndex] = dataPoint;
                }
                this.dataSeries[i] = sery;
            }
        }
    },

    getLegendMarkerIcon:function(dataSeries, colors){
        var itemIcon = new FR.Chart.LineMarkerIcon();
        var marker = new FR.Chart.CircleFilledMarker();
        var attrBackground = this.conditionCollection.getConditionAttr(new FR.Chart.AttrColor(), dataSeries, colors);
        marker.background = FR.Chart.ColorBackground.getInstance(attrBackground.seriesColor);
        itemIcon.marker = marker;
        this.dealCondition(itemIcon, dataSeries, colors);
        return itemIcon;
    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        return this._getDataPointByXY(para);
    },

    /**
     * 刷新坐标轴
     */
    refreshAxisGlyph: function () {
        this.refreshAxisGlyph4XY();
    },

    /**
     * 是否支持两步的画图,散点，气泡，雷达，地图支持
     */
    isSupportTwoStepDraw:function(){
        return true;
    },

    /**
     *  返回系列点 标记的大小
     * @returns {*}
     */
    getMarkerSize: function () {
        return this.maxBubblePixel;
    },


    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        var plotBounds = this.getBounds();
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex += 1) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                dataPoint.lastGlyph = dataPoint.glyph;
                if (dataPoint.valueIsNull) {
                    continue;
                }

                var value = dataPoint.sizeValue;
                if (value < 0) {
                    if (!this.isShowNegativeBubble) {
                        continue;
                    }
                }

                var bubbleX = this.xAxisGlyph.getPoint2D(dataPoint.x).x;
                var bubbleY = this.getSeriesAxisGlyph(seriesIndex).getPoint2D(dataPoint.value).y;

                var radius = this.getRadius(value, this.getMaxSize());
                var minX = plotBounds.x;
                var maxX = plotBounds.x + plotBounds.width;
                var minY = plotBounds.y;
                var maxY = plotBounds.y + plotBounds.height;

                if ((bubbleX + radius / 2) < minX || (bubbleX - radius / 2) > maxX
                    || (bubbleY + radius / 2) < minY || (bubbleY - radius / 2) > maxY) {
                    dataPoint.glyph = null; // kunsnat: 全部超出 就不画.
                    continue;
                }

                var bubble = new FR.Chart.Bubble(bubbleX, bubbleY, radius / 2);
                dataPoint.glyph = bubble;

                // 默认Border. 负值白色填充
                bubble.colorInfo.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());

                bubble.toolTipBackground = new FR.Chart.ColorBackground({'color': bubble.colorInfo.getSeriesColor()});

                if (this.isShowNegativeBubble && value < 0) {// 负值用灰色表示.
                    bubble.colorInfo.seriesAttrColor.seriesColor = 'gray';
                }

                this.dealDataPointLabel(dataPoint);
            }
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     * 返回 系列中 最大值
     * @returns {number}
     */
    getMaxSize: function () {

        //优先返回自定义的最大值
        if(this.customizedMaxValue > 0){
            return this.customizedMaxValue;
        }

        var maxSize = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex += 1) {
            var dataSeries = this.dataSeries[seriesIndex];

            for (var i = 0; dataSeries && i < dataSeries.getDataPointCount(); i++) {
                var dataPoint = dataSeries.points[i];
                var value = dataPoint.sizeValue;

                if (this.isShowNegativeBubble) {
                    maxSize = Math.max(maxSize, Math.abs(value));
                } else {
                    maxSize = Math.max(maxSize, value);
                }
            }
        }
        return maxSize;
    },

    /**
     * 返回 值对应的半径
     * @param value 气泡值大小
     * @param maxValue 最大值
     * @returns {number} 返回半径大小
     */
    getRadius: function (value, maxValue) {
        if (maxValue <= 0) {
            return 0;
        }
        var radius = 0;
        if (this.seriesEqualsBubbleInWidthOrArea == FR.Chart.Constants.BubbleType_Width) {
            radius = this.maxBubblePixel * Math.abs(value) / maxValue;
        } else {
            radius = Math.sqrt(this.maxBubblePixel * this.maxBubblePixel * Math.abs(value) / maxValue);
        }
        return radius;
    },

    /**
     * 根据标签位置 返回标签边界
     * @param labelPreDim 标签占用大小
     * @param dataPointRect 系列点边界
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 返回标签边界
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        var labelX = dataPointRect.x + (dataPointRect.width - labelPreDim.width) / 2;
        var labelY;
        var bubbleGap4Label = 3;
        if (position == FR.Chart.Constants.OUTSIDE) {
            labelY = dataPointRect.y - labelPreDim.height - bubbleGap4Label;
        } else {
            labelY = dataPointRect.y + (dataPointRect.height - labelPreDim.height) / 2;
        }
        return new FR.Chart.Bounds(labelX, labelY, labelPreDim.width, labelPreDim.height);
    },

    /**
     * 返回动态步骤个数
     * @returns {number} 动态步骤个数.
     */
    getAnimateStepNumber: function () {
        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var i = 0;
        while (this.dataSeries[i] && !(this.dataSeries[i].getAnimateStepNumber() > 0
            && this.dataSeries[i].paintState != FR.Chart.ChartPaintConstants.STATE_UNENABLE)) {
            i++;
        }
        if (this.dataSeries[i]) { // 暂时判断是否为空值, 还有其他空值情况..
            return this.dataSeries[i].getAnimateStepNumber();
        } else {
            return -1;
        }
    },

    //气泡图从小到大开始寻找
    redoListDrawDataPoint:function(){
        var dataPointList = [];
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            for (var cate = 0; cate < dataSeries.getDataPointCount(); cate++) {
                var dataPoint = dataSeries.points[cate];
                if (dataPoint && dataPoint.glyph){
                    dataPointList[dataPointList.length] = dataPoint;
                }
            }
        }

        dataPointList.sort(function compare(x, y) {
            return x.sizeValue - y.sizeValue;
        });

        return dataPointList;
    },


    /**
     * 寻找鼠标移动 所在的Glyph
     * @param mousePos 鼠标位置
     * @param event 事件
     * @param ctx4OnMove 最上层触发的ctx图层.
     */
    findOnMoveGlyph: function (mousePos, event, ctx4OnMove) {
        var bounds = this.getBounds();

        if (this.xAxisGlyph) {
            this.xAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }
        if (this.yAxisGlyph) {
            this.yAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }
        if (this.secondAxisGlyph) {
            this.secondAxisGlyph.findOnMoveGlyph(mousePos, event, ctx4OnMove);
        }

        if (event._hasfound) {
            return;
        }

        if (mousePos.y < bounds.y || mousePos.x < bounds.x
            || mousePos.x > bounds.x + bounds.width || mousePos.y > bounds.y + bounds.height) {
            return;
        }

        var drawablePoints = this.redoListDrawDataPoint();
        for(var i = 0; i < drawablePoints.length; i++){
            var dataPoint = drawablePoints[i];
            if(dataPoint.containsMousePos(mousePos)){
                event._hasfound = true;
                event._lastHover = dataPoint;
                if(dataPoint.hyperlink){
                    event.cursor = ChartHandler.POINTER;
                }
                if(event._isMouseDown && !event._isMouseMoving){
                    var series = this.dataSeries[dataPoint.seriesIndex];
                    if(series){
                        event._draggingTarget = series; //记录需要拖拽的系列
                        series.paintState = FR.Chart.ChartPaintConstants.STATE_TRANSPARENT;
                    }
                }

                break;
            }
        }
    }
});

// RangePlotGlyph.start
/**
 * 全距图绘图区
 *      @example
 *      var range = new FR.Chart.RangePlotGlyph();
 *
 * @class FR.Chart.RangePlotGlyph 全距图绘图区
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {FR.Chart.RangeAxisGlyph} [js.rangeAxisGlyph] 全距图坐标轴
 * @param {Number} [js.seriesWidth = 2] 系列宽度
 */
FR.Chart.RangePlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.rangeAxisGlyph = new FR.Chart.RangeAxisGlyph(js.rangeAxisGlyph);

        this.seriesWidth = FR.pick(js.seriesWidth, 2);

        FR.Chart.RangePlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        this.rangeAxisGlyphList = [];

        var cateCount = this.getCategoryCount();

        var plotBounds = this.getBounds();

        var plotHeight = plotBounds.height;
        var plotWidth = plotBounds.width;
        var axisHeight = plotHeight / (cateCount * 2) > 50 ? 50 : plotHeight / (cateCount * 2);

        var cloneBounds = plotBounds.clone();//kunsnat: 全距图 不改变绘图区大小, 所以clone不变.

        for (var categoryIndex = 0; categoryIndex < cateCount; categoryIndex++) {
            var rangeAxisGlyphY = plotBounds.y + (categoryIndex + 1) * plotHeight / (1 + cateCount) - axisHeight / 2;

            var bounds = new FR.Chart.Bounds(plotBounds.x, rangeAxisGlyphY, plotWidth, axisHeight);

            var rangeAxisGlyph = new FR.Chart.RangeAxisGlyph(this.rangeAxisGlyph);

            rangeAxisGlyph.dealPlotBoundsWithAxisLabel(cloneBounds);//kunsnat: plotBounds 只是限制最高的标签高度信息.
            rangeAxisGlyph.plotZeroBounds = cloneBounds;

            rangeAxisGlyph.init(bounds, plotHeight);
            rangeAxisGlyph.calculateAxisLengthUnit();

            this.rangeAxisGlyphList.push(rangeAxisGlyph);
        }

        this.dealPlotBounds4JSDraw(this.getBounds());
    },

    /**
     * 刷新全距图坐标轴最大最小值.
     */
    refreshAxisGlyph: function () {
        this.rangeAxisGlyph.oldMin = this.rangeAxisGlyph.minValue;
        this.rangeAxisGlyph.oldMax = this.rangeAxisGlyph.maxValue;
        this.rangeAxisGlyph.oldTick = this.rangeAxisGlyph.tickLength;

        this.rangeAxisGlyph.initMinMaxValue(this.getMinValueFromData(this.rangeAxisGlyph.isLog), this.getMaxValueFromData());
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        for (var categoryIndex = 0; categoryIndex < this.getCategoryCount(); categoryIndex++) {

            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataSeries = this.dataSeries[seriesIndex];
                if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                    continue;
                }
                var dataPoint = dataSeries.points[categoryIndex];
                dataPoint.lastGlyph = dataPoint.glyph;

                if (dataPoint.valueIsNull) {
                    dataPoint.glyph = null;
                    continue;
                }

                var rangeAxisGlyph = this.rangeAxisGlyphList[categoryIndex];

                var valuePoint = rangeAxisGlyph.getPoint2D(dataPoint.value);

                var height = rangeAxisGlyph.bounds.height;

                var value_rect = new FR.Chart.Bounds(
                    valuePoint.x - this.seriesWidth * 0.5, valuePoint.y - height * 0.5, this.seriesWidth, height
                );
                // 标记动态方式
                value_rect.is4RangePlot = true;

                var value_shape = new FR.Chart.ShapeGlyph();
                value_shape.shape = value_rect;

                value_shape.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
                value_shape.toolTipBackground = value_shape.info.getBackground();

                dataPoint.glyph = value_shape;

                this.dealDataPointLabel(dataPoint);
            }
        }

        this.dealLabelBoundsInOrder();
    },

    /**
     *  画出全距图坐标轴的网格线
     * @param ctx  画布
     */
    drawAxisGrid: function (ctx) {
        if (this.rangeAxisGlyphList != null && this.rangeAxisGlyphList.length > 0) {
            for (var index = 0; index < this.rangeAxisGlyphList.length; index++) {
                this.rangeAxisGlyphList[index].drawAxisGrid(ctx);
            }
        }
    },

    /**
     * 根据坐标轴标签位置 返出 标签边界
     * @param labelPreDim 标签占用的大小
     * @param dataPointRect 系列的边界
     * @param position 标签位置
     * @returns {FR.Chart.Bounds} 标签边界
     */
    getDataPointLabelBoundsWithPosition: function (labelPreDim, dataPointRect, position) {
        var labelX = dataPointRect.x + (dataPointRect.width - labelPreDim.width) / 2;
        var labelY = 0;
        var gap4Label = 2;
        switch (position) {
            case FR.Chart.Constants.TOP :
            {
                labelY = dataPointRect.y - labelPreDim.height - gap4Label;
                break;
            }
            case FR.Chart.Constants.BOTTOM :
            {
                labelY = dataPointRect.y + dataPointRect.height + gap4Label;
                break;
            }
            default :
            {
                labelX = dataPointRect.x + dataPointRect.width + gap4Label;
                labelY = dataPointRect.y + (dataPointRect.height - labelPreDim.height) / 2;
            }
        }

        return new FR.Chart.Bounds(labelX, labelY, labelPreDim.width, labelPreDim.height);
    },

    /**
     * 画出全距图的绘图区
     * @param ctxs 画布吃呢国际
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        this.drawInfo(ctxs);

        this.drawAxisGrid(ctxs[1]);

        if (this.rangeAxisGlyphList != null && this.rangeAxisGlyphList.length > 0) {
            for (var index = 0; index < this.rangeAxisGlyphList.length; index++) {
                this.rangeAxisGlyphList[index].draw(ctxs[1]);
            }
        }

        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            var stepNumber = 12; // actionModel 动态变化的方式
            if (FR.Browser.isIE6()) {
                stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[1];
                time = FR.Chart.ChartPaintConstants.ACTION_TIME[1];
            } else {
                if (actionModel) {
                    stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[actionModel];
                    time = FR.Chart.ChartPaintConstants.ACTION_TIME[actionModel];
                }
            }
            this.dataSeries[i].getAnimateGlyph(stepNumber, actionModel);
        }

        var step = this.getAnimateStepNumber();
        if (step > 0) {
            this.animateGlyphDraw(ctxs[2], time, step);
        } else {
            this.animateGlyphDraw(ctxs[2], 1, 1);
        }
    }
});

// GanttPlotGlyph.start
/**
 * 甘特图绘图区
 *
 *     @example
 *     var plan = new FR.Chart.GanttPlotGlyph();
 *
 * @class FR.Chart.GanttPlotGlyph
 * @extends FR.Chart.RectanglePlotGlyph
 * @param {JSON} js 配置属性
 *
 * @param {String}[this.errorImageIESrc=''] 报错 ie图片加载src
 * @param {String}[this.errorImageSrc=''] 报错图片加载src
 * @param {String}[this.errorToolTip=''] 报错 热点提示文字
 * @param {Number}[this.planAlphaFromUI=0.8] UI规定透明度
 */
FR.Chart.GanttPlotGlyph = FR.extend(FR.Chart.RectanglePlotGlyph, {

    initConstants: function (js) {
        js = js || {};

        this.errorImageIESrc = js.iesrc;
        this.errorImageSrc = js.src;
        this.errorToopTip = js.errorToopTip;
        this.planAlphaFromtUI = 0.8;

        FR.Chart.GanttPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    refreshAxisGlyph:function(){

    },

    //GanttChartData
    configAxisLabels:function(chartData){
        //DateAxisGlyph
        var xAxisGlyph = this.xAxisGlyph;
        xAxisGlyph.initMinMaxValue(this.getMinValueFromData(chartData),this.getMaxValueFromData(chartData));

        //CategoryAxisGlyph
        var yAxis = this.yAxisGlyph
        yAxis.categoryLabelList = chartData.stepName;
        if (yAxis.categoryLabelList) {
            yAxis.maxValue = yAxis.categoryLabelList.length;
            yAxis.minValue = 0;
            yAxis.tickLength = yAxis.isCustomMainUnit ? yAxis.tickLength : 1;
            yAxis.smallTickLength = yAxis.isCustomSecUnit ? yAxis.smallTickLength : yAxis.tickLength/5;
        }
    },

    configAxisLabelsWithOptionData:function(data){
        if(data && data.length){
            var minValue = Number.MAX_VALUE;
            var maxValue = Number.MIN_VALUE;
            var steps = [];

            for(var i = 0; i < data.length; i++){
                var singleData = data[i].value;
                if(singleData){
                    for(var j = 0; j < singleData.length; j++){
                        var comp = singleData[j];
                        if(i === 0){
                            steps.push(comp[0]);
                        }
                        for(var k = 1; k < comp.length - 1; k++){
                            minValue = Math.min(minValue, comp[k]);
                            maxValue = Math.max(maxValue, comp[k]);
                        }
                    }
                }
            }

            //DateAxis
            var xAxisGlyph = this.xAxisGlyph;
            xAxisGlyph.initMinMaxValue(minValue, maxValue);

            //CategoryAxis
            var yAxis = this.yAxisGlyph
            yAxis.categoryLabelList = steps;
            if (yAxis.categoryLabelList){
                yAxis.maxValue = yAxis.categoryLabelList.length;
                yAxis.minValue = 0;
                yAxis.tickLength = yAxis.isCustomMainUnit ? yAxis.tickLength : 1;
                yAxis.smallTickLength = yAxis.isCustomSecUnit ? yAxis.smallTickLength : yAxis.tickLength/5;
            }
        }
    },

    //根据装载好的数据，创建图例项
    createLegendItems:function(){
        var legendItems = [];
        var seriesSize = this.dataSeries.length;
        for (var seriesIndex = 0; seriesIndex < seriesSize; seriesIndex++) {
            var series = this.dataSeries[seriesIndex];
            if (series.getDataPointCount() > 0) {
                var colors = this.createColors4Series();
                legendItems.push(this.createPlanItem(series, seriesSize, colors));
                legendItems.push(this.createRealStartItem(series, seriesSize));
                legendItems.push(this.createRealEndItem(series, seriesSize));
                legendItems.push(this.createProgressItem(series, seriesSize, colors));
            }
        }

        return legendItems;
    },

    createPlanItem:function(series, idCount, colors){
        var idName = idCount > 1 ? series.seriesName + "-" : "";
        var label = idName + FR.i18nText("Fine-Dec_Chart_Gantt_PlanTime");
        var item = new FR.Chart.LegendItem();
        var attrColor = this.conditionCollection.getConditionAttr(new FR.Chart.AttrColor(), series, colors);
        var attrAlpha = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAlpha(), series, colors);
        var marker = new FR.Chart.SquareFilledMarker();
        marker.background = FR.Chart.ColorBackground.getInstance(attrColor.seriesColor);

        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();
        lineMarkerIcon.alpha = attrAlpha.alpha;
        lineMarkerIcon.marker = marker;
        item.label = label;
        item.lineMarkerIcon = lineMarkerIcon;

        return item;
    },

    createRealStartItem:function(series, idCount){
        var idName = idCount > 1 ? series.seriesName + "-" : "";
        var label = idName + FR.i18nText("Fine-Dec_Chart_Gantt_RealStartTime");
        var item = new FR.Chart.LegendItem();

        var marker = new FR.Chart.TriangleFilledMarker();
        marker.background = FR.Chart.ColorBackground.getInstance("black");
        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();
        lineMarkerIcon.marker = marker;

        item.label = label;
        item.lineMarkerIcon = lineMarkerIcon;
        return item;
    },

    createProgressItem:function(series, idCount, colors){
        var idName = idCount > 1 ? series.seriesName + "-" : "";
        var label = idName + FR.i18nText("Fine-Dec_Chart_Gantt_Progress");
        var item = new FR.Chart.LegendItem();
        var attrColor = this.conditionCollection.getConditionAttr(new FR.Chart.AttrColor(), series, colors);
        var attrAlpha = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAlpha(), series, colors);
        var marker = new FR.Chart.SquareFilledMarker();
        marker.background = FR.Chart.ColorBackground.getInstance(attrColor.seriesColor);

        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();
        lineMarkerIcon.alpha = attrAlpha.alpha;
        lineMarkerIcon.marker = marker;
        item.label = label;
        item.lineMarkerIcon = lineMarkerIcon;

        return item;
    },

    createRealEndItem:function(series, idCount){
        var idName = idCount > 1 ? series.seriesName + "-" : "";
        var label = idName + FR.i18nText("Fine-Dec_Chart_Gantt_RealEndTime");
        var item = new FR.Chart.LegendItem();

        var marker = new FR.Chart.DiamondFilledMarker();
        marker.background = FR.Chart.ColorBackground.getInstance("rgb(122, 225, 108)");
        var lineMarkerIcon = new FR.Chart.LineMarkerIcon();
        lineMarkerIcon.marker = marker;

        item.label = label;
        item.lineMarkerIcon = lineMarkerIcon;
        return item;

    },

    getMaxValueFromData:function(cd){
        var maxValue = Number.MIN_VALUE;

        for(var idIndex = 0; idIndex < cd.getIDCount(); idIndex++ ) {
            for (var i = 0; i < cd.getCategoryLabelCount(); i ++) {
                maxValue = Math.max(maxValue, cd.getPlanStart(idIndex, i));
                maxValue = Math.max(maxValue, cd.getPlanEnd(idIndex, i));
                maxValue = Math.max(maxValue, cd.getRealStart(idIndex, i));
                maxValue = Math.max(maxValue, cd.getRealEnd(idIndex, i));
            }
        }

        if(maxValue == this.getMinValueFromData(cd)) {
            return maxValue + 10;
        }

        return maxValue;
    },

    getMinValueFromData:function(cd){
        var minValue = Number.MAX_VALUE;
        for(var idIndex = 0; idIndex < cd.getIDCount(); idIndex++ ) {
            for (var i = 0; i < cd.getCategoryLabelCount(); i ++) {
                minValue = Math.min(minValue, cd.getPlanStart(idIndex, i));
                minValue = Math.min(minValue, cd.getPlanEnd(idIndex, i));
                minValue = Math.min(minValue, cd.getRealStart(idIndex, i));
                minValue = Math.min(minValue, cd.getRealEnd(idIndex, i));
            }
        }
        return minValue;
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        if(data){
            this.dataSeries = [];
            for(var idIndex = 0; idIndex < data.length; idIndex++) {
                var singleSeries = data[idIndex];
                var series = new FR.Chart[plotTypes.dataSeries];
                series.seriesIndex = idIndex;
                series.points = [];
                series.seriesName = singleSeries.seriesName;
                var value = singleSeries.value;
                for (var i = 0; i < value.length; i++) {
                    var singleValue = value[i];
                    if(singleValue.length === 6){
                        var dataPoint = new FR.Chart[plotTypes.dataPoint];
                        dataPoint.categoryIndex = i;
                        dataPoint.seriesIndex = idIndex;
                        dataPoint.seriesName = series.seriesName;
                        series.points[i] = dataPoint;

                        dataPoint.categoryName = singleValue[0];
                        dataPoint.planStart = singleValue[1];
                        dataPoint.planEnd = singleValue[2];
                        dataPoint.realStart = singleValue[3];
                        dataPoint.realEnd = singleValue[4];
                        dataPoint.value = singleValue[5];
                    }
                }

                this.dataSeries[idIndex] = series;
            }
        }
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.dataSeries = [];
        for(var idIndex = 0; idIndex < chartData.getIDCount(); idIndex++) {
            var series = new FR.Chart[plotTypes.dataSeries];
            series.seriesIndex = idIndex;
            series.points = [];

            if(chartData.getIDCount() === 1) {//  项目ID == 系列名
                series.seriesName = "1";
            } else {
                series.seriesName = chartData.getIDName(idIndex);
            }

            for (var i = 0; i < chartData.getCategoryLabelCount(); i++) {
                var dataPoint = new FR.Chart[plotTypes.dataPoint];
                this.initDataPointValue(dataPoint, chartData, i, idIndex);
                dataPoint.categoryIndex = i;
                dataPoint.categoryName = chartData.getCategoryLabel(i);
                dataPoint.seriesIndex = idIndex;
                dataPoint.seriesName = series.seriesName;
                series.points[i] = dataPoint;
            }

            this.dataSeries[idIndex] = series;
        }
    },

    initDataPointValue:function(dataPoint, chartData, i, idIndex){
        var planStartValue = -1, planEndValue = -1, progressValue = -1, realStartValue = -1, realEndValue = -1;

        var number = chartData.getPlanStart(idIndex,i);
        if (number != null) {
            planStartValue = number;
        }
        number = chartData.getPlanEnd(idIndex, i);
        if (number != null) {
            planEndValue = number;
        }
        number = chartData.getRealStart(idIndex, i);
        if (number != null) {
            realStartValue = number;
        }
        number = chartData.getRealEnd(idIndex, i);
        if (number != null) {
            realEndValue = number;
        }
        number = chartData.getProgress(idIndex, i);
        if (number != null) {
            progressValue = number;
        }

        if (planStartValue == -1 || planEndValue == -1) {
            dataPoint.valueIsNull = true;
        } else {
            dataPoint.planStart = planStartValue;
            dataPoint.planEnd = planEndValue;
            dataPoint.value = progressValue;
            dataPoint.realStart = realStartValue;
            dataPoint.realEnd = realEndValue;
        }
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        this.layoutDoubleAxisGlyph(this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph);
        // 特别处理 左边的 分类轴. 原点不对.
        this.yAxisGlyph.originPoint = new FR.Chart.Point2D(this.yAxisGlyph.originPoint.x, this.yAxisGlyph.getBounds().y);
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        if (this.errorImageSrc || this.errorImageIESrc) {
            var newLine = '<br />'
            var resultHTML = '';
            resultHTML += FR.i18nText('Fine-Dec_Chart_Gantt_ErrorTip') + newLine;
            resultHTML += this.errorToopTip + FR.i18nText('Fine-Dec_Chart_Use_Data') + newLine;
            resultHTML += FR.i18nText('Fine-Dec_Chart_Gantt_Error_Check');
            this.errorDataTip = resultHTML;
            return;
        }

        var barPlotGlyph = new FR.Chart.Bar2DPlotGlyph();
        // 需要逐层判断系列的Size
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                if (dataPoint.valueIsNull) {
                    continue;
                }
                var color = this.conditionCollection.getConditionAttr(new FR.Chart.AttrColor(), dataPoint, this.createColors4Series());
                var alpha = this.conditionCollection.getConditionAttr(new FR.Chart.AttrAlpha(), dataPoint, this.createColors4Series());

                // 计划时间
                var planRect = barPlotGlyph.getBarShape4GanttPlot(dataPoint.planStart, dataPoint.planEnd,
                    categoryIndex,
                    this.yAxisGlyph, this.xAxisGlyph, this.dataSeries.length, seriesIndex);
                planRect.isHorizontal4Bounds = true;

                if (dataPoint.value > 0) {// 百分比的值存在 如果有进度百分比,则计划条不动态
                    planRect.isUnanimate = true;
                }

                var planGlyph = new FR.Chart.ShapeGlyph();
                planGlyph.shape = planRect;
                planGlyph.setBackground(FR.Chart.ColorBackground.getInstance(FR.Chart.Color.saturationDown(color.seriesColor)));
                planGlyph.setAlpha(this.planAlphaFromtUI);
                planGlyph.toolTipBackground = planGlyph.info.getBackground();

                dataPoint.glyph = planGlyph;

                if (dataPoint.value >= 0) {
                    var realRect = barPlotGlyph.getBarShape4GanttPlot(dataPoint.planStart,
                        dataPoint.planStart + (dataPoint.planEnd - dataPoint.planStart) * dataPoint.value, categoryIndex,
                        this.yAxisGlyph, this.xAxisGlyph, this.dataSeries.length, seriesIndex);
                    realRect.isHorizontal4Bounds = true;

                    var realGlyph = new FR.Chart.ShapeGlyph(realRect);
                    realGlyph.shape = realRect;
                    realGlyph.setBackground(FR.Chart.ColorBackground.getInstance(color.seriesColor));
                    realGlyph.setAlpha(alpha.alpha);
                    realGlyph.toolTipBackground = realGlyph.info.getBackground();
                    dataPoint.progressImpl = realGlyph;
                }

                if (dataPoint.realStart >= 0) {
                    var startDate = FR.Chart.ChartUtils.int2Date(dataPoint.realStart, FR.Chart.Constants.Second);
                    var axisStart = FR.Chart.ChartUtils.date2Int(startDate, this.xAxisGlyph.mainType);

                    var realStartX = this.xAxisGlyph.getPoint2D(axisStart).x;
                    var rectHeight = planRect.height;
                    // 实际开始标记
                    var startMarker = new FR.Chart.TriangleFilledMarker();
                    startMarker.size = rectHeight / 4;
                    startMarker.background = FR.Chart.ColorBackground.getInstance('rgb(0, 0, 0)');
                    var startMarkerGlyph = new FR.Chart.MarkerGlyph();
                    startMarkerGlyph.marker = startMarker;
                    startMarkerGlyph.color = 'rgb(0, 0, 0)';
                    startMarkerGlyph.setShape(new FR.Chart.Bounds(realStartX - rectHeight / 4, planRect.y + rectHeight * 0.5,
                        rectHeight * 0.5, rectHeight * 0.5));

                    startMarkerGlyph.toolTipBackground = startMarkerGlyph.background;
                    dataPoint.realStartImpl = startMarkerGlyph;
                }

                if (dataPoint.realEnd >= 0) {
                    var endDate = FR.Chart.ChartUtils.int2Date(dataPoint.realEnd, FR.Chart.Constants.Second);
                    var axisEnd = FR.Chart.ChartUtils.date2Int(endDate, this.xAxisGlyph.mainType);

                    var realEndX = this.xAxisGlyph.getPoint2D(axisEnd).x;
                    var rectHeight = planRect.height;
                    // 实际结束标记
                    var endMarker = new FR.Chart.DiamondFilledMarker();
                    endMarker.size = rectHeight / 4;
                    endMarker.background = FR.Chart.ColorBackground.getInstance('rgb(122, 225, 108)');
                    var endMarkerGlyph = new FR.Chart.MarkerGlyph();
                    endMarkerGlyph.marker = endMarker;
                    endMarkerGlyph.color = 'rgb(122, 225, 108)';
                    endMarkerGlyph.setShape(new FR.Chart.Bounds(realEndX - rectHeight * 0.25, planRect.y + rectHeight * 0.5,
                        rectHeight * 0.5, rectHeight * 0.5));

                    endMarkerGlyph.toolTipBackground = endMarkerGlyph.background;
                    dataPoint.realEndImpl = endMarkerGlyph;
                }
            }
        }
    },

    /**
     * 甘特图 画出绘图区
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        if (this.errorImageSrc || this.errorImageIESrc) {
            var ctx = ctxs[2];
            if(!ctx){     return; } ctx.save();

            var image = new Image();

            var imageX = this.getBounds().x + this.getBounds().width / 2;
            var imageY = this.getBounds().y + this.getBounds().height / 2;

            image.onload = function () {
                // kunsnat: 预防IE9的概率性 加载失败, 但是现在仍然小概率发生, 没辙了.
                setTimeout(function () {
                        ctx.drawImage(image,
                            imageX - image.width / 2, imageY - image.height / 2,
                            image.width, image.height);
                    },
                    100);
            };

            if (FR.Browser.isIE() || FR.Chart.ChartUtils.isIE11()) {
                image.src = FR.servletURL + "?op=fr_attach&cmd=ah_image&id=" + this.errorImageIESrc;
            } else {
                image.src = this.errorImageSrc;
            }

            ctx.restore();
        }

        FR.Chart.GanttPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    /**
     * GanttPlot, 画完动态之后的drawReal
     * @param ctx 画布
     * @param currentDivKey 当前div key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            for (var j = 0; j < this.dataSeries[i].points.length; j++) {
                this.dataSeries[i].points[j].drawReal(ctx, currentDivKey);
            }
        }
    }
});

/**
 *  仪表盘样式
 *     @example
 *     var op = {
 *            designType : 0,
   *            order : 0,
   *            dialShape : 360,
   *            meterAngle : 270,
   *            maxArrowAngle : 180,
   *            startValue : 0,
   *            endValue : 400,
   *            tickSize : 50,
   *            units : '单位',
   *            unitFormat : null,
   *            tickLabelsVisible : true,
   *            titleTextAttr : new FR.Chart.TextAttr(),
   *            intervals : [
   *               new FR.Chart.MeterInterval({
   *                        beginValue : 0,
   *                        endValue : 100,
   *                        backgroundColor : 'red'
   *               }),
   *               new FR.Chart.MeterInterval({
   *                        beginValue : 100,
   *                        endValue : 200,
   *                        backgroundColor : 'blue'
   *               }),
   *                new FR.Chart.MeterInterval({
   *                        beginValue : 200,
   *                        endValue : 400,
   *                        backgroundColor : 'green'
   *               })
   *            ]
 *     };
 *     var meterStyle = new FR.Chart.MeterStyle(op);
 *
 * @class FR.Chart.MeterStyle
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.designType = 0] 仪表盘样式 自动 或者 自定义
 * @param {Number} [js.order = 0] 数量级.
 * @param {Number} [js.dialShape = 360] 仪表盘外观形状
 * @param {Number} [js.meterAngle = 360] 仪表盘角度大小
 * @param {Number} [js.maxArrowAngle = 180] 仪表盘最大角度
 * @param {Number} [js.startValue = 0] 仪表盘起始值
 * @param {Number} [js.endValue = 0] 仪表盘结束值
 * @param {Number} [js.tickSize = 0] 仪表盘刻度大小
 * @param {String}[js.units = ''] 仪表盘单位
 * @param {String}[js.unitFormat=null] 仪表盘单位格式
 * @param {Boolean}{js.tickLabelsVisible = true} 仪表盘标签是否可见
 * @param {FR.Chart.TextAttr} [js.titleTextAttr = new FR.Chart.TextAttr()] 仪表盘标题文本格式.
 * @param {Array}[js.intervals = []] 数组 FR.Chart.MeterInterval
 *
 */
FR.Chart.MeterStyle = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};
        this.designType = FR.pick(js.designType, 0);
        this.order = FR.pick(js.order, 0);
        this.dialShape = FR.pick(js.dialShape, 360);

        this.meterAngle = FR.pick(js.meterAngle, 270);
        this.maxArrowAngle = FR.pick(js.maxArrowAngle, 180);

        this.startValue = FR.pick(js.startValue, 0.0);
        this.endValue = FR.pick(js.endValue, 400);
        this.tickSize = FR.pick(js.tickSize, 50.0);
        this.units = FR.pick(js.units, FR.i18nText('Fine-Dec_Chart_Use_Unit'));
        this.unitFormat = js.unitFormat;
        this.tickLabelsVisible = FR.pick(js.tickLabelsVisible, true);
        this.titleTextAttr = new FR.Chart.TextAttr(js.titleTextAttr);
        this.valueTextAttr = new FR.Chart.TextAttr(js.valueTextAttr);
        this.unitTextAttr = new FR.Chart.TextAttr(js.unitTextAttr);
        this.meterType = js.meterType;
        this.intervals = [];
        if (js.intervals && js.intervals.length > 0) {
            for (var i = 0; i < js.intervals.length; i++) {
                var meterInterval = new FR.Chart.MeterInterval(js.intervals[i]);
                this.intervals.push(meterInterval);
            }
        }
        this.isShowTitle = FR.pick(js.isShowTitle, true);
    },

    configSeries:function(seriesConfig){
        this.maxArrowAngle = FR.pick(seriesConfig.maxArrowAngle, 180);
        this.units = FR.pick(seriesConfig.units, FR.i18nText('Fine-Dec_Chart_Use_Unit'));
        this.order = FR.pick(seriesConfig.order, 0);
        this.meterType = FR.pick(seriesConfig.meterType, 0);
        this.isShowTitle = FR.pick(seriesConfig.isShowTitle);

        if(seriesConfig.intervals){
            this.designType = 1;
            for (var i = 0; i < seriesConfig.intervals.length; i++) {
                var meterInterval = new FR.Chart.MeterInterval(seriesConfig.intervals[i]);
                this.intervals.push(meterInterval);
            }
        }else{
            this.designType = 0;
        }

        if(seriesConfig.titleTextAttr){
            this.titleTextAttr = new FR.Chart.TextAttr(seriesConfig.titleTextAttr);
        }else{
            this.titleTextAttr = new FR.Chart.TextAttr();
            this.titleTextAttr.frFont = new FR.Chart.FRFont({size:10, fontName:"Microsoft Yahei"});
        }

        if(seriesConfig.valueTextAttr){
            this.valueTextAttr = new FR.Chart.TextAttr(seriesConfig.valueTextAttr);
        }else{
            this.valueTextAttr = new FR.Chart.TextAttr();
            this.valueTextAttr.frFont = new FR.Chart.FRFont({size:18, fontName:"Century Gothic", style:1});
        }

        if(seriesConfig.unitTextAttr){
            this.unitTextAttr = new FR.Chart.TextAttr(seriesConfig.unitTextAttr);
        }else{
            this.unitTextAttr = new FR.Chart.TextAttr();
            this.unitTextAttr.frFont = new FR.Chart.FRFont({size:12, fontName:"Microsoft Yahei"});
        }
    },

    /**
     * 根据值 自动重置 最大最小值.
     * @private
     * @param value 值
     */
    resetAutoDesignIntervals: function (value) {
        if (this.designType === 0 && value !== 0) {
            if(value < 0){
                var minValue = value * 3;
                this.startValue = minValue;
                this.endValue = 0;

                var meterInterval = this.getInterval(0);
                meterInterval.resetInterval(this.startValue, this.startValue - value, "rgb(231, 71, 62)");

                meterInterval = this.getInterval(1);
                meterInterval.resetInterval(this.startValue - value, this.startValue - 2 * value, "rgb(252, 210, 9)")

                meterInterval = this.getInterval(2);
                meterInterval.resetInterval(this.startValue - 2 * value, this.endValue, "rgb(77, 184, 73)");

            }else{
                var maxValue = ((value) * 3 / 2);
                this.startValue = 0;
                this.endValue = maxValue;

                var meterInterval = this.getInterval(0);
                meterInterval.resetInterval(this.startValue, this.endValue / 3, "rgb(231, 71, 62)");

                meterInterval = this.getInterval(1);
                meterInterval.resetInterval(this.endValue / 3, this.endValue * 2 / 3, "rgb(252, 210, 9)")

                meterInterval = this.getInterval(2);
                meterInterval.resetInterval(this.endValue * 2 / 3, this.endValue, "rgb(77, 184, 73)");
            }
        } else {
            this.startValue = this.getInterval(0).beginValue;
            this.endValue = this.getInterval(this.intervals.length - 1).endValue;
        }
    },

    /**
     * 添加 间隔色段
     * @param interval 间隔色段
     */
    addInterval: function (interval) {
        this.intervals.push(interval);
    },

    /**
     * 根据序号 返回间隔色段
     * @param index 序号
     * @returns {FR.Chart.MeterInterval} 间隔色段
     */
    getInterval: function (index) {
        if (index < 0 || index >= this.intervals.length) {
            this.intervals.push(new FR.Chart.MeterInterval());
        }
        return this.intervals[index];
    },

    /**
     * 复制
     * @returns {FR.Chart.MeterStyle}
     */
    clone: function () {
        var meterStyle = new FR.Chart.MeterStyle();
        meterStyle.designType = this.designType;
        meterStyle.order = this.order;
        meterStyle.dialShape = this.dialShape;
        meterStyle.meterAngle = this.meterAngle;
        meterStyle.maxArrowAngle = this.maxArrowAngle;
        meterStyle.startValue = this.startValue;
        meterStyle.endValue = this.endValue;
        meterStyle.tickSize = this.tickSize;
        meterStyle.units = this.units;
        meterStyle.unitFormat = this.unitFormat;
        meterStyle.tickLabelsVisible = this.tickLabelsVisible;

        meterStyle.titleTextAttr = this.titleTextAttr.clone();
        meterStyle.valueTextAttr = this.valueTextAttr.clone();
        meterStyle.unitTextAttr = this.unitTextAttr.clone();

        meterStyle.intervals = [];

        for (var i = 0; i < this.intervals.length; i++) {
            var meterInterval = this.intervals[i].clone();
            meterStyle.intervals.push(meterInterval);
        }

        meterStyle.isShowTitle = this.isShowTitle;

        return meterStyle;
    },

    /**
     * 清除所有间隔色段.
     */
    clearAllInterval: function () {
        if (this.intervals == null) {
            return;
        }
        this.intervals = [];
    }
});
$.extend(FR.Chart.MeterStyle, /**@class FR.Chart.MeterStyle*/{
    /**
     * @static
     * @property 默认的边框宽度
     */
    DEFAULT_BORDER_SIZE: 3,

    /**
     * @static
     * @property 默认的圆形半径
     */
    DEFAULT_CIRCLE_SIZE: 10,

    /**
     * 默认360度形状
     * @static
     * @property
     */
    CIRCLE: 360,

    /**
     * 默认180度形状
     * @static
     * @property
     */
    PIE: 180,

    /**
     * 默认270形状
     * @static
     * @property
     */
    PIE270: 270
});

/**
 *     仪表盘间隔色段
 *     @example
 *     var interval = new FR.Chart.MeterInterval({
   *                        beginValue : 200,
   *                        endValue : 400,
   *                        backgroundColor : 'green'
   *               })
 * @class FR.Chart.MeterInterval
 * @extends FR.Chart.ChartJsInit
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number}[js.beginValue = 0] 开始值
 * @param {Number}[js.endValue = 0] 结束值
 * @param {Color}[js.backgroundColor ='red'] 背景色
 */
FR.Chart.MeterInterval = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function (js) {
        js = js || {};

        this.beginValue = FR.pick(js.beginValue, 0);
        this.endValue = FR.pick(js.endValue, 0);
        this.backgroundColor = js.backgroundColor;
    },

    /**
     * 复制
     * @returns {FR.Chart.MeterInterval}
     */
    clone: function () {
        var meterInterval = new FR.Chart.MeterInterval();
        meterInterval.beginValue = this.beginValue;
        meterInterval.endValue = this.endValue;
        meterInterval.backgroundColor = this.backgroundColor;
        return meterInterval;
    },

    /**
     * 重置数据颜色
     * @param begin 开始值
     * @param end 结束值
     * @param color 颜色
     */
    resetInterval: function (begin, end, color) {
        this.beginValue = begin;
        this.endValue = end;
        this.backgroundColor = color;
    }
});

/**
 *   仪表盘绘图区
 *    @example
 *    var meter = new FR.Chart.MeterPlotGlyph();
 *
 * @class FR.Chart.MeterPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 * @param {FR.Chart.MeterStyle}[js.meterStyle=new FR.Chart.MeterStyle()] 仪表盘样式
 */
FR.Chart.MeterPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.meterStyle = js.meterStyle == undefined ? null : new FR.Chart.MeterStyle(js.meterStyle);

        FR.Chart.MeterPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    configSeries:function(seriesConfig){
        this.meterStyle = new FR.Chart.MeterStyle();
        this.meterStyle.configSeries(seriesConfig);
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        var sery = new FR.Chart[plotTypes.dataSeries]();
        sery.points = [];
        this.dataSeries.push(sery);

        for(var i = 0; i < data.length; i++){
            var singleData = data[i];
            var dataPoint = new FR.Chart[plotTypes.dataPoint]();
            dataPoint.categoryIndex = i;
            dataPoint.seriesIndex = 0;
            dataPoint.categoryName = singleData[0];
            dataPoint.value = singleData[1];
            sery.points.push(dataPoint);
        }
    },

    isSupportLegend:function(){
        return false;
    },

    addSeries2PlotGlyph:function(chartData, plotTypes){
        var namesCount = chartData.getCategoryLabelCount();
        this.dataSeries = [];

        var series = new FR.Chart[plotTypes.dataSeries];
        series.seriesIndex = 0;
        series.seriesName = "1";
        series.points = [];
        this.dataSeries.push(series);

        for (var nameIndex = 0; nameIndex < namesCount; nameIndex++) {
            var dataPoint = new FR.Chart[plotTypes.dataPoint];
            dataPoint.categoryInedex = nameIndex;
            dataPoint.seriesIndex = 0;

            var number = chartData.getValue(nameIndex);
            dataPoint.value = number ? number : 0;
            if(number == null){
                dataPoint.valueIsNull = true;
            }

            dataPoint.categoryName = chartData.getCategoryLabel(nameIndex);
            dataPoint.seriesName = "1";
            series.points[nameIndex] = dataPoint;
        }
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {
        // do nothing
    },

    /**
     * 布局数据系列
     */
    layoutDataSeriesGlyph: function () {
        var plotBounds = this.getBounds();

        var seriesCount = this.dataSeries.length;
        var categoryCount = this.getCategoryCount();

        var gapWidth = 0;
        var gapHeight = 0;

        var eachHeight = (plotBounds.height - ((seriesCount + 1) * gapHeight)) / seriesCount;
        var eachWidth = (plotBounds.width - ((categoryCount + 1) * gapWidth)) / categoryCount;

        for (var seriesIndex = 0; seriesIndex < seriesCount; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];

            for (var categoryIndex = 0; categoryIndex < categoryCount; categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                if (dataPoint.valueIsNull) {
                    dataPoint.glyph = null;
                    continue;
                }
                var title = dataPoint.categoryName;
                var dim = FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(title, this.meterStyle.titleTextAttr);
                dim = this.meterStyle.isShowTitle ? dim : new FR.Chart.Dimension2D();

                var titleGlyph = new FR.Chart.TextGlyph({text: title, textAttr: this.meterStyle.titleTextAttr});
                titleGlyph.textAttr.frFont = this.meterStyle.titleTextAttr.frFont;

                var meterStyle = this.meterStyle;
                if (this.meterStyle.designType === 0) {
                    meterStyle = this.meterStyle.clone();
                }
                var meter;
                if(this.meterStyle.meterType === 0){
                    meter =  new FR.Chart.Meter(titleGlyph,dataPoint.value, this.meterStyle.clone());
                } else if(this.meterStyle.meterType === 1) {
                    meter =  new FR.Chart.BlueMeter(titleGlyph,dataPoint.value, this.meterStyle.clone());
                } else if(this.meterStyle.meterType === 2) {
                    meter =  new FR.Chart.SimpleMeter(titleGlyph,dataPoint.value, this.meterStyle.clone());
                }
                dataPoint.glyph = meter;

                meter.bounds = new FR.Chart.Bounds(categoryIndex * eachWidth + plotBounds.x,
                    seriesIndex * eachHeight + dim.height + plotBounds.y,
                    eachWidth, eachHeight - dim.height);

                // 记录绝对位置. 便于画meter中的vaulelabel
                meter.plotX = plotBounds.x;
                meter.plotY = plotBounds.y;

                dataPoint.dataLabel = null; // 不要标签了
            }
        }
    },

    doOnMove:function() {

    },

    /**
     * 画出仪表盘绘图区
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {

        this.drawInfo(ctxs);

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var categoryIndex = 0, categoryCount = dataSeries.getDataPointCount(); categoryIndex < categoryCount; categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];
                var needReset = true;
                //自定义的不管超不超出范围都重画
                if(this.lastPlotGlyph && this.lastPlotGlyph.dataPointDoExit(seriesIndex, categoryIndex) && this.meterStyle.designType === 0){
                    var lastMeter = this.getLastDataPointGlyph(dataPoint);

                    //判断是否超出最大最小值，超出则重新计算
                    var betweenInterval = lastMeter && lastMeter.meterStyle.startValue <= dataPoint.value && lastMeter.meterStyle.endValue >= dataPoint.value;

                    if(betweenInterval){
                        dataPoint.glyph.meterStyle = lastMeter.meterStyle;
                        dataPoint.lastGlyph = lastMeter;
                        needReset = false;
                    }
                }
                if(dataPoint.glyph){
                    dataPoint.glyph.drawMeterBottom(ctxs[1], needReset);
                }
            }
        }

        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            var stepNumber = 12; // actionModel 动态变化的方式
            if (FR.Browser.isIE6()) {
                stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[1];
                time = FR.Chart.ChartPaintConstants.ACTION_TIME[1];
            } else {
                if (actionModel) {
                    stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[actionModel];
                    time = FR.Chart.ChartPaintConstants.ACTION_TIME[actionModel];
                }
            }
            this.dataSeries[i].getAnimateGlyph(stepNumber, actionModel);
        }

        var step = this.getAnimateStepNumber();
        if (step > 0) {
            this.animateGlyphDraw(ctxs[2], time, step);
        } else {
            this.animateGlyphDraw(ctxs[2], 1, 1);
        }
    }
});

/**
 * CustomPlotGlyph.start
 * 组合图绘图区
 *
 *     @example
 *     var custom = new FR.Chart.CustomPlotGlyph();
 *
 * @class FR.Chart.CustomPlotGlyph
 * @extends FR.Chart.CategoryPlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @property {FR.Chart.Bar2DPlotGlyph} [js.bar2DPlotGlyph]默认柱形图样式
 * @property {FR.Chart.Bar2DPlotGlyph} [js.bar2DPlotGlyphStack] 默认柱形图堆积样式
 * @property {FR.Chart.LinePlotGlyph}[js.linePlotGlyph] 默认折线图样式
 * @property {FR.Chart.AreaPlotGlyph}[js.areaPlotGlyphStack] 默认堆积面积图样式
 */
FR.Chart.CustomPlotGlyph = FR.extend(FR.Chart.CategoryPlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.bar2DPlotGlyph = new FR.Chart.Bar2DPlotGlyph(js.bar2DPlotGlyph);
        this.bar2DPlotGlyphStack = new FR.Chart.Bar2DPlotGlyph(js.bar2DPlotGlyphStack);
        this.linePlotGlyph = new FR.Chart.LinePlotGlyph(js.linePlotGlyph);
        this.areaPlotGlyphStack = new FR.Chart.AreaPlotGlyph(js.areaPlotGlyphStack)
        this.bar3DPlotGlyph = new FR.Chart.Bar3DPlotGlyph(js.bar3DPlotGlyph);
        this.bar3DPlotGlyphStack = new FR.Chart.Bar3DPlotGlyph(js.bar3DPlotGlyphStack);
        this.customTypeCondition = new FR.Chart.ConditionCollection(js.customTypeCondition);

        this.has3DShape = FR.pick(js.has3DShape, false);
        this.seriesOverlapPercent = FR.pick(js.seriesOverlapPercent, -0.25);
        this.categoryIntervalPercent = FR.pick(js.categoryIntervalPercent, 1.0);
        this.projection = new FR.Chart.Projection(js.projection);
        this.projection.calculateSINCOS();
        this.stackedList = [];

        FR.Chart.CustomPlotGlyph.superclass.initConstants.apply(this, [js]);
    },

    isSupportLabelWrap:function(){
        return true;
    },

    /**
     * 获取第二个值轴
     */
    getSecondValueAxisGlyph:function(){
        return this.secondAxisGlyph;
    },

    //装载完数据才能初始化条件显示
    configCondition:function(plotConfig){
        if(!plotConfig || !plotConfig.typeConfig){
            return;
        }

        var conditionConfig = FR.pick(plotConfig.conditionConfig, {});
        var typeConfig = FR.pick(plotConfig.typeConfig, {});

        this.conditionCollection.addAll(conditionConfig, this.dataSeries);
        this.customTypeCondition.addAllWithDefault(typeConfig, this.dataSeries);

        this._syncConditionCollection();
        this._syncPlotFillStyle();
        this._syncDataSeries();
        this._syncPlotStyle();
    },

    _syncConditionCollection:function(){
        this.bar2DPlotGlyph.conditionCollection = this.conditionCollection;
        this.bar2DPlotGlyphStack.conditionCollection = this.conditionCollection;
        this.linePlotGlyph.conditionCollection = this.conditionCollection;
        this.areaPlotGlyphStack.conditionCollection = this.conditionCollection;
        this.bar3DPlotGlyph.conditionCollection = this.conditionCollection;
        this.bar3DPlotGlyphStack.conditionCollection = this.conditionCollection;
    },

    _syncPlotFillStyle:function(){
        this.bar2DPlotGlyph.plotFillStyle = this.plotFillStyle;
        this.bar2DPlotGlyphStack.plotFillStyle = this.plotFillStyle;
        this.linePlotGlyph.plotFillStyle = this.plotFillStyle;
        this.areaPlotGlyphStack.plotFillStyle = this.plotFillStyle;
        this.bar3DPlotGlyph.plotFillStyle = this.plotFillStyle;
        this.bar3DPlotGlyphStack.plotFillStyle = this.plotFillStyle;
    },

    _synStockChart:function(){
        this.bar2DPlotGlyph.isStockChart = this.isStockChart;
        this.bar2DPlotGlyphStack.isStockChart = this.isStockChart;
        this.linePlotGlyph.isStockChart = this.isStockChart;
        this.areaPlotGlyphStack.isStockChart = this.isStockChart;
        this.bar3DPlotGlyph.isStockChart = this.isStockChart;
        this.bar3DPlotGlyphStack.isStockChart = this.isStockChart;
    },

    _syncDataSeries:function(){
        this.bar2DPlotGlyph.dataSeries = this.dataSeries;
        this.bar2DPlotGlyphStack.dataSeries = this.dataSeries;
        this.linePlotGlyph.dataSeries = this.dataSeries;
        this.areaPlotGlyphStack.dataSeries = this.dataSeries;
        this.bar3DPlotGlyph.dataSeries = this.dataSeries;
        this.bar3DPlotGlyphStack.dataSeries = this.dataSeries;
    },

    _syncPlotStyle:function(){
        this.bar2DPlotGlyph.plotStyle = this.plotStyle;
        this.bar2DPlotGlyphStack.plotStyle = this.plotStyle;
        this.linePlotGlyph.plotStyle = this.plotStyle;
        this.areaPlotGlyphStack.plotStyle = this.plotStyle;
        this.bar3DPlotGlyph.plotStyle = this.plotStyle;
        this.bar3DPlotGlyphStack.plotStyle = this.plotStyle;
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph:function(){
        if(this.has3DShape){
            this.layoutAxisGlyph4ThreeD(this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph)
        }else{
            this.layoutDoubleAxisGlyph(this.xAxisGlyph, this.yAxisGlyph, this.secondAxisGlyph);
        }
    },

    /**
     * 返回3D的深度
     * @returns {*} 深度
     */
    get3DDeep:function(){
        return 70;
    },

    isNeedDefaultDeep:function(){
        return true;
    },

    draw: function (ctxs, time, actionModel) {
        if(this.has3DShape){
            this.draw3DInfo(ctxs);
            if (this.xAxisGlyph) {
                this.xAxisGlyph.draw4ThreeD(ctxs[1]);
            }
            if (this.yAxisGlyph) {
                this.yAxisGlyph.draw4ThreeD(ctxs[1]);
            }
            if(this.secondAxisGlyph){
                this.secondAxisGlyph.draw4ThreeD(ctxs[1]);
            }
        }

        FR.Chart.CustomPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);
    },

    /**
     * 布局系列
     */
    layoutDataSeriesGlyph: function () {
        this._synStockChart();
        this.normalBarCount = 0;
        this.stackedList = [];
        for (var i = 0; i < this.dataSeries.length; i++) {
            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                continue;
            }
            var seriesAttr = this.customTypeCondition.getAttr(this.dataSeries[i]);
            var render = seriesAttr.useRenderer;
            if (render == FR.Chart.CustomAttr.BAR_RENDERER || render == FR.Chart.CustomAttr.BAR3D) {
                this.normalBarCount++;
            }
        }

        var stackedBarCount = 0;
        for(var i = 0; i < this.customTypeCondition.getAttrSize(); i++){
            var attr = this.customTypeCondition.getAttrByIndex(i);
            if(attr.isStackedBarShape()){
                var stackList = attr.getStackedSeriesList(this.dataSeries);
                this.stackedList.push(stackList);

                var hasVisualSeries = false;
                for(var tmpIndex = 0; tmpIndex < stackList.length; tmpIndex++){
                    var tmpSeries = this.dataSeries[stackList[tmpIndex]];
                    if(tmpSeries && tmpSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                        hasVisualSeries = true;
                    }
                }
                if(hasVisualSeries){
                    stackedBarCount++;
                }
            }
        }

        //处理默认的情况，比较复杂
        var defaultAttr = this.customTypeCondition.defaultAttr;
        if(defaultAttr.isStackedBarShape()){
            var unUsedSeriesIndex = this.customTypeCondition.getUnUsedSeriesIndex(this.dataSeries);
            this.stackedList.push(unUsedSeriesIndex);

            var hasVisualSeries = false;
            for(var tmpIndex = 0; tmpIndex < unUsedSeriesIndex.length; tmpIndex++){
                var tmpSeries = this.dataSeries[unUsedSeriesIndex[tmpIndex]];
                if(tmpSeries && tmpSeries.paintState > FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                    hasVisualSeries = true;
                }
            }
            if(hasVisualSeries){
                stackedBarCount++;
            }
        }

        this._dealSeriesSingle(this.normalBarCount + stackedBarCount);

        if(this.bar2DPlotGlyphStack.bounds){
            this.bar2DPlotGlyphStack.setTotalRectForStackedBar(this);
        }

        this.dealLabelBoundsInOrder();
    },

    getBarPercent: function (seriesCount) {
        return 1.0 / (seriesCount - ((seriesCount - 1 ) * this.seriesOverlapPercent) + this.categoryIntervalPercent);
    },

    getBarStartPercent:function(bar_percent, barIndex, xAxisGlyph, categoryIndex){
        return bar_percent * (1 - this.seriesOverlapPercent) * barIndex + xAxisGlyph.getTickIndex4Value(categoryIndex) +
            0.5 * this.categoryIntervalPercent * bar_percent;
    },

    _getAxisPosition:function(seriesIndex){
        var customSeriesAttr = this.customTypeCondition.getAttr(this.dataSeries[seriesIndex]);
        return customSeriesAttr.getAxisPosition();
    },

    _getRenderer:function(seriesIndex){
        var customSeriesAttr = this.customTypeCondition.getAttr(this.dataSeries[seriesIndex]);
        if(customSeriesAttr){
            return customSeriesAttr.useRenderer;
        }
    },

    getSeriesAxisGlyph:function(seriesIndex){
        var position = this._getAxisPosition(seriesIndex);
        return position == FR.Chart.CustomAttr.Axis_RIGHT ? this.secondAxisGlyph : this.yAxisGlyph;
    },

    _getStackedBarIndex:function(inIndex){
        for(var i = 0; i < this.stackedList.length; i++){
            var singleBar = this.stackedList[i];
            for(var j = 0; j < singleBar.length; j++){
                if(singleBar[j] == inIndex){
                    return this.normalBarCount + i;
                }
            }
        }
        return -1;
    },

    _getStackedSeriesList:function(seriesIndex){
        for(var i = 0; i < this.stackedList.length; i++){
            var singleBar = this.stackedList[i];
            for(var j = 0; j < singleBar.length; j++){
                if(seriesIndex == singleBar[j]){
                    return singleBar;
                }
            }
        }
        return [];
    },

    _dealSeriesSingle:function(totalBarCount){
        var barSeriesIndex = 0;
        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            var customSeriesAttr = this.customTypeCondition.getAttr(dataSeries);
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            var render = customSeriesAttr.useRenderer;
            var currentValueAxisGlyph = this.getSeriesAxisGlyph(seriesIndex);
            var stackedSeriesBarIndex = this._getStackedBarIndex(seriesIndex);

            if (render == FR.Chart.CustomAttr.BAR_RENDERER) {
                this.bar2DPlotGlyph.bounds = this.getBounds();
                var adjustType = this.bar2DPlotGlyph.dealShape4Bar(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, barSeriesIndex);
                if(adjustType !== -1){
                    this.bar2DPlotGlyph.dealBarShape4LargeDataStockChart(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, barSeriesIndex,adjustType);
                }
                if(this.isStockChart){
                    this.bar2DPlotGlyph.refreshDataPointLabel();
                }
                barSeriesIndex++;
            } else if (render == FR.Chart.CustomAttr.BAR_STACK) {
                this.bar2DPlotGlyphStack.bounds = this.getBounds();
                var adjustType = this.bar2DPlotGlyphStack.dealShape4Bar(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, stackedSeriesBarIndex);
                if(adjustType !== -1){
                    this.bar2DPlotGlyphStack.dealBarShape4LargeDataStockChart(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, stackedSeriesBarIndex,adjustType);
                }
                if(this.isStockChart){
                    this.bar2DPlotGlyphStack.refreshDataPointLabel();
                }
            } else if (render == FR.Chart.CustomAttr.AREA_STACK) {
                this.areaPlotGlyphStack.bounds = this.getBounds();
                var adjustType = this.areaPlotGlyphStack.dealAreaShape4PlotGlyph(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph);
                if(adjustType !== -1){
                    this.areaPlotGlyphStack.dealAreaLargeData4StockChart(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph,adjustType);
                }
                if(this.isStockChart){
                    this.areaPlotGlyphStack.refreshDataPointLabel();
                }
            } else if (render == FR.Chart.CustomAttr.LINE_RENDERER){
                this.linePlotGlyph.bounds = this.getBounds();
                var adjustType = this.linePlotGlyph.dealLine4PlotGlyph(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph);
                //-1为成功布局
                if(adjustType !== -1){
                    this.linePlotGlyph.dealLine4StockChartLargeData(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph);
                }
                if(this.isStockChart){
                    this.linePlotGlyph.refreshDataPointLabel();
                }
            }else if(render == FR.Chart.CustomAttr.BAR3D){
                this.bar3DPlotGlyph.dealShape4CubicBar(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, barSeriesIndex);
                barSeriesIndex++;
            }else if(render == FR.Chart.CustomAttr.BAR3D_STACK){
                this.bar3DPlotGlyphStack.dealShape4CubicBar(dataSeries, this, this.xAxisGlyph, currentValueAxisGlyph, totalBarCount, stackedSeriesBarIndex);
            }
        }
    },

    /**
     * 返回堆积的前柱形值
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @param isNegative 是否负值
     * @returns {Number} 堆积值
     */
    getPreSum4Bar: function (seriesIndex, categoryIndex, isNegative) {
        var stackList = this._getStackedSeriesList(seriesIndex);
        var pre_sum = 0;
        var pre_sum_neg = 0;
        for (var i = 0; i < stackList.length; i++) {
            var preIndex = stackList[i];
            if(seriesIndex == preIndex){
                break;
            }

            if(this.dataSeries[preIndex].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                continue;
            }

            var seriesValue = this.getDataPointPercentValue(preIndex, categoryIndex);
            if (seriesValue > 0){
                pre_sum += seriesValue;
            }else{
                pre_sum_neg += seriesValue;
            }
        }

        if (isNegative) {
            return pre_sum_neg;
        } else {
            return pre_sum;
        }
    },

    /**
     * 堆积面积图值, 包括兼容之前版本的 堆积折线图
     * @param seriesIndex 系列序号
     * @param categoryIndex 分类序号
     * @returns {Number} 堆积面积图值
     */
    getPreSum4Area:function(seriesIndex, categoryIndex){
        var axisPosition = this._getAxisPosition(seriesIndex);
        var pre_sum = 0;
        for(var i = seriesIndex - 1; i >= 0; i--){

            if(this.dataSeries[i].paintState < FR.Chart.ChartPaintConstants.STATE_ENABLE){
                continue;
            }

            var otherPosition = this._getAxisPosition(i);
            var renderer = this._getRenderer(i);
            if (FR.equals(otherPosition, axisPosition) && FR.equals(renderer, FR.Chart.CustomAttr.AREA_STACK)){
                pre_sum += this.getDataPointPercentValue(i, categoryIndex);
            }
        }
        return pre_sum;
    },


    /**
     *返回指定系列 日期范围的堆积值
     * @param seriesIndex 系列
     * @param date 日期
     * @param dateType 范围类型
     */
    getPreSum4AreaDataRange:function(seriesIndex,baseDate,dateType,xAxisGlyph){
        var axisPosition = this._getAxisPosition(seriesIndex);
        var pre_sum = 0;
        for (var i = seriesIndex - 1; i >= 0; i--) {
            var dataSeries =this.dataSeries[i];
            var seriesSum = 0;
            var seriesCount = 0;
            if (dataSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }

            var otherPosition = this._getAxisPosition(i);
            var renderer = this._getRenderer(i);

            if(!FR.equals(otherPosition, axisPosition) || !FR.equals(renderer, FR.Chart.CustomAttr.AREA_STACK)){
                continue;
            }

            for (var categoryIndex = 0; categoryIndex < dataSeries.getDataPointCount(); categoryIndex++) {
                var dataPoint = dataSeries.points[categoryIndex];

                if(dataPoint.valueIsNull){
                    continue ;
                }
                var cateDate = xAxisGlyph.getDate4Value(categoryIndex);
                if(FR.Chart.ChartUtils.isBelongToSameUnit(baseDate,cateDate,dateType)){
                    seriesSum += this.getDataPointPercentValue(i, categoryIndex);
                    seriesCount++;
                }
            }
            if(seriesCount>0){
                pre_sum += seriesSum/seriesCount;
            }
        }

        return pre_sum;
    },

    /**
     * 重置 计算期间的最大最小值
     * @param isLog 是否为对数
     */
    resetValue: function (isLog) {
        this.pointMaxMinValue = 0;
        this.barStackMaxMinValue = 0;
        this.areaStackMaxMinValue = 0;

        if (isLog) {
            this.pointMaxMinValue = 1;
        }
    },

    /**
     * 根据render类型 累计相应的 最大最小值
     * @param render 图类型
     * @param isMax 累计最大值 或者最小值
     * @param dataPointValue 系列点值
     */
    addValueInRender: function (render, isMax, dataPointValue) {
        if (render == FR.Chart.CustomAttr.BAR_STACK || render == FR.Chart.CustomAttr.BAR3D_STACK) {
            this.barStackMaxMinValue += dataPointValue;
        } else if (render == FR.Chart.CustomAttr.AREA_STACK || render == FR.Chart.CustomAttr.LINE_STACK) {
            this.areaStackMaxMinValue += dataPointValue;
        } else {
            if (isMax) {
                this.pointMaxMinValue = Math.max(this.pointMaxMinValue, dataPointValue);
            } else {
                this.pointMaxMinValue = Math.min(this.pointMaxMinValue, dataPointValue);
            }
        }
    },

    /**
     * 从系列中获取最小值
     * @param isLog 是否为对数
     * @param leftOrRight  左侧坐标轴或者右侧坐标轴
     * @returns {Number}
     */
    getMinValueFromData: function (isLog, leftOrRight) {
        var minValue = 0.0;
        var crossValue = 0.0;
        if (isLog) {
            minValue = Number.MAX_VALUE;
            crossValue = 1;
        }

        for (var cateIndex = 0; cateIndex < this.getCategoryCount(); cateIndex++) {
            this.resetValue(isLog);

            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataPoint = this.dataSeries[seriesIndex].points[cateIndex];
                if (dataPoint.valueIsNull || this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
                    || !this.isInCategoryField(this.xAxisGlyph, dataPoint.categoryName)) {
                    continue;
                }

                var pointValue = dataPoint.value;

                var seriesAttr = this.customTypeCondition.getAttr(seriesIndex);
                var seriesPosition = this._getAxisPosition(seriesIndex);
                if (pointValue < crossValue && seriesPosition == leftOrRight) {
                    this.addValueInRender(seriesAttr.renderer, false, pointValue);
                    minValue = (isLog && pointValue < crossValue && minValue > pointValue) ? pointValue : minValue;
                }
            }

            if (!isLog) {
                minValue = Math.min(minValue, Math.min(this.barStackMaxMinValue,
                    Math.min(this.areaStackMaxMinValue, this.pointMaxMinValue)));
            }
        }

        if (minValue == Number.MAX_VALUE) {
            return 1;
        }
        return minValue;
    },

    /**
     * 从系列中获取最大值
     * @param leftOrRight 左侧坐标轴或者右侧坐标轴
     * @returns {Number} 最大值
     */
    getMaxValueFromData: function (leftOrRight) {
        var maxValue = 0.0;

        for (var cateIndex = 0; cateIndex < this.getCategoryCount(); cateIndex++) {
            this.resetValue(false);

            for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
                var dataPoint = this.dataSeries[seriesIndex].points[cateIndex];
                if (dataPoint.valueIsNull || this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE
                    || !this.isInCategoryField(this.xAxisGlyph, dataPoint.categoryName)) {
                    continue;
                }

                var dataPointValue = dataPoint.value;

                var seriesAttr = this.customTypeCondition.getAttr(seriesIndex);
                var seriesPosition = this._getAxisPosition(seriesIndex);
                if (dataPointValue >= 0.0 && seriesPosition == leftOrRight) {
                    this.addValueInRender(seriesAttr.renderer, true, dataPointValue);
                }
            }

            maxValue = Math.max(maxValue, Math.max(this.barStackMaxMinValue,
                Math.max(this.areaStackMaxMinValue, this.pointMaxMinValue)));
        }

        if (maxValue == this.getMinValueFromData(false, leftOrRight)) {
            return maxValue + 10;
        }
        return maxValue;
    },

    /**
     * 返回 组合图系列的绘画顺序. 默认为 先堆积面积图在底层,  其次为柱形图,  然后是折线图.
     * @returns {Array}
     */
    getDrawSequence: function () {
        var order = [];
        for(var i = 0, seriesSize = this.dataSeries.length ; i < seriesSize; i++){
            order.push(i);
        }

        //计算顺序
        var bar3DSeries = [];
        var barSeries = [];
        var areaSeries = [];
        var lineSeries = [];

        for(var i = 0, seriesSize = this.dataSeries.length ; i < seriesSize; i++){
            var series = this.dataSeries[i];
            var cusAttr = this.customTypeCondition.getAttr(series);
            var useRenderer = cusAttr.useRenderer;
            if(useRenderer == FR.Chart.CustomAttr.BAR_RENDERER){
                barSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.BAR3D){
                bar3DSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.AREA_STACK){
                areaSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.LINE_RENDERER){
                lineSeries.push(i);
            }
        }

        var currentSeriesIndex = 0;

        this._addIndexToOrder(barSeries, order, currentSeriesIndex);
        currentSeriesIndex += barSeries.length;

        if(!this._is3DStackedBar()){
            for(var i = 0; i < this.stackedList.length; i++){
                var singleStacked = this.stackedList[i];
                this._addIndexToOrder(singleStacked, order, currentSeriesIndex);
                currentSeriesIndex += singleStacked.length;
            }
        }

        this._addIndexToOrder(areaSeries, order, currentSeriesIndex);
        currentSeriesIndex += areaSeries.length;

        this._addIndexToOrder(lineSeries, order, currentSeriesIndex);
        currentSeriesIndex += lineSeries.length;

        this._addIndexToOrder(bar3DSeries, order, currentSeriesIndex);
        currentSeriesIndex += bar3DSeries.length;

        if(this._is3DStackedBar()){
            for(var i = 0; i < this.stackedList.length; i++){
                var singleStacked = this.stackedList[i];
                this._addIndexToOrder(singleStacked, order, currentSeriesIndex);
                currentSeriesIndex += singleStacked.length;
            }
        }

        return order;
    },

    _is3DStackedBar:function(){
        if(this.stackedList != null){
            for(var i = 0; i < this.stackedList.length; i++){
                var singleStacked = this.stackedList[i];
                for(var j = 0; j < singleStacked.length; j++){
                    var seriesIndex = singleStacked[j];
                    var seriesAttr = this.customTypeCondition.getAttr(this.dataSeries[seriesIndex]);
                    if(seriesAttr.useRenderer == FR.Chart.CustomAttr.BAR3D_STACK){
                        return true;
                    }
                }
            }
        }
        return false;
    },

    _addIndexToOrder:function(seriesIndex, order, startIndex){
        for(var i = 0; i < seriesIndex.length; i++){
            order[i + startIndex] = seriesIndex[i];
        }
    },

    /**
     * 返回动态步骤数目.
     * @returns  {Number}步骤数目
     */
    getAnimateStepNumber: function () {
        var maxStepNumber = 0;

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            if (this.dataSeries[seriesIndex].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            maxStepNumber = Math.max(maxStepNumber, this.dataSeries[seriesIndex].getAnimateStepNumber());
        }

        return maxStepNumber;
    },

    /**
     * 因为组合图 的stepNumber是所有系列的最大Number
     *
     * 在animateGlyphDraw时 无法控制 是否画当前系列, 只能在此判断是否画.
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        var bar3DStackDataSeries = [];
        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            if (plotGlyph.dataSeries[seq[i]].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var seriesAttr = this.customTypeCondition.getAttr(plotGlyph.dataSeries[seq[i]]);
            if(seriesAttr.useRenderer == FR.Chart.CustomAttr.BAR3D_STACK){
                //三维堆积柱形图按分类画
                bar3DStackDataSeries[bar3DStackDataSeries.length] = plotGlyph.dataSeries[seq[i]];
                continue;
            }
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);

            if (plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine) {
                plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine(ctx, count);
            }
        }

        //三维堆积柱形图按分类画
        this.drawBar3DStackSeries(ctx, plotGlyph, count, bar3DStackDataSeries);

        for (var i = plotGlyph.dataSeries.length - 1; i >= 0; i--) {
            if (plotGlyph.dataSeries[seq[i]].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            plotGlyph.dataSeries[i].drawAnimateGlyph4TrendLine(ctx, count);
        }
    },

    /**
     * 动态结束后 画标签.
     * @param ctx 画布
     * @param currentDivKey 当前 div key
     */
    drawMarkersLabelAfterSeriesAnimateDraw: function (ctx, currentDivKey) {
        for (var i = 0, len = this.dataSeries.length; i < len; i++) {
            if (this.dataSeries[i].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            this.dataSeries[i].drawLabel(ctx, currentDivKey);

            if(this.customTypeCondition){
                var render = this.customTypeCondition.getAttr(i).renderer;
                if (render == FR.Chart.CustomAttr.LINE_RENDERER
                    || render == FR.Chart.CustomAttr.AREA_STACK) {
                    var dataPoints = this.dataSeries[i].points;
                    for (var j = 0; j < dataPoints.length; j++) {
                        dataPoints[j].draw(ctx, currentDivKey);
                    }
                }
            }
        }
        this.drawAlertLinesAfterSeriesAnimateDraw(ctx);
    },

    findOnMoveGlyph:function(mousePos, event, ctx4OnMove) {
        var chosenSeries = this.dataSeries[this.currentChosenSeriesIndex];
        if(chosenSeries && this._isLineSeries(chosenSeries)){
            this._findOnMoveGlyph4Line(mousePos, event, ctx4OnMove);
        }else{
            FR.Chart.CustomPlotGlyph.superclass.findOnMoveGlyph.apply(this, [mousePos, event, ctx4OnMove]);
        }
    },

    findClickOnGlyph:function(mousePos, event){
        if(this.bounds && this.bounds.containsPoint(mousePos.x, mousePos.y)){
            var chosenSeries = this.dataSeries[this.currentChosenSeriesIndex];
            if(chosenSeries && this._isLineSeries(chosenSeries)){
                this._findSeriesGlyphPoint(chosenSeries, mousePos, event);
            }

            if(!event._hasfound){
                this._findNearByDataPoint(mousePos, event);
            }
        }
    },

    /**
     * 判断数据点对应的shape的部分是否在鼠标画出来的矩形中，组合图需要分别判断一下
     * @param dataPoint{DataPoint} 数据点对象，里面存有该数据点对应的图形对象
     * @param drawBounds{Bounds} 鼠标画出来的矩形
     * @returns {boolean} 返回矩形是否包含图形
     */
    categoryInBounds: function (dataPoint, drawBounds) {
        return this.bar2DPlotGlyph.categoryInBounds(dataPoint, drawBounds)
            || this.bar2DPlotGlyphStack.categoryInBounds(dataPoint, drawBounds)
            || this.linePlotGlyph.categoryInBounds(dataPoint, drawBounds)
            || this.areaPlotGlyphStack.categoryInBounds(dataPoint, drawBounds);
    }

});

/**
 * bug:75664 解决了这段代码删掉，为了避免订制
 */
FR.Chart.ThreeDCustomPlotGlyph = FR.extend(FR.Chart.CustomPlotGlyph, {
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var seq = plotGlyph.getDrawSequence();
        var bar3DStackDataSeries = [];

        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            if (plotGlyph.dataSeries[seq[i]].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var seriesAttr = this.customTypeCondition.getAttr(plotGlyph.dataSeries[seq[i]]);
            if(seriesAttr.useRenderer == FR.Chart.CustomAttr.BAR3D_STACK){
                bar3DStackDataSeries[bar3DStackDataSeries.length] = plotGlyph.dataSeries[seq[i]];
                continue;
            }
        }
        this.drawBar3DStackSeries(ctx, plotGlyph, count, bar3DStackDataSeries);

        for (var i = 0; i < plotGlyph.dataSeries.length; i++) {
            if (plotGlyph.dataSeries[seq[i]].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            var seriesAttr = this.customTypeCondition.getAttr(plotGlyph.dataSeries[seq[i]]);
            if(seriesAttr.useRenderer == FR.Chart.CustomAttr.BAR3D_STACK){
                continue;
            }
            plotGlyph.dataSeries[seq[i]].drawAnimateGlyph(ctx, count);

            if (plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine) {
                plotGlyph.dataSeries[seq[i]].drawAnimateGlyph4TopFoldLine(ctx, count);
            }
        }

        for (var i = plotGlyph.dataSeries.length - 1; i >= 0; i--) {
            if (plotGlyph.dataSeries[seq[i]].paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                continue;
            }
            plotGlyph.dataSeries[i].drawAnimateGlyph4TrendLine(ctx, count);
        }

    },

    getDrawSequence: function () {
        var order = [];
        for(var i = 0, seriesSize = this.dataSeries.length ; i < seriesSize; i++){
            order.push(i);
        }

        var bar3DSeries = [];
        var barSeries = [];
        var areaSeries = [];
        var lineSeries = [];

        for(var i = 0, seriesSize = this.dataSeries.length ; i < seriesSize; i++){
            var series = this.dataSeries[i];
            var cusAttr = this.customTypeCondition.getAttr(series);
            var useRenderer = cusAttr.useRenderer;
            if(useRenderer == FR.Chart.CustomAttr.BAR_RENDERER){
                barSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.BAR3D){
                bar3DSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.AREA_STACK){
                areaSeries.push(i);
            }else if(useRenderer == FR.Chart.CustomAttr.LINE_RENDERER){
                lineSeries.push(i);
            }
        }

        var currentSeriesIndex = 0;

        this._addIndexToOrder(barSeries, order, currentSeriesIndex);
        currentSeriesIndex += barSeries.length;

        if(!this._is3DStackedBar()){
            for(var i = 0; i < this.stackedList.length; i++){
                var singleStacked = this.stackedList[i];
                this._addIndexToOrder(singleStacked, order, currentSeriesIndex);
                currentSeriesIndex += singleStacked.length;
            }
        }

        this._addIndexToOrder(areaSeries, order, currentSeriesIndex);
        currentSeriesIndex += areaSeries.length;

        this._addIndexToOrder(bar3DSeries, order, currentSeriesIndex);
        currentSeriesIndex += bar3DSeries.length;

        if(this._is3DStackedBar()){
            for(var i = 0; i < this.stackedList.length; i++){
                var singleStacked = this.stackedList[i];
                this._addIndexToOrder(singleStacked, order, currentSeriesIndex);
                currentSeriesIndex += singleStacked.length;
            }
        }

        this._addIndexToOrder(lineSeries, order, currentSeriesIndex);

        return order;
    }
});

// GisMapPlotGlyph.start
/**
 * Gis地图 绘图区
 *
 *     @example
 *     var gis = new FR.Chart.GisMapPlotGlyph();
 *
 * @class FR.Chart.GisMapPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {Number} [js.gisType] Gis地图类型 (百度或者谷歌)
 * @param {String}[js.addressType] 位置地址类型 (经纬度 或者详细地址)
 * @param {String}[js.key] 地图产品使用key
 * @property {Array} [pointPosition] 系列点的位置数组.
 */
FR.Chart.GisMapPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};
        this.gisType = js.gisType;
        this.addressType = js.addressType;
        this.key = js.key;
        FR.Chart.GisMapPlotGlyph.superclass.initConstants.apply(this, [js]);

        this.pointPosition = [];
    },

    //根据导出的配置里的data来构造系列
    installSeriesWithOptionData:function(data, plotTypes){
        this.dataSeries = [];
        var series = new FR.Chart.DataSeries();
        this.dataSeries.push(series);
        if(data){
            for(var i = 0; i < data.length; i++){
                var singleData = data[i];
                var title = singleData.seriesName;
                var value = singleData.value;
                for(var j = 0; j < value.length; j++){
                    var singleValue = value[j];
                    var dataPoint;
                    if(i === 0){
                        dataPoint = new FR.Chart[plotTypes.dataPoint]();
                        series.points.push(dataPoint);
                        dataPoint.categoryName = singleValue[0];
                        dataPoint.value = singleValue[singleValue.length - 1];
                        if(singleValue.length > 2){
                            dataPoint.addressName = singleValue[1];
                        }
                    }else{
                        dataPoint = series.points[j];
                    }
                    dataPoint.mapAreaValue.addTitleValue(new FR.Chart.MapTitleValue({title:title, value:singleValue[singleValue.length-1]}));
                }
            }
        }
    },


    isSupportLegend:function(){
        return false;
    },

    addSeries2PlotGlyph:function(gisMapChartData, plotTypes){
        this.key = "OkCharts";
        this.dataSeries = [];
        var series = new FR.Chart.DataSeries();
        this.dataSeries.push(series);

        var addressTittleValue = gisMapChartData.addressTitleValue;
        var addressName = gisMapChartData.addressName;
        var isAddress = gisMapChartData.isAddress;
        var isLngFirst = gisMapChartData.isLngFirst;
        var addresses = gisMapChartData.addresses;
        for(var i = 0; i < addresses.length; i++){
            var address = addresses[i];
            var point = new FR.Chart[plotTypes.dataPoint]();
            if(isAddress){
                point.categoryName = address;
            }else{
                var result = address;
                var comp = address.split(',');
                if(comp.length !== 2){
                    continue;
                }
                if(this.isDegreeFormat(result)){
                    result = this.degree2number(comp[0]) + "," + this.degree2number(comp[1]);
                }
                if(this.gisType != isLngFirst){
                    result = comp[1] + ',' + comp[0];
                }
                point.categoryName = result;
            }

            var tittleVale = addressTittleValue[address];
            if(tittleVale.isTitleValueNull()) {
                point.value = 0;
                point.valueIsNull = true;
            } else {
                var firstValue = tittleVale.getFirstTitleValue();
                point.value = firstValue;
                point.mapAreaValue = tittleVale;
            }
            if(addressName && !FR.isEmpty(addressName[address])){
                point.addressName = addressName[address];
            }
            series.points.push(point);
        }
    },

    degree2number:function(address){
        var first = null; //度
        var second = null; //分
        var third = null;//秒

        var backIndex = 0;
        var forwardIndex = 0;
        for(; forwardIndex < address.length; forwardIndex++){
            if(address.charAt(forwardIndex) == '°'){
                first = address.substring(backIndex, forwardIndex);
                backIndex = forwardIndex + 1;
            }else if(address.charAt(forwardIndex) == '′'){
                second = address.substring(backIndex, forwardIndex);
                backIndex = forwardIndex + 1;
            }else if(address.charAt(forwardIndex) == '″'){
                third = address.substring(backIndex, forwardIndex);
                break;
            }
        }

        var result = 0;
        if(third != null){
            result += parseFloat(third) / (60 * 60);
        }
        if(second != null){
            result += parseFloat(second) / 60;
        }
        if(first != null){
            result += parseFloat(first);
        }

        return result;
    },

    isDegreeFormat:function(address){
        if(address){
            return address.indexOf('°') != -1
                || address.contains('′') != -1
                    || address.contains('″') != -1;
        }
        return false;
    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        var address = para[0];
        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            var points = series.points;
            for(var j = 0, cateCount = points.length; j < cateCount; j++){
                var dataPoint = series.points[j];
                if(FR.equals(address, dataPoint.categoryName)){
                    return dataPoint;
                }
            }
        }
    },

    /**
     * 设置 父级的 div
     * @param div 元素div
     */
    setDivID: function (div) {
        this.parentDiv = div;
    },

    /**
     * 加载 Gis地图信息
     */
    draw: function () {
        var id = this.parentDiv.getAttribute("id") + "GisMap";
        var div4gis = document.getElementById(id);
        if(!div4gis){
            var dom = $('<div>').attr('id', id).css('width', this.bounds.width)
                .css('height', this.bounds.height).css('position', 'absolute')
                .css('left', this.bounds.x).css('top', this.bounds.y + "px").css('z-index', 10);
            div4gis = dom[0];
            this.parentDiv.appendChild(div4gis);
        }

        if(!this.dataSeries[0] || !this.dataSeries[0].points.length){
            return ;
        }

        if (this.gisType) {
            if (FR.isEmpty(this.key)) {
                this.dealWithNoKey(div4gis);
            } else {
                this.drawBaiduMap(id);
            }
        } else {
            if (FR.isEmpty(this.key)) {
                this.dealWithNoKey(div4gis);
            } else {
                this.drawGoogleMap(id);
            }
        }
    },

    /**
     * 处理下key为空的情况，防止报错出现
     * @param div4gis 地图Gis div
     */
    dealWithNoKey: function (div4gis) {
        var img = document.createElement("img");
        img.src = FR.servletURL + '?op=resource&resource=com/fr/chart/base/GisError.jpg';
        img.style.width = div4gis.style.width;
        img.style.height = div4gis.style.height;
        div4gis.appendChild(img);
    },

    _getDataPointByCategoryName:function(cateName){
        var pointSize = this.dataSeries[0].points.length;
        for (var i = 0; i < pointSize; i++) {
            var dataPoint = this.dataSeries[0].points[i];
            if(FR.equals(dataPoint.categoryName, cateName)){
                return dataPoint;
            }
        }
    },

    setTooltipValue:function(paras){
        if(paras.length !== 2){
            return;
        }
        var cateName = paras[0];
        var tooltipValue = paras[1];
        var dataPoint = this._getDataPointByCategoryName(cateName);
        if(dataPoint && dataPoint.marker){
            dataPoint.isCustomTooltip = true;
            dataPoint.tooltip = tooltipValue;
        }
    },

    triggerTooltip:function(paras){
        if(paras.length !== 1){
            return;
        }
        var cateName = paras[0];
        var dataPoint = this._getDataPointByCategoryName(cateName);
        if(dataPoint && dataPoint.marker){
            var marker = dataPoint.marker;
            var map = marker.getMap();
            var tooltip = this.getToolTipString(dataPoint, this.hotTooltipStyle);
            var infowindow = new BMap.InfoWindow('<p>'+ tooltip +'</p>');
            var pixel = map.pointToPixel(marker.getPosition());
            pixel.y -= 20;
            map.openInfoWindow(infowindow, map.pixelToPoint(pixel));
        }
    },

    /**
     * 加载百度地图
     * @param id4plot 百度地图div的id
     */
    drawBaiduMap: function (id4plot) {
        //初始化地图
        var map = new BMap.Map(id4plot);
        this.map = map;
        if (this.addressType) {
            var myGeo = new BMap.Geocoder();
            myGeo.getPoint(this.dataSeries[0].points[0].categoryName, function (point) {
                map.centerAndZoom(point, 8);
            });
        } else {
            var addArray = this.dataSeries[0].points[0].categoryName.split(",");
            var point = new BMap.Point(addArray[0], addArray[1]);
            map.centerAndZoom(point, 8);
        }
        map.enableScrollWheelZoom();

        function ComplexCustomOverlay(point, dataPoint, attrContents){
            this._point = point;
            this._dataPoint = dataPoint;
            this._attrContents = attrContents;
        }

        var self = this;
        ComplexCustomOverlay.prototype = new BMap.Overlay();
        ComplexCustomOverlay.prototype.initialize = function(map){
            this._labelContent = self.getLabelString(this._dataPoint, this._attrContents);
            var frFont = this._attrContents.textAttr.frFont;
            var fontSize = frFont.size;

            this._map = map;
            var div = this._div = document.createElement("div");
            div.style.position = "absolute";
            div.style.whiteSpace = "nowrap";
            div.style.MozUserSelect = "none";
            div.style.zIndex = BMap.Overlay.getZIndex(this._point.lat);
            div.style.backgroundColor = "rgb(85,148,255)";
            div.style.border = "1px solid rgb(85,148,255)";

            div.style.color = frFont.color;
            div.style.font = frFont.style + " " + frFont.size + "px " + frFont.fontName;

            div.style.borderRadius = "4px";
            div.style.lineHeight = fontSize + "px";
            div.style.padding = "8px 4px";
            div.style.textAlign = "center";

            var span = this._span = document.createElement("span");
            div.appendChild(span);
            span.innerHTML = this._labelContent.label;

            var arrow = this._arrow = document.createElement("div");
            var url = FR.servletURL + '?op=resource&resource=/com/fr/chart/base/tri.png';
            arrow.style.background = "url(" + url + ") no-repeat";
            arrow.style.position = "absolute";
            arrow.style.width = "9px";
            arrow.style.height = "7px";

            arrow.style.top = fontSize * this._labelContent.lineCount + 14 + "px";
            arrow.style.left = this._labelContent.maxLineWidth / 2 + 4 + "px";
            arrow.style.overflow = "hidden";
            div.appendChild(arrow);

            this._map.getPanes().labelPane.appendChild(div);
            return div;
        }

        ComplexCustomOverlay.prototype.draw = function(){
            var fontSize = this._attrContents.textAttr.frFont.size;

            var map = this._map;
            var pixel = map.pointToOverlayPixel(this._point);

            this._div.style.left = pixel.x - this._labelContent.maxLineWidth / 2 - 10 + "px";
            this._div.style.top  = pixel.y -  fontSize * this._labelContent.lineCount - 51 + "px";
        }

        //添加标注点
        var pointSize = this.dataSeries[0].points.length;
        for (var i = 0; i < pointSize; i++) {
            var dataPoint = this.dataSeries[0].points[i];
            this.resolveBaidu(dataPoint, map, ComplexCustomOverlay);
        }
    },

    /**
     * 处理百度 位置 信息点
     * @param address 地址
     * @param tooltip 热点提示
     * @param map 地图
     * @param hyperlink 超链
     */
    resolveBaidu: function (dataPoint, map, ComplexCustomOverlay) {
        var address = dataPoint.categoryName;
        var myGeo = new BMap.Geocoder();
        var detx = this.bounds.x;
        var dety = this.bounds.y;
        var chartEvent = {'clientX': detx, 'clientY': dety};
        var positions = this.pointPosition;
        var self = this;

        var addMarker = function (point, dataPoint, map) {
            var marker = new BMap.Marker(new BMap.Point(point.lng, point.lat));
            dataPoint.marker = marker;

            map.addOverlay(marker);
            map.centerAndZoom(point, 15);

            var hyperlink = dataPoint.hyperlink;

            if(self.conditionCollection.defaultAttr.isContains(new FR.Chart.AttrContents())){
                var attrContents = self.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
                var myCompOverlay = new ComplexCustomOverlay(point, dataPoint, attrContents);
                map.addOverlay(myCompOverlay);

                dataPoint.location = point;
                dataPoint.lastOverlay = myCompOverlay;
                dataPoint.ComplexCustomOverlay = ComplexCustomOverlay;
                dataPoint.map = map;
                dataPoint.attrContents = attrContents;
            }

            marker.addEventListener('click', function () {
                if(!FR.isEmpty(hyperlink)){
                    var pixel = map.pointToPixel(point);
                    chartEvent.clientX = pixel.x + detx;
                    chartEvent.clientY = pixel.y + dety;
                    FR.doHyperlink(chartEvent, (new Function("return " + hyperlink))(), true);
                }
            });

            marker.addEventListener('mouseover', function () {
                var tooltip = self.getToolTipString(dataPoint, self.hotTooltipStyle);
                if(!FR.isEmpty(tooltip)){
                    var infowindow = new BMap.InfoWindow('<p>'+tooltip+'</p>');
                    this.openInfoWindow(infowindow);
                }
            });

            marker.addEventListener('mouseout', function () {
                this.closeInfoWindow();
            });

            positions.push(point);
            map.setViewport(positions);
        };

        if (this.addressType) {
            myGeo.getPoint(address, function (point) {
                if (point) {
                    addMarker(point, dataPoint, map);
                }
            });
        } else if (address && address.split(",").length === 2) {
            var addArray = address.split(",");
            var point = new BMap.Point(addArray[0], addArray[1]);
            addMarker(point, dataPoint, map);
        }
    },

    /**
     * 加载谷歌地图
     * @param id4plot  绘图区的id
     */
    drawGoogleMap: function (id4plot) {
        //初始化地图
        var geocoder = new google.maps.Geocoder();
        var startPoint;
        if (this.addressType) {
            geocoder.geocode({'address': this.dataSeries[0].points[0].categoryName}, function (results, status) {
                if (status == google.maps.GeocoderStatus.OK) {
                    startPoint = results[0].geometry.location;
                }
            });
        } else {
            var addArray = this.dataSeries[0].points[0].categoryName.split(",");
            startPoint = new google.maps.LatLng(addArray[0], addArray[1]);
        }

        var mapOptions = {
            center: startPoint,
            zoom: 8,
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };
        var map = new google.maps.Map(document.getElementById(id4plot),
            mapOptions);

        //添加标注点
        var pointSize = this.dataSeries[0].points.length;
        for (var i = 0; i < pointSize; i++) {
            var point = this.dataSeries[0].points[i];
            var tooltip = this.getToolTipString(point, this.hotTooltipStyle);
            var address = point.categoryName;
            this.resolveGoogle(address, tooltip, map, point.hyperlink, point);
        }
    },

    /**
     * 处理谷歌位置信息
     * @param address 地址
     * @param tooltip 热点
     * @param map 地图
     * @param hyperlink 超链
     */
    resolveGoogle: function (address, tooltip, map, hyperlink, dataPoint) {
        var geocoder = new google.maps.Geocoder();
        var detx = this.bounds.x;
        var dety = this.bounds.y;
        var centerX = this.bounds.width/2;
        var centerY = this.bounds.height/2;
        var chartEvent = {'clientX': detx, 'clientY': dety};
        var positions = this.pointPosition;
        var self = this;
        //添加标注点
        var addMarker = function (location, tooltip, map, hyperlink) {

            if (self.conditionCollection.defaultAttr.isContains(new FR.Chart.AttrContents())) {

                var attrContents = self.conditionCollection.getConditionAttr(new FR.Chart.AttrContents(), dataPoint);
                var _labelContent = self.getLabelString(dataPoint, attrContents);

                //没找到italic的配置项
                var markerLabel = {
                    color: attrContents.textAttr.frFont.color,
                    fontFamily: attrContents.textAttr.frFont.fontName,
                    fontWeight: attrContents.textAttr.frFont.style.indexOf('bold') != -1 ? 'bold' : 'normal',
                    fontSize: attrContents.textAttr.frFont.size + 'px',
                    text: _labelContent.label
                };
            }

            var marker = new google.maps.Marker({position: location, label: markerLabel});
            marker.setMap(map);
            var infowindow = new google.maps.InfoWindow({content: tooltip});
            if (hyperlink != null) {
                google.maps.event.addListener(marker, 'click', function () {
                    var pixel = arguments[0].pixel;
                    chartEvent.clientX = pixel.x + detx + centerX;
                    chartEvent.clientY = pixel.y + dety + centerY;
                    FR.doHyperlink(chartEvent, (new Function("return " + hyperlink))(), true);
                });
            } else {
                google.maps.event.addListener(marker, 'click', function () {
                    infowindow.open(map, marker);
                });
            }

            google.maps.event.addListener(marker, 'mouseover', function () {
                infowindow.open(map, marker);
            });

            google.maps.event.addListener(marker, 'mouseout', function () {
                infowindow.close();
            });

            positions.push(location);
            var bounds = new google.maps.LatLngBounds();
            for (var i = 0, LtLgLen = positions.length; i < LtLgLen; i++) {
                bounds.extend(positions[i]);
            }
            map.fitBounds(bounds);
        }

        if (this.addressType) {
            geocoder.geocode({ 'address': address}, function (results, status) {
                if (status == google.maps.GeocoderStatus.OK) {
                    var location = results[0].geometry.location;
                    addMarker(location, tooltip, map, hyperlink);
                }
            });
        } else if (address && address.split(",").length === 2) {
            var addArray = address.split(",");
            var location = new google.maps.LatLng(addArray[0], addArray[1]);
            addMarker(location, tooltip, map, hyperlink);
        }
    },

    //判断第index个value是不是不是数字，字符串
    _getValueString:function(value, format){
        if(isNaN(parseFloat(value))){
            return value;
        }
        return FR.contentFormat(value, format);
    },

    _getSeprator:function(seriesLabel){
        var separator = ',';
        var separators = FR.Chart.Separator;
        for(var i = 0; i < separators.length; i++){
            if(seriesLabel.indexOf(separators[i]) != -1){
                separator = separators[i];
                break;
            }
        }
        if(separator.indexOf('{BR}') != -1){
            separator = '<br />';
        }
        return separator;
    },

    getLabelString:function(point, attrContents){
        var seriesLabel = attrContents.seriesLabel;
        if(FR.isEmpty(seriesLabel)){
            return "";
        }
        var separator = this._getSeprator(seriesLabel);
        var labelString = this.getToolTipString(point, attrContents, separator);
        if(FR.isEmpty(labelString)){
            return {label:"", lineCount:0, maxLineWidth:0};
        }
        labelString = labelString.substr(0, labelString.length - 1);

        var components = [labelString];

        if(point.isCustomLabel && point.dataLabel){
            components = [point.dataLabel.text];
        }else{
            if(labelString.indexOf('<br />') !== -1){
                components = labelString.split(separator)
            }
        }

        var maxLineWidth = 0;
        var textAttr = attrContents.textAttr;
        for(var i = 0; i < components.length; i++){
            var str = components[i];
            maxLineWidth = Math.max(maxLineWidth, FR.Chart.TextUtils.calculateTextDimensionWithNoRotation(str, textAttr).width);
        }

        return {label:labelString, lineCount:components.length, maxLineWidth:maxLineWidth};
    },

    /**
     *  返回系列点的热点提示信息
     * @param point 系列点
     * @returns {String} 热点文本
     */
    getToolTipString: function (point, attrContents, separator) {

        //如果通过接口改动过的话直接返回自定义的内容
        if(point.isCustomTooltip){
            return point.tooltip;
        }

        if(separator == null || separator == undefined){
            separator = "<br />";
        }
        var tooltip = "";
        var labelString = attrContents.seriesLabel;
        var format = attrContents.format ? attrContents.format: '#.##';
        if (FR.isEmpty(labelString)){
            return tooltip;
        }
        if (labelString.indexOf('{ADDRESS}') != -1) {
            var address = point.categoryName;
            tooltip += address + separator;
        }
        if (labelString.indexOf('{ADDRESS_NAME}') != -1) {
            var addressName = point.addressName;
            tooltip += addressName + separator;
        }
        if (labelString.indexOf('{AREA_TITTLE}') != -1 && labelString.indexOf('{VALUE}') != -1) {
            var tittleValues = point.mapAreaValue.titleValues;
            for (var i = 0; i < tittleValues.length; i++) {
                var title = tittleValues[i].title;
                var value = this._getValueString(tittleValues[i].value, format);
                tooltip += title + ":" + value + separator;
            }
        } else if (labelString.indexOf('{AREA_TITTLE}') == -1 && labelString.indexOf('{VALUE}') != -1) {
            var tittleValues = point.mapAreaValue.titleValues;
            for (var i = 0; i < tittleValues.length; i++) {
                var value = this._getValueString(tittleValues[i].value, format);
                tooltip += value + separator;
            }
        } else if (labelString.indexOf('{AREA_TITTLE}') != -1 && labelString.indexOf('{VALUE}') == -1) {
            var tittleValues = point.mapAreaValue.titleValues;
            for (var i = 0; i < tittleValues.length; i++) {
                var title = tittleValues[i].title;
                tooltip += title + separator;
            }
        }
        return tooltip;
    },

    layoutAxisGlyph: function (leftOffset) {

    },

    layoutDataSeriesGlyph: function () {

    }
});

// MapPlotGlyph.start
/**
 *      地图绘图区
 *
 *      @example
 *      var map = new FR.Chart.MapPlotGlyph();
 *
 * @class FR.Chart.MapPlotGlyph
 * @extends FR.Chart.PlotGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {String}[js.mapName] 地图名称
 * @param {String} [js.returnIE] ie时的返回按钮图片
 * @param {String} [js.returnNormal] 非IE时的返回按钮图片.
 * @param {Array} [js.pathMap] 存储的路径数组. key 地图名 value 地图路径
 * @param {Array}[js.imageBounds]图片的边界存储数组 key 地图名 value 数组
 * @param {Array}[js.imageSrc] 图片的src存储  key 地图名 value 地图src地址
 * @param {FR.Chart.MapHotAreaColor}[js.mapAreaColor] 地图区域颜色
 *
 * @property {Array}[js.cacheImages] 存储已经加载过的图片.
 * @property {Array} [js.addedBounds] 已经添加的标签边界
 * @property {Number} [js.showLayerIndex] 当前展示的层级序号
 * @property {String}[js.fromName] 地图钻去时, 来源地图名称.
 */
FR.Chart.MapPlotGlyph = FR.extend(FR.Chart.PlotGlyph, {
    initConstants: function (js) {
        js = js || {};

        this.isSvgMap = FR.pick(js.isSvgMap, true);
        this.mapName = js.mapName;
        this.currentAttr = "";//当前展示的地图的mapAttr
        this.mapAreaColor = new FR.Chart.MapHotAreaColor(js.mapAreaColor);

        this.isHeatMap = FR.pick(js.isHeatMap, true);
        this.heatIndex = FR.pick(js.heatIndex, 0);
        this.mapType = FR.pick(js.mapType, FR.Chart.MapType.Map_Normal);

        this.showLayerIndex = 0;
        this.fromName = [];
        this.zoom = [1,1];
        this.offsetX = [0,0];
        this.offsetY = [0,0];
        this.totalOffsetX = [0,0];
        this.totalOffsetY = [0,0];

        if(js.mapFilePath){//从导出的属性来
            this.mapFilePath = FR.pick(js.mapFilePath, "./map");
            this.isSvgMap = true;
        }

        if (!this.isSvgMap) {//mapplotglyph.java 里只对非svg地图配置了这个
            if (FR.Browser.isIE() || FR.Chart.ChartUtils.isIE11()) {
                this.returnIE = new Image();
                this.returnIE.src = FR.servletURL + "?op=fr_attach&cmd=ah_image&id=" + js.normalIEsrc;
            } else if (js.normalsrc) {
                this.returnNormal = new Image();
                this.returnNormal.src = js.normalsrc;
            }
        }

        FR.Chart.MapPlotGlyph.superclass.initConstants.apply(this, [js]);

        if(this.isSvgMap){
            this.initSvgMap(this.mapName, true);
            this.currentAttr = FR.Chart.MapHelper.getMapAttr(this.mapName, this.isSvgMap);
        }else{
            this.initBitMap(js);
        }

        this._createHotTooltip4CombineGlyph();//按照设计文档的方式创建数据点提示

        this.initZoomOffset();
    },

    /**
     * 初始化svg地图
     * @param js
     */
    initSvgMap:function(mapName, initGet){

        var serviceName = initGet ? 'svginit' : 'chartrelate';

        if(FR.Chart.MapHelper.containsSvgMap(mapName)){
            this.fromName[this.showLayerIndex] = this.mapName;
            return;
        }

        var location = FR.servletURL;
        if(!FR.isEmpty(this.mapFilePath)){
            location = this.mapFilePath+"/"+mapName+".svg";
        }

        var self = this;
        FR.ajax({
            type: 'GET',
            url: location,
            data: {
                op: serviceName,
                cmd: 'map_getsvg',
                mapName: mapName,
                __time: new Date().getTime()
            },
            dataType: 'xml',
            async: false,
            success: function (svgRoot) {
                self.fromName[self.showLayerIndex] = self.mapName;
                FR.Chart.MapHelper.addSvgMapAttr(mapName, new FR.Chart.MapSvgAttr(svgRoot));
            }
        });
    },

    /**
     * 初始化位图地图
     * @param js 配置
     */
    initBitMap:function(js){
        if (js.imageBounds && js.pathMap) {
            var tmp = js.imageBounds[0];
            var mapName = tmp.mapName;

            if(FR.Chart.MapHelper.containsBitMap(mapName)){
                this.currentAttr = FR.Chart.MapHelper.getBitMapAttr(mapName);
                return;
            }

            var imageBounds = new FR.Chart.Bounds(0, 0, tmp.mapWidth, tmp.mapHeight);
            var src = {};
            src.onie = tmp.onie;
            src.onsrc = tmp.onsrc;

            this.currentAttr = new FR.Chart.MapAttr(js.pathMap, imageBounds, src);
            FR.Chart.MapHelper.addBitMapAttr(mapName, this.currentAttr);
        }
    },

    installSeriesWithOptionData:function(data, plotTypes){
        if(data){
            var chartData = new FR.Chart.MapChartData();
            var areaNames = [];
            var titleValues = [];
            var seriesSize = data.length;
            for(var i = 0; i < seriesSize; i++){
                var singleSeries = data[i];
                var title = singleSeries.seriesName;
                for(var j = 0; j < singleSeries.value.length; j++){
                    var singleValue = singleSeries.value[j];
                    areaNames[j] = singleValue[0];
                    titleValues.push(new FR.Chart.MapTitleValue({title:title, value:singleValue[1]}));
                }
            }

            for(var i = 0; i < areaNames.length; i++){
                var mapAreaValue = new FR.Chart.MapAreaValue();
                for(var j = 0; j < seriesSize; j++){
                    mapAreaValue.addTitleValue(titleValues[j*areaNames.length + i]);
                }
                chartData.addMapData(areaNames[i], mapAreaValue);
            }

            this.addSeries2PlotGlyph(chartData, plotTypes);
        }
    },

    //装载数据
    addSeries2PlotGlyph:function(chartData, plotTypes){
        this.fromMapNames = {};//记录下上层的地图

        var legendSize = this.mapAreaColor.getAreaNumber();
        var noValueSeries = 1;
        for (var i = 0; i < legendSize + noValueSeries; i++) {
            var series = new FR.Chart[plotTypes.dataSeries]();
            series.seriesIndex = i;
            this.dataSeries[i] = series;
        }

        var layerIndex = 0;
        this.add4NextSeries4Svg(this.mapName, this.mapName, layerIndex, chartData);
    },

    //地图不需要计算
    calculatePercentValue:function(){

    },

    /**
     * 不同的plotGlyph根据不用的para找到相应的dataPoint
     * @param para 默认是通过分类名和系列名来找数据点
     */
    getDataPointByPara:function(para){
        var areaName = para[0];
        for(var i = 0; i < this.dataSeries.length; i++){
            var series = this.dataSeries[i];
            var points = series.points || [];
            for(var j = 0, cateCount = points.length; j < cateCount; j++){
                var dataPoint = series.points[j];
                if(dataPoint && FR.equals(areaName, dataPoint.categoryName) && FR.equals(dataPoint.seriesName, this.mapName)){
                    return dataPoint;
                }
            }
        }
    },

    add4NextSeries4Svg:function(mapFrom, useMapName, layerIndex, mapChartData){
        var mapAttr = FR.Chart.MapHelper.getSvgMapAttr(useMapName);
        if(!mapAttr){
            return new FR.Chart.MapAreaValue();
        }

        var editingMap = FR.Chart.MapHelper.getSvgMapAttr(this.mapName);
        var currentTotal = new FR.Chart.MapAreaValue();

        var shapeNames = mapAttr.names;
        for(var nameIndex = 0; nameIndex < shapeNames.length; nameIndex++){
            var name  = shapeNames[nameIndex];
            var point = new FR.Chart.DataPoint4Map();
            point.nameFrom = mapFrom;
            point.layerIndex = layerIndex;
            point.seriesName = useMapName;

            if(!FR.equals(useMapName, mapFrom)){
                this.fromMapNames[useMapName] = mapFrom;
            }

            var nameLayerTo = editingMap.getLayerTo(name);
            var dataUseName = this.getDataUseName4Svg(editingMap, name, useMapName);

            if(mapChartData.hasLayerTo){
                point.hasNextTo = !FR.isEmpty(nameLayerTo);
                point.nameLayerTo = nameLayerTo;
            }else{
                point.hasNextTo = false;
                point.nameLayerTo = '';
            }

            point.categoryName = dataUseName;
            point.categoryOriginalName = name;

            this.initDataPoint4Map(nameLayerTo, useMapName, dataUseName, layerIndex, mapChartData, point, currentTotal)
        }

        return currentTotal;

    },

    initDataPoint4Map:function(nameLayerTo, useMapName, dataUseName, layerIndex, mapChartData, point, currentTotal){
        var seriesIndex = 0;
        var mapAreaValue = mapChartData.getMapData(dataUseName);

        if(this.isPointComNext(point, mapChartData, useMapName, nameLayerTo)) {// 从底层开始  递归取数据.

            var next = this.add4NextSeries4Svg(useMapName, nameLayerTo, layerIndex+1, mapChartData);;

            if(next.isTitleValueNull()) {
                if(!mapAreaValue.isTitleValueNull()){
                    var firstValue = mapAreaValue.getFirstTitleValue();
                    point.value = firstValue;
                    point.areaValue = mapAreaValue;
                    this.checkMapAreaTitle(currentTotal, mapAreaValue);
                    seriesIndex = this.mapAreaColor.getColorIndexWithValue(firstValue);
                }else{
                    point.value = 0;
                    point.valueIsNull = true;
                    seriesIndex = this.dataSeries.length - 1;
                }
            } else {
                var firstValue = next.getFirstTitleValue();
                point.value = firstValue;
                point.areaValue = next;
                seriesIndex = this.mapAreaColor.getColorIndexWithValue(firstValue);
            }
        } else {
            if(mapAreaValue.isTitleValueNull()) {
                point.value = 0;
                point.valueIsNull = true;
                seriesIndex = this.dataSeries.length - 1;
            } else {
                var firstValue = mapAreaValue.getFirstTitleValue();
                point.value = firstValue;
                point.areaValue = mapAreaValue;
                this.checkMapAreaTitle(currentTotal, mapAreaValue);
                seriesIndex = this.mapAreaColor.getColorIndexWithValue(firstValue);
            }
        }

        point.seriesIndex = seriesIndex;
        this.dataSeries[seriesIndex].points.push(point);
    },

    checkMapAreaTitle:function(com, ob){
        if(com.isTitleValueNull()) {
            var size = ob.titleValues.length;
            for(var i = 0; i < size; i++) {
                var titleValue = ob.titleValues[i];
                com.titleValues.push(new FR.Chart.MapTitleValue(titleValue.title, titleValue.value));
            }
        } else {
            var size = ob.titleValues.length;
            for(var i = 0; i < size; i++) {
                var titleValue = ob.titleValues[i];
                var comTV = com.titleValues[i];
                comTV.value += titleValue.value;
            }
        }
    },

    isPointComNext:function(point, mapChartData, useMapName, nameLayerTo){
        return point.hasNextTo && mapChartData.hasLayerTo
            && !FR.equals(useMapName, nameLayerTo) && !FR.isEmpty(nameLayerTo);
    },

    getDataUseName4Svg:function(editingMap, name, useMapName){
        var dataUseName = editingMap.getNameToValue(name);
        if(!FR.equals(useMapName, this.mapName)){
            var useMapAttr = FR.Chart.MapHelper.getSvgMapAttr(useMapName);
            if(useMapAttr){
                dataUseName = useMapAttr.getNameToValue(name);
            }
        }
        return dataUseName;
    },

    //根据装载好的数据，创建图例项
    createLegendItems:function(){
        return [];
    },

    /**
     * 返回到上一层
      */
    returnBackToLayer: function () {
        this.mapName = this.getBackNameFrom();//  根据当前名字 得到上一层 名称的包含关系 所属
        this.showLayerIndex = this.showLayerIndex - 1;
        this.currentAttr = FR.Chart.MapHelper.getMapAttr(this.mapName, this.isSvgMap);
        this.checkShowIndex();
    },

    /**
     * 返回地图 来源名称
     * @returns {String}
     */
    getBackNameFrom: function () {
        if(this.showLayerIndex <= 0){
            return this.mapName;
        }

        return this.fromName[this.showLayerIndex - 1];
    },

    hasNextLayer:function() {
        for (var seriesIndex = 0, seriesLength = this.dataSeries.length; seriesIndex < seriesLength; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var cate = 0, count = dataSeries.getDataPointCount(); cate < count; cate++) {
                var dataPoint = dataSeries.points[cate];
                if (dataPoint.seriesName != this.mapName) {
                    continue;
                }

                if(dataPoint.hasNextTo && dataPoint.nameLayerTo){
                    return true;
                }
            }
        }

        return false;
    },
    /**
     * 地图钻去到 系列点的下一层级.
     * @param dataPoint 系列点.
     * @param event 鼠标事件.
     */
    layerToDataPointNext: function (dataPoint, event) {
        if (!(dataPoint.hasNextTo && dataPoint.nameLayerTo)) {
            return;
        }
        event.needRefresh = true;
        var tmpMapName = dataPoint.nameLayerTo;
        if(this.isSvgMap){
            this.initSvgMap(tmpMapName);
            this._dealLayerIndexAndMapName(dataPoint);
            if (FR.Browser.isIE()) {
                event.doRefresh();
            }
            return;
        }

        //对位图的处理
        if (FR.Chart.MapHelper.containsBitMap(tmpMapName)) {
            this.fromName[this.showLayerIndex] = this.mapName;
            this._dealLayerIndexAndMapName(dataPoint);
            if (FR.Browser.isIE()) {
                event.doRefresh();
            }
        } else {
            var plotGlyph = this;
            FR.ajax({
                type: 'GET',
                url: FR.servletURL,
                data: {
                    op: 'chartrelate',
                    cmd: 'map_layerto',
                    mapName: tmpMapName,
                    __time: new Date().getTime()
                },
                dataType: 'JSON',
                async: false,
                success: function (mapJs) {
                    plotGlyph.fromName[plotGlyph.showLayerIndex] = plotGlyph.mapName;
                    plotGlyph.initBitMap(mapJs);
                    plotGlyph._dealLayerIndexAndMapName(dataPoint);
                    if (FR.Browser.isIE()) {
                        event.doRefresh();
                    }
                },
                error: function () {
                    alert('error in Map drill!');
                }
            });
        }
    },

    _dealLayerIndexAndMapName: function (dataPoint) {
        this.mapName = dataPoint.nameLayerTo;
        this.showLayerIndex = this.showLayerIndex + 1;
        this.currentAttr = FR.Chart.MapHelper.getMapAttr(this.mapName, this.isSvgMap);
        this.checkShowIndex();
        this.initZoomOffset();
    },

    /**
     * 检查校验 当前展示层级.
     */
    checkShowIndex: function () {
        if (this.showLayerIndex >= this.dataSeries.length) {
            this.showLayerIndex = this.dataSeries.length - 1;
        }
        if (this.showLayerIndex < 0) {
            this.showLayerIndex = 0;
        }
    },

    /**
     * 布局坐标轴
     */
    layoutAxisGlyph: function () {

    },

    /**
     * 是否支持两步的画图,散点，气泡，雷达，地图支持
     */
    isSupportTwoStepDraw:function(){
        return true && this.lastPlotGlyph && this.lastPlotGlyph.mapName == this.mapName;
    },

    /**
     * 包含在[minIndex, maxIndex]之间的系列是有效的
     * @param minIndex 开始的有效系列
     * @param maxIndex 结束的有效系列
     */
    relayoutDataSeriesWithIndex:function(minIndex, maxIndex){
        for(var i = 0; i < minIndex; i++){
            var dataSeries = this.dataSeries[i];
            dataSeries.paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
        }

        for(var i = maxIndex + 1; i < this.dataSeries.length; i++){
            var dataSeries = this.dataSeries[i];
            dataSeries.paintState = FR.Chart.ChartPaintConstants.STATE_UNENABLE;
        }

        for(var i = minIndex; i <= maxIndex; i++){
            var dataSeries = this.dataSeries[i];
            dataSeries.paintState = FR.Chart.ChartPaintConstants.STATE_ENABLE;
        }

        this.layoutDataSeriesGlyph();
    },

    getSeriesPointsOfCurrentMap:function(seriesIndex){
        var dataPoints = [];
        var series = this.dataSeries[seriesIndex];
        for(var pointIndex = 0; pointIndex < series.points.length; pointIndex++){
            var dataPoint = series.points[pointIndex];
            if(dataPoint && dataPoint.glyph && dataPoint.seriesName == this.mapName){
                dataPoints.push(dataPoint);
            }
        }
        return dataPoints;
    },

    _createHotTooltip4CombineGlyph:function(){
        for(var seriesIndex = 0, seriesLen = this.dataSeries.length; seriesIndex < seriesLen; seriesIndex++){
            var mapSeries = this.dataSeries[seriesIndex];
            for (var cate = 0, count = mapSeries.getDataPointCount(); cate < count; cate++) {
                var dataPoint = mapSeries.points[cate];
                if (dataPoint.seriesName == this.mapName && dataPoint.combineGlyph) {
                    var plotGlyph = dataPoint.combineGlyph.plotGlyph;
                    if(plotGlyph && plotGlyph.dataSeries){
                        for(var i = 0; i < plotGlyph.dataSeries.length; i++){
                            var series = plotGlyph.dataSeries[i];
                            for(var j = 0; j < series.points.length; j++){
                                var combinePoint = series.points[j];
                                combinePoint.createHotTooltip4Map(dataPoint, this.isHeatMap, this.heatIndex, this.hotTooltipStyle);
                            }
                        }
                    }
                }
            }
        }
    },

    /**
     * 布局地图的系列
     */
    layoutDataSeriesGlyph:function() {
        var affineTransform = this.getTransform();
        var boundsManager = new FR.Chart.BoundsManager();
        var leadLinePoints = [];

        // 颜色区域, 创建和图例 应该是一样的个数.
        for (var seriesIndex = 0, seriesLength = this.dataSeries.length; seriesIndex < seriesLength; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];

            for (var cate = 0, count = dataSeries.getDataPointCount(); cate < count; cate++) {
                var dataPoint = dataSeries.points[cate];
                dataPoint.leadLine = null;

                if (dataPoint.seriesName == this.mapName) {// 等于当前的展示地图
                    if (dataPoint.valueIsNull || dataSeries.paintState == FR.Chart.ChartPaintConstants.STATE_UNENABLE) {
                        dataPoint.glyph = null;// 消除 不该显示的部分
                        dataPoint.animateGlyph = null;
                        if (dataPoint.dataLabel) {
                            dataPoint.dataLabel.bounds = null;
                        }
                        continue;
                    }

                    if(this.isSvgMap && !this.isHeatMap){
                        continue;
                    }

                    dataPoint.lastGlyph = dataPoint.glyph;
                    var shapeGlyph = new FR.Chart.ShapeGlyph();
                    var path = this.currentAttr.getPathByName(dataPoint.categoryOriginalName);
                    var shape = path.transform(affineTransform);

                    shape = this._getZoomShape(shape);//处理缩放

                    shapeGlyph.setShape(shape);
                    shapeGlyph.isFade = true;

                    // 处理条件的顺序.
                    var seriesCondition = this.conditionCollection.getAttr(dataPoint).isContains(new FR.Chart.AttrBackground());
                    if (seriesCondition != null) {
                        shapeGlyph.info.dealCondition(this.conditionCollection, dataPoint, this.createColors4Series());
                    } else {
                        if (this.mapAreaColor) {
                            var color = this.mapAreaColor.getColorWithValue(dataPoint.value);
                            if (dataPoint.valueIsNull) {
                                shapeGlyph.info.setBackground(null);// kunsnat: 空值时 不加任何背景.

                                shapeGlyph.info.setBorderStyle(FR.Chart.Constants.LINE_NONE);
                            } else {
                                var background = FR.Chart.ColorBackground.getInstance(color);
                                shapeGlyph.info.setBackground(background);

                                shapeGlyph.info.setBorderStyle(FR.Chart.Constants.LINE_NONE);
                            }
                        }
                    }
                    dataPoint.glyph = shapeGlyph;
                    shapeGlyph.toolTipBackground = shapeGlyph.info.getBackground();
                    this._layoutDataPointLabelBounds(dataPoint, boundsManager, leadLinePoints);
                }else{
                    dataPoint.glyph = null;// 消除 不该显示的部分
                    dataPoint.animateGlyph = null;
                    if (dataPoint.dataLabel) {
                        dataPoint.dataLabel.bounds = null;
                    }
                }
            }
        }

        //处理下需要牵引线的数据点
        var leftPoints = [];
        var rightPoints = [];
        for(var i = 0; i < leadLinePoints.length; i++){
            var dataPoint = leadLinePoints[i];
            var shape = dataPoint.glyph.shape;
            var bBox = shape.getBounds2D();
            var middleX = bBox.getMiddleX();
            var plotMiddle = this.bounds.getMiddleX();
            middleX < plotMiddle ? leftPoints.push(dataPoint) : rightPoints.push(dataPoint);
        }
        this._dealLeadLineDataPointLabel(leftPoints, true);
        this._dealLeadLineDataPointLabel(rightPoints, false);

        //处理被组合的图表
        if(this.isSvgMap){
            this.layoutShapeForEachDataPoint();
        }
    },

    layoutShapeForEachDataPoint:function(){
        var affineTransform = this.getTransform();

        for (var seriesIndex = 0, seriesLength = this.dataSeries.length; seriesIndex < seriesLength; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];

            for (var cate = 0, count = dataSeries.getDataPointCount(); cate < count; cate++) {
                var dataPoint = dataSeries.points[cate];
                if (dataPoint.seriesName == this.mapName && dataPoint.combineGlyph) {

                    var path = this.currentAttr.getPathByName(dataPoint.categoryOriginalName);
                    var shape = path.transform(affineTransform);

                    shape = this._getZoomShape(shape);

                    var bounds = shape.getBounds2D();
                    var plotGlyph = dataPoint.combineGlyph.plotGlyph;
                    var size = plotGlyph.combinedSize;
                    var x = bounds.getMiddleX() - size / 2;
                    var y = bounds.getMiddleY() - size / 2;
                    if(this.mapType == FR.Chart.MapType.Map_Column){
                        y = bounds.y + bounds.height - size;
                    }
                    var resultBounds = new FR.Chart.Bounds(x, y, size, size);
                    dataPoint.combineGlyph.bounds = resultBounds;
                    dataPoint.combineGlyph.doLayout();
                }
            }
        }
    },

    _getZoomShape:function(shape){
        if(shape){
            var zoom = this.zoom[this.showLayerIndex];
            var trans = (zoom - 1) / 2;
            var offsetX = this.totalOffsetX[this.showLayerIndex] + this.offsetX[this.showLayerIndex];
            var offsetY = this.totalOffsetY[this.showLayerIndex] + this.offsetY[this.showLayerIndex];
            return shape.transform(new FR.Chart.AffineTransform(zoom ,0 ,0 ,zoom ,
                -this.bounds.width * trans + offsetX,-this.bounds.height * trans + offsetY));
        }
        return new FR.Chart.GeneralPath();
    },

    drawCombineGlyph:function(time){
        if(this.mapType == FR.Chart.MapType.Map_Normal){
            return;
        }

        var combineGlyph = [];
        for (var seriesIndex = 0, seriesLength = this.dataSeries.length; seriesIndex < seriesLength; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var cate = 0, count = dataSeries.getDataPointCount(); cate < count; cate++) {
                var dataPoint = dataSeries.points[cate];
                if (dataPoint.seriesName == this.mapName && dataPoint.combineGlyph) {
                    var tmpChartGlyph = dataPoint.combineGlyph;
                    var plotGlyph = tmpChartGlyph.plotGlyph;
                    for(var i = 0; i < plotGlyph.dataSeries.length; i++) {
                        var tmpSeries = plotGlyph.dataSeries[i];
                        if(tmpSeries.paintState <= FR.Chart.ChartPaintConstants.STATE_UNENABLE){
                            continue;
                        }
                        for (var j = 0; j < tmpSeries.points.length; j++) {
                            var tmpDataPoint = tmpSeries.points[j];
                            tmpDataPoint.lastGlyph = null;
                            tmpDataPoint.dataPoint4Map = dataPoint;
                            combineGlyph.push(tmpDataPoint);
                        }
                    }
                }
            }
        }

        var stepNumber = 36;
        if(this.actionModel){
            stepNumber = FR.Chart.ChartPaintConstants.ACTION_STEP_Number[this.actionModel]
        }
        var series = new FR.Chart.DataSeries();
        series.points = combineGlyph;
        series.getAnimateGlyph(stepNumber, FR.Chart.ChartPaintConstants.ACTION_MODEL_CHANGE);
        var plotGlyph = new FR.Chart.PlotGlyph();
        plotGlyph.dataSeries = [series];
        plotGlyph.bounds = this.bounds;
        plotGlyph.animateGlyphDraw(this.combineGlyphCtx, time, stepNumber);
    },

    _layoutDataPointLabelBounds:function(dataPoint, boundsManager, leadLinePoints){
        var textGlyph = dataPoint.dataLabel;
        if(this.notDealDataPointLabel(dataPoint, textGlyph)){
            if(textGlyph) {
                textGlyph.bounds = null;
            }
            return;
        }

        var labelBounds = this._getCenterBoundsForText(dataPoint);
        if (!this.bounds.contains(labelBounds.x, labelBounds.y, labelBounds.width, labelBounds.height)) {
            textGlyph.bounds = null;
            return;
        }
        if(boundsManager.isInteract(labelBounds)){
            leadLinePoints.push(dataPoint);
        }else{
            textGlyph.bounds = labelBounds;
            boundsManager.addBounds(labelBounds);
        }
    },

    _dealLeadLineDataPointLabel:function(leadLinePoints, isLeft){
        if(leadLinePoints.length === 0){
            return;
        }

        var labelHeight = leadLinePoints[0].dataLabel.preferredDimension().height + 3;

        var self = this;
        var maxY = this.bounds.y;
        var minY = this.bounds.y + this.bounds.height;
        function sortByY(a, b){
            var aBox = self._getCenterBoundsForText(a);
            var bBox = self._getCenterBoundsForText(b);
            return aBox.getMiddleY() - bBox.getMiddleY();
        }

        leadLinePoints.sort(sortByY);

        var topPoint = leadLinePoints[0];
        var bottomPoint = leadLinePoints[leadLinePoints.length - 1];

        minY = Math.min(this._getCenterBoundsForText(topPoint).y, minY);
        maxY = Math.max(this._getCenterBoundsForText(bottomPoint).y + labelHeight, maxY);

        if(leadLinePoints.length * labelHeight > maxY - minY){
            var lessHeight = leadLinePoints.length * labelHeight - (maxY - minY);
            minY -= lessHeight / 2;
        }

        for(var i = 0; i < leadLinePoints.length; i++){
            var dataPoint = leadLinePoints[i]
            var label = dataPoint.dataLabel;
            var labelBounds = this._getCenterBoundsForText(dataPoint);

            //牵引线
            var path = new FR.Chart.GeneralPath();
            path.moveTo(labelBounds.getMiddleX(), labelBounds.getMiddleY());

            labelBounds.x = isLeft ? this.bounds.x : this.bounds.x + this.bounds.width - labelBounds.width;
            labelBounds.y = minY + i * labelHeight;

            label.bounds = labelBounds;

            if(isLeft){
                path.lineTo(labelBounds.x + labelBounds.width, labelBounds.y + labelBounds.height / 2);
            }else{
                path.lineTo(labelBounds.x, labelBounds.y + labelBounds.height / 2);
            }

            //构造牵引线
            var foldLine = new FR.Chart.FoldLine(path);
            var lineStyle = new FR.Chart.AttrLineStyle();
            lineStyle.lineStyle = FR.Chart.Constants.LINE_THIN;
            foldLine.lineStyleInfo.attrLineStyle = lineStyle;
            var lineColor = new FR.Chart.AttrColor();
            lineColor.seriesColor = lineColor.seriesColor = "rgb(73, 79, 51)";
            foldLine.lineStyleInfo.attrLineColor = lineColor;
            dataPoint.leadLine = foldLine;
        }
    },

    _getCenterBoundsForText:function(dataPoint){
        var textGlyph = dataPoint.dataLabel;
        var shape = dataPoint.glyph.shape;

        var dim = textGlyph.preferredDimension();

        var bBox = null;
        if(!shape.getLargeShapeBounds4Map){
            bBox = shape.getBounds2D();
        }else{
            bBox = shape.getLargeShapeBounds4Map();
        }

        var width = dim.width;
        var height = dim.height;
        var x = bBox.getMiddleX() - width / 2;
        var y = bBox.getMiddleY() - height / 2;

        return new FR.Chart.Bounds(x, y, width, height);
    },

    /**
     * 返回 移动端 点击 范围内的 列表.
     * @param mousePos 点击位置
     * @returns {Array} 列表数组
     */
    getMenuItemsInBounds: function (mousePos) {
        var item = [];
        var roundArea = 5;
        var pressBounds = new FR.Chart.Bounds(mousePos.x - roundArea, mousePos.y - roundArea, roundArea * 2, roundArea * 2);

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];

            var cateItem = [];
            for (var cate = 0; cate < dataSeries.getDataPointCount(); cate++) {
                var dataPoint = dataSeries.points[cate];

                if (dataPoint
                    && (dataPoint.hyperlink || (dataPoint.nameLayerTo && dataPoint.hasNextTo))
                    && dataPoint.glyph && dataPoint.glyph.shape) {

                    var shape = dataPoint.glyph.shape;

                    if (shape &&
                        (shape.intersects(pressBounds.x, pressBounds.y, pressBounds.width, pressBounds.height)
                            || shape.contains(pressBounds.x, pressBounds.y, pressBounds.width, pressBounds.height)
                            || shape.contains(mousePos.x, mousePos.y, 1, 1)
                            || this._dealEveryPointInBounds(shape, pressBounds)
                            )) {
                        cateItem[cateItem.length] = dataPoint;
                    }
                }
            }

            if (cateItem.length > 0) {
                item[item.length] = cateItem;
            }
        }
        return item;
    },

    _dealEveryPointInBounds: function (path, pressBounds) {
        if (path && pressBounds) {
            var width = pressBounds.width || 0;
            var height = pressBounds.height || 0;
            for (var i = pressBounds.x; i < pressBounds.x + width; i++) {
                for (var h = pressBounds.y; h < pressBounds.y + height; h++) {
                    if (path.contains(i, h, 1, 1)) {
                        return true;
                    }
                }
            }
        }
        return false;
    },

    /**
     * 画出返回按钮
     * @private
     * @param ctx 画布
     */
    drawReturnButton: function (ctx) {
        if (this.showLayerIndex === 0) {
            this.returnButton = null;
            return;
        }

        var startX = this.getBounds().width;
        var startY = this.bounds.y;

        var returnIcon = new FR.Chart.MapReturnSmallIcon(
            new FR.Chart.Bounds(startX-22, startY, 22, 22),FR.Chart.ChartIconState.STATE_SELECT);
        returnIcon.draw(ctx);

    },

    /**
     * 画出 使用的图片背景 地图区域
     * @private
     * @param ctx 画布
     */
    drawUseImage:function(ctx){
        this.currentAttr.drawBackgroundImage(ctx, this.getTransform(), this.bounds);
    },

    /**
     * kunsnat: 主要是预防一些 大区域包含小区域, 如果先画小区域, 导致被后画的大区域覆盖掉.
     * 但是 实际大多数的 地图不会遇到这种情况.
     * 第一种: 画的时候, 小区域要后画
     * 第二种: 取热点提示的时候,  小区域要优先判断取出.
     */
    redoListDrawDataPoint: function (useType) {
        // 得到所有的DataPoint, 然后比较Path的长度, 优先画出长(代表区域大)
        var dataPointList = [];
        var combinePointList = [];

        for (var seriesIndex = 0; seriesIndex < this.dataSeries.length; seriesIndex++) {
            var dataSeries = this.dataSeries[seriesIndex];
            for (var cate = 0; cate < dataSeries.getDataPointCount(); cate++) {
                var dataPoint = dataSeries.points[cate];
                if (dataPoint && dataPoint.glyph && dataPoint.glyph.shape){
                    dataPointList[dataPointList.length] = dataPoint;
                }else if(dataPoint && dataPoint.combineGlyph && dataPoint.seriesName == this.mapName){
                    combinePointList.push(dataPoint);
                }
            }
        }

        dataPointList.sort(function compare(x, y){
            var xlength = 0;
            var ylength = 0;

            if(x && x.glyph && x.glyph.shape && x.glyph.shape.x && x.glyph.shape.x.length){
                xlength = x.glyph.shape.x.length;
            }

            if(y && y.glyph && y.glyph.shape && y.glyph.shape.x && y.glyph.shape.x.length){
                ylength = y.glyph.shape.x.length;
            }

            if (useType) {
                return ylength - xlength;
            } else {
                return xlength - ylength;
            }
        });

        return combinePointList.concat(dataPointList);
    },

    /**
     * MapPlotGlyph  动态的绘图
     * @param ctx 画布
     * @param plotGlyph 指定画的绘图区
     * @param count 绘制步骤
     */
    doAnimate4SeriesDraw: function (ctx, plotGlyph, count) {
        var dataPointList = this.redoListDrawDataPoint(true);// 按照Length的整个长度比较, 然后顺序画出系列点

        if (dataPointList && dataPointList.length > 0) {
            for (var i = 0; i < dataPointList.length; i++) {
                var dataPoint = dataPointList[i];
                if(dataPoint && dataPoint.glyph && dataPoint.glyph.shape){
                    dataPoint.drawAnimateGlyph(ctx, count);
                }
            }
        }

        if(!ctx){     return; } ctx.save();
        if(this.isSvgMap) {//只有svg地图能读出来边框
            if (dataPointList && dataPointList.length > 0) {
                for (var i = 0; i < dataPointList.length; i++) {
                    var dataPoint = dataPointList[i];
                    if(dataPoint && dataPoint.glyph && dataPoint.glyph.shape){
                        var path = dataPoint.glyph.shape;
                        this.currentAttr.drawBorder(ctx, dataPoint.categoryOriginalName, path);
                    }
                }
            }
        }
        ctx.restore();
    },

    /**
     * 地图画绘图区
     * @param ctxs 画布层级
     * @param time 动态时间
     * @param actionModel 动态方式
     */
    draw: function (ctxs, time, actionModel) {
        if(this.isSvgMap){
            for (var i = 0; i < ctxs.length; i ++) {
                ctxs[i].save();
                ctxs[i].rect(this.bounds.x, this.bounds.y, this.bounds.width, this.bounds.height);
                ctxs[i].clip();
            }
            var zoom = this.zoom[this.showLayerIndex];
            var trans = (zoom - 1) / 2;
            var offsetX = this.totalOffsetX[this.showLayerIndex] + this.offsetX[this.showLayerIndex];
            var offsetY = this.totalOffsetY[this.showLayerIndex] + this.offsetY[this.showLayerIndex];
            ctxs[0].transform(zoom,0 ,0 ,zoom ,-this.bounds.width * trans + offsetX ,-this.bounds.height * trans + offsetY);
        }
        this.drawInfo(ctxs);
        FR.Chart.MapPlotGlyph.superclass.draw.apply(this, [ctxs, time, actionModel]);

        if(this.isSvgMap){
            for (var i = 0; i < ctxs.length; i ++) {
                ctxs[i].restore();
            }
        }
    },

    /**
     * 绘制背景图片 和 返回按钮
     * @param ctxs 画布层级
     */
    drawInfo: function (ctxs) {
        this.drawUseImage(ctxs[0]);
        this.drawReturnButton(ctxs[1]);
    },

    /**
     * 返回动态步骤个数
     * @returns {Number} 动态个数
     */
    getAnimateStepNumber: function () {
        // getAnimateStepNumber 直接得到已经创建的animateGlyph.length, 不拿stepNumber, 虽然大部分是和stepNumber相等
        var i = 0;
        while (this.dataSeries[i] && !(this.dataSeries[i].getAnimateStepNumber() > 0)) {
            i++;
        }
        if (this.dataSeries[i]) { // 暂时判断是否为空值, 还有其他空值情况..
            return this.dataSeries[i].getAnimateStepNumber();
        } else {
            return -1;
        }
    },

    /**
     * 当前鼠标位置 查找到的Glyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件.
     */
    findOnMoveGlyph: function (mousePos, event) {

        if (this.showLayerIndex > 0) {

            if (!this.returnButton) {
                this.returnButton = new FR.Chart.MapReturnButton(new FR.Chart.Bounds(this.getBounds().width - 30, this.getBounds().y, 30, 30));
            }

            if (this.returnButton.containsPoint(mousePos.x, mousePos.y)) {
                event.cursor = ChartHandler.POINTER;
                event._lastHover = this.returnButton;
                event._hasfound = true;
                return;
            }
        }

        var newMousePos = mousePos;
        var bounds = this.getBounds();
        if (newMousePos.y < bounds.y || newMousePos.x > bounds.x + bounds.width) {
            return;
        }

        var dataPointList = this.redoListDrawDataPoint(false);// 按照Length的整个长度比较, 然后顺序画出系列点
        if (dataPointList && dataPointList.length > 0) {
            for (var i = 0, pointLength = dataPointList.length; i < pointLength; i++) {
                dataPointList[i].findOnMoveGlyph(newMousePos, event);
                if (event._hasfound) {
                    return;
                }
            }
        }
    },

    /**
     * 返回地图在当前大小下的放缩比例
     * @param mapName 地图名
     * @returns {FR.Chart.AffineTransform} 放缩比例
     */
    getTransform: function () {// 对应当前展示地图

        var imageBounds = this.currentAttr.imageBounds;

        if (!imageBounds) {
            return  new FR.Chart.AffineTransform(0, 0, 0, 0, 0, 0);
        }

        var realBounds = new FR.Chart.Bounds(this.bounds.x, this.bounds.y, this.bounds.width, this.bounds.height);

        var d1 = realBounds.width / imageBounds.width;
        var d2 = realBounds.height / imageBounds.height;

        d1 = d2 = Math.min(d1, d2);

        var d3 = realBounds.x - d1 * imageBounds.x;
        var d4 = realBounds.y - d2 * imageBounds.y;

        if (d1 * imageBounds.width < realBounds.width) {
            d3 += (realBounds.width - d1 * imageBounds.width) / 2;
        }
        if (d2 * imageBounds.height < realBounds.height) {
            d4 += (realBounds.height - d2 * imageBounds.height) / 2;
        }

        return new FR.Chart.AffineTransform(d2, 0, 0, d2, d3, d4);
    },

    /**
     * 是否支持缩放
     * @returns {boolean}
     */
    isSupportZoom: function() {
        return this.isSvgMap;
    },

    /**
     * 设置缩放
     * @param zoom
     */
    setZoom:function(zoom) {
        this.zoom[this.showLayerIndex] += zoom;
        if (this.zoom[this.showLayerIndex] < 1) {
            this.zoom[this.showLayerIndex] = 1;
            return false;
        }
        if (this.zoom[this.showLayerIndex] === 1) {
            this.totalOffsetX[this.showLayerIndex] = 0;
            this.totalOffsetY[this.showLayerIndex] = 0;
        } else {
            var bounds = this.bounds;
            if (bounds) {
                var trans = (this.zoom[this.showLayerIndex] - 1) / 2;
                var offsetX = this.totalOffsetX[this.showLayerIndex];
                var offsetY = this.totalOffsetY[this.showLayerIndex];
                if (offsetX > bounds.width * trans) {
                    this.totalOffsetX[this.showLayerIndex] = bounds.width * trans;
                }
                if (offsetX < -bounds.width * trans) {
                    this.totalOffsetX[this.showLayerIndex] = -bounds.width * trans;
                }
                if (offsetY > bounds.height * trans) {
                    this.totalOffsetY[this.showLayerIndex] = bounds.height * trans;
                }
                if (offsetY < -bounds.height * trans) {
                    this.totalOffsetY[this.showLayerIndex] = -bounds.height * trans;
                }
            }
        }

        return true;
    },

    /**
     * 设置过程偏移
     * @param x
     * @param y
     */
    setCurrentOffset:function(x, y) {
        if (Math.abs(x) < 2 && Math.abs(y) < 2) {
            return false;
        }
        var offsetX = this.totalOffsetX[this.showLayerIndex];
        var offsetY = this.totalOffsetY[this.showLayerIndex];
        var bounds = this.bounds;
        if (bounds == undefined) {
            this.offsetX[this.showLayerIndex] = x;
            this.offsetY[this.showLayerIndex] = y;
            return true;
        }
        var trans = (this.zoom[this.showLayerIndex] - 1) / 2;
        if (offsetX + x > bounds.width * trans) {
            x = bounds.width * trans - offsetX;
        }
        if (offsetX + x < -bounds.width * trans) {
            x = -bounds.width * trans  - offsetX;
        }
        if (offsetY + y > bounds.height * trans) {
            y = bounds.height * trans - offsetY;
        }
        if (offsetY + y < -bounds.height * trans) {
            y = -bounds.height * trans - offsetY;
        }
        this.offsetX[this.showLayerIndex] = x;
        this.offsetY[this.showLayerIndex] = y;
        return true;
    },

    /**
     * 设置结果偏移
     * @param x
     * @param y
     */
    setTotalOffset:function() {
        this.totalOffsetX[this.showLayerIndex] += this.offsetX[this.showLayerIndex];
        this.totalOffsetY[this.showLayerIndex] += this.offsetY[this.showLayerIndex];
        this.offsetX[this.showLayerIndex] = 0;
        this.offsetY[this.showLayerIndex] = 0;
    },

    /**
     * 初始化缩放偏移
     */
    initZoomOffset: function(){
        this.offsetX[this.showLayerIndex] = 0;
        this.offsetY[this.showLayerIndex] = 0;
        this.totalOffsetX[this.showLayerIndex] = 0;
        this.totalOffsetY[this.showLayerIndex] = 0;
        this.zoom[this.showLayerIndex] = 1;
    }
});

// PreChartGlyph.start
/**
 *    图表放缩时的 缩略demo
 *    @example
 *    var preDemo = new FR.Chart.PreChartGlyph();
 *
 * @class FR.Chart.PreChartGlyph
 * @extends FR.Chart.ChartJsInit
 *
 * @property {FR.Chart.MoveButton}[js.leftMoveButton] 左侧移动按钮
 * @property {FR.Chart.MoveButton}[js.rightMoveButton] 右侧移动按钮
 * @property {FR.Chart.Bounds}[js.moveBar] 缩略图下方滚动条
 *
 * @property {FR.Chart.PlotGlyph} [js.demoPlot] 缩略的绘图区
 * @property {FR.Chart.Bounds} [js.bounds] 缩略图绘图区的Bounds
 * @property {FR.Chart.AxisGlyph} [js.xAxisGlyph] 绑定的缩放坐标轴 (分类轴)
 * @property {FR.Chart.AxisGlyph}[js.yAxisGlyph] 值轴
 * @property {FR.Chart.Bounds} [js.barBackground] 滚动条背景
 * @property {Boolean}[js.dragStyle] 是否为拖拽样式
 * @property {Boolean}[js.isscroll] 是否支持滚动
 *
 */
FR.Chart.PreChartGlyph = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function () {

        this.leftMoveButton = null;
        this.rightMoveButton = null;
        this.moveBar = null;

        this.demoPlot = null;// 缩略图绘图区
        this.bounds = null;// 换成demoPlot的bounds, 所有的都根据demo的bounds来..外围
        this.xAxisGlyph = null;// 绑定的缩放坐标轴
        this.yAxisGlyph = null;
        this.barBackground = null;

        this.isScroll = true;
        this.dragStyle = false;
    },

    // 根据总体的bounds, 确定每个button啊 什么的各种详细的位置
    // 这个需要在bounds之外, 因为 此次的bounds是和绘图区共宽度的, 如果内减的话 就无法对齐
    doLayout: function () {
        if (this.bounds) {
            // 根据现有的 坐标轴的forward和backward等 再次确定下 TODO
            this.initMoveBar(this.xAxisGlyph);
            this.initMoveButton();

            this.barBackground = new FR.Chart.Bounds(this.bounds.x, this.bounds.y + this.bounds.height,
                this.bounds.width, FR.Chart.PreChartGlyph.MOVE_BACKGROUND_HEIGHT - 3);
        }
    },

    // 根据 坐标轴的前后缩放位置 初始化移动条的位置
    initMoveBar: function (axisGlyph) {// 面积图 计算的时候 少一个分类, 因为是标签在刻度下的
        if (this.demoPlot.xAxisGlyph && axisGlyph) {
            var categoryCount = Math.max(1, this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue());
            var minWidth = this.bounds.width / categoryCount;

            var width = (categoryCount + axisGlyph.zoomForward + axisGlyph.zoomBackward) * this.bounds.width / categoryCount;
            width = width < minWidth ? minWidth : width;

            var x = Math.abs(axisGlyph.zoomBackward) * this.bounds.width / categoryCount + this.bounds.x;
            x = Math.min(x, this.bounds.x + this.bounds.width - width);// 不能超出右侧Bar范围边界了.

            var y = this.bounds.y + this.bounds.height;// 滚动样式的y

            this.moveBar = new FR.Chart.RoundBounds(
                x,
                y + (FR.Chart.PreChartGlyph.MOVE_BACKGROUND_HEIGHT - FR.Chart.PreChartGlyph.MOVE_BAR_HEIGHT) / 2,
                width,
                FR.Chart.PreChartGlyph.MOVE_BAR_HEIGHT - 3, 4, 4);
        }
    },

    // 根据横轴移动条 初始化两个移动按钮的位置.
    initMoveButton: function () {
        if (this.moveBar && this.bounds) {
            var x = this.moveBar.x;
            var y = this.bounds.y;

            var moveButtonY = y + this.bounds.height / 2 - FR.Chart.MoveButton.HEIGHT / 2;
            this.leftMoveButton = new FR.Chart.MoveButton(x - FR.Chart.MoveButton.WIDTH / 2, moveButtonY);
            this.leftMoveButton.isScroll = true;

            this.rightMoveButton = new FR.Chart.MoveButton(x + this.moveBar.width - FR.Chart.MoveButton.WIDTH / 2, moveButtonY);
            this.rightMoveButton.isScroll = true;
        }
    },

    draw: function (ctx) {
        if(!ctx){     return; } ctx.save();

        this.drawBackgroundDemoPath(ctx);
        this.drawBackgroundLine(ctx);// 移动的线

        if (this.barBackground) {// 移动条的背景
            ctx.fillStyle = 'rgb(243,243,243)';
            this.barBackground.paint(ctx);
            ctx.strokeStyle = 'rgb(172,180,177)'
            this.barBackground.draw(ctx);
        }

        if (this.leftMoveButton) {
            this.leftMoveButton.draw(ctx);
        }

        if (this.rightMoveButton) {
            this.rightMoveButton.draw(ctx);
        }

        this.drawMoveBar(ctx);
        ctx.restore();
    },

    save4CtxBounds: function (ctx, bounds) {
        ctx.beginPath();
        ctx.moveTo(bounds.x, bounds.y);
        ctx.lineTo(bounds.x + bounds.width, bounds.y);
        ctx.lineTo(bounds.x + bounds.width, bounds.y + bounds.height);
        ctx.lineTo(bounds.x, bounds.y + bounds.height);
        ctx.closePath();
        ctx.clip();
    },

    // 缩略图的Path 路径, 只画简单的. 其他各种阴影等效果都不要.
    drawBackgroundDemoPath: function (ctx) {
        if (this.demoPlot) {

            var leftX = this.moveBar.x;
            var rightX = this.moveBar.x + this.moveBar.width;

            if(!ctx){     return; } ctx.save();

            ctx.fillStyle = 'rgb(59,195,236)';
            ctx.strokeStyle = 'rgba(25,79,151, 0.8)';

//            this.save4CtxBounds(ctx, this.bounds);// TODO 自定义范围的 截取  超出范围的 需要出重叠 不画出.

            this.demoPlot.draw4Demo(ctx);

            ctx.fillStyle = 'rgba(255,255,255, 0.5)';
            ctx.fillRect(this.bounds.x, this.bounds.y, leftX - this.bounds.x, this.bounds.height);
            ctx.fillRect(rightX, this.bounds.y, this.bounds.x + this.bounds.width - rightX, this.bounds.height);

            //画标签
            if (this.xAxisGlyph && this.yAxisGlyph) {
                var fontText = new FR.Chart.TextAttr();

                var font = new FR.Chart.FRFont();
                font.fontName = FR.i18nText('Fine-Dec_Chart_Use_MSBold');
                font.color = 'rgb(0, 0, 0)';
                fontText.frFont = font;

                ctx.strokeStyle = 'rgb(218,218,218)';
                ctx.lineWidth = 1;
                var originY = this.xAxisGlyph.originPoint.y;

                if(this.yAxisGlyph.axisReversed){
                    this.xAxisGlyph.originPoint.y = this.bounds.y + this.xAxisGlyph.bounds.height;
                }else{
                    this.xAxisGlyph.originPoint.y = this.xAxisGlyph.originPoint.y + this.bounds.height;
                }
                this.xAxisGlyph.demoGridLength = this.bounds.height;
                this.xAxisGlyph.draw4Demo(ctx, fontText); // no刻度 no旋转 no 线型
                this.xAxisGlyph.originPoint.y = originY;
            }

            ctx.restore();
        }
    },

    drawMoveBar: function (ctx) {
        if (this.moveBar) {
            if(!ctx){     return; } ctx.save();

            ctx.strokeStyle = 'rgb(101,96,72)';
            this.moveBar.draw(ctx);

            var x = this.moveBar.x;
            var y = this.moveBar.y;
            var width = this.moveBar.width;
            var height = this.moveBar.height;

            var grd = ctx.createLinearGradient(x, y, x, y + height);
            if (this.dragStyle) {
                grd.addColorStop(0, 'rgb(210,210,210)');
                grd.addColorStop(1, 'rgb(250,250,250)');
            } else {
                grd.addColorStop(0, 'rgb(250,250,250)');
                grd.addColorStop(1, 'rgb(210,210,210)');
            }
            ctx.fillStyle = grd;
            this.moveBar.paint(ctx);

            ctx.lineWidth = 1;
            ctx.strokeStyle = 'rgb(102,102,102)';

            var path = new FR.Chart.GeneralPath();

            path.moveTo(x + width / 2 - 2, y + height / 2 - 2);
            path.lineTo(x + width / 2 - 2, y + height / 2 + 2);

            path.moveTo(x + width / 2, y + height / 2 - 2);
            path.lineTo(x + width / 2, y + height / 2 + 2);

            path.moveTo(x + width / 2 + 2, y + height / 2 - 2);
            path.lineTo(x + width / 2 + 2, y + height / 2 + 2);

            path.draw(ctx);

            ctx.restore();
        }
    },

    drawBackgroundLine: function (ctx) {// 随按钮移动的线条
        if(!ctx){     return; } ctx.save();
        var path = new FR.Chart.GeneralPath();

        var x = this.bounds.x;
        var y = this.bounds.y;
        var width = this.bounds.width;
        var height = this.bounds.height;

        path.moveTo(x, y);

        var leftX = this.leftMoveButton.bounds.x + this.leftMoveButton.bounds.width / 2;
        path.lineTo(leftX, y);
        path.lineTo(leftX, y + height);

        path.moveTo(x + width, y);
        var rightX = this.rightMoveButton.bounds.x + this.rightMoveButton.bounds.width / 2;
        path.lineTo(rightX, y);
        path.lineTo(rightX, y + height);

        ctx.strokeStyle = 'rgb(0, 0, 0)';
        path.draw(ctx);
        ctx.restore();
    },

    findOnMoveGlyph: function (mousePos, event) {
        if (this.xAxisGlyph) {
            var obj = this.xAxisGlyph;
            if (obj && obj.isZoom) {
                if ((this.leftMoveButton && this.leftMoveButton.containsPoint(mousePos.x, mousePos.y))
                    || (this.rightMoveButton && this.rightMoveButton.containsPoint(mousePos.x, mousePos.y))
                    ) {
                    event._lastHover = this;
                    event._hasfound = true;
                    event._draggingTarget = event._isMouseDown ? this : null;
                } else if (this.moveBar.getBounds2D().containsPoint(mousePos.x, mousePos.y)) {
                    event._lastHover = this;
                    event._hasfound = true;
                    if (event._isMouseDown) {
                        event._mouseStyle = "drag";
                        event._draggingTarget = this;
                        event.needRefresh = !this.dragStyle;
                        this.dragStyle = true;
                    } else {
                        event._mouseStyle = "hand";
                        event.needRefresh = this.dragStyle;
                        this.dragStyle = false;
                    }
                }
            }
        }
    },

    move4Mobile: function (downPos, mousePos, ctx, event) {
        event.zoomRefresh = false;
        var mouseStartPos = downPos;

        var startPos = mousePos;

        var isAcition = false;
        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var offsetX = Math.abs(startPos.x - mousePos.x);

            var maxNumber = this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue();
            var oneTickLength = this.xAxisGlyph.axisLength / maxNumber;
            var number = (parseInt(offsetX / oneTickLength));

            if (number !== 0) {
                if (startPos.x < mousePos.x) {
                    if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                        isAcition = this.xAxisGlyph.onZoomBarLeftDown(number);
                    } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                        isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                    } else {
                        isAcition = this.xAxisGlyph.onZoomBarIncrease(number);
                    }
                } else if (startPos.x > mousePos.x) {
                    if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                        isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                    } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                        isAcition = this.xAxisGlyph.onZoomBarRightDown(number);
                    } else {
                        isAcition = this.xAxisGlyph.onZoomBarDecrease(number);
                    }
                }
            }
        }

        // draw on ctx 画 this.moveBar.x this.moveBar.width 双向箭头 标线
        if(!ctx){     return; } ctx.save();

        ctx.fillStyle = "rgb(51, 51, 51)";
        ctx.strokeStyle = "rgb(51, 51, 51)";

        var arrowX = mousePos.x - 4.5;
        var arrowY = this.bounds.y - 2;

        ctx.beginPath();
        ctx.moveTo(arrowX + 4.5, arrowY + 2);
        ctx.lineTo(arrowX + 4.5, arrowY + 2 + this.bounds.height);
        ctx.stroke();

        ctx.beginPath();// 左下角最下方靠近this.bounds为起点
        ctx.moveTo(arrowX, arrowY);
        ctx.lineTo(arrowX - 3, arrowY - 2.5);
        ctx.lineTo(arrowX, arrowY - 5);
        ctx.closePath();
        ctx.fill();

        ctx.beginPath();
        ctx.moveTo(arrowX + 9, arrowY);
        ctx.lineTo(arrowX + 9, arrowY - 5);
        ctx.lineTo(arrowX + 12, arrowY - 2.5);
        ctx.closePath();
        ctx.fill();

        ctx.fillRect(arrowX, arrowY - 3.5, 9, 2);

        if (isAcition) {
            event.zoomRefresh = true;
        }
    },

    /**
     * 在preChartGlyph里响应拖拽结束事件
     * @param mouseStartPos 拖拽开始的位置
     * @param mousePos 当前的鼠标位置
     */
    onDragEnd: function (mouseStartPos, mousePos) {
        var isAcition = false;
        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var offsetX = Math.abs(mouseStartPos.x - mousePos.x);

            var maxNumber = this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue();
            var oneTickLength = this.xAxisGlyph.axisLength / maxNumber;
            var number = (Math.round(offsetX / oneTickLength));

            if (mouseStartPos.x < mousePos.x) {// 根据鼠标位置, 得到需要移动的个数  ===
                if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarLeftDown(number);
                } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                } else {
                    isAcition = this.xAxisGlyph.onZoomBarIncrease(number);
                }
            } else if (mouseStartPos.x > mousePos.x) {
                if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarRightDown(number);
                } else {
                    isAcition = this.xAxisGlyph.onZoomBarDecrease(number);
                }
            }
        }
    },

    onDrag: function (mouseStartPos, mousePos, ctx4OnMove, event, stupidBrowser) {
        if (stupidBrowser) {
            return;
        }
        var isAcition = false;
        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var offsetX = Math.abs(mouseStartPos.x - mousePos.x);

            var maxNumber = this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue();
            var oneTickLength = this.xAxisGlyph.axisLength / maxNumber;
            var number = parseInt(offsetX / oneTickLength);

            if (number < 1) {
                event.zoomRefresh = false;
                return;
            }

            if (mouseStartPos.x < mousePos.x) {// 根据鼠标位置, 得到需要移动的个数  ===
                if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarLeftDown(number);
                } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                } else {
                    isAcition = this.xAxisGlyph.onZoomBarIncrease(number);
                }
            } else if (mouseStartPos.x > mousePos.x) {
                if (this.leftMoveButton && this.leftMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                } else if (this.rightMoveButton && this.rightMoveButton.containsPoint(mouseStartPos.x, mouseStartPos.y)) {
                    isAcition = this.xAxisGlyph.onZoomBarRightDown(number);
                } else {
                    isAcition = this.xAxisGlyph.onZoomBarDecrease(number);
                }
            }

            if (isAcition) {
                event.zoomRefresh = isAcition;
                mouseStartPos.x = mouseStartPos.x + number * (mouseStartPos.x - mousePos.x < 0 ? oneTickLength : -oneTickLength);
                this.initMoveBar(this.xAxisGlyph);
                this.initMoveButton();
            }
        }
    },

    /**
     * 鼠标到滑块的时候鼠标形状变成抓紧的手型
     * @param mousePos
     * @param event
     */
    onPress: function (mousePos, event) {
        if (this.xAxisGlyph) {
            var obj = this.xAxisGlyph;
            if (obj && obj.isZoom) {
                if (this.moveBar.getBounds2D().containsPoint(mousePos.x, mousePos.y)) {
                    event.cursor = ChartHandler.DRAG;
                }
            }
        }
    }
});

$.extend(FR.Chart.PreChartGlyph, /**@class FR.Chart.PreChartGlyph*/{
    /**
     * 缩放绘图区占的比例
     * @static
     * @property
     */
    PLOT_HEIGHT_SCALE: 0.09,
    /**
     * 下方滚动条 占用背景高度
     * @static
     * @property
     */
    MOVE_BACKGROUND_HEIGHT: 12,
    /**
     * 下方滚动条高度
     * @static
     * @property
     */
    MOVE_BAR_HEIGHT: 12,
    /**
     * 最小的缩略图区域限制高度
     * @static
     * @property
     */
    MIN_DEMO_HEIGHT: 20,
    /**
     * 最大的缩略图限制高度
     * @static
     * @property
     */
    MAX_DEMO_HEIGHT: 42,
    /**
     * 缩略图分类的 间隔
     * @static
     * @property
     */
    PRE_CATE_GAP: 8
});
// PreChartGlyph.end

// ChartGlyph.start
/**

 * 图表 Glyph
 *
 *    @example
 *    var chartGlyph = new FR.Chart.ChartGlyph();
 *
 * @class FR.Chart.ChartGlyph
 * @extends FR.Chart.RectangleGlyph
 *
 * @param {JSON} js 配置属性
 *
 * @param {FR.Chart.LegendGlyph} [js.legendGlyph] 图表区 图例部分
 * @param {FR.Chart.TitleGlyph} [js.titleGlyph] 图表区 标题部分
 * @param {FR.Chart.PlotGlyph} [js.plotGlyph] 图表区 绘图部分
 * @param {FR.Chart.DataSheetGlyph}[js.dataSheetGlyph] 图表区 数据表部分
 * @param {Boolean} [js.isUseChangeChart = false] 使用使用切换图表控件
 * @param {String}[js.canvasInfo] 图表使用canvas层级的信息
 * @paran {FR.Chart.AttrFillStyle} [js.currentStyle] 服务器部分的风格
 *
 *
 */
FR.Chart.ChartGlyph = FR.extend(FR.Chart.RectangleGlyph, {
    initConstants: function (js) {
        js = js || {};

        if (js.currentStyle) {// 初始化服务器端的预定义风格. 拆分之后就不需要传入, 直接每个定义.
            FR.Chart.CurrentPreStyle = new FR.Chart.AttrFillStyle(js.currentStyle);
        } else {
            FR.Chart.CurrentPreStyle = null;
        }

        this.legendGlyph = js.legendGlyph == undefined ? null : new FR.Chart.LegendGlyph(js.legendGlyph);
        this.titleGlyph = js.titleGlyph == undefined ? null : new FR.Chart.TitleGlyph(js.titleGlyph);

        if (js.plotGlyph == undefined || js.plotType == undefined) {
            this.plotGlyph = null;
        } else {
            this.plotGlyph = new FR.Chart[js.plotType](js.plotGlyph);
            this.initDemoPlotGlyphIfNeed(js);
        }

        if(!this.plotGlyph || !this.plotGlyph.isSupportLegend()){
            this.legendGlyph = null ;
        }

        if (js.dataSheetGlyph != undefined) {
            this.dataSheetGlyph = new FR.Chart.DataSheetGlyph(js.dataSheetGlyph);
        } else {
            this.dataSheetGlyph = null;
        }

        this.isUseChangeChart = js.isUseChangeChart;
        this.isUseDragRefresh = js.isUseDragRefresh;

        this.canvasInfo = FR.pick(js.canvasInfo, '');

        FR.Chart.ChartGlyph.superclass.initConstants.apply(this, [js]);

        if (this.legendGlyph && this.plotGlyph) {
            this.legendGlyph.seriesList = this.plotGlyph.dataSeries;
        }

        if (this.dataSheetGlyph && this.plotGlyph) {
            this.dataSheetGlyph.seriesList = this.plotGlyph.dataSeries;
        }

        this.timeSwitchGlyph = null;
        this.mapControlBar = null;
    },


    /**
     * 设置对应的glyph属性
     * @param option  自定义的图表属性.
     */
    setOption: function (options) {

        this.isUseChangeChart = FR.pick(options.isUseChangeChart, false);
        this.isUseDragRefresh = FR.pick(options.isUseDragRefresh, true);

        this.info = new FR.Chart.GeneralInfo(options.background)

        this.initTitleConfig(options.title);

        this.initPlotConfigWithTableData(options);

        this.initLegendConfig(options.legend);

        this.initDataSheetConfig(options);

        this.initInteractionInfo(options);

        var items = [];
        if((this.legendGlyph || this.dataSheetGlyph) && this.plotGlyph){
            items = this.plotGlyph.createLegendItems();
        }

        if (this.legendGlyph && this.plotGlyph) {
            this.legendGlyph.seriesList = this.plotGlyph.dataSeries;
            this.legendGlyph.items = items;
            this.legendGlyph.endItemIndex = items.length;
        }

        if (this.dataSheetGlyph && this.plotGlyph) {
            this.dataSheetGlyph.seriesList = this.plotGlyph.dataSeries;
            this.dataSheetGlyph.items = items;
        }
    },

    initTitleConfig : function(titleConfig) {
        if (titleConfig) {
            this.titleGlyph = new FR.Chart.TitleGlyph(titleConfig);
        }
    },

    initLegendConfig : function(legendConfig) {
        if (legendConfig) {
            this.legendGlyph = new FR.Chart.LegendGlyph(legendConfig);
        }
    },

    initPlotConfigWithTableData:function(options){
        if(!options.chartType || !FR.Chart.Type[options.chartType]){
            return;
        }

        var plotTypes = FR.Chart.Type[options.chartType];

        this.plotGlyph = new FR.Chart[plotTypes.plotType](options.plot);

        this.initPlotGlyphAttrs(this.plotGlyph, options);
    },

    initPlotGlyphAttrs:function(plotGlyph, options){
        var plotTypes = FR.Chart.Type[options.chartType];

        plotGlyph.configSeries(options.series || {});

        plotGlyph.configTooltipAndLabels(options || {});//数据点提示和标签

        plotGlyph.installSeriesWithOptionData(options.data, plotTypes);

        plotGlyph.calculatePercentValue();

        plotGlyph.configCondition(options.plot);//数据装载完再去判断条件显示

        plotGlyph.configAxisLabelsWithOptionData(options.data);

        plotGlyph.refreshAxisGlyph();

        plotGlyph.createHotTooltip();

        plotGlyph.createDataLabel4EverySeries();

        if(options.plot && options.plot.hyperlink){
            plotGlyph.createHyperlink(options.plot.hyperlink);
        }
    },

    initDataSheetConfig : function(options) {
        if(options && options.dataSheet) {
            this.dataSheetGlyph = new FR.Chart.DataSheetGlyph(options.dataSheet);
            if (this.plotGlyph) {
                this.dataSheetGlyph.seriesList = this.plotGlyph.dataSeries;
            }
        }
    },

    initInteractionInfo:function(options){
        var interaction = FR.pick(options.interaction, {});
        if(interaction.zoom){
            this.plotGlyph.xAxisGlyph.isZoom = true;
            var plotTypes = FR.Chart.Type[options.chartType];
            this.smallDemoPlotGlyph = new FR.Chart[plotTypes.plotType](options.plot);
            this.initPlotGlyphAttrs(this.smallDemoPlotGlyph, options);
            this._initDemoPlotGlyph();
        }

        if(interaction.draggable){
            var series = this.plotGlyph.dataSeries;
            for(var i = 0; i < series.length; i++){
                series[i].seriesDragEnable = true;
            }
        }
    },

    /**
     * 用于缩略图  判断属性是否导入.
     */
    initDemoPlotGlyphIfNeed: function (js) {
        if (this.plotGlyph && this.plotGlyph.xAxisGlyph && this.plotGlyph.xAxisGlyph.isZoom) {
            this.smallDemoPlotGlyph = new FR.Chart[js.plotType](js.plotGlyph);
            if(this.smallDemoPlotGlyph.isStacked){
                this.smallDemoPlotGlyph.isStacked = false;
                //因为缩略图是按照折线图来画的，所以要重新计算最大最小值,因为堆积的时候，最大最小值是按照堆积的来的
                if(this.smallDemoPlotGlyph.yAxisGlyph){
                    this.smallDemoPlotGlyph.yAxisGlyph.minValue = this.smallDemoPlotGlyph.getMinValueFromData();
                    this.smallDemoPlotGlyph.yAxisGlyph.maxValue = this.smallDemoPlotGlyph.getMaxValueFromData();
                }
            }
            this._initDemoPlotGlyph();
        }
    },

    _initDemoPlotGlyph:function(){
        if (this.smallDemoPlotGlyph.xAxisGlyph) {// 覆盖所有坐标轴中 会占用空间, 挤压绘图区的属性, 为了方便布局.
            this.smallDemoPlotGlyph.xAxisGlyph.isShowAxisLabel = false;
            this.smallDemoPlotGlyph.xAxisGlyph.titleGlyph = null;
            this.smallDemoPlotGlyph.xAxisGlyph.tickMarkType = FR.Chart.Constants.TICK_MARK_NONE;
        }
        if (this.smallDemoPlotGlyph.yAxisGlyph) {
            this.smallDemoPlotGlyph.yAxisGlyph.isShowAxisLabel = false;
            this.smallDemoPlotGlyph.yAxisGlyph.titleGlyph = false;
            this.smallDemoPlotGlyph.yAxisGlyph.tickMarkType = FR.Chart.Constants.TICK_MARK_NONE;
        }
        if (this.smallDemoPlotGlyph.secondAxisGlyph) {
            this.smallDemoPlotGlyph.secondAxisGlyph.isShowAxisLabel = false;
            this.smallDemoPlotGlyph.secondAxisGlyph.titleGlyph = false;
            this.smallDemoPlotGlyph.secondAxisGlyph.tickMarkType = FR.Chart.Constants.TICK_MARK_NONE;
        }
    },

    /**
     * 返回背景canvas层级
     * @returns {Object} canvas层级
     */
    getChartBackgroundCtx: function () {
        return this.ctxs[0];
    },

    /**
     * 如果图表区域 有图片背景, 另开一层.
     * 如果图表区域没有, 则在第一层 和图表区域同层.
     */
    getPlotBackgroundCtx: function () {
        return this.canvasInfo.indexOf('chart') != -1 ? this.ctxs[1] : this.ctxs[0];
    },

    /**
     * 返回绘图区 绘图部分的canvas层级
     * @returns {Object} canvas层级
     */
    getNormalAllCtx: function () {
        var hasChart = this.canvasInfo.indexOf('chart') != -1;
        var hasPlot = this.canvasInfo.indexOf('plot') != -1;

        if (hasChart) {
            return hasPlot ? this.ctxs[2] : this.ctxs[1];
        } else {
            return hasPlot ? this.ctxs[1] : this.ctxs[0];
        }
    },

    /**
     * 返回动态步骤使用的canvas层级
     * @returns {Obejct} canvas层级
     */
    getAnimateCtx: function () {
        var hasChart = this.canvasInfo.indexOf('chart') != -1;
        var hasPlot = this.canvasInfo.indexOf('plot') != -1;

        if (hasChart) {
            return hasPlot ? this.ctxs[3] : this.ctxs[2];
        } else {
            return hasPlot ? this.ctxs[2] : this.ctxs[1];
        }
    },

    getAnimateCtxIndex:function(){
        var hasChart = this.canvasInfo.indexOf('chart') != -1;
        var hasPlot = this.canvasInfo.indexOf('plot') != -1;

        if (hasChart) {
            return hasPlot ? 3 : 2;
        } else {
            return hasPlot ? 2 : 1;
        }
    },

    /**
     * 返回OnMove事件的Ctx画布.
     * @returns {Object} 返回热点提示灯 使用的canvas层级
     */
    getOnMoveCtx: function () {
        return this.ctxs[this.ctxs.length - 1];
    },

    getCombineGlyphCtx:function(){
        return this.ctxs[this.ctxs.length - 2];
    },

    getMapOnMoveCtx:function(){
        if(this.hasMapInnerCtx4OnMove()){
            return this.ctxs[this.getAnimateCtxIndex() + 1];
        }else{
            return this.getOnMoveCtx();
        }
    },

    //是否有组合的图表
    hasCombineGlyph:function(){
        return this.plotGlyph && this.plotGlyph.mapType
            && this.plotGlyph.mapType != FR.Chart.MapType.Map_Normal;
    },

    hasMapInnerCtx4OnMove:function(){
        return this.hasCombineGlyph() && this.plotGlyph.isHeatMap;
    },

    /**
     * 弹性的层级ctx,  图表区( +图片), 绘图区( +图片), 数据表格区域, 标题区域,  图例, 缩略图, 坐标轴区域.
     *
     * 系列值刷新(单独一层),
     * 热点区域显示(单独一层)
     *
     * chartBackgroundCtx, plotBackgroundCtx, normalAllCtx,  animateCtx, tipCtx;
     *
     * @param {Number} [actionModel]动态方式
     */
    draw:function(actionModel, actionTime){
        this.bounds = FR.Chart.ChartUtils.rectangle2RoundRectangle(this.bounds, this.isRoundBorder());

        this.drawBackground();

        FR.Chart.DivCon.clearDiv(this.parentDiv);
        FR.Chart.GraphHelper.ctx = this.ctxs[0]; // 这个是用于GraphHelper的计算字体
        FR.Chart.GraphHelper.isDrawingEnd = false;

        this.doLayout();

        this.drawWhenLayoutEnd(actionModel, actionTime);
    },

    /**
     * 另外 CellHtmlWriter在计算实际高度 cellHeight 和 valueWrapperHeight (有误差2像素的样子)
     * 然后同样在 单元格元素padding-left border 这些css元素都会影响 canvas的位置和边界 缝隙
     * kunsnat:  图表边框背景, 边框隐患: 和实际的外单元格或悬浮元素 有1-2像素的缝隙..
     * @private
     */
    drawBackground:function(){
        var shape = this.bounds;
        var ctx = this.getChartBackgroundCtx();

        var info = this.info;

        if (shape && info) {
            if(!ctx){     return; } ctx.save();
            ctx.globalAlpha = info.seriesAttrAlpha.alpha;
            var width = parseFloat(ctx.canvas.style.width);
            var height = parseFloat(ctx.canvas.style.height);

            //预留宽度是因为图表区边框问题
            var isChartBorderExists = info.seriesAttrBorder.borderStyle != FR.Chart.Constants.LINE_NONE
                            && info.seriesAttrBorder.borderColor != null;
            var gap = isChartBorderExists ? 5 : 0;
            if (info.seriesAttrBackground.seriesBackground) {
                var fillShape = new FR.Chart.Bounds(0, 0, Math.max(0, width - gap), Math.max(0, height - gap));
                var newFill = FR.Chart.ChartUtils.rectangle2RoundRectangle(fillShape, this.isRoundBorder());
                info.seriesAttrBackground.seriesBackground.paint(ctx, newFill);
            }

            if (isChartBorderExists) {
                var lineWidth = FR.Chart.GraphHelper.getLineStyleSize(info.seriesAttrBorder.borderStyle);
                var strokeShape = new FR.Chart.Bounds(lineWidth/2, lineWidth/2, Math.max(0, width - gap - lineWidth), Math.max(0, height - gap - lineWidth));
                ctx.strokeStyle = info.seriesAttrBorder.borderColor;
                ctx.lineWidth = lineWidth;
                var newShape = FR.Chart.ChartUtils.rectangle2RoundRectangle(strokeShape, this.isRoundBorder());
                newShape.draw(ctx);
            }

            ctx.restore();
        }
    },

    /**
     * 布局结束之后 画图表
     * @param actionModel 动态方式
     * @private
     */
    drawWhenLayoutEnd: function (actionModel, actionTime) {
        var plotBackCtx = this.getPlotBackgroundCtx();
        var normalCtx = this.getNormalAllCtx();
        var animateCtx = this.getAnimateCtx();

        if(this.plotGlyph && this.legendGlyph){
            this.plotGlyph.dealAttr4LegendGlyph(this.legendGlyph);
        }

        if (this.legendGlyph) {
            this.legendGlyph.draw(normalCtx);
        }

        if (this.titleGlyph) {
            this.titleGlyph.draw(normalCtx);
        }

        if(this.timeSwitchGlyph){
            this.timeSwitchGlyph.draw(normalCtx);
        }

        if(this.mapControlBar){
            this.mapControlBar.draw(normalCtx);
        }

        if (this.dataSheetGlyph) {
            this.dataSheetGlyph.draw(normalCtx);
        }

        if (this.plotGlyph) {
            if (this.plotGlyph.setDivID) {
                this.plotGlyph.setDivID(this.parentDiv);
            }

            this.plotGlyph.setCombieGlyphCtx(this.getCombineGlyphCtx());

            if (this.preChartGlyph) {
                this.preChartGlyph.draw(normalCtx);
            }

            var plotCtxs = [plotBackCtx, normalCtx, animateCtx];

            this.plotGlyph.chartBounds = this.bounds;
            this.plotGlyph.draw(plotCtxs, actionTime || 1000, actionModel);// 三个ctx, 背景, 坐标轴区域, 动态区域.
            this.plotGlyph.lastPlotGlyph = null;
        }
    },

    /**
     * mobile中滚动条的即时变化和图例变化
     */
    clearLegendAndDraw: function () {
        var ctxLegend = this.getNormalAllCtx();
        if (this.legendGlyph && this.legendGlyph.bounds) {
            var legendBounds = this.legendGlyph.bounds;
            ctxLegend.clearRect(legendBounds.x, legendBounds.y, legendBounds.width, legendBounds.height);
            this.legendGlyph.draw(ctxLegend);
        }
    },

    /**
     * 重新画出Plot的系列, 非动态
     * 标签等 因为在整个Div管理中, 所以未清除.
     *
     */
    reDrawPlotGlyphNoAnimate:function(){
        if(this.plotGlyph != null) {
            var ctx = this.getAnimateCtx();
            this.clear(ctx);
            this.plotGlyph.doAnimate4SeriesDraw(ctx, this.plotGlyph, this.plotGlyph.getAnimateStepNumber() - 1);
        }
    },

    /**
     * 在鼠标onMove时 响应
     * @param ctx 画布
     */
    draw4OnMove: function (ctx) {
        if (this.legendGlyph) {
            this.legendGlyph.draw4OnMove(ctx);
        }
    },

    /**
     * 清除 响应的热点提示.
     */
    clearOnMoveCtx: function () {
        this.clear(this.getOnMoveCtx());
        if(this.hasMapInnerCtx4OnMove()){
            this.clear(this.getMapOnMoveCtx());
        }
    },

    /**
     * 清除组合图的层级
     */
    clearCombineGlyphCtx:function(){
        this.clear(this.getCombineGlyphCtx());
    },

    /**
     * 清除对应canvas中的数据等内容
     * @param ctx 画布
     */
    clear: function (ctx) {
        var width = parseFloat(ctx.canvas.style.width);
        var height = parseFloat(ctx.canvas.style.height);

        ctx.clearRect(0, 0, width, height);
    },

    /**
     * 清除掉 所有的canvas层级数据.
     */
    clearAll: function () {
        var animateCtxIndex = this.getAnimateCtxIndex();
        var ctxCount = this.ctxs.length;
        for (var i = 0; i < ctxCount; i++) {
            if(i != animateCtxIndex){
                this.clear(this.ctxs[i]);
            }
        }
        FR.Chart.GraphHelper.isDrawingEnd = false;
    },

    /**
     * 返回到上一层 地图
     */
    returnBackToLayer: function () {
        if (this.plotGlyph != null) {
            this.plotGlyph.returnBackToLayer();
        }
    },

    /**
     * 钻去到当前系列点的下一层
     * @param dataPoint 系列点
     * @param event 鼠标事件
     */
    layerToDataPointNext: function (dataPoint, event) {
        if (this.plotGlyph != null) {
            this.plotGlyph.layerToDataPointNext(dataPoint, event);
        }
    },

    /**
     * 在鼠标按压时 响应的内容 画
     * @param ctx 画布
     */
    drawOnPress: function (ctx) {
        if (this.legendGlyph) {
            this.legendGlyph.drawOnPress(ctx);
        }
    },

    /**
     * 响应onPress事件
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onPress: function (mousePos, event) {
        var bounds = this.getBounds();
        if (mousePos.y < bounds.y || mousePos.x > bounds.x + bounds.width) {
            return;
        }

        if (this.preChartGlyph) {
            this.preChartGlyph.onPress(mousePos, event);
        }

        if (this.legendGlyph) {
            this.legendGlyph.onPress(mousePos, event);
        }
    },


    /**
     * 根据鼠标位置 查找OnGlyph
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findOnMoveGlyph: function (mousePos, event) {
        if (this.legendGlyph) {
            if (this.plotGlyph) {
                if (this.plotGlyph.isLegendInteractive()) {
                    this.legendGlyph.findOnMoveGlyph(mousePos, event);
                } else {
                    //图例没有交互是指图例项,翻页按钮还是要有的.
                    this.legendGlyph.findOnMoveLegendButtonGlyph(mousePos, event);
                }
            } else {
                this.legendGlyph.findOnMoveGlyph(mousePos, event);
            }
        }

        if (this.dataSheetGlyph && this.plotGlyph && !event._hasfound) {
            this.dataSheetGlyph.findOnMoveGlyph(mousePos, event);
        }

        if (this.plotGlyph && !event._hasfound) {
            this.plotGlyph.findOnMoveGlyph(mousePos, event);
        }


        if(this.timeSwitchGlyph &&  !event._hasfound){
            this.timeSwitchGlyph.findOnMoveGlyph(mousePos,event);
        }

        if (this.preChartGlyph && !event._hasfound) {
            this.preChartGlyph.findOnMoveGlyph(mousePos, event);
        }

        if(this.mapControlBar && !event._hasfound){
            this.mapControlBar.findOnMoveGlyph(mousePos, event);
        }
    },

    /**
     * 微信集成的点击找悬浮的徒刑
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    findClickOnGlyph:function(mousePos, event){
        if (this.legendGlyph) {
            if (this.plotGlyph && this.plotGlyph.isLegendInteractive()) {
                this.legendGlyph.findOnMoveGlyph(mousePos, event);
            }
        }

        if (this.plotGlyph && !event._hasfound) {
            this.plotGlyph.findClickOnGlyph(mousePos, event);
        }

        if(this.timeSwitchGlyph &&  !event._hasfound){
            this.timeSwitchGlyph.findOnMoveGlyph(mousePos,event);
        }

        if(this.mapControlBar && !event._hasfound){
            this.mapControlBar.findOnMoveGlyph(mousePos, event);
        }
    },

    /**
     *  图表总体, 主要是响应 提示信息(坐标轴提示, 趋势线提示, 空值错误提示)
     * @param mousePos 鼠标位置
     */
    doOnMove: function (mousePos) {
        if (this.plotGlyph) {
            this.plotGlyph.doOnMove(this.getOnMoveCtx(), mousePos);
        }
    },

    /**
     * 设置自动刷新和联动之前的plotGlyph
     * @param lastGlyph 数据变化之前的glyph
     */
    setLastGlyph:function(lastGlyph){
        if(this.plotGlyph && lastGlyph.plotGlyph){
            this.plotGlyph.setLastGlyph(lastGlyph.plotGlyph);
        }
    },

    /**
     * 布局
     */
    doLayout: function () {
        var layout = new FR.Chart.ChartGlyphLayout();
        layout.bounds = this.getBounds();
        if(this.plotGlyph != null){
            var background = this.getBackground();
            if(background && background.backgroundType == "ColorBackground"){
                this.plotGlyph.wholeChartBackground = FR.Chart.ColorBackground.getInstance(background.color);
            }
        }
        layout.doLayout(this);
    },

    /**
     * 刷新图例标签
     */
    refreshLegendLabel: function () {
        if (this.legendGlyph && this.plotGlyph) {
            var items = this.legendGlyph.items;
            var dataSeries = this.plotGlyph.dataSeries;
            if (items.length == dataSeries.length) {
                for (var i = 0; i < items.length; i++) {
                    if (dataSeries[i].lastState && dataSeries[i].lastState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
                        dataSeries.splice(i, 1);
                        items.splice(i, 1);
                    }
                }

                for (var i = 0; i < items.length; i++) {
                    dataSeries[i].seriesIndex = i;
                    var points = dataSeries[i].points;
                    for (var j = 0; j < points.length; j++) {
                        points[j].seriesIndex = i;
                    }
                }

                for (var i = 0; i < items.length; i++) {
                    items[i].label = dataSeries[i].seriesName;
                }
            }

            //同步系列和图例的颜
            this.legendGlyph.synchronizeLegendColorWithSeries(this.plotGlyph.plotFillStyle, dataSeries);
        }

        if (this.plotGlyph) {
            var dataSeries = this.plotGlyph.dataSeries;
            for (var i = 0; i < dataSeries.length; i++) {
                if (dataSeries[i].lastState && dataSeries[i].lastState == FR.Chart.ChartPaintConstants.STATE_TRANSPARENT) {
                    dataSeries.splice(i, 1);
                }
            }
            for (var i = 0; i < dataSeries.length; i++) {
                dataSeries[i].seriesIndex = i;
                var points = dataSeries[i].points;
                for (var j = 0; j < points.length; j++) {
                    points[j].seriesIndex = i;
                }
            }
        }
    }
});


FR.Chart.BackgroundGlyph = function(parentDom, items, selectedIndex, isNeedRefresh, size){
    this.ctx = null;
    this.iconDeoms = [];
    this.domWidth = 0;
    this.domHeight = 0;
    this.selectedIndex = selectedIndex;
    this.isNeedRefreshButton = isNeedRefresh;
    this.canvas = null;
    this.items = items;
    this.initBackgroundCtx(parentDom, selectedIndex, size);
};
$.extend(FR.Chart.BackgroundGlyph.prototype,{
    initBackgroundCtx:function(parentDom, selectedIndex, size){
        //创建背景的canvas
        this.domWidth = size ? size.width: parentDom.width();
        this.domHeight = size ? size.height: parentDom.height();
        this.canvas = $('<canvas>').css('position', 'absolute').css('left', 0).css('top', 0)
                        .css('width', this.domWidth + 'px').css('height', this.domHeight + 'px');
        this.canvas.appendTo(parentDom);
        if (window.G_vmlCanvasManager) {
            window.G_vmlCanvasManager.initElement(this.canvas[0]);
        }
        this.ctx = this.canvas[0].getContext("2d");

        this.resize(this.domWidth, this.domHeight);
    },

     _fixedPosition:function(event) {
         if(!event){
             event = window.event;
         }
        var ev = event
        if (event.targetTouches && event.targetTouches.length > 0) {
            ev = event.targetTouches[0];
        } else {
            if (event && event.touches) {
                if (event.touches.length > 0) {
                    ev = event.touches[0];
                } else if (event.changedTouches && event.changedTouches.length > 0) {
                    ev = event.changedTouches[0];
                }
            }
        }

        var pos = {x: 0, y: 0};

        var scales = 1;

        pos.x += ev.clientX;	// 加上鼠标在视窗中的位置
        pos.y += ev.clientY;

        var offset = this.canvas.offset();
        pos.x -= offset.left;
        pos.y -= offset.top;

        var contentPane = contentPane;
        var pagePane = undefined;
        if (!FR.BIDezi) {
            if (contentPane) {
                pagePane = contentPane;
            } else if (ev.view && ev.view.contentPane) {
                pagePane = ev.view.contentPane;
            } else {
                if (FR.Browser.isIE()) {
                    try {
                        pagePane = window.parent.contentPane;// frame 权限
                    } catch (e) {
                        pagePane = null;
                    }
                }
            }

            if (pagePane && pagePane.zoom) {
                scales = pagePane.zoom;
            }
        }
        pos.x = pos.x / scales;
        pos.y = pos.y / scales;

        return pos;
    } ,

    init4MouseHandler:function(){
        var doc = this.canvas[0];
        var background = this;
        doc.onmousemove = function(event){
            var mousePos =background._fixedPosition(event);
            background.doOnMove(mousePos);
        }

        doc.onclick = function(event){
            var mousePos = background._fixedPosition(event);
            background.doOnClick(mousePos);
        }
    },

    refresh:function(){

        var width = parseFloat(this.ctx.canvas.style.width);
        var height = parseFloat(this.ctx.canvas.style.height);
        this.ctx.clearRect(0, 0, width, height);

        if(this.isNeedRefreshButton){
            this.iconDeoms[this.iconDeoms.length-1].draw(this.ctx);
        }
        for(var iconIndex = 0; iconIndex < (this.isNeedRefreshButton ?(this.iconDeoms.length-1):this.iconDeoms.length); iconIndex++){
            this.iconDeoms[iconIndex].draw(this.ctx);
        }
    },

    findSelectedIndex:function(mousePos){
        if(mousePos.x < this.domWidth-this.iconDeoms.length*26){
            return -1;
        }
        var xPosition = mousePos.x - (this.domWidth-this.iconDeoms.length*26);
        var xIndex = parseInt(xPosition / 26);
        var yIndex = parseInt(mousePos.y / 26);
        var endIndex = parseInt(this.domWidth / 26) * yIndex + xIndex;
        return this.iconDeoms.length > endIndex ? endIndex : -1;
    },

    doOnMove:function(mousePos){
        var index = this.findSelectedIndex(mousePos);
        var isMoveOnRefresh = false;
        if(this.isNeedRefreshButton){
            isMoveOnRefresh = index=== this.iconDeoms.length-1;
        }

        for(var i =0;i<this.iconDeoms.length;i++){
            if(i !== this.selectedIndex){
                this.iconDeoms[i].state = FR.Chart.ChartIconState.STATE_NORMAL;
            }else{
                this.iconDeoms[i].state = FR.Chart.ChartIconState.STATE_SELECT;
            }
        }

        if(!isMoveOnRefresh && (index < 0 || index >= this.iconDeoms.length)){
            this.refresh();
            return;
        }

        if(index !== -1 && !isMoveOnRefresh && index !== this.selectedIndex){
            if(this.iconDeoms[index].state !== undefined){
                this.iconDeoms[index].state = FR.Chart.ChartIconState.STATE_OVER;
            }
        }

        this.refresh();

        return this.items[index] && !isMoveOnRefresh ? this.items[index].name : null;
    },

    doOnClick:function(mousePos){
        var index = this.findSelectedIndex(mousePos);
        if(index!=-1 && this.isNeedRefreshButton && index === this.iconDeoms.length-1){
            var chartsArray = this.chartWidget.chartArray;
            var fineChart = chartsArray[this.selectedIndex].fineChart;
            if(fineChart.chartAttr){
                fineChart.changeData(fineChart.chartAttr, fineChart.actionModel);
            }else{
                fineChart.changeTableData();
            }
        } else {
            if( index === -1 || index === this.selectedIndex || !this.chartWidget) {
                return ;
            }

            this.chartWidget.changeChartImage(index);
        }
    },

    resize:function(width, height){
        this.domWidth = width;
        this.domHeight = height;

        FR.Chart.ChartUtils.retinaOptimization(this.canvas[0], this.ctx, this.domWidth, this.domHeight);

        if(this.isNeedRefreshButton && this.items.length === 1){
            this.items = [];
        }

        var items = this.items;
        var selectedIndex = this.selectedIndex;
        this.iconDeoms = [];

        //创建各个IconDemo
        var pw = 22;
        var ph = 22;
        var startX =this.domWidth-26*items.length-(this.isNeedRefreshButton?26:0);
        var startY = 0;

        for(var index = 0; index <items.length &&( parseInt(26*(index+1)) / this.domWidth) < 2; index++){
            var bounds = new FR.Chart.Bounds(startX, startY, pw, ph);
            var state = index == selectedIndex ? FR.Chart.ChartIconState.STATE_SELECT : FR.Chart.ChartIconState.STATE_NORMAL;
            var chartOption = items[index];
            var chartType = chartOption.simpleIconType;

            if(!chartType){
                chartType = FR.Chart.Type.getSimpleIconType(chartOption);
            }

            var iconDemo = new FR.Chart[chartType+"SmallIcon"](bounds, state);
            this.iconDeoms.push(iconDemo);

            //更新下下个图表的位置
            startX += 26;
            startY = parseInt(startX / this.domWidth) * 26;
        }

        if(this.isNeedRefreshButton){
            var bounds = new FR.Chart.Bounds(this.domWidth - 26 , startY, pw, ph);
            var refresh = new FR.Chart.RefreshSmallIcon(bounds,FR.Chart.ChartIconState.STATE_NORMAL) ;
            this.iconDeoms.push(refresh);
        }

        this.refresh();
    }
});


FR.Chart.TimeSwitchGlyph = FR.extend(FR.Chart.RectangleGlyph,{
    initConstants: function (js) {
        js = js || {};
        this.visible = FR.pick(js.visible, true);

        this.bounds = null;

        this.switchButtonWidth = 0;

        this.switchButtons = [];

        this.xAxisGlyph = null;

        this.timeSwitchArray = null;

        this.lastClickButton = null;

        this.lastOverButton = null;

        FR.Chart.RectangleGlyph.superclass.initConstants.apply(this, [js]);
    },

    // 根据总体的bounds, 确定每个button啊 什么的各种详细的位置
    // 这个需要在bounds之外, 因为 此次的bounds是和绘图区共宽度的, 如果内减的话 就无法对齐
    doLayout: function () {
        if(!this.timeSwitchArray || this.timeSwitchArray.length <1 || !this.bounds) {
            return;
        }

        var startX = this.bounds.x;


        if(this.switchButtons.length>0){
            for(var index = 0 ;index < this.timeSwitchArray.length ;index++){
                var timeSwitchButton = this.switchButtons[index];
                timeSwitchButton.bounds =  new FR.Chart.Bounds(startX,this.bounds.y,
                    this.switchButtonWidth, this.bounds.height);
                startX += this.switchButtonWidth+2;
            }
            this.globalTimeButton.bounds =  new FR.Chart.Bounds(startX,this.bounds.y,
                this.switchButtonWidth, this.bounds.height);
            this.globalTimeButton.isGlobal = true;
            return;
        }

        for(var index = 0 ;index < this.timeSwitchArray.length ;index++){
            var timeSwitchButton = new FR.Chart.TimeSwitchButton();
            this.switchButtons[index] = timeSwitchButton;
            timeSwitchButton.bounds =  new FR.Chart.Bounds(startX,this.bounds.y,
                this.switchButtonWidth, this.bounds.height);
            timeSwitchButton.time = this.timeSwitchArray[index].timeUnit;
            timeSwitchButton.unit = this.timeSwitchArray[index].timeType;
            startX += this.switchButtonWidth+2;
        }

        //全部按钮
        this.globalTimeButton = new FR.Chart.TimeSwitchButton();
        this.globalTimeButton.bounds =  new FR.Chart.Bounds(startX,this.bounds.y,
            this.switchButtonWidth, this.bounds.height);
        this.globalTimeButton.isGlobal = true;
        //一开始的时候默认选中全部的
        this.globalTimeButton.isPress = true;

    } ,


    findButton:function(mousePos){
        if(!this.timeSwitchArray || this.timeSwitchArray.length <1 || !this.bounds) {
            return null;
        }

        var moveOnButton = null;
        for(var index = 0; index<this.switchButtons.length ; index ++){
            if(this.switchButtons[index].containsPoint(mousePos.x,mousePos.y)){
                moveOnButton =this.switchButtons[index];
                break;
            }
        }
        if(!moveOnButton && this.globalTimeButton.containsPoint(mousePos.x,mousePos.y)){
            moveOnButton = this.globalTimeButton;
        }

        return moveOnButton;
    },

    clearButtonOver:function(){
        if(this.lastOverButton){
            this.lastOverButton.isOver = false;
        }
    },


    clearButtonPress:function(){
        if(this.lastClickButton){
            this.lastClickButton.isPress = false;
            this.lastClickButton.isOver = false;
        }
        this.globalTimeButton.isPress = false;
        this.globalTimeButton.isOver = false;
    },

    clearButtonsChoosen:function(){
        for(var index = 0; index<this.switchButtons.length ; index ++){
            this.switchButtons[index].isPress = false;
        }
        this.globalTimeButton.isPress = false;
    },


    findOnMoveGlyph: function (mousePos, event) {
        this.clearButtonOver();
        this.lastOverButton = this.findButton(mousePos);
        if( this.lastOverButton){
            this.lastOverButton.isOver = true;
            event._lastHover = this;
            event._hasfound = true;
        }
    },

    doOnMove: function (ctx, mousePos) {
        if( this.lastOverButton){
            this.lastOverButton.draw(ctx);
        }
    },

    /**
     * 数据点响应点击
     * @param mousePos 鼠标位置
     * @param event 鼠标事件
     */
    onClick: function (mousePos, event) {
        this.clearButtonPress();
        this.lastClickButton = this.findButton(mousePos);
        if(this.lastClickButton){
            this.lastClickButton.isPress = true;
            this.timeChange();
        }
        event.needRefresh = true;
    },

    timeChange:function(){
        if(!this.lastClickButton || !this.lastClickButton.isPress){
            return;
        }

        //是全部按钮
        if(this.lastClickButton.isGlobal){
            this.xAxisGlyph.zoomBackward=0;
            this.xAxisGlyph.zoomForward=0;
            return ;
        }

        var endDiff = this.xAxisGlyph.getOriginalArrowValue()-
            this.xAxisGlyph.getEndValue4UnitBasedMinDate(this.lastClickButton.unit,parseInt(this.lastClickButton.time));
        this.xAxisGlyph.zoomForward = 0;
        if(endDiff >=0){
            this.xAxisGlyph.onZoomBarRightDown(endDiff,true);
        }else{
            this.xAxisGlyph.zoomBackward=0;
            var startDiff = this.xAxisGlyph.getStartValue4UnitBasedOriginalMaxDate(this.lastClickButton.unit,parseInt(this.lastClickButton.time)) -
                this.xAxisGlyph.getOriginalCrossValue();
            this.xAxisGlyph.onZoomBarLeftDown(startDiff);
        }

    },


    draw: function (ctx) {
        if(!this.timeSwitchArray || this.timeSwitchArray.length <1 || !this.bounds) {
            return;
        }

        for(var index = 0; index<this.switchButtons.length ; index ++){
            this.switchButtons[index].draw(ctx);
        }

        this.globalTimeButton.draw(ctx);
    }

});

$.extend(FR.Chart.TimeSwitchGlyph,{
    /**
     * 时间切换控件的高度比列
     * @static
     * @property
     */
    WIDGET_HEIGHT_SCALE: 0.06,

    /**
     * 最小的控件区域限制高度
     * @static
     * @property
     */
    MIN_WIDGET_HEIGHT: 20,
    /**
     * 最大的控件限制高度
     * @static
     * @property
     */
    MAX_WIDGET_HEIGHT: 30 ,

        //按钮最小宽度
    BUTTOM_MIN_WIDTH:50,

    //按钮最大宽度
    BUTTON_MAX_WIDTH:80,

    //按钮宽度比例
    BUTTON_WIDTH_SCALE:0.12
});



// GlobalDemoGlyph.start
/**
 *    图表放缩时的 缩略demo
 *    @example
 *    var globalDemo = new FR.Chart.GlobalDemoGlyph();
 *
 * @class FR.Chart.GlobalDemoGlyph
 * @extends FR.Chart.ChartJsInit
 *
 * @property {FR.Chart.MoveButton}[js.leftMoveButton] 左侧移动按钮
 * @property {FR.Chart.MoveButton}[js.rightMoveButton] 右侧移动按钮
 * @property {FR.Chart.Bounds}[js.moveBar] 缩略图下方滚动条
 *
 * @property {FR.Chart.PlotGlyph} [js.demoPlot] 缩略的绘图区
 * @property {FR.Chart.Bounds} [js.bounds] 缩略图绘图区的Bounds
 * @property {FR.Chart.AxisGlyph} [js.xAxisGlyph] 绑定的缩放坐标轴 (分类轴)
 * @property {FR.Chart.AxisGlyph}[js.yAxisGlyph] 值轴
 * @property {FR.Chart.Bounds} [js.barBackground] 滚动条背景
 * @property {Boolean}[js.dragStyle] 是否为拖拽样式
 * @property {Boolean}[js.isScroll] 是否支持滚动
 *
 */
FR.Chart.GlobalDemoGlyph = FR.extend(FR.Chart.ChartJsInit, {
    initConstants: function () {

        this.dragBar = null;

        this.demoPlot = null;// 缩略图绘图区
        this.bounds = null;// 换成demoPlot的bounds, 所有的都根据demo的bounds来..外围
        this.xAxisGlyph = null;// 绑定的缩放坐标轴
        this.yAxisGlyph = null;
        this.barBackground = null;
        this.timeSwitchGlyph = null;
        this.isScroll = true;
        this.dragStyle = false;
        this.focusOnLeftButton = false; //焦点是否应该在左按钮上
        this.focusOnRightButton = false; //焦点是否应该在右按钮上
    },

    //确定是不是拖拽左button还是右button
    makeSureDragFocus : function(mouseStartPos){
        if(!this.dragBar || !mouseStartPos ){
            return;
        }
        this.focusOnLeftButton = this.dragBar.isMoveOnLeftDragButton(mouseStartPos.x, mouseStartPos.y) && this.isScroll;
        this.focusOnRightButton = this.dragBar.isMoveOnRightDragButton(mouseStartPos.x, mouseStartPos.y) && this.isScroll;
    },


    // 根据总体的bounds, 确定每个button啊 什么的各种详细的位置
    // 这个需要在bounds之外, 因为 此次的bounds是和绘图区共宽度的, 如果内减的话 就无法对齐
    doLayout: function () {
        if (this.bounds) {
            // 根据现有的 坐标轴的forward和backward等 再次确定下 TODO
            this.initDragBar(this.xAxisGlyph);
            this.barBackground = new FR.Chart.Bounds(this.bounds.x, this.bounds.y + this.bounds.height,
                this.bounds.width, FR.Chart.PreChartGlyph.MOVE_BACKGROUND_HEIGHT - 3);
        }
    },


    // 根据坐标轴的前后缩放位置 初始化移动条的位置
    initDragBar: function (axisGlyph) {// 面积图 计算的时候 少一个分类, 因为是标签在刻度下的
        if (this.demoPlot.xAxisGlyph && axisGlyph) {
            var categoryCount = Math.max(1, this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue());
            var minWidth = this.bounds.width / categoryCount;

            var width = (categoryCount + axisGlyph.zoomForward + axisGlyph.zoomBackward) * this.bounds.width / categoryCount;
            width = width < minWidth ? minWidth : width;

            var x = Math.abs(axisGlyph.zoomBackward) * this.bounds.width / categoryCount + this.bounds.x;
            x = Math.min(x, this.bounds.x + this.bounds.width - width);// 不能超出右侧Bar范围边界了.
            this.dragBar = new FR.Chart.GlobalDragBar();
            this.dragBar.bounds = new FR.Chart.Bounds(x,this.bounds.y,width,this.bounds.height);
            this.dragBar.parentBounds = this.bounds;
        }
    },


    draw: function (ctx) {
        if(!ctx){     return; } ctx.save();
        this.drawBackgroundDemoPath(ctx);
        this.dragBar.draw(ctx);
        ctx.restore();
    },



    // 缩略图的Path 路径, 只画简单的. 其他各种阴影等效果都不要.
    drawBackgroundDemoPath: function (ctx) {
        if (this.demoPlot) {
            this.demoPlot.draw4Demo(ctx);
        }
    },


    findOnMoveGlyph: function (mousePos, event) {
        if (this.xAxisGlyph) {
            var obj = this.xAxisGlyph;
            if (obj && obj.isZoom) {
                if(!this.dragBar.bounds.containsPoint(mousePos.x, mousePos.y)){
                    return;
                }
                event._lastHover = this;
                event._hasfound = true;
                event._draggingTarget = event._isMouseDown ? this : null;
                if(this.dragBar.isMoveOnDragButton(mousePos.x, mousePos.y)){
                    event._mouseStyle = "arrow";
                    if(event._isMouseDown) {
                        event._draggingTarget = this;
                        event.needRefresh = !this.dragStyle;
                        this.dragStyle = true;
                    }else{
                        event.needRefresh = this.dragStyle;
                        this.dragStyle = false;
                    }
                } else{
                    event._mouseStyle = "hand";
                }
            }
        }
    },


    /**
     * 在GlobalDemoGlyph里响应拖拽结束事件
     * @param mouseStartPos 拖拽开始的位置
     * @param mousePos 当前的鼠标位置
     */
    onDragEnd: function (mouseStartPos, mousePos) {
        this.focusOnLeftButton = false;
         this.focusOnRightButton = false;
    },

    onDrag: function (mouseStartPos, mousePos, ctx4OnMove, event, stupidBrowser) {
        if (stupidBrowser) {
            return;
        }
        if(!this.lastMousePos){
            this.lastMousePos = mouseStartPos;
        }

        var isAcition = false;
        if (this.xAxisGlyph && (this.xAxisGlyph.position == FR.Chart.Constants.BOTTOM || this.xAxisGlyph.position == FR.Chart.Constants.TOP)) {
            var offsetX = Math.abs(mouseStartPos.x - mousePos.x);

            var maxNumber = this.xAxisGlyph.getOriginalArrowValue() - this.xAxisGlyph.getOriginalCrossValue();
            var oneTickLength = this.xAxisGlyph.axisLength / maxNumber;
            var number = parseInt(offsetX / oneTickLength);
            if (number < 1) {
                event.zoomRefresh = false;
                return;
            }

            if(event._mouseStyle === "hand"){
                if(mouseStartPos.x < mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarIncrease(number);
                }else if(mouseStartPos.x > mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarDecrease(number);
                }
            }else if(this.dragBar && this.dragBar.isDragButtonCross()){
                if(this.lastMousePos.x < mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                    this.focusOnLeftButton = false;
                    this.focusOnRightButton = true;
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }else if(this.lastMousePos.x > mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                    this.focusOnLeftButton = true;
                    this.focusOnRightButton = false;
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }
            }else if(this.dragBar && this.focusOnLeftButton){
                if(this.lastMousePos.x < mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarLeftDown(number);
                    if(!isAcition  || this.dragBar.isMoveOnRightDragButton(mousePos.x, mousePos.y)){
                        isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                        this.focusOnLeftButton = false;
                        this.focusOnRightButton = true;
                    }
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }else if(this.lastMousePos.x > mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }
            }else if(this.dragBar && this.focusOnRightButton){
                if(this.lastMousePos.x < mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarRightUp(number);
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }else if(this.lastMousePos.x > mousePos.x){
                    isAcition = this.xAxisGlyph.onZoomBarRightDown(number);
                    if(!isAcition || this.dragBar.isMoveOnLeftDragButton(mousePos.x, mousePos.y)){
                        isAcition = this.xAxisGlyph.onZoomBarLeftUp(number);
                        this.focusOnLeftButton = true;
                        this.focusOnRightButton = false;
                    }
                    if(this.timeSwitchGlyph){
                        this.timeSwitchGlyph.clearButtonsChoosen();
                    }
                }
            }


            if (isAcition) {
                event.zoomRefresh = isAcition;
                mouseStartPos.x = mouseStartPos.x + number * (mouseStartPos.x - mousePos.x < 0 ? oneTickLength : -oneTickLength);
                this.initDragBar(this.xAxisGlyph);
            }
        }

        this.lastMousePos = mousePos;
    },

    /**
     * 鼠标到滑块的时候鼠标形状变成抓紧的手型
     * @param mousePos
     * @param event
     */
    onPress: function (mousePos, event) {
        if (this.xAxisGlyph) {
            var obj = this.xAxisGlyph;
            if (obj && obj.isZoom) {
                if (this.dragBar.bounds.containsPoint(mousePos.x,mousePos.y)) {
                    event.cursor = ChartHandler.DRAG;
                }
            }
        }
    }
});

$.extend(FR.Chart.GlobalDemoGlyph, /**@class FR.Chart.PreChartGlyph*/{
    /**
     * 缩放绘图区占的比例
     * @static
     * @property
     */
    DEMO_HEIGHT_SCALE: 0.09,

    /**
     * 最小的缩略图区域限制高度
     * @static
     * @property
     */
    MIN_DEMO_HEIGHT: 20,
    /**
     * 最大的缩略图限制高度
     * @static
     * @property
     */
    MAX_DEMO_HEIGHT: 42,
    /**
     * 缩略图分类的 间隔
     * @static
     * @property
     */
    PRE_CATE_GAP: 8


});
// GlobalDemo.end

FR.Chart.GlobalDragBar = FR.extend(FR.Chart.ChartJsInit,{
    initConstants: function () {
        this.bounds = null;
        this.parentBounds = null;
    },


    draw: function (ctx) {
        if(!ctx){     return; } ctx.save();
        this.drawLeftAreaGray(ctx);
        this.drawRightAreaGray(ctx);
        this.drawLeftButton(ctx);
        this.drawRightButton(ctx);
        this.drawUpDownLine(ctx)
        ctx.restore();
    } ,


    drawUpDownLine:function(ctx){
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        ctx.strokeStyle = FR.Chart.GlobalDragBar.DARK_BLUE;
        ctx.lineWidth = 1;
        ctx.lineCap = 'round';
        var arcWidth = 3;
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+arcWidth, y),
            new FR.Chart.Point2D(x+width -arcWidth, y));
        line.draw(ctx);
        line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+arcWidth, y+height),
                    new FR.Chart.Point2D(x+width -arcWidth, y+height));
        line.draw(ctx);
    },

    drawLeftAreaGray:function(ctx){
        ctx.fillStyle = FR.Chart.GlobalDragBar.GRAY;
        var arcWidth = 3;
        var arcHeight = 3;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var parentX = Math.round(this.parentBounds.x), parentY = Math.round(this.parentBounds.y),
            parentHeight = Math.round(this.parentBounds.height), parentWidth = Math.round(this.parentBounds.width);
        ctx.beginPath();
        ctx.moveTo(parentX, parentY + arcHeight);
        ctx.lineTo(parentX, parentY + parentHeight - arcHeight);
        ctx.quadraticCurveTo(parentX, parentY + parentHeight, parentX + arcWidth, parentY + parentHeight);
        ctx.lineTo(x+arcWidth,y+height);
        ctx.lineTo(x+arcWidth,y);
        ctx.lineTo(parentX+arcWidth,parentY);
        ctx.quadraticCurveTo(parentX,parentY,parentX, parentY + arcHeight);
        ctx.closePath();
        ctx.fill();
    },

    drawRightAreaGray:function(ctx){
        ctx.fillStyle = FR.Chart.GlobalDragBar.GRAY;
        var arcWidth = 3;
        var arcHeight = 3;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var parentX = Math.round(this.parentBounds.x), parentY = Math.round(this.parentBounds.y),
            parentHeight = Math.round(this.parentBounds.height), parentWidth = Math.round(this.parentBounds.width);
        ctx.beginPath();

        ctx.moveTo(parentX+parentWidth, parentY + arcHeight);
        ctx.lineTo(parentX+parentWidth, parentY + parentHeight - arcHeight);
        ctx.quadraticCurveTo(parentX+parentWidth, parentY + parentHeight, parentX+parentWidth- arcWidth, parentY + parentHeight);
        ctx.lineTo(x+width-arcWidth,y+height);
        ctx.lineTo(x+width-arcWidth,y);
        ctx.lineTo(parentX+parentWidth-arcWidth,parentY);
        ctx.quadraticCurveTo(parentX+parentWidth,parentY,parentX+parentWidth, parentY + arcHeight);
        ctx.closePath();
        ctx.fill();
    },




    drawLeftButton:function(ctx){
        ctx.fillStyle = FR.Chart.GlobalDragBar.DARK_BLUE;
        var arcWidth = 3;
        var arcHeight = 3;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);

        ctx.beginPath();
        ctx.moveTo(x, y + arcHeight);
        ctx.lineTo(x, y + height - arcHeight);
        ctx.quadraticCurveTo(x, y + height, x + arcWidth, y + height);
        ctx.lineTo(x+ FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH,y+height);
        ctx.lineTo(x+ FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH,y);
        ctx.lineTo(x+arcWidth,y);
        ctx.quadraticCurveTo(x,y,x,y+arcHeight);
        ctx.closePath();
        ctx.fill();

        var lineGap = height/3;
        var lineLen = height/3;
        ctx.strokeStyle = FR.Chart.GlobalDragBar.LIGHT_BLUE;
        ctx.lineWidth = 2;
        ctx.lineCap = 'round';
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+3, this.bounds.y+lineGap),
            new FR.Chart.Point2D(x+3, this.bounds.y+lineLen+lineGap));
        line.draw(ctx);
        line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+7, this.bounds.y+lineGap),
                    new FR.Chart.Point2D(x+7, this.bounds.y+lineLen+lineGap));
        line.draw(ctx);
    },

    drawRightButton:function(ctx){
        ctx.fillStyle = FR.Chart.GlobalDragBar.DARK_BLUE;
        var arcWidth = 3;
        var arcHeight = 3;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        ctx.beginPath();
        ctx.moveTo(x+width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH,y);
        ctx.lineTo(x + width - arcWidth, y);
        ctx.quadraticCurveTo(x + width, y, x + width, y + arcHeight);
        ctx.lineTo(x + width, y +height- arcHeight);
        ctx.quadraticCurveTo(x + width, y+height, x + width - arcWidth, y+height);
        ctx.lineTo(x+width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH,y+height);
        ctx.lineTo(x+width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH,y);
        ctx.closePath();
        ctx.fill();

        var lineGap = height/3;
        var lineLen =height/3;
        ctx.strokeStyle = FR.Chart.GlobalDragBar.LIGHT_BLUE;
        ctx.lineWidth = 2;
        ctx.lineCap = 'round';
        var line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+width-3, this.bounds.y+lineGap),
            new FR.Chart.Point2D(x+width-3, this.bounds.y+lineLen+lineGap));
        line.draw(ctx);
        line = new FR.Chart.Line2D(new FR.Chart.Point2D(x+width-7, this.bounds.y+lineGap),
                    new FR.Chart.Point2D(x+width-7, this.bounds.y+lineLen+lineGap));
        line.draw(ctx);
    },

    isMoveOnDragButton: function (mouseX, mouseY) {
        var gap = 2;
        var handWidth = 10;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var leftBounds = new FR.Chart.Bounds(x-gap, y-gap, FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH+gap*2, height+gap*2);
        var rightBounds = new FR.Chart.Bounds(x+width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH-gap-handWidth,y-gap,FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH+gap*2+handWidth,height+gap*2);
        return leftBounds.contains(mouseX, mouseY,1,1) || rightBounds.contains(mouseX,mouseY,1,1);
    },

    /**
     *是否重叠
     */
    isDragButtonCross : function(){
        return this.bounds.width <= FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH*2;
    },

    isMoveOnLeftDragButton: function (mouseX, mouseY) {
        var gap = 2;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var leftBounds = new FR.Chart.Bounds(x-gap, y-gap, FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH+gap*2, height+gap*2);
        return leftBounds.contains(mouseX, mouseY,1,1);
    },

    isOnBarWhite:function(mouseX, mouseY){
        if(this.isDragButtonCross()){
            return false;
        }
        var gap = 2;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var whiteBounds = new FR.Chart.Bounds(x+FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH+gap ,y-gap,width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH*2-gap*2,height+gap*2);
        return whiteBounds.contains(mouseX,mouseY,1,1);
    },

    isMoveOnRightDragButton: function (mouseX, mouseY) {
        var gap = 2;
        var handWidth = 10;
        var x = Math.round(this.bounds.x), y = Math.round(this.bounds.y),
            height = Math.round(this.bounds.height), width = Math.round(this.bounds.width);
        var rightBounds = new FR.Chart.Bounds(x+width-FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH-gap - handWidth ,y-gap,FR.Chart.GlobalDragBar.DRAG_BAR_WIDTH+gap*2 +handWidth,height+gap*2);
        return  rightBounds.contains(mouseX,mouseY,1,1);
    }

});




$.extend(FR.Chart.GlobalDragBar,{

    //深蓝色
    DARK_BLUE:'rgb(41,171,226)',

    //浅蓝色
    LIGHT_BLUE:'rgb(122,218,224)',

    GRAY:'rgba(220,221,221,0.6)',

    /**
     * 拖拽的蓝色按钮的宽度
     */
    DRAG_BAR_WIDTH:10
});